(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    912108,      24483]
NotebookOptionsPosition[    844380,      22586]
NotebookOutlinePosition[    865073,      23036]
CellTagsIndexPosition[    864092,      23002]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Trees", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 11}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
This chapter is devoted to exploring the computational aspects of the study \
of trees. Recall from the textbook that a tree is a connected simple graph \
with no simple circuits.\
\>", "Text"],

Cell[TextData[{
 "First, we will discuss how to represent, display, and work with trees using \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Specifically, we will see how to represent rooted trees and ordered \
rooted trees in addition to simple trees. We then use these representations \
to explore many of the topics discussed in the textbook. In particular, we \
will see how to use binary trees to store data in such a way as to make \
searching more efficient and we will see an implementation of Huffman codes. \
We will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to carry out the different tree traversal methods described in the text. \
We will see how to construct spanning trees using both depth-first and \
breadth-first search and how to use backtracking to solve a variety of \
interesting problems. Finally, we will implement Prim's algorithm and \
Kruskal's algorithm for finding spanning trees of minimum weight for a \
weighted graph."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.1 Introduction to Trees", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "In this section we will focus on how to construct trees in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and how to check basic properties, such as determining if a tree is \
balanced. To begin, we will consider the simplest case, unrooted trees, \
before moving on to rooted and ordered trees."
}], "Text"],

Cell[CellGroupData[{

Cell["Unrooted Trees", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 "Recall that a tree is defined to be a graph that is undirected, connected, \
and has no simple circuits (or cycles). To create a tree, we can just create \
a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " as we did in the previous chapter."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"firstTree", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e"}, {
      Null, {{1, 2}, {1, 3}, {2, 4}, {2, 5}}}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8944271909999159, 1.7888543819998317`}, 
           0.020874891668923312`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4472135954999579, 0.8944271909999159}, 
           0.020874891668923312`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3416407864998738`, 0.8944271909999159}, 
           0.020874891668923312`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.020874891668923312`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8944271909999159, 0.}, 0.020874891668923312`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.296119561098699`, 
       324.8812836844811}, {-214.3271504119902, 198.95572522134523`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The first thing you may notice is that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has automatically drawn this in the traditional way, which suggests that \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " recognized the graph as a tree. The ",
 StyleBox[ButtonBox["TreeGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraphQ"], "FunctionLink"],
 " function can be used to check if a graph is a tree."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraphQ", "[", "firstTree", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the order of the vertices can affect which vertex ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draws at the top of the tree. For example, in the tree above, we can \
encourage ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to draw the vertex ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " at the top of the tree by giving a list of the vertices as the first \
argument. We list the vertices in layered order, that is, the top vertex will \
be first, followed by the vertices we wish to see in the second layer, then \
the vertices in the third layer and so forth. Note that changing the order of \
vertices within a layer can affect their horizontal position. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<b\>\"", ",", "\"\<e\>\"", ",", "\"\<d\>\"", ",", "\"\<a\>\"", ",", 
     "\"\<c\>\""}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"b", "e", "d", "a", "c"}, {
      Null, {{4, 1}, {4, 5}, {1, 3}, {1, 2}}}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 1.6329931618554523`}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.8164965809277261}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.6329931618554523`, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{1.6329931618554523`, 0.}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 349.7886658983125}, {-178.06066589831266`, 
       161.928}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "There is a limit to the amount of influence this provides, however. For \
example, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a strong preference for shorter, more balanced, trees. Attempting to \
use the vertex order to create a deeper tree will often be ignored. For \
example, below we attempt, and fail, to get ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " at the top of the tree."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<e\>\"", ",", "\"\<b\>\"", ",", "\"\<d\>\"", ",", "\"\<a\>\"", ",", 
     "\"\<c\>\""}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"e", "b", "d", "a", "c"}, {
      Null, {{4, 2}, {4, 5}, {2, 3}, {2, 1}}}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.8164965809277261}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 1.6329931618554523`}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.6329931618554523`, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{1.6329931618554523`, 0.}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 349.7886658983125}, {-178.0606658983126, 
       161.92800000000005`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides functions for creating certain kinds of trees. Recall that a \
rooted tree is called ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "-ary if every vertex has no more than ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " children. When ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "2"}], TraditionalForm]]],
 ", we say that it is a binary tree. Also recall that the height of a rooted \
tree is the maximum number of levels in the tree, i.e., the height is the \
length of the largest path from the root to any other vertex. (",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " says that the tree has ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " levels, which is synonymous with height ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 ".) Given the height as the first argument and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " as the second, the function ",
 StyleBox[ButtonBox["CompleteKaryTree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CompleteKaryTree"], "FunctionLink"],
 " produces the complete ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "-ary tree of height ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 ". If the second argument is omitted, it defaults as 2, producing a complete \
binary tree of the specified height."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteKaryTree", "[", 
  RowBox[{"3", ",", "4", ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
       20, 21}, {Null, 
       SparseArray[
       Automatic, {21, 21}, 0, {
        1, {{0, 4, 9, 14, 19, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 
          36, 37, 38, 39, 40}, {{2}, {3}, {4}, {5}, {1}, {6}, {7}, {8}, {9}, {
          1}, {10}, {11}, {12}, {13}, {1}, {14}, {15}, {16}, {17}, {1}, {
          18}, {19}, {20}, {21}, {2}, {2}, {2}, {2}, {3}, {3}, {3}, {3}, {
          4}, {4}, {4}, {4}, {5}, {5}, {5}, {5}}}, Pattern}]}, {
      ImagePadding -> 5, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$11", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$12", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$13", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$14", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$15", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$16", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$17", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$18", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$19", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$20", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$21", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.061862178478973, 0.8164965809277261}, 
           0.04911078443249145], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.6123724356957946, 0.4082482904638631}, 
           0.04911078443249145], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.2453655975512468`, 0.4082482904638631}, 
           0.04911078443249145], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.878358759406699, 0.4082482904638631}, 
           0.04911078443249145], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{5.511351921262151, 0.4082482904638631}, 
           0.04911078443249145], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.04911078443249145], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.4082482904638631, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8164965809277261, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["8", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{1.2247448713915892`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["9", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{1.6329931618554523`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["10", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"], 
       TagBox[{
         TagBox[
          DiskBox[{2.041241452319315, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$11"], 
         InsetBox[
          FormBox["11", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$11", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$11"], 
       TagBox[{
         TagBox[
          DiskBox[{2.4494897427831783`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$12"], 
         InsetBox[
          FormBox["12", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$12", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$12"], 
       TagBox[{
         TagBox[
          DiskBox[{2.8577380332470415`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$13"], 
         InsetBox[
          FormBox["13", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$13", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$13"], 
       TagBox[{
         TagBox[
          DiskBox[{3.2659863237109046`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$14"], 
         InsetBox[
          FormBox["14", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$14", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$14"], 
       TagBox[{
         TagBox[
          DiskBox[{3.6742346141747673`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$15"], 
         InsetBox[
          FormBox["15", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$15", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$15"], 
       TagBox[{
         TagBox[
          DiskBox[{4.08248290463863, 0.}, 0.04911078443249145], "DynamicName",
           BoxID -> "VertexID$16"], 
         InsetBox[
          FormBox["16", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$16", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$16"], 
       TagBox[{
         TagBox[
          DiskBox[{4.4907311951024935`, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$17"], 
         InsetBox[
          FormBox["17", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$17", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$17"], 
       TagBox[{
         TagBox[
          DiskBox[{4.898979485566357, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$18"], 
         InsetBox[
          FormBox["18", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$18", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$18"], 
       TagBox[{
         TagBox[
          DiskBox[{5.30722777603022, 0.}, 0.04911078443249145], "DynamicName",
           BoxID -> "VertexID$19"], 
         InsetBox[
          FormBox["19", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$19", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$19"], 
       TagBox[{
         TagBox[
          DiskBox[{5.715476066494083, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$20"], 
         InsetBox[
          FormBox["20", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$20", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$20"], 
       TagBox[{
         TagBox[
          DiskBox[{6.123724356957946, 0.}, 0.04911078443249145], 
          "DynamicName", BoxID -> "VertexID$21"], 
         InsetBox[
          FormBox["21", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$21", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$21"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{11.000000000000004`, 
       360.9303246388799}, {-40.21032463887959, 23.116973550473283`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->5]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Rooted Trees", "Subsection",
 CellTags->"Section1b"],

Cell["\<\
Next we consider rooted trees. Recall that a rooted tree is a directed graph \
whose underlying undirected graph is a tree and in which one vertex is \
designated as the root and all edges are directed away from the root. For \
example, the following graph is a rooted tree.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"firstRooted", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<g\>\""}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g"}, {{{1, 2}, {1, 3}, {1, 4}, {2, 5}, {
       2, 6}, {3, 7}}, Null}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.031839716508632585`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3772342551463013`, 1.5739820058814873`}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.3934955014703718, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.5739820058814873`, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.360973008822231, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02547454323586698], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7869910029407436, 0.}, 0.02547454323586698], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1.5739820058814873`, 0.}, 0.02547454323586698], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       350.897382972899}, {-125.169382972899, 108.4262751470043}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " drew this graph as a tree, just as you would hope. In fact, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not make a distinction between directed edges and undirected edges \
for trees, and ",
 StyleBox[ButtonBox["TreeGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraphQ"], "FunctionLink"],
 " will identify this as a tree."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraphQ", "[", "firstRooted", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "While ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does recognize trees with directed edges, it does not identify a root. \
When drawing the tree, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s emphasis is on minimizing the height, as the following \
example illustrates."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"secondRooted", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "4"}], ",", 
      RowBox[{"2", "\[Rule]", "5"}], ",", 
      RowBox[{"4", "\[Rule]", "6"}], ",", 
      RowBox[{"4", "\[Rule]", "7"}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 
      7}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {4, 6}, {4, 7}}, Null}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.03404239059267076], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.7869910029407436}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9837387536759296, 1.5739820058814873`}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.0223470323119013], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.1804865044111155`, 0.7869910029407436}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.9674775073518591`, 0.7869910029407436}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7869910029407436, 0.}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1.5739820058814873`, 0.}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 350.84537622862524`}, {-146.11737622862515`, 
       130.09798994085637`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has drawn this tree with 2 at the top of the image, despite the fact that \
it does not satisfy the definition of a root, as it has an edge directed \
towards it. "
}], "Text"],

Cell[TextData[{
 "To draw the tree as intended, we can apply ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 ". This is related to the ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " function discussed in the previous chapter, used for multigraphs. ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 " accepts up to three arguments. The first argument is a list of edges that \
form the tree. The first argument can also be a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. The second argument, which is optional, specifies where the root \
of the tree should be drawn. Possible positions are: ",
 StyleBox[ButtonBox["Top",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Top"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Bottom",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Bottom"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Left",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Left"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Right",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Right"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Center",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Center"], "FunctionLink"],
 ". The third argument, which is also optional but can only be used when the \
second is present, specifies one of the vertices in the tree to be the root. \
After all arguments are given, you may use the same options as described for ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " in Chapter 10. The following illustrates how to draw ",
 StyleBox["secondRooted", "Input"],
 " with the root 1 in its proper position."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreePlot", "[", 
  RowBox[{"secondRooted", ",", "Top", ",", "1", ",", 
   RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{1.3416407864998738`, 2.6832815729997477`}, {
    0.8944271909999159, 1.788854381999832}, {1.7888543819998317`, 
    1.788854381999832}, {0.4472135954999579, 0.894427190999916}, {
    1.3416407864998738`, 0.894427190999916}, {0., 0.}, {0.8944271909999159, 
    0.}}, {
     {RGBColor[0.5, 0., 0.], Arrowheads[{{0.030286132981502786`, 0.7}}], 
      ArrowBox[{1, 2}], ArrowBox[{1, 3}], ArrowBox[{2, 4}], ArrowBox[{2, 5}], 
      ArrowBox[{4, 6}], ArrowBox[{4, 7}]}, {InsetBox[
       FrameBox["1",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["2",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["3",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["4",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4], InsetBox[
       FrameBox["5",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 5], InsetBox[
       FrameBox["6",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 6], InsetBox[
       FrameBox["7",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 7]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{1.3416407864998738`, 2.6832815729997477`}, {
     0.8944271909999159, 1.788854381999832}, {1.7888543819998317`, 
     1.788854381999832}, {0.4472135954999579, 0.894427190999916}, {
     1.3416407864998738`, 0.894427190999916}, {0., 0.}, {0.8944271909999159, 
     0.}}]& ],
  AspectRatio->1.224744871391589,
  FrameTicks->None,
  ImagePadding->10,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Identifying Roots and Rooted Trees", "Subsubsection"],

Cell[TextData[{
 "As mentioned above, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["TreeGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraphQ"], "FunctionLink"],
 " will tell you if a graph is a tree. For directed graphs, it will return \
false if the graph is not a rooted tree. However, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not include a function for finding the root of a rooted tree."
}], "Text"],

Cell["\<\
We begin by creating an example of a directed graph which is not a rooted \
tree, but whose underlying undirected graph is a tree.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"notRooted", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "4"}], ",", 
      RowBox[{"2", "\[Rule]", "5"}], ",", 
      RowBox[{"4", "\[Rule]", "6"}], ",", 
      RowBox[{"7", "\[Rule]", "4"}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 
      7}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {4, 6}, {7, 4}}, Null}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.028937842778793414`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[{
         DynamicLocation["VertexID$1", Automatic, Center], 
         DynamicLocation["VertexID$2", Automatic, Center]}], 
       ArrowBox[{
         DynamicLocation["VertexID$1", Automatic, Center], 
         DynamicLocation["VertexID$3", Automatic, Center]}], 
       ArrowBox[{
         DynamicLocation["VertexID$2", Automatic, Center], 
         DynamicLocation["VertexID$4", Automatic, Center]}], 
       ArrowBox[{
         DynamicLocation["VertexID$2", Automatic, Center], 
         DynamicLocation["VertexID$5", Automatic, Center]}], 
       ArrowBox[{
         DynamicLocation["VertexID$4", Automatic, Center], 
         DynamicLocation["VertexID$6", Automatic, Center]}], 
       ArrowBox[{
         DynamicLocation["VertexID$7", Automatic, Center], 
         DynamicLocation["VertexID$4", Automatic, Center]}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 3.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 2.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 2.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 1.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 1.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 2.}, 0.030239520958083826`], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{14.627219622606997`, 
       293.2798177480974}, {-215.14392540265132`, 199.59786916229731`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This is identical to ",
 StyleBox["secondRooted", "Input"],
 ", except for the direction of the edge between vertex 4 and vertex 7. Note \
that ",
 StyleBox[ButtonBox["TreeGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraphQ"], "FunctionLink"],
 " identifies it correctly."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraphQ", "[", "notRooted", "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["However, the underlying undirected graph is a tree.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraphQ", "[", 
  RowBox[{"UndirectedGraph", "[", "notRooted", "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "It will be useful to be able to distinguish, however, between a rooted tree \
and a non-rooted tree. That is, we will need a function that returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " for ",
 StyleBox["firstRooted", "Input"],
 ", but ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " on its undirected underlying graph."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rootedTreeQ", "[", "T_Graph", "]"}], ":=", 
  RowBox[{
   RowBox[{"TreeGraphQ", "[", "T", "]"}], "&&", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"DirectedGraphQ", "[", "T", "]"}], "||", 
     RowBox[{
      RowBox[{"Length", "[", 
       RowBox[{"VertexList", "[", "T", "]"}], "]"}], "<", "2"}]}], 
    ")"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The condition on the length of the vertex list is included in the above \
because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will not recognize a singleton graph or the empty graph as being directed. \
As promised, this distinguishes between ",
 StyleBox["firstRooted", "Input"],
 " and its underlying graph."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rootedTreeQ", "[", "firstRooted", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rootedTreeQ", "[", 
  RowBox[{"UndirectedGraph", "[", "firstRooted", "]"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The root of a rooted tree is necessarily the unique vertex with in-degree \
0. We can use ",
 StyleBox[ButtonBox["VertexInDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexInDegree"], "FunctionLink"],
 " to obtain a list of the in-degrees of the vertices of a directed graph."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexInDegree", "[", "firstRooted", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " function, applied to the list of in-degree, will tell us which location in \
the list is 0. Note that the output of ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " is a list of the position specifications, themselves lists. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{
   RowBox[{"VertexInDegree", "[", "firstRooted", "]"}], ",", "0"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", "1", "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since ",
 StyleBox[ButtonBox["TreeGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraphQ"], "FunctionLink"],
 " has identified this as a tree, we know that there will be exactly one \
vertex with in-degree 0, so we confidently use ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") to obtain the root\[CloseCurlyQuote]s position in the list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Position", "[", 
   RowBox[{
    RowBox[{"VertexInDegree", "[", "firstRooted", "]"}], ",", "0"}], "]"}], 
  "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since the output of ",
 StyleBox[ButtonBox["VertexInDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexInDegree"], "FunctionLink"],
 " matches the order of vertices from ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 ", we can use the previous result with ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ")",
 " ",
 "and ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 " to obtain the name of the root."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VertexList", "[", "firstRooted", "]"}], "[", 
  RowBox[{"[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Position", "[", 
     RowBox[{
      RowBox[{"VertexInDegree", "[", "firstRooted", "]"}], ",", "0"}], "]"}], 
    "[", 
    RowBox[{"[", 
     RowBox[{"1", ",", "1"}], "]"}], "]"}], "\[IndentingNewLine]", "]"}], 
  "]"}]], "Input"],

Cell[BoxData["\<\"a\"\>"], "Output"]
}, Open  ]],

Cell["We create a function based on this example.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findRoot", "[", 
   RowBox[{"G_", "?", "rootedTreeQ"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"VertexList", "[", "G", "]"}], "[", 
   RowBox[{"[", 
    RowBox[{
     RowBox[{"Position", "[", 
      RowBox[{
       RowBox[{"VertexInDegree", "[", "G", "]"}], ",", "0"}], "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"1", ",", "1"}], "]"}], "]"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "With this function in hand, we can automate the use of ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 " to draw a rooted tree in its proper form."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"rootedPlot", "[", 
   RowBox[{
    RowBox[{"G_", "?", "rootedTreeQ"}], ",", "opts___"}], "]"}], ":=", 
  RowBox[{"TreePlot", "[", 
   RowBox[{"G", ",", "Top", ",", 
    RowBox[{"findRoot", "[", "G", "]"}], ",", "opts"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We use this function to draw ",
 StyleBox["secondRooted", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rootedPlot", "[", 
  RowBox[{"secondRooted", ",", 
   RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{1.3416407864998738`, 2.6832815729997477`}, {
    0.8944271909999159, 1.788854381999832}, {1.7888543819998317`, 
    1.788854381999832}, {0.4472135954999579, 0.894427190999916}, {
    1.3416407864998738`, 0.894427190999916}, {0., 0.}, {0.8944271909999159, 
    0.}}, {
     {RGBColor[0.5, 0., 0.], Arrowheads[{{0.030286132981502786`, 0.7}}], 
      ArrowBox[{1, 2}], ArrowBox[{1, 3}], ArrowBox[{2, 4}], ArrowBox[{2, 5}], 
      ArrowBox[{4, 6}], ArrowBox[{4, 7}]}, {InsetBox[
       FrameBox["1",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["2",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["3",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["4",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4], InsetBox[
       FrameBox["5",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 5], InsetBox[
       FrameBox["6",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 6], InsetBox[
       FrameBox["7",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 7]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{1.3416407864998738`, 2.6832815729997477`}, {
     0.8944271909999159, 1.788854381999832}, {1.7888543819998317`, 
     1.788854381999832}, {0.4472135954999579, 0.894427190999916}, {
     1.3416407864998738`, 0.894427190999916}, {0., 0.}, {0.8944271909999159, 
     0.}}]& ],
  AspectRatio->1.224744871391589,
  FrameTicks->None,
  ImagePadding->10,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The drawback to ",
 StyleBox["rootedPlot", "Input"],
 " is that, while it produces the expected image of the tree, its output is \
an image not a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. In particular, ",
 StyleBox["secondRooted", "Input"],
 " still displays with 2 at the top of the image."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["secondRooted"], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 
      7}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {4, 6}, {4, 7}}, Null}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.03404239059267076], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.7869910029407436}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9837387536759296, 1.5739820058814873`}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.0223470323119013], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.1804865044111155`, 0.7869910029407436}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.9674775073518591`, 0.7869910029407436}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7869910029407436, 0.}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1.5739820058814873`, 0.}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 350.84537622862524`}, {-146.11737622862515`, 
       130.09798994085637`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can, however, leverage the ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 " function to modify the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object in order to cause it to draw as a rooted tree. To do this, we need \
to first be able to modify arguments to a function."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Modifying arguments with ",
 StyleBox["HoldFirst", "Input"]
}], "Subsubsection"],

Cell[TextData[{
 "We use the ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " attribute to make arguments modifiable. Normally when you provide an \
argument to a function, the function cannot assign to the argument. Consider \
the following simple function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"five", "=", "5"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"addOne", "[", "n_", "]"}], ":=", 
  RowBox[{"n", "=", 
   RowBox[{"n", "+", "1"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"addOne", "[", "five", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Set", "::", "setraw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot assign to raw object \
\[NoBreak]\\!\\(5\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/Set/setraw\\\", ButtonNote -> \
\\\"Set::setraw\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData["6"], "Output"]
}, Open  ]],

Cell[TextData[{
 "This function produces an error when we attempt to assign a value to the \
argument ",
 StyleBox["n", "Input"],
 ". This is a feature of programming in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that is designed to encourage good programming practices. In particular, \
for a function to modify one of its arguments, we need to be very explicit \
that we really want to do so. This helps prevent unintended consequences \
\[LongDash] accidentally modifying an argument can cause serious bugs in your \
programs."
}], "Text"],

Cell[TextData[{
 "You can think about what's going on in the ",
 StyleBox["addOne", "Input"],
 " function this way: when you call the function with the syntax ",
 StyleBox["addOne[five]", "Input"],
 ", all of the occurrences of the name ",
 StyleBox["n", "Input"],
 " are resolved to the object ",
 StyleBox["5", "Input"],
 ", which is the value stored in ",
 StyleBox["five", "Input"],
 ". So the command ",
 StyleBox["n=n+1", "Input"],
 " resolves to ",
 StyleBox["5=5+1", "Input"],
 ". Clearly that's not a legal command."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " attribute provides a way around this. If we attach the ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " attribute to ",
 StyleBox["addOne", "Input"],
 ", by applying ",
 StyleBox[ButtonBox["SetAttributes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetAttributes"], "FunctionLink"],
 " to the function name and the attribute, we are telling ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to not evaluate the parameter ",
 StyleBox["n", "Input"],
 " into the object it refers to, but to evaluate the argument into a symbol. \
Instead of evaluating the symbol ",
 StyleBox["five", "Input"],
 " to get the object ",
 StyleBox["5", "Input"],
 " and replacing the parameter ",
 StyleBox["n", "Input"],
 " with ",
 StyleBox["5", "Input"],
 ", the symbol ",
 StyleBox["five", "Input"],
 " is held, and the parameter ",
 StyleBox["n", "Input"],
 " is replaced with the symbol ",
 StyleBox["five", "Input"],
 ". This means that the command becomes ",
 StyleBox["five=five+1", "Input"],
 "."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"addOne2", ",", "HoldFirst"}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"addOne2", "[", "n_", "]"}], ":=", 
  RowBox[{"n", "=", 
   RowBox[{"n", "+", "1"}]}]}]}], "Input"],

Cell["\<\
Now we can see that this function modifies the variable it is given.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"addOne2", "[", "five", "]"}]], "Input"],

Cell[BoxData["6"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["five"], "Input"],

Cell[BoxData["6"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To summarize: we can create functions that modify an argument that is \
passed as a symbol by applying ",
 StyleBox[ButtonBox["SetAttributes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetAttributes"], "FunctionLink"],
 " to the function name and the attribute ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 ". This allows the parameter to appear on the left side of an assignment and \
modify the symbol passed as input to the function."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Rooting a ",
 StyleBox["Graph", "Input"],
 " Object"
}], "Subsubsection"],

Cell[TextData[{
 "Now, in order to draw a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object as a rooted tree, we need to better understand the output of ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 ". Consider the example below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"treeplotExample", "=", 
  RowBox[{"TreePlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", "\[Rule]", "2"}], "}"}], ",", "Top", ",", "1"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0., 1.}, {0., 0.}}, {
     {RGBColor[0.5, 0., 0.], LineBox[{{1, 2}}]}, 
     {RGBColor[0, 0, 0.7], 
      TagBox[
       TooltipBox[PointBox[1],
        "1"],
       Annotation[#, 1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[2],
        "2"],
       Annotation[#, 2, "Tooltip"]& ]}}],
   Annotation[#, VertexCoordinateRules -> {{0., 1.}, {0., 0.}}]& ],
  AspectRatio->Automatic,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We use ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " to expose the underlying data."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"TreePlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", "\[Rule]", "2"}], "}"}], ",", "Top", ",", "1"}], "]"}], "//",
   "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Graphics", "[", 
    RowBox[{
     RowBox[{"Annotation", "[", 
      RowBox[{
       RowBox[{"GraphicsComplex", "[", 
        RowBox[{
         RowBox[{"List", "[", 
          RowBox[{
           RowBox[{"List", "[", 
            RowBox[{"0.`", ",", "1.`"}], "]"}], ",", 
           RowBox[{"List", "[", 
            RowBox[{"0.`", ",", "0.`"}], "]"}]}], "]"}], ",", 
         RowBox[{"List", "[", 
          RowBox[{
           RowBox[{"List", "[", 
            RowBox[{
             RowBox[{"RGBColor", "[", 
              RowBox[{"0.5`", ",", "0.`", ",", "0.`"}], "]"}], ",", 
             RowBox[{"Line", "[", 
              RowBox[{"List", "[", 
               RowBox[{"List", "[", 
                RowBox[{"1", ",", "2"}], "]"}], "]"}], "]"}]}], "]"}], ",", 
           RowBox[{"List", "[", 
            RowBox[{
             RowBox[{"RGBColor", "[", 
              RowBox[{"0", ",", "0", ",", "0.7`"}], "]"}], ",", 
             RowBox[{"Tooltip", "[", 
              RowBox[{
               RowBox[{"Point", "[", "1", "]"}], ",", "1"}], "]"}], ",", 
             RowBox[{"Tooltip", "[", 
              RowBox[{
               RowBox[{"Point", "[", "2", "]"}], ",", "2"}], "]"}]}], "]"}]}],
           "]"}], ",", 
         RowBox[{"List", "[", "]"}]}], "]"}], ",", 
       RowBox[{"Rule", "[", 
        RowBox[{"VertexCoordinateRules", ",", 
         RowBox[{"List", "[", 
          RowBox[{
           RowBox[{"List", "[", 
            RowBox[{"0.`", ",", "1.`"}], "]"}], ",", 
           RowBox[{"List", "[", 
            RowBox[{"0.`", ",", "0.`"}], "]"}]}], "]"}]}], "]"}]}], "]"}], 
     ",", 
     RowBox[{"Rule", "[", 
      RowBox[{"FrameTicks", ",", "None"}], "]"}], ",", 
     RowBox[{"Rule", "[", 
      RowBox[{"PlotRange", ",", "All"}], "]"}], ",", 
     RowBox[{"Rule", "[", 
      RowBox[{"PlotRangePadding", ",", 
       RowBox[{"Scaled", "[", "0.1`", "]"}]}], "]"}], ",", 
     RowBox[{"Rule", "[", 
      RowBox[{"AspectRatio", ",", "Automatic"}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Notice, in the middle of the output above, the ",
 StyleBox[ButtonBox["VertexCoordinateRules",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinateRules"], "FunctionLink"],
 " symbol. This is always present in the output from a ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 " and is identified, via a ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Input"],
 "), with a list of vertex locations. Our strategy will be to extract those \
locations and impose them on the corresponding ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "To extract the vertex coordinates, we make use of the ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") operator. We typically use ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") with an expression and a rule or a list of rules as below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"x", "^", "2"}], "+", 
   RowBox[{"5", "y"}]}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", "3"}], ",", 
    RowBox[{"y", "\[Rule]", "7"}]}], "}"}]}]], "Input"],

Cell[BoxData["44"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The output from ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 " contains a rule identifying the symbol ",
 StyleBox[ButtonBox["VertexCoordinateRules",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinateRules"], "FunctionLink"],
 " with the list of vertex locations. But it is not a list of rules, so it \
cannot be used with ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 "). The ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " function, however, will apply to any expression and produce a list of all \
the subexpressions matching a pattern. The first argument to ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " is an expression, the second is a pattern to search for, and the third \
optional argument is a level specification. Since the ",
 StyleBox[ButtonBox["VertexCoordinateRules",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinateRules"], "FunctionLink"],
 " rule is below the top level, we will need to tell ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " to delve down to find it. The easist level specification to use here is ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 ". Observe the result of applying ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " to our simple example with pattern ",
 StyleBox["_Rule", "Input"],
 " and level specification ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{"treeplotExample", ",", "_Rule", ",", "Infinity"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"VertexCoordinateRules", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.`", ",", "1.`"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.`", ",", "0.`"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", "None"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"PlotRangePadding", "\[Rule]", 
    RowBox[{"Scaled", "[", "0.1`", "]"}]}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", "Automatic"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using this as the list of rules in conjunction with the ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") operator allows us to access the list of vertex locations."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexCoordinateRules", "/.", 
  RowBox[{"Cases", "[", 
   RowBox[{"treeplotExample", ",", "_Rule", ",", "Infinity"}], 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.`", ",", "1.`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.`", ",", "0.`"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now that we know how to extract the vertex coordinates from the result of \
",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 ", we need to be able to set the locations in a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. We do this for an individual vertex by using the ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " function in the form ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"PropertyValue", "[", "{"}], 
   StyleBox["graph", "CodeVar"], ",", 
   StyleBox["vertex", "CodeVar"], "}", ",", "VertexCoordinates", "]", "=", 
   "{", 
   StyleBox["x", "CodeVar"], ",", 
   StyleBox["y", "CodeVar"], "}"}]], "myCode"],
 ". Our function will loop over all the vertices of the graph and assign the \
vertex\[CloseCurlyQuote]s coordinate to that obtained from the ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 ". "
}], "Text"],

Cell[TextData[{
 "The function is below. It uses ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " in order to modify the graph object it is passed."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"drawRooted", ",", "HoldFirst"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"drawRooted", "[", 
   RowBox[{
    RowBox[{"G_", "?", "rootedTreeQ"}], ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"plot", ",", "vertList", ",", "vertLocs"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"plot", "=", 
      RowBox[{"TreePlot", "[", 
       RowBox[{"G", ",", "Top", ",", 
        RowBox[{"findRoot", "[", "G", "]"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vertList", "=", 
      RowBox[{"VertexList", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vertLocs", "=", 
      RowBox[{"VertexCoordinateRules", "/.", 
       RowBox[{"Cases", "[", 
        RowBox[{"plot", ",", "_Rule", ",", "Infinity"}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "vertList", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"G", ",", 
            RowBox[{"vertList", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
          "VertexCoordinates"}], "]"}], "=", 
        RowBox[{"vertLocs", "[", 
         RowBox[{"[", "i", "]"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "G"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Appling this function to ",
 StyleBox["secondRooted", "Input"],
 " moves the vertices so that it is drawn in the typical places."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drawRooted", "[", "secondRooted", "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 
      7}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {4, 6}, {4, 7}}, Null}, {
      ImagePadding -> 10, 
       VertexCoordinates -> {{1.3416407864998738`, 2.6832815729997477`}, {
        0.8944271909999159, 1.788854381999832}, {1.7888543819998317`, 
        1.788854381999832}, {0.4472135954999579, 0.894427190999916}, {
        1.3416407864998738`, 0.894427190999916}, {0., 0.}, {
        0.8944271909999159, 0.}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.030286132981502786`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3416407864998738`, 2.6832815729997477`}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8944271909999159, 1.788854381999832}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.7888543819998317`, 1.788854381999832}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4472135954999579, 0.894427190999916}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3416407864998738`, 0.894427190999916}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.027924334567009212`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8944271909999159, 0.}, 0.027924334567009212`], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{14.628923110791934`, 
       293.32640701418063`}, {-215.10540839901583`, 199.5962338136397}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell["Note that this effect is permanent. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData["secondRooted"], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 
      7}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {4, 6}, {4, 7}}, Null}, {
      ImagePadding -> 10, 
       VertexCoordinates -> {{1.3416407864998738`, 2.6832815729997477`}, {
        0.8944271909999159, 1.788854381999832}, {1.7888543819998317`, 
        1.788854381999832}, {0.4472135954999579, 0.894427190999916}, {
        1.3416407864998738`, 0.894427190999916}, {0., 0.}, {
        0.8944271909999159, 0.}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.030286132981502786`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3416407864998738`, 2.6832815729997477`}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8944271909999159, 1.788854381999832}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.7888543819998317`, 1.788854381999832}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4472135954999579, 0.894427190999916}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3416407864998738`, 0.894427190999916}, 
           0.027924334567009212`], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.027924334567009212`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8944271909999159, 0.}, 0.027924334567009212`], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{14.628923110791934`, 
       293.32640701418063`}, {-215.10540839901492`, 199.5962338136397}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Parents, Children, Leaves, and Internal Vertices of Rooted Trees\
\>", "Subsection",
 CellTags->"Section1c"],

Cell["\<\
We now consider functions related to identifying particular vertices in a \
rooted tree.\
\>", "Text"],

Cell["\<\
We begin with the question of whether one vertex is the parent of another. \
Given the two vertices, checking this requires determining whether the \
directed edge from the parent to the child is actually in the tree. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"parentQ", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "p_", ",", "c_"}], "]"}], ":=", 
  RowBox[{"EdgeQ", "[", 
   RowBox[{"T", ",", 
    RowBox[{"DirectedEdge", "[", 
     RowBox[{"p", ",", "c"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"parentQ", "[", 
  RowBox[{"firstRooted", ",", "\"\<b\>\"", ",", "\"\<f\>\""}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"parentQ", "[", 
  RowBox[{"firstRooted", ",", "\"\<b\>\"", ",", "\"\<d\>\""}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Next, we consider the question of finding the parent of a given vertex. \
This can be done by searching the ",
 StyleBox[ButtonBox["EdgeList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeList"], "FunctionLink"],
 " output for those edges that have the given vertex at the terminal end. \
Assuming the graph is in fact a rooted tree, there can be at most one such \
edge. If the vertex is the root, there will be no parent and the function \
will return ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ". Note the second argument in ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " means that it will search the edge list for those edges from the variable ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " to the given edge ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " but will put only ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " in the output."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findParent", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "v_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"P", ",", "p"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"P", "=", 
      RowBox[{"Cases", "[", 
       RowBox[{
        RowBox[{"EdgeList", "[", "T", "]"}], ",", 
        RowBox[{
         RowBox[{"DirectedEdge", "[", 
          RowBox[{"p_", ",", "v"}], "]"}], "\[Rule]", "p"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "P", "]"}], "\[NotEqual]", "1"}], ",", "Null", 
       ",", 
       RowBox[{"P", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findParent", "[", 
  RowBox[{"firstRooted", ",", "\"\<d\>\""}], "]"}]], "Input"],

Cell[BoxData["\<\"a\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"findParent", "[", 
  RowBox[{"firstRooted", ",", "\"\<a\>\""}], "]"}]], "Input"],

Cell["\<\
For the related question of determining all children of the given vertex, we \
take the same approach but with the given vertex at the initial end.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findChildren", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "v_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "c", "}"}], ",", 
    RowBox[{"Cases", "[", 
     RowBox[{
      RowBox[{"EdgeList", "[", "T", "]"}], ",", 
      RowBox[{
       RowBox[{"DirectedEdge", "[", 
        RowBox[{"v", ",", "c_"}], "]"}], "\[Rule]", "c"}]}], "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findChildren", "[", 
  RowBox[{"firstRooted", ",", "\"\<a\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findChildren", "[", 
  RowBox[{"firstRooted", ",", "\"\<f\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["findChildren", "Input"],
 " function also indicates how we can test a vertex to determine if it is an \
internal vertex or a leaf."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"internalVertexQ", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "v_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Length", "[", 
    RowBox[{"findChildren", "[", 
     RowBox[{"T", ",", "v"}], "]"}], "]"}], "\[NotEqual]", "0"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"leafQ", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "v_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Length", "[", 
    RowBox[{"findChildren", "[", 
     RowBox[{"T", ",", "v"}], "]"}], "]"}], "\[Equal]", "0"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"internalVertexQ", "[", 
  RowBox[{"firstRooted", ",", "\"\<a\>\""}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"leafQ", "[", 
  RowBox[{"firstRooted", ",", "\"\<a\>\""}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"leafQ", "[", 
  RowBox[{"firstRooted", ",", "\"\<f\>\""}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can determine all the leaves of a given tree by testing each vertex with \
",
 StyleBox["leafQ", "Input"],
 ". Here we use ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " in order to test all of the vertices of the graph with a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") based on ",
 StyleBox["leafQ", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findLeaves", "[", 
   RowBox[{"T_", "?", "rootedTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "V", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V", "=", 
      RowBox[{"VertexList", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Select", "[", 
      RowBox[{"V", ",", 
       RowBox[{
        RowBox[{"leafQ", "[", 
         RowBox[{"T", ",", "#"}], "]"}], "&"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLeaves", "[", "firstRooted", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"d\"\>", ",", "\<\"e\"\>", ",", "\<\"f\"\>", ",", "\<\"g\"\>"}],
   "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Ordered Rooted Trees", "Subsection",
 CellTags->"Section1d"],

Cell["\<\
Recall that an ordered rooted tree is a rooted tree in which the children of \
each internal vertex are ordered. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Representing Ordered Rooted Trees", "Subsubsection"],

Cell[TextData[{
 "To represent an ordered rooted tree in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we'll need to store the order of children. There are many ways to \
accomplish this, but perhaps the most straightforward is to mark each vertex \
with its order among its siblings. By way of illustration, we'll make an \
ordered version of ",
 StyleBox["firstRooted", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["firstRooted"], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g"}, {{{1, 2}, {1, 3}, {1, 4}, {2, 5}, {
       2, 6}, {3, 7}}, Null}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.031839716508632585`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3772342551463013`, 1.5739820058814873`}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.3934955014703718, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.5739820058814873`, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.360973008822231, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02547454323586698], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7869910029407436, 0.}, 0.02547454323586698], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1.5739820058814873`, 0.}, 0.02547454323586698], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       350.897382972899}, {-125.16938297289923`, 108.4262751470043}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " automatically draws the vertices in alphabetical order. But suppose \
instead we wanted the children of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " to be in the order ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ", and the children of ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " to be in the order ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " then ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ". Internally, we'll represent this by assigning, for each vertex, a \
property called \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote]. We set \
the order of the root to be 0, and for all other vertices, the order \
attribute will represent the position of that vertex among its siblings. In \
our example, ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " will have \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] 1, ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " will have \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] 2, and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " will have \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] 3. We will \
define a function that will use ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " to set the \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] property \
for each vertex."
}], "Text"],

Cell["\<\
The first argument to this function will be the name of a rooted tree. The \
second argument will be a list of values representing the order value to be \
assigned to each vertex. The function will loop through the vertices of the \
tree and set the \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] \
property.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"setChildOrder", "::", "argx"}], "=", 
   "\"\<List of orders is of incorrect length.\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"setChildOrder", "[", 
   RowBox[{
    RowBox[{"G_", "?", "rootedTreeQ"}], ",", "orderL_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"T", "=", "G"}], ",", "V", ",", "v"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V", "=", 
      RowBox[{"VertexList", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Check", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "V", "]"}], "\[NotEqual]", 
          RowBox[{"Length", "[", "orderL", "]"}]}], ",", 
         RowBox[{"Message", "[", 
          RowBox[{"setChildOrder", "::", "argx"}], "]"}]}], "]"}], ",", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "V", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"T", ",", 
            RowBox[{"V", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", "\"\<order\>\""}], 
         "]"}], "=", 
        RowBox[{"orderL", "[", 
         RowBox[{"[", "i", "]"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Since the order values in the list given to the function must match the \
order of the vertices that is returned from ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 ", it is a good idea to double-check the output of ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 " before using this function. Also note that this function outputs a new \
graph, it does not modify the original input, so we will need to make an \
assignment to the output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexList", "[", "firstRooted", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"e\"\>", ",", "\<\"f\"\>", ",", "\<\"g\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"firstOrdered", "=", 
  RowBox[{"setChildOrder", "[", 
   RowBox[{"firstRooted", ",", 
    RowBox[{"{", 
     RowBox[{
     "0", ",", "2", ",", "1", ",", "3", ",", "2", ",", "1", ",", "1"}], 
     "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g"}, {{{1, 2}, {1, 3}, {1, 4}, {2, 5}, {
       2, 6}, {3, 7}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "d" -> {"order" -> 3}, "c" -> {"order" -> 1}, "f" -> {"order" -> 1}, 
         "b" -> {"order" -> 2}, "e" -> {"order" -> 2}, "g" -> {"order" -> 1}, 
         "a" -> {"order" -> 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.031839716508632585`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3772342551463013`, 1.5739820058814873`}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.3934955014703718, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.5739820058814873`, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.360973008822231, 0.7869910029407436}, 
           0.02547454323586698], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02547454323586698], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7869910029407436, 0.}, 0.02547454323586698], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1.5739820058814873`, 0.}, 0.02547454323586698], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       350.897382972899}, {-125.1693829728986, 108.4262751470043}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the vertices in ",
 StyleBox["firstOrdered", "Input"],
 " now have the \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] property \
set, but we still need to make the tree drawn in the appropriate way."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Test for Ordered Rooted Trees", "Subsubsection"],

Cell[TextData[{
 "Now that we know how to represent an ordered rooted tree in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we will create a function to test a graph object and ensure that it does \
in fact represent an ordered rooted tree. This will help us make functions we \
write later be more robust. The requirement for being an ordered rooted tree \
are: the object must be a tree, it must be directed, every vertex must have \
the order property set, and the root\[CloseCurlyQuote]s order must be 0. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"orderedRootedTreeQ", "[", "T_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "v", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", "rootedTreeQ"}], ",", 
        RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Do", "[", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], "===",
            "$Failed"}], ",", 
          RowBox[{"Throw", "[", "False", "]"}]}], "]"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"{", 
         RowBox[{"v", ",", 
          RowBox[{"VertexList", "[", "T", "]"}]}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", 
             RowBox[{"findRoot", "[", "T", "]"}]}], "}"}], ",", 
           "\"\<order\>\""}], "]"}], "\[NotEqual]", "0"}], ",", 
        RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orderedRootedTreeQ", "[", "firstRooted", "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orderedRootedTreeQ", "[", "firstOrdered", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Drawing Ordered Rooted Trees", "Subsubsection"],

Cell[TextData[{
 "While ",
 StyleBox["firstOrdered", "Input"],
 " is now officially an ordered rooted tree and is storing the order of \
children, it will not be drawn with children in the proper order. To \
influence the order in which the vertices appear in the drawing, we will use \
the fact that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draws vertices in the same order as is output by ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 ". Consider the following example,"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orderExample1", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "5"}], ",", 
      RowBox[{"1", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "\"\<bannana\>\""}], ",", 
      RowBox[{"2", "\[Rule]", "\"\<apple\>\""}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      1, 2, 5, 3, "bannana", "apple"}, {{{1, 2}, {1, 3}, {1, 4}, {2, 5}, {2, 
       6}}, Null}, {ImagePadding -> 10, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.033600216651739116`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.2247448713915892`, 1.6329931618554523`}, 
           0.022945218072026957`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4082482904638631, 0.8164965809277261}, 
           0.022945218072026957`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.2247448713915892`, 0.8164965809277261}, 
           0.022945218072026957`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.041241452319315, 0.8164965809277261}, 
           0.022945218072026957`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.022945218072026957`], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"bannana\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8164965809277261, 0.}, 0.022945218072026957`], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"apple\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       350.856109087606}, {-146.12810908760372`, 130.0833322317783}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexList", "[", "orderExample1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "5", ",", "3", ",", "\<\"bannana\"\>", 
   ",", "\<\"apple\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The order of the vertex list is determined by the order of the edges. For \
example, the edge 1\[Rule]5 appears before the edge 1\[Rule]3 in the \
definition of the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ", so vertex 5 appears before vertex 3. When drawing the tree, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses the order of the vertices in the vertex list to determine the \
relative positions of vertices within a level. In the above, since vertex 5 \
appears before vertex 3, it also appears to the left."
}], "Text"],

Cell[TextData[{
 "By explicitly providing a list of vertices as an argument to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ", we can influence the positions in the image."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orderExample2", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "1", ",", "5", ",", "2", ",", "3", ",", "\"\<apple\>\"", ",", 
      "\"\<bannana\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "5"}], ",", 
      RowBox[{"1", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "\"\<bannana\>\""}], ",", 
      RowBox[{"2", "\[Rule]", "\"\<apple\>\""}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      1, 5, 2, 3, "apple", "bannana"}, {{{1, 3}, {1, 2}, {1, 4}, {3, 6}, {3, 
       5}}, Null}, {ImagePadding -> 10, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.0362469412612264], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 1.6329931618554523`}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.8164965809277261}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.6329931618554523`, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.4082482904638631, 0.}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"apple\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{1.2247448713915892`, 0.}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"bannana\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 350.7886658983125}, {-178.06066589831192`, 
       161.92800000000005`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell["\<\
We will use this feature in order to properly draw ordered rooted graphs. \
Given an ordered rooted tree, we'll explicitly sort the vertices based on the \
order property. \
\>", "Text"],

Cell[TextData[{
 "To create the ordered list of vertices, we will begin by initializing a \
symbol, ",
 StyleBox["OVerts", "Input"],
 ", to the list containing only the root and we initialize a counter ",
 StyleBox["i", "Input"],
 " to 1. We then use ",
 StyleBox["findChildren", "Input"],
 " to find all of the children of the root. We sort the children according to \
their order property, using ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " with a ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") as the second argument that compares the order properties. Once the \
children are sorted, they are appended to the ",
 StyleBox["OVerts", "Input"],
 " list. Then, increment the counter ",
 StyleBox["i", "Input"],
 " and repeat. At each step, the counter ",
 StyleBox["i", "Input"],
 " is used as an index into ",
 StyleBox["OVerts", "Input"],
 " to determine which vertex is being processed. If that vertex has any \
children, they are sorted and added to the list."
}], "Text"],

Cell[TextData[{
 "Once the vertex list is properly ordered, we recreate the graph. We use ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " again to allow this function to modify its argument."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"plotORTree", "[", 
   RowBox[{
    RowBox[{"T_", "?", "orderedRootedTreeQ"}], ",", "opts___"}], "]"}], ":=", 
  
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "E", ",", "R", ",", "OVerts", ",", "i", ",", "numverts", ",", "children",
       ",", "G", ",", "VP", ",", "v", ",", "p"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"E", "=", 
      RowBox[{"EdgeList", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"R", "=", 
      RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"OVerts", "=", 
      RowBox[{"{", "R", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "OVerts", "]"}]}], ",", "\[IndentingNewLine]", 
       
       RowBox[{
        RowBox[{"children", "=", 
         RowBox[{"findChildren", "[", 
          RowBox[{"T", ",", 
           RowBox[{"OVerts", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"children", "=", 
         RowBox[{"Sort", "[", 
          RowBox[{"children", ",", 
           RowBox[{
            RowBox[{
             RowBox[{"PropertyValue", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"T", ",", "#1"}], "}"}], ",", "\"\<order\>\""}], 
              "]"}], "\[LessEqual]", 
             RowBox[{"PropertyValue", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"T", ",", "#2"}], "}"}], ",", "\"\<order\>\""}], 
              "]"}]}], "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"OVerts", "=", 
         RowBox[{"Join", "[", 
          RowBox[{"OVerts", ",", "children"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "++"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"G", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{"OVerts", ",", "E", ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"G", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"T", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", "OVerts"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "G"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell["\<\
Finally, we can draw our ordered tree correctly. Note that to retain  the \
order, we make an explicit reassignment of the symbol.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"firstOrdered", "=", 
  RowBox[{"plotORTree", "[", 
   RowBox[{"firstOrdered", ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "c", "b", "d", "g", "f", "e"}, {{{1, 3}, {1, 2}, {1, 4}, {3, 7}, {
       3, 6}, {2, 5}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "d" -> {"order" -> 3}, "c" -> {"order" -> 1}, "f" -> {"order" -> 1}, 
         "b" -> {"order" -> 2}, "e" -> {"order" -> 2}, "g" -> {"order" -> 1}, 
         "a" -> {"order" -> 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.03404239059267076], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9837387536759296, 1.5739820058814873`}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.7869910029407436}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.1804865044111155`, 0.7869910029407436}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.9674775073518591`, 0.7869910029407436}, 
           0.0223470323119013], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.0223470323119013], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7869910029407436, 0.}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1.5739820058814873`, 0.}, 0.0223470323119013], 
          "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 350.84537622862524`}, {-146.11737622862526`, 
       130.09798994085637`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Properties of Trees", "Subsection",
 CellTags->"Section1e"],

Cell["\<\
We conclude this section with functions for calculating the level of a \
vertex, the height of a tree, and for determining if a tree is balanced or \
not.\
\>", "Text"],

Cell["\<\
The level of a vertex in a rooted tree is the length of the path from the \
root to the vertex. We compute the level in reverse. We first initialize a \
counter to 0. If the given vertex is the root, then the level is 0. \
Otherwise, increment the counter and look at the parent of the original \
vertex. If this vertex is the root, then the counter holds the level. \
Otherwise, increment the counter and back up to the parent of the current \
vertex. When we reach the root, then the value of the counter is the level of \
the vertex.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findLevel", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "V_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"v", ",", "level"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"level", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"v", "=", "V"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"findParent", "[", 
         RowBox[{"T", ",", "v"}], "]"}], "=!=", "Null"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"level", "++"}], ";", "\[IndentingNewLine]", 
        RowBox[{"v", "=", 
         RowBox[{"findParent", "[", 
          RowBox[{"T", ",", "v"}], "]"}]}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "level"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can compute the levels of ",
 Cell[BoxData[
  FormBox["g", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " in the ",
 StyleBox["firstOrdered", "Input"],
 " tree."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLevel", "[", 
  RowBox[{"firstOrdered", ",", "\"\<g\>\""}], "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLevel", "[", 
  RowBox[{"firstOrdered", ",", "\"\<d\>\""}], "]"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]],

Cell["\<\
The height of a tree is the maximum of the levels of the vertices. We can \
compute the height by checking each vertex's level. We use a variable to hold \
the largest level and each time we find a vertex with a level larger than the \
current maximum, we update the variable.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findHeight", "[", 
   RowBox[{"T_", "?", "rootedTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"v", ",", "height", ",", "level"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"height", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"level", "=", 
         RowBox[{"findLevel", "[", 
          RowBox[{"T", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"level", ">", "height"}], ",", 
          RowBox[{"height", "=", "level"}]}], "]"}]}], "\[IndentingNewLine]", 
       ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "T", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "height"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findHeight", "[", "firstOrdered", "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Recall that a rooted tree of height ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is balanced if all leaves are at level ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "-", "1"}], TraditionalForm]]],
 ". To determine if a given tree is balanced, we need to: (1) calculate the \
height of the tree, (2) find all the leaves of the tree with the ",
 StyleBox["findLeaves", "Input"],
 " function we wrote earlier, and (3) test each leaf's level and return ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " if it is higher than level ",
 Cell[BoxData[
  FormBox[
   RowBox[{"h", "-", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"balancedTreeQ", "[", 
   RowBox[{"T_", "?", "rootedTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"height", ",", "leaves", ",", "v"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"height", "=", 
      RowBox[{"findHeight", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"leaves", "=", 
      RowBox[{"findLeaves", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"findLevel", "[", 
             RowBox[{"T", ",", "v"}], "]"}], "<", 
            RowBox[{"height", "-", "1"}]}], ",", 
           RowBox[{"Throw", "[", "False", "]"}]}], "]"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", "leaves"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We see that our ",
 StyleBox["firstOrdered", "Input"],
 " tree is balanced."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"balancedTreeQ", "[", "firstOrdered", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["11.2 Applications of Trees", "Section",
 CellTags->"Section2"],

Cell["\<\
This section is concerned with applications of trees, particularly binary \
trees. Specifically, we consider the use of trees in binary search algorithms \
as well as in Huffman codes. The reason we use binary trees is that we can \
use the binary structure of the tree to make binary decisions (e.g., less \
than/greater than) regarding search paths or insertion of elements. \
Additionally, the binary tree structure corresponds well with the way \
computers store information as binary data.\
\>", "Text"],

Cell["\<\
Recall that a tree is called a binary tree if all vertices in the tree have \
at most two children. In this section, we will be using ordered binary trees. \
The fact that the vertices are ordered means that the children of a vertex \
can be considered to be either a left child or a right child. By convention, \
we consider the left child to be the first child and the right child to be \
second.\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Representation in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"]
}], "Subsection",
 CellTags->"Section2a"],

Cell[TextData[{
 "Before we get to the applications, we will discuss how we can represent \
binary trees in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and develop some functions to help us manipulate them. Since a binary tree \
is a particular kind of ordered rooted tree, our work here should be \
consistent with what we did above."
}], "Text"],

Cell[CellGroupData[{

Cell["A Binary Tree Test", "Subsubsection"],

Cell[TextData[{
 "We will construct a test, ",
 StyleBox["binaryTreeQ", "Input"],
 " to test whether a graph represents a binary tree. We will impose three \
conditions for an object to be considered a binary tree. First, it must be an \
ordered rooted tree, i.e., it must pass ",
 StyleBox["orderedRootedTreeQ", "Input"],
 ". Second, it must be binary, that is, each vertex must have at most two \
children. And third, each vertex other than the root must have order \
attribute 1 or 2, with 1 indicating that the vertex is a left child and 2 for \
right. The root will have its order attribute set to 0."
}], "Text"],

Cell["\<\
First, let's construct an example of a binary tree. The tree we construct is \
the binary search tree for the letters D, B, F, A, C, E.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"firstBinaryTree", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<D\>\"", ",", "\"\<B\>\"", ",", "\"\<F\>\"", ",", "\"\<A\>\"", ",", 
      "\"\<C\>\"", ",", "\"\<E\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<D\>\"", "\[Rule]", "\"\<B\>\""}], ",", 
      RowBox[{"\"\<D\>\"", "\[Rule]", "\"\<F\>\""}], ",", 
      RowBox[{"\"\<B\>\"", "\[Rule]", "\"\<A\>\""}], ",", 
      RowBox[{"\"\<B\>\"", "\[Rule]", "\"\<C\>\""}], ",", 
      RowBox[{"\"\<F\>\"", "\[Rule]", "\"\<E\>\""}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "D", "B", "F", "A", "C", "E"}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 
       6}}, Null}, {ImagePadding -> 10, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.03552057386854859], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.0859306069076735`, 1.7374889710522776`}, 
           0.02044525157230459], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"D\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4343722427630694, 0.8687444855261388}, 
           0.02044525157230459], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"B\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.7374889710522776`, 0.8687444855261388}, 
           0.02044525157230459], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"F\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02044525157230459], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"A\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8687444855261388, 0.}, 0.02044525157230459], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"C\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{1.7374889710522776`, 0.}, 0.02044525157230459], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"E\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       351.80803137831595`}, {-178.08003137831554`, 161.92800000000005`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"firstBinaryTree", "=", 
   RowBox[{"setChildOrder", "[", 
    RowBox[{"firstBinaryTree", ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "2", ",", "1", ",", "2", ",", "1"}], 
      "}"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now that we have an example, let's create the test. To check that the tree \
is in fact binary, we can use the ",
 StyleBox[ButtonBox["VertexOutDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexOutDegree"], "FunctionLink"],
 " function to count the number of children of each vertex. If any vertex has \
more than two children, then the tree is not binary. And we'll make sure that \
each vertex is marked with an order of 1 or 2, or 0 in the case of the vertex."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"binaryTreeQ", "[", "T_Graph", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"R", ",", "v", ",", "vpos"}], "}"}], ",", "\[IndentingNewLine]", 
    
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"orderedRootedTreeQ", "[", "T", "]"}]}], ",", 
        RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"R", "=", 
       RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"VertexOutDegree", "[", 
              RowBox[{"T", ",", "v"}], "]"}], ">", "2"}], ",", 
            RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"vpos", "=", 
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"vpos", "\[Equal]", "0"}], "&&", 
               RowBox[{"v", "\[NotEqual]", "R"}]}], ")"}], "||", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"vpos", "\[NotEqual]", "0"}], "&&", 
               RowBox[{"vpos", "\[NotEqual]", "1"}], "&&", 
               RowBox[{"vpos", "\[NotEqual]", "2"}]}], ")"}]}], ",", 
            RowBox[{"Throw", "[", "False", "]"}]}], "]"}]}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", 
           RowBox[{"VertexList", "[", "T", "]"}]}], "}"}]}], "]"}], 
       "\[IndentingNewLine]", 
       RowBox[{"Throw", "[", "True", "]"}]}]}], "\[IndentingNewLine]", 
     "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"binaryTreeQ", "[", "firstBinaryTree", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Drawing Binary Trees", "Subsubsection"],

Cell[TextData[{
 "Next we'll write a function for drawing binary trees. Note that, while ",
 StyleBox["plotORTree", "Input"],
 " will draw a binary tree, that function will display vertices that are only \
children directly below their parent rather than to the left or right. Here \
we take more control over the locations by explicitly calculating the \
position of each vertex."
}], "Text"],

Cell[TextData[{
 "Think about the tree as being drawn in a 1 by 1 box with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"0", ",", "0"}], ")"}], TraditionalForm]]],
 " at the bottom left corner. The ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "-coordinate of each vertex will depend on the height of the tree and the \
level of the vertex. Specifically, the ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " coordinate of any vertex is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "-", 
    RowBox[{"l", "/", "h"}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["l", TraditionalForm]]],
 " is the level of the vertex and ",
 Cell[BoxData[
  FormBox["h", TraditionalForm]]],
 " is the height of the tree. This way, the root, which is at level 0, has ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "-coordinate 1 and the vertices in the last level have ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "-coordinate 0."
}], "Text"],

Cell[TextData[{
 "For the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinates, the position of the vertex depends on the position of its \
parent and its level. We start by setting the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-position of the root to ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "2"], TraditionalForm]]],
 ". The left child of the root will be drawn at ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "4"], TraditionalForm]]],
 " and the right child at ",
 Cell[BoxData[
  FormBox[
   FractionBox["3", "4"], TraditionalForm]]],
 ". We can think about the children of the root as being drawn ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "4"], TraditionalForm]]],
 " to the left of the root and ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", "4"], TraditionalForm]]],
 " to the right of the root, respectively. That is, the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate of the root's left child is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", "2"], "-", 
    FractionBox["1", "4"]}], TraditionalForm]]],
 ". and the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate of the right child is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", "2"], "+", 
    FractionBox["1", "4"]}], TraditionalForm]]],
 ". Generally, for a vertex in level ",
 Cell[BoxData[
  FormBox["l", TraditionalForm]]],
 ", we can calculate its ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate as the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate of its parent plus (for right children) or minus (for left) ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    SuperscriptBox["2", 
     RowBox[{"l", "+", "1"}]]], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox["drawBinaryTree", "Input"],
 " function is below. It begins by calculating the height of the tree with \
the ",
 StyleBox["findHeight", "Input"],
 " function we created earlier and finds the root with ",
 StyleBox["findRoot", "Input"],
 ". It then processes the root of the tree by setting its position to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], ",", "1"}], ")"}], TraditionalForm]]],
 ". (Note that we can set the position of the vertex by using ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " to set the ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " property. This is in some ways more convenient than assembling all of the \
vertex positions and then using ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " within a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " definition.) We also create a list, ",
 StyleBox["verts", "Input"],
 ", and populate it with the root's children. "
}], "Text"],

Cell[TextData[{
 "We then begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. We initialize a counter ",
 StyleBox["i", "Input"],
 " to 1. This counter serves as an index into the ",
 StyleBox["verts", "Input"],
 " list. At each step in the loop, we do several things. First, we use the ",
 StyleBox["findLevel", "Input"],
 " function to determine the level of the vertex. Second, the ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 "-coordinate is calculated by the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "+", 
    FractionBox["l", "h"]}], TraditionalForm]]],
 ". Third, the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate is calculated by accessing the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-coordinate of the parent and adding or subtracting ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    SuperscriptBox["2", 
     RowBox[{"l", "+", "1"}]]], TraditionalForm]]],
 ". Fourth, we set the ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " property for the vertex. And finally, the children of the current vertex \
are added to the ",
 StyleBox["verts", "Input"],
 " list and the counter is incremented."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"drawBinaryTree", ",", "HoldFirst"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"drawBinaryTree", "[", 
   RowBox[{"T_", "?", "binaryTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "height", ",", "v", ",", "i", ",", "level", ",", "verts", ",", "x", ",", 
      "y", ",", "parent", ",", "side"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"height", "=", 
      RowBox[{"findHeight", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"v", "=", 
      RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "v"}], "}"}], ",", "VertexCoordinates"}], "]"}], 
      "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", "/", "2"}], ",", "1"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"verts", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "verts", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"v", "=", 
         RowBox[{"verts", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"level", "=", 
         RowBox[{"findLevel", "[", 
          RowBox[{"T", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"y", "=", 
         RowBox[{"1", "-", 
          RowBox[{"level", "/", "height"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"parent", "=", 
         RowBox[{"findParent", "[", 
          RowBox[{"T", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"x", "=", 
         RowBox[{
          RowBox[{"PropertyValue", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"T", ",", "parent"}], "}"}], ",", "VertexCoordinates"}], 
           "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"side", "=", 
         RowBox[{"Switch", "[", 
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
           ",", "1", ",", 
           RowBox[{"-", "1"}], ",", "2", ",", "1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"x", "=", 
         RowBox[{"x", "+", 
          RowBox[{"side", "*", 
           RowBox[{"1", "/", 
            RowBox[{"(", 
             RowBox[{"2", "^", 
              RowBox[{"(", 
               RowBox[{"level", "+", "1"}], ")"}]}], ")"}]}]}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "v"}], "}"}], ",", "VertexCoordinates"}], "]"}],
          "=", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y"}], "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"verts", "=", 
         RowBox[{"Join", "[", 
          RowBox[{"verts", ",", 
           RowBox[{"findChildren", "[", 
            RowBox[{"T", ",", "v"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "++"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",\

 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drawBinaryTree", "[", "firstBinaryTree", "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "D", "B", "F", "A", "C", "E"}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 
       6}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "E" -> {"order" -> 1}, "A" -> {"order" -> 1}, "F" -> {"order" -> 2}, 
         "B" -> {"order" -> 1}, "D" -> {"order" -> 0}, "C" -> {"order" -> 2}},
        VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[1, 4], 
          Rational[1, 2]}, {
          Rational[3, 4], 
          Rational[1, 2]}, {
          Rational[1, 8], 0}, {
          Rational[3, 8], 0}, {
          Rational[5, 8], 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"D\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.5}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"B\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.5}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"F\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"A\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"C\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"E\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{14.309776216040198`, 
       278.42519396366794`}, {-215.14877421029945`, 199.9026148326014}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parents and Children", "Subsubsection"],

Cell[TextData[{
 "In the previous section, we created the function ",
 StyleBox["findParent", "Input"],
 ", which returns the parent of a given vertex in the given tree. This \
function works on binary trees just as well."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findParent", "[", 
  RowBox[{"firstBinaryTree", ",", "\"\<C\>\""}], "]"}]], "Input"],

Cell[BoxData["\<\"B\"\>"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We had also created the ",
 StyleBox["findChildren", "Input"],
 " function, which we can also use with binary trees. But for binary trees, \
we'll want to be more specific and be able to determine the left and right \
children of a given vertex. Finding the left (respectively, right) child of a \
given vertex can be done by looking at each child of the vertex and checking \
the order attribute. The child with order 1 is the left child and is returned \
by the function (respectively, 2 and right child). If there is no left \
(respectively, right), child, the function will return ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"findLeftChild", "[", 
    RowBox[{
     RowBox[{"T_", "?", "binaryTreeQ"}], ",", "v_"}], "]"}], "/;", 
   RowBox[{"VertexQ", "[", 
    RowBox[{"T", ",", "v"}], "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"children", ",", "w", ",", "pos"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"children", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"pos", "=", 
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "w"}], "}"}], ",", "\"\<order\>\""}], "]"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"pos", "\[Equal]", "1"}], ",", 
            RowBox[{"Throw", "[", "w", "]"}]}], "]"}]}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"w", ",", "children"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Throw", "[", "Null", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"findRightChild", "[", 
    RowBox[{
     RowBox[{"T_", "?", "binaryTreeQ"}], ",", "v_"}], "]"}], "/;", 
   RowBox[{"VertexQ", "[", 
    RowBox[{"T", ",", "v"}], "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"children", ",", "w", ",", "pos"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"children", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"pos", "=", 
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "w"}], "}"}], ",", "\"\<order\>\""}], "]"}]}],
           ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"pos", "\[Equal]", "2"}], ",", 
            RowBox[{"Throw", "[", "w", "]"}]}], "]"}]}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"w", ",", "children"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Throw", "[", "Null", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findLeftChild", "[", 
  RowBox[{"firstBinaryTree", ",", "\"\<F\>\""}], "]"}]], "Input"],

Cell[BoxData["\<\"E\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"findRightChild", "[", 
  RowBox[{"firstBinaryTree", ",", "\"\<F\>\""}], "]"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Building Binary Trees", "Subsubsection"],

Cell["\<\
We will also want functions to create and build up a binary tree. \
Specifically, we'll create a function that, given the label for the root of a \
binary tree, creates the tree with that vertex as its root. We will then \
write functions that, given a binary tree, a vertex in the tree, and a label \
for a new vertex, adds the new vertex as the left or right child of the given \
vertex.\
\>", "Text"],

Cell[TextData[{
 "The ",
 StyleBox["newBinaryTree", "Input"],
 " function creates the binary tree consisting of a single vertex, the root \
of the tree. This function also accepts a list of options to be passed to the \
",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"newBinaryTree", "[", 
   RowBox[{"r_", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "T", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"{", "r", "}"}], ",", 
        RowBox[{"{", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "r"}], "}"}], ",", "\"\<order\>\""}], "]"}], "=", 
      "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "T", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Adding a child to a vertex in a binary tree requires three basic steps. We \
must add a vertex to the graph with the ",
 StyleBox[ButtonBox["VertexAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexAdd"], "FunctionLink"],
 " function, add a directed edge from the parent to the new vertex with the \
",
 StyleBox[ButtonBox["EdgeAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeAdd"], "FunctionLink"],
 " function, and then use ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " to identify the new child as left or right by setting the order attribute \
to 1 or 2, respectively. We conclude the functions with a call to ",
 StyleBox["drawBinaryTree", "Input"],
 " so as to ensure the resulting tree appears with the vertices in the \
correct order."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addLeftChild", "[", 
    RowBox[{
     RowBox[{"T_", "?", "binaryTreeQ"}], ",", "v_", ",", "newV_"}], "]"}], "/;", 
   RowBox[{"VertexQ", "[", 
    RowBox[{"T", ",", "v"}], "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"newT", ",", "newedge", ",", "temp", ",", "w"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"findLeftChild", "[", 
         RowBox[{"T", ",", "v"}], "]"}], "\[NotEqual]", "Null"}], ",", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"VertexAdd", "[", 
       RowBox[{"T", ",", "newV"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"EdgeAdd", "[", 
       RowBox[{"newT", ",", 
        RowBox[{"DirectedEdge", "[", 
         RowBox[{"v", ",", "newV"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newV"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
      "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "newT", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"addRightChild", "[", 
    RowBox[{
     RowBox[{"T_", "?", "binaryTreeQ"}], ",", "v_", ",", "newV_"}], "]"}], "/;", 
   RowBox[{"VertexQ", "[", 
    RowBox[{"T", ",", "v"}], "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"newT", ",", "newedge", ",", "temp", ",", "w"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"findRightChild", "[", 
         RowBox[{"T", ",", "v"}], "]"}], "\[NotEqual]", "Null"}], ",", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"VertexAdd", "[", 
       RowBox[{"T", ",", "newV"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"EdgeAdd", "[", 
       RowBox[{"newT", ",", 
        RowBox[{"DirectedEdge", "[", 
         RowBox[{"v", ",", "newV"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newV"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
      "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "newT", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "With ",
 StyleBox["newBinaryTree", "Input"],
 ", ",
 StyleBox["addLeftChild", "Input"],
 ", and ",
 StyleBox["addRightChild", "Input"],
 ", we can now construct binary trees one vertex at a time. We will \
illustrate this by creating the binary search tree described in Example 1 of \
the text by following the steps illustrated in Figure 1. We abbreviate the \
words in order to make the image more readable."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"newBinaryTree", "[", 
    RowBox[{"\"\<Math\>\"", ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
     RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addRightChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Math\>\"", ",", "\"\<Phys\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addLeftChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Math\>\"", ",", "\"\<Geog\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addRightChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Phys\>\"", ",", "\"\<Zoo\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addLeftChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Phys\>\"", ",", "\"\<Meteo\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addRightChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Geog\>\"", ",", "\"\<Geol\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addLeftChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Zoo\>\"", ",", "\"\<Psy\>\""}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fig1Tree", "=", 
   RowBox[{"addLeftChild", "[", 
    RowBox[{"fig1Tree", ",", "\"\<Geog\>\"", ",", "\"\<Chem\>\""}], "]"}]}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["fig1Tree"], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "Math", "Phys", "Geog", "Zoo", "Meteo", "Geol", "Psy", "Chem"}, {{{1, 
       2}, {1, 3}, {2, 4}, {2, 5}, {3, 6}, {4, 7}, {3, 8}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "Chem" -> {"order" -> 1}, "Geol" -> {"order" -> 2}, 
         "Phys" -> {"order" -> 2}, "Geog" -> {"order" -> 1}, 
         "Psy" -> {"order" -> 1}, "Math" -> {"order" -> 0}, 
         "Zoo" -> {"order" -> 2}, "Meteo" -> {"order" -> 1}}, 
       VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[3, 4], 
          Rational[2, 3]}, {
          Rational[1, 4], 
          Rational[2, 3]}, {
          Rational[7, 8], 
          Rational[1, 3]}, {
          Rational[5, 8], 
          Rational[1, 3]}, {
          Rational[3, 8], 
          Rational[1, 3]}, {
          Rational[13, 16], 0}, {
          Rational[1, 8], 
          Rational[1, 3]}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"Math\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"Phys\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"Geog\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"Zoo\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"Meteo\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"Geol\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"Psy\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"Chem\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.300206106122634`, 
       336.96317280995027`}, {-214.20487574806418`, 198.95180213812228`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Binary Insertion", "Subsection",
 CellTags->"Section2b"],

Cell["\<\
A key benefit of binary search trees is that the search time required to find \
a specific element of the tree is logarithmic in the number of vertices of \
the tree. The drawback is that the initial insertion of a vertex is more \
expensive.\
\>", "Text"],

Cell[TextData[{
 "The procedure for constructing a binary search tree by insertion is \
described in Algorithm 1 in Section 11.2 of the textbook. We will implement \
this algorithm as the function ",
 StyleBox["binaryInsertion", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox["binaryInsertion", "Input"],
 " function will accept two input values: a binary search tree and a vertex \
to be found or added. The function returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the vertex is found to already be in the tree, and if not, it will add \
the vertex to the tree and return ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "We begin by locating the root of the tree and setting the local variable ",
 StyleBox["v", "Input"],
 " to the root. Then we begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. This loop continues provided two conditions are met. First, that ",
 StyleBox["v", "Input"],
 " is not ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ". If we discover that the value we're searching for is not in the tree, \
then we will add it to the tree and set ",
 StyleBox["v", "Input"],
 " to ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", to indicate that we had to add a vertex, and this terminates the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. The second condition is that ",
 StyleBox["v", "Input"],
 " is not ",
 StyleBox["x", "Input"],
 ", where ",
 StyleBox["x", "Input"],
 " is the value we are searching for. If ",
 Cell[BoxData[
  RowBox[{"v", "=", "x"}]], "myCode"],
 ", then we have found the vertex and thus the loop should terminate. (Note \
that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " identifies a vertex with its label, so, unlike the text, we do not \
distinguish between ",
 StyleBox["v", "Input"],
 " and its label.)"
}], "Text"],

Cell[TextData[{
 "Within the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop, there are two possibilities. Either the sought-after value is less \
than ",
 StyleBox["v", "Input"],
 " or it is greater than ",
 StyleBox["v", "Input"],
 ". They cannot be equal because that is one of the terminating conditions \
for the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. If the target value is less than ",
 StyleBox["v", "Input"],
 ", then we consider the left child of ",
 StyleBox["v", "Input"],
 ". If there is no left child, then we know that the value is not in the tree \
and so we add the value as the left child of ",
 StyleBox["v", "Input"],
 " and then set ",
 StyleBox["v", "Input"],
 " to ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 " to indicate that the desired value was not already in the tree. If there \
is a left child, then we set ",
 StyleBox["v", "Input"],
 " equal to it and continue the loop. If the target value is greater than ",
 StyleBox["v", "Input"],
 ", we proceed in exactly the same way, substituting right for left."
}], "Text"],

Cell[TextData[{
 "Once the loop terminates, we check the value of ",
 StyleBox["v", "Input"],
 ". If ",
 StyleBox["v", "Input"],
 " is ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", then we know that the desired value was not found and the algorithm \
returns ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ". If ",
 StyleBox["v", "Input"],
 " is not ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", then we return ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 "."
}], "Text"],

Cell["Here, now, is the binary insertion algorithm.", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"binaryInsertion", ",", "HoldFirst"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"binaryInsertion", "[", 
   RowBox[{
    RowBox[{"BST_", "?", "binaryTreeQ"}], ",", "x_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "v", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"v", "=", 
      RowBox[{"findRoot", "[", "BST", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"v", "=!=", "Null"}], "&&", 
        RowBox[{"v", "\[NotEqual]", "x"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Order", "[", 
           RowBox[{"x", ",", "v"}], "]"}], "\[Equal]", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"findLeftChild", "[", 
             RowBox[{"BST", ",", "v"}], "]"}], "===", "Null"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{"no", " ", "left", " ", "child"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"BST", "=", 
             RowBox[{"addLeftChild", "[", 
              RowBox[{"BST", ",", "v", ",", "x"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"v", "=", "Null"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{"left", " ", "child", " ", "exists"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"v", "=", 
            RowBox[{"findLeftChild", "[", 
             RowBox[{"BST", ",", "v"}], "]"}]}]}], "]"}], 
         "\[IndentingNewLine]", ",", " ", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"else", " ", "x"}], ">", "v"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"findRightChild", "[", 
             RowBox[{"BST", ",", "v"}], "]"}], "===", "Null"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{"no", " ", "right", " ", "child"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"BST", "=", 
             RowBox[{"addRightChild", "[", 
              RowBox[{"BST", ",", "v", ",", "x"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"v", "=", "Null"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{"right", " ", "child", " ", "exists"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"v", "=", 
            RowBox[{"findRightChild", "[", 
             RowBox[{"BST", ",", "v"}], "]"}]}]}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     " ", 
     RowBox[{"(*", " ", 
      RowBox[{"end", " ", "while", " ", "loop"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"v", "=!=", "Null"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that we use the ",
 StyleBox[ButtonBox["Order",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Order"], "FunctionLink"],
 " function to compare ",
 StyleBox["x", "Input"],
 " and ",
 StyleBox["v", "Input"],
 ". ",
 StyleBox[ButtonBox["Order",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Order"], "FunctionLink"],
 " returns 1 if the first argument is before (\[OpenCurlyDoubleQuote]less \
than\[CloseCurlyDoubleQuote]) the second in the canonical order, i.e., \
alphabetically for strings, numerically for numbers. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not allow you to use the less-than operator, ",
 StyleBox[ButtonBox["Less",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Less"], "FunctionLink"],
 " (",
 StyleBox["<", "Input"],
 ") with strings. ",
 StyleBox[ButtonBox["Order",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Order"], "FunctionLink"],
 " is generic and will allow both string and integer vertex names. "
}], "Text"],

Cell[TextData[{
 "Also note that ",
 StyleBox["binaryInsertion", "Input"],
 " requires that its argument already be a binary tree. A new tree must be \
created with ",
 StyleBox["newBinaryTree", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Now let's see if oceanography is in the ",
 StyleBox["fig1Tree", "Input"],
 " of academic subjects."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"binaryInsertion", "[", 
  RowBox[{"fig1Tree", ",", "\"\<Oc\>\""}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The function returned ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " indicating that \[OpenCurlyDoubleQuote]Oc\[CloseCurlyDoubleQuote] was not \
found in the tree. Displaying ",
 StyleBox["fig1Tree", "Input"],
 ", we see that it was added as a child of meteorology."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["fig1Tree"], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "Math", "Phys", "Geog", "Zoo", "Meteo", "Geol", "Psy", "Chem", 
       "Oc"}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 6}, {4, 7}, {3, 8}, {5, 
       9}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "Chem" -> {"order" -> 1}, "Geol" -> {"order" -> 2}, 
         "Phys" -> {"order" -> 2}, "Geog" -> {"order" -> 1}, 
         "Psy" -> {"order" -> 1}, "Oc" -> {"order" -> 2}, 
         "Math" -> {"order" -> 0}, "Zoo" -> {"order" -> 2}, 
         "Meteo" -> {"order" -> 1}}, VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[3, 4], 
          Rational[2, 3]}, {
          Rational[1, 4], 
          Rational[2, 3]}, {
          Rational[7, 8], 
          Rational[1, 3]}, {
          Rational[5, 8], 
          Rational[1, 3]}, {
          Rational[3, 8], 
          Rational[1, 3]}, {
          Rational[13, 16], 0}, {
          Rational[1, 8], 
          Rational[1, 3]}, {
          Rational[11, 16], 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"Math\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"Phys\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"Geog\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"Zoo\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"Meteo\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"Geol\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"Psy\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"Chem\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.6875, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"Oc\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.300206106122634`, 
       336.96317280995027`}, {-214.20487574806418`, 198.95180213812228`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell["\<\
On the other hand, zoology is already in the tree and so the tree is not \
modified.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"binaryInsertion", "[", 
  RowBox[{"fig1Tree", ",", "\"\<Zoo\>\""}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["fig1Tree"], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "Math", "Phys", "Geog", "Zoo", "Meteo", "Geol", "Psy", "Chem", 
       "Oc"}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 6}, {4, 7}, {3, 8}, {5, 
       9}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "Chem" -> {"order" -> 1}, "Geol" -> {"order" -> 2}, 
         "Phys" -> {"order" -> 2}, "Geog" -> {"order" -> 1}, 
         "Psy" -> {"order" -> 1}, "Oc" -> {"order" -> 2}, 
         "Math" -> {"order" -> 0}, "Zoo" -> {"order" -> 2}, 
         "Meteo" -> {"order" -> 1}}, VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[3, 4], 
          Rational[2, 3]}, {
          Rational[1, 4], 
          Rational[2, 3]}, {
          Rational[7, 8], 
          Rational[1, 3]}, {
          Rational[5, 8], 
          Rational[1, 3]}, {
          Rational[3, 8], 
          Rational[1, 3]}, {
          Rational[13, 16], 0}, {
          Rational[1, 8], 
          Rational[1, 3]}, {
          Rational[11, 16], 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"Math\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"Phys\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"Geog\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"Zoo\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"Meteo\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"Geol\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"Psy\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"Chem\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.6875, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"Oc\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.300206106122634`, 
       336.96317280995027`}, {-214.20487574806418`, 198.95180213812228`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Constructing a Binary Search Tree from a List", "Subsubsection"],

Cell[TextData[{
 "To conclude our discussion of binary search trees, we will create a \
function that takes a list of values and successively uses the ",
 StyleBox["binaryInsertion", "Input"],
 " function to create a binary search tree for the given list."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeBST", "[", 
   RowBox[{"L_List", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"T", ",", "v"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"T", "=", 
      RowBox[{"newBinaryTree", "[", 
       RowBox[{
        RowBox[{"L", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"binaryInsertion", "[", 
        RowBox[{"T", ",", 
         RowBox[{"L", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "2", ",", 
         RowBox[{"Length", "[", "L", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell["We use this to complete Exercise 1 from Section 11.2.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise1", "=", 
  RowBox[{"makeBST", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<banana\>\"", ",", "\"\<peach\>\"", ",", "\"\<apple\>\"", ",", 
      "\"\<pear\>\"", ",", "\"\<coconut\>\"", ",", "\"\<mango\>\"", ",", 
      "\"\<papaya\>\""}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "15"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "banana", "peach", "apple", "pear", "coconut", "mango", "papaya"}, {{{1,
        2}, {1, 3}, {2, 4}, {2, 5}, {5, 6}, {6, 7}}, Null}, {
      ImagePadding -> 15, 
       Properties -> {
        "banana" -> {"order" -> 0}, "peach" -> {"order" -> 2}, 
         "pear" -> {"order" -> 2}, "mango" -> {"order" -> 2}, 
         "coconut" -> {"order" -> 1}, "apple" -> {"order" -> 1}, 
         "papaya" -> {"order" -> 2}}, VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[3, 4], 
          Rational[3, 4]}, {
          Rational[1, 4], 
          Rational[3, 4]}, {
          Rational[7, 8], 
          Rational[1, 2]}, {
          Rational[5, 8], 
          Rational[1, 2]}, {
          Rational[11, 16], 
          Rational[1, 4]}, {
          Rational[23, 32], 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"banana\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.75}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"peach\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.75}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"apple\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.5}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"pear\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.5}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"coconut\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.6875, 0.25}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"mango\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.71875, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"papaya\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{19.180898152544067`, 
       288.1456287407921}, {-210.30795703969025`, 195.02633777355763`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->15]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Huffman Coding", "Subsection",
 CellTags->"Section2c"],

Cell["\<\
Huffman coding is a method for constructing an efficient prefix code for a \
set of characters. It is based on a greedy algorithm, where at each step the \
vertices with the least weight are combined. It can be shown that Huffman \
coding produces optimal prefix codes. The algorithm that we will implement is \
described in Algorithm 2 of Section 11.2.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Creating the Initial Forest", "Subsubsection"],

Cell["\<\
We begin with a list of symbols and their weights, or frequencies. The first \
step is to create the initial forest, which we will implement as a list of \
trees. For each symbol, we will create the binary tree consisting of a single \
vertex corresponding to the symbol.\
\>", "Text"],

Cell[TextData[{
 "We create a function, similar to ",
 StyleBox["newBinaryTree", "Input"],
 ", which, in addition to creating the binary tree, also assigns a \
\[OpenCurlyDoubleQuote]weight\[CloseCurlyDoubleQuote] attribute to the graph \
to store the weight of the symbol."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"newHTree", "[", 
   RowBox[{"s_", ",", "w_", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "T", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"{", "s", "}"}], ",", 
        RowBox[{"{", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{"T", ",", "\"\<weight\>\""}], "]"}], "=", "w"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "s"}], "}"}], ",", "\"\<order\>\""}], "]"}], "=", 
      "0"}], ";", "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Now we write a function to create the initial forest. We assume that the data \
is provided as a list of pairs consisting of the symbol and the weight.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"createForest", "[", 
   RowBox[{"L_List", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"forest", "=", 
       RowBox[{"{", "}"}]}], ",", "M", ",", "v", ",", "w", ",", "G"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"v", ",", "w"}], "}"}], "=", "M"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"forest", ",", 
          RowBox[{"newHTree", "[", 
           RowBox[{"v", ",", "w", ",", "opts"}], "]"}]}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"M", ",", "L"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     "forest"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Using this function, we form the initial forest for Exercise 23 from \
Section 11.2. We use the ",
 StyleBox[ButtonBox["ImageSize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ImageSize"], "FunctionLink"],
 " option to limit the width of the images. Ordinarily, you would simply \
suppress the output from this function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex23Forest", "=", 
  RowBox[{"createForest", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\"\<a\>\"", ",", "0.20"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<b\>\"", ",", "0.10"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<c\>\"", ",", "0.15"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<d\>\"", ",", "0.25"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<e\>\"", ",", "0.30"}], "}"}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "50"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"a"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.2}, "a" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"a\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.349414869256858`, 
         32.6567383988394}, {-16.656738398838975`, -0.3494148692563641}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"b"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.1}, "b" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"b\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.34941486925686, 
         33.65673839883942}, {-16.656738398838982`, -0.3494148692563712}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"c"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.15}, "c" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"c\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.349414869256858`, 
         32.656738398839366`}, {-16.656738398838975`, -0.3494148692563641}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"d"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.25}, "d" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"d\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.349414869256858`, 
         33.656738398839366`}, {-16.656738398838975`, -0.3494148692563641}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"e"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.3}, "e" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"e\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.349414869256858`, 
         32.656738398839366`}, {-16.656738398838975`, -0.3494148692563641}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The main work of the Huffman coding algorithm is to determine the two members \
of the forest with the smallest weights. These two trees are then assembled \
into a single tree whose root is a new vertex and with the trees with the \
lowest weight and second lowest weight as the right and left subtrees of the \
root. The new tree's weight is the sum of the weights of the two original \
trees.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sorting the Forest", "Subsubsection"],

Cell[TextData[{
 "Recall that the ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " function accepts an optional second argument, specifically, a function on \
two arguments that returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the first argument precedes the second in the desired order. The \
following function will be of this kind, accepting two binary trees as input \
and comparing their weights."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"compareTrees", "[", 
   RowBox[{
    RowBox[{"A_", "?", "binaryTreeQ"}], ",", 
    RowBox[{"B_", "?", "binaryTreeQ"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"a", "=", 
      RowBox[{"PropertyValue", "[", 
       RowBox[{"A", ",", "\"\<weight\>\""}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"b", "=", 
      RowBox[{"PropertyValue", "[", 
       RowBox[{"B", ",", "\"\<weight\>\""}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"a", "<", "b"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex23Forest", "=", 
  RowBox[{"Sort", "[", 
   RowBox[{"ex23Forest", ",", "compareTrees"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"b"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.1}, "b" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"b\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.55438596491229, 
         33.875756877782244`}, {-16.87575687778607, -0.5543859649122886}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"c"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.15}, "c" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"c\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.55438596491229, 
         32.87575687778224}, {-16.87575687778607, -0.5543859649122886}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"a"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.2}, "a" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"a\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.55438596491229, 
         32.875756877782244`}, {-16.87575687778607, -0.5543859649122886}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"d"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.25}, "d" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"d\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.55438596491229, 
         33.875756877782244`}, {-16.87575687778607, -0.5543859649122886}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1], ",", 
   GraphicsBox[
    NamespaceBox["NetworkGraphics",
     DynamicModuleBox[{Typeset`graph = HoldComplete[
       Graph[{"e"}, {}, {
        ImageSize -> 50, 
         Properties -> {
          "GraphProperties" -> {"weight" -> 0.3}, "e" -> {"order" -> 0}}, 
         VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
      GraphicsGroupBox[{{}, {
         Directive[
          Hue[0.6, 0.2, 0.8], 
          EdgeForm[
           Directive[
            GrayLevel[0], 
            Opacity[0.7]]]], 
         TagBox[{
           TagBox[
            DiskBox[{0., 0.}, 0.004014598540145964], "DynamicName", BoxID -> 
            "VertexID$1"], 
           InsetBox[
            FormBox["\"e\"", TraditionalForm], 
            Offset[{2, 2}, 
             DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
            ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
          BoxID -> "VertexLabelID$1"]}}], $CellContext`flag}, 
      TagBox[
       DynamicBox[GraphComputation`NetworkGraphicsBox[
        3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
        CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
         SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
        
        ImageSizeCache->{{22.55438596491229, 
         32.875756877782244`}, {-16.87575687778607, -0.5543859649122886}}],
       MouseAppearanceTag["NetworkGraphics"]],
      AllowKernelInitialization->False,
      UnsavedVariables:>{$CellContext`flag}]],
    DefaultBaseStyle->{
     "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
    
    FrameTicks->None,
    ImageSize->50,
    PlotRangePadding->0.1]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combining Two Trees", "Subsubsection"],

Cell["\<\
Next, we need to take two binary trees and create a new binary tree with one \
tree as the left subtree of the new root and the other as the right subtree. \
This function will require three arguments: the name of the new root, the \
left subtree, and the right subtree.\
\>", "Text"],

Cell["\<\
We create the new tree by: (1) combining the vertex lists of the original \
trees and adding the new vertex; (2) merging the two sets of edges of the \
original tress and adding two new edges linking the new root to the previous \
roots; (3) copying the \
\[OpenCurlyDoubleQuote]position\[CloseCurlyDoubleQuote] attribute from the \
original trees and then changing the position attributes for the two original \
roots to reflect their new status as left and right children in the new tree; \
(4) copying the edge labels and assigning the new edges the labels 0 and 1; \
and (5) setting the \[OpenCurlyDoubleQuote]weight\[CloseCurlyDoubleQuote] \
attribute to be the sum of the weights of the two original trees.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"joinHTrees", "[", 
   RowBox[{"newR_", ",", 
    RowBox[{"A_", "?", "binaryTreeQ"}], ",", 
    RowBox[{"B_", "?", "binaryTreeQ"}], ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "newT", ",", "newVerts", ",", "Aroot", ",", "Broot", ",", "newEdges", 
      ",", "v", ",", "e", ",", "p", ",", "w"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newVerts", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "newR", "}"}], ",", 
        RowBox[{"VertexList", "[", "A", "]"}], ",", 
        RowBox[{"VertexList", "[", "B", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Aroot", "=", 
      RowBox[{"findRoot", "[", "A", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Broot", "=", 
      RowBox[{"findRoot", "[", "B", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newEdges", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"EdgeList", "[", "A", "]"}], ",", 
        RowBox[{"EdgeList", "[", "B", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"DirectedEdge", "[", 
           RowBox[{"newR", ",", "Aroot"}], "]"}], ",", 
          RowBox[{"DirectedEdge", "[", 
           RowBox[{"newR", ",", "Broot"}], "]"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{"newVerts", ",", "newEdges", ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"A", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "A", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"B", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "B", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "Aroot"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "Broot"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newR"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
      "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "e"}], "}"}], ",", "EdgeLabels"}], "]"}], "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"A", ",", "e"}], "}"}], ",", "EdgeLabels"}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"e", ",", 
         RowBox[{"EdgeList", "[", "A", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "e"}], "}"}], ",", "EdgeLabels"}], "]"}], "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"B", ",", "e"}], "}"}], ",", "EdgeLabels"}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"e", ",", 
         RowBox[{"EdgeList", "[", "B", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", 
          RowBox[{"DirectedEdge", "[", 
           RowBox[{"newR", ",", "Aroot"}], "]"}]}], "}"}], ",", 
        "EdgeLabels"}], "]"}], "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", 
          RowBox[{"DirectedEdge", "[", 
           RowBox[{"newR", ",", "Broot"}], "]"}]}], "}"}], ",", 
        "EdgeLabels"}], "]"}], "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"w", "=", 
      RowBox[{
       RowBox[{"PropertyValue", "[", 
        RowBox[{"A", ",", "\"\<weight\>\""}], "]"}], "+", 
       RowBox[{"PropertyValue", "[", 
        RowBox[{"B", ",", "\"\<weight\>\""}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{"newT", ",", "\"\<weight\>\""}], "]"}], "=", "w"}], ";", 
     "\[IndentingNewLine]", "newT"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For example, we'll join the first two graphs in our sorted ",
 StyleBox["ex23Forest", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleJoin", "=", 
  RowBox[{"joinHTrees", "[", 
   RowBox[{"\"\<newR\>\"", ",", 
    RowBox[{"ex23Forest", "[", 
     RowBox[{"[", "1", "]"}], "]"}], ",", 
    RowBox[{"ex23Forest", "[", 
     RowBox[{"[", "2", "]"}], "]"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"newR", "b", "c"}, {{{1, 2}, {1, 3}}, Null}, {
      EdgeLabels -> {
        DirectedEdge["newR", "b"] -> 0, DirectedEdge["newR", "c"] -> 1}, 
       ImagePadding -> 10, 
       Properties -> {
        "GraphProperties" -> {"weight" -> 0.25}, "c" -> {"order" -> 2}, 
         "b" -> {"order" -> 1}, "newR" -> {"order" -> 0}}, 
       VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Arrowheads[0.04266728990926259], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$2", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$3", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4472135954999579, 0.8944271909999159}, 
           0.011386058141428929`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"newR\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.011386058141428929`], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8944271909999159, 0.}, 0.011386058141428929`], 
          "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       349.72032971286444`}, {-177.99232971286872`, 161.92799999999997`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementing the Main Function", "Subsubsection"],

Cell[TextData[{
 "We now have the major pieces of the Huffman algorithm assembled and we can \
write the ",
 StyleBox["huffmanCode", "Input"],
 " function. This function will accept as input the same list of \
symbol-weight pairs as the ",
 StyleBox["createForest", "Input"],
 " function did. The function's first step is to create the forest ",
 StyleBox["F", "Input"],
 ". Then we begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop that continues as long as the list ",
 StyleBox["F", "Input"],
 " contains more than one element. Inside the loop, we first use the ",
 StyleBox["compareTrees", "Input"],
 " function to sort the forest in increasing order of weight. Then, we use \
the ",
 StyleBox["joinHTrees", "Input"],
 " function to join the first two trees in the forest and we add that new \
tree to the list, replacing the original two."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"huffmanCode", "[", 
   RowBox[{"L_List", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"F", ",", "i", ",", "tempT"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"F", "=", 
      RowBox[{"createForest", "[", 
       RowBox[{"L", ",", "opts"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "F", "]"}], ">", "1"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"F", "=", 
         RowBox[{"Sort", "[", 
          RowBox[{"F", ",", "compareTrees"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "++"}], ";", "\[IndentingNewLine]", 
        RowBox[{"tempT", "=", 
         RowBox[{"joinHTrees", "[", 
          RowBox[{
           RowBox[{"\"\<I\>\"", "<>", 
            RowBox[{"ToString", "[", "i", "]"}]}], ",", 
           RowBox[{"F", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ",", 
           RowBox[{"F", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "opts"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"F", "=", 
         RowBox[{"Append", "[", 
          RowBox[{
           RowBox[{"F", "[", 
            RowBox[{"[", 
             RowBox[{"3", ";;", 
              RowBox[{"-", "1"}]}], "]"}], "]"}], ",", "tempT"}], "]"}]}]}]}],
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"F", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that we need a name for the new root when we join two trees. Since \
these will be the internal vertices of the final tree, we'll call them I1, \
I2, I3, etc. We keep a counter ",
 StyleBox["i", "Input"],
 " and use the string concatenation operator ",
 StyleBox[ButtonBox["StringJoin",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringJoin"], "FunctionLink"],
 " (",
 StyleBox["<>", "Input"],
 ") to create the names of the internal vertices."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ex23HCode", "=", 
   RowBox[{"huffmanCode", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<a\>\"", ",", "0.20"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<b\>\"", ",", "0.10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<c\>\"", ",", "0.15"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<d\>\"", ",", "0.25"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<e\>\"", ",", "0.30"}], "}"}]}], "}"}], ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
     RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"drawBinaryTree", "[", "ex23HCode", "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "I4", "I3", "e", "d", "I2", "I1", "c", "b", "a"}, {{{2, 3}, {2, 4}, {6, 
       7}, {6, 8}, {5, 6}, {5, 9}, {1, 2}, {1, 5}}, Null}, {
      EdgeLabels -> {
        DirectedEdge["I1", "c"] -> 0, DirectedEdge["I2", "I1"] -> 0, 
         DirectedEdge["I3", "d"] -> 1, DirectedEdge["I4", "I3"] -> 0, 
         DirectedEdge["I2", "a"] -> 1, DirectedEdge["I3", "e"] -> 0, 
         DirectedEdge["I4", "I2"] -> 1, DirectedEdge["I1", "b"] -> 1}, 
       ImagePadding -> 10, 
       Properties -> {
        "GraphProperties" -> {"weight" -> 1.}, "I3" -> {"order" -> 1}, 
         "d" -> {"order" -> 2}, "c" -> {"order" -> 1}, "I1" -> {"order" -> 1},
          "I4" -> {"order" -> 0}, "b" -> {"order" -> 2}, 
         "e" -> {"order" -> 1}, "I2" -> {"order" -> 2}, 
         "a" -> {"order" -> 2}}, VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[1, 4], 
          Rational[2, 3]}, {
          Rational[1, 8], 
          Rational[1, 3]}, {
          Rational[3, 8], 
          Rational[1, 3]}, {
          Rational[3, 4], 
          Rational[2, 3]}, {
          Rational[5, 8], 
          Rational[1, 3]}, {
          Rational[9, 16], 0}, {
          Rational[11, 16], 0}, {
          Rational[7, 8], 
          Rational[1, 3]}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$2", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$5", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$2", Automatic, Center], 
            DynamicLocation["VertexID$3", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$3"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$3", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$2", Automatic, Center], 
            DynamicLocation["VertexID$4", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$4"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$4", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$5", Automatic, Center], 
            DynamicLocation["VertexID$6", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$5"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$5", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$5", Automatic, Center], 
            DynamicLocation["VertexID$9", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$6"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$6", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$6", Automatic, Center], 
            DynamicLocation["VertexID$7", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$7"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$7", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$6", Automatic, Center], 
            DynamicLocation["VertexID$8", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$8"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$8", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"I4\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"I3\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"I2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"I1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.5625, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.6875, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.300206106122634`, 
       324.96317280995027`}, {-214.20487574806464`, 198.95180213812228`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Encoding Strings Using the Huffman Code Tree", "Subsubsection"],

Cell[TextData[{
 "We conclude this section by writing a function to encode a string of \
symbols using a given Huffman tree. Note that you can use ",
 StyleBox[ButtonBox["Characters",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Characters"], "FunctionLink"],
 " to decompose a string into a list of the individual letters. For example,"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Characters", "[", "\"\<Hello\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"H\"\>", ",", "\<\"e\"\>", ",", "\<\"l\"\>", ",", "\<\"l\"\>", 
   ",", "\<\"o\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since we encode a string by assembling the codes for individual characters, \
we'll start with a function for encoding a single character. We assemble the \
character's code right to left. Beginning with the vertex corresponding to \
the desired character, we find that vertex's parent. The last digit of the \
code is the weight of the corresponding edge. The next rightmost digit is the \
weight of the edge connecting the next parent. We continue until we reach the \
root. We use the ",
 StyleBox[ButtonBox["StringLength",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringLength"], "FunctionLink"],
 " function in this function to determine the length of the input string and \
ensure it contains only a single character."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"encodeCharacter", "[", 
    RowBox[{
     RowBox[{"H_", "?", "binaryTreeQ"}], ",", "c_String"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"StringLength", "[", "c", "]"}], "\[Equal]", "1"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"code", ",", "vertex", ",", "parent", ",", "digit"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vertex", "=", "c"}], ";", "\[IndentingNewLine]", 
     RowBox[{"code", "=", "\"\<\>\""}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"findParent", "[", 
         RowBox[{"H", ",", "vertex"}], "]"}], "=!=", "Null"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"parent", "=", 
         RowBox[{"findParent", "[", 
          RowBox[{"H", ",", "vertex"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"digit", "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"H", ",", 
             RowBox[{"DirectedEdge", "[", 
              RowBox[{"parent", ",", "vertex"}], "]"}]}], "}"}], ",", 
           "EdgeLabels"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"code", "=", 
         RowBox[{
          RowBox[{"ToString", "[", "digit", "]"}], "<>", "code"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"vertex", "=", "parent"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "code"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"encodeCharacter", "[", 
  RowBox[{"ex23HCode", ",", "\"\<c\>\""}], "]"}]], "Input"],

Cell[BoxData["\<\"100\"\>"], "Output"]
}, Open  ]],

Cell["\<\
To encode a string, we encode each character and assemble the results. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"encodeString", "[", 
   RowBox[{
    RowBox[{"H_", "?", "binaryTreeQ"}], ",", "S_String"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"charList", ",", "code", ",", "char", ",", " ", "charcode"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"charList", "=", 
      RowBox[{"Characters", "[", "S", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"code", "=", "\"\<\>\""}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"charcode", "=", 
         RowBox[{"encodeCharacter", "[", 
          RowBox[{"H", ",", "char"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"code", "=", 
         RowBox[{"code", "<>", "charcode"}]}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"char", ",", "charList"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "code"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We use this to encode the word \[OpenCurlyDoubleQuote]ace\
\[CloseCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"encodeString", "[", 
  RowBox[{"ex23HCode", ",", "\"\<ace\>\""}], "]"}]], "Input"],

Cell[BoxData["\<\"1110000\"\>"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["11.3 Tree Traversal", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section we show how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to carry out tree traversals. Recall that a tree traversal algorithm is a \
procedure for systematically visiting every vertex of an ordered rooted tree. \
We will provide procedures for three important tree traversal algorithms: \
preorder traversal, inorder traversal, and postorder traversal. We will then \
show how to use these traversal methods to produce the prefix, infix, and \
postfix notations for arithmetic expressions."
}], "Text"],

Cell[TextData[{
 "These tree traversal algorithms all require that the tree be rooted and \
ordered. Recall how we implemented ordered rooted trees in Section 1. A tree \
that satisfies ",
 StyleBox["orderedRootedTree", "Input"],
 " is a",
 " ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " that is a rooted tree and with the additional restriction that each vertex \
has an \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] attribute."
}], "Text"],

Cell[TextData[{
 "Also recall that we illustrated in Section 1 how to use ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " to sort lists of vertices based on the \[OpenCurlyDoubleQuote]order\
\[CloseCurlyDoubleQuote] attribute. This is done by giving ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " a second argument, which is a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") that accesses and compares the \[OpenCurlyDoubleQuote]order\
\[CloseCurlyDoubleQuote] attribute of two vertices of the tree."
}], "Text"],

Cell["\<\
To begin, we will create an ordered tree to use as an example as we explore \
the three traversal algorithms. This example is a reproduction of Figure 3 \
from Section 11.3.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"fig3Tree", "=", 
   RowBox[{"Graph", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<i\>\""}], ",", 
       RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<j\>\""}], ",", 
       RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<k\>\""}], ",", 
       RowBox[{"\"\<g\>\"", "\[Rule]", "\"\<l\>\""}], ",", 
       RowBox[{"\"\<g\>\"", "\[Rule]", "\"\<m\>\""}], ",", 
       RowBox[{"\"\<k\>\"", "\[Rule]", "\"\<n\>\""}], ",", 
       RowBox[{"\"\<k\>\"", "\[Rule]", "\"\<o\>\""}], ",", 
       RowBox[{"\"\<k\>\"", "\[Rule]", "\"\<p\>\""}]}], "}"}], ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
     RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexList", "[", "fig3Tree", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"e\"\>", ",", "\<\"f\"\>", ",", "\<\"g\"\>", ",", "\<\"h\"\>", 
   ",", "\<\"i\"\>", ",", "\<\"j\"\>", ",", "\<\"k\"\>", ",", "\<\"l\"\>", 
   ",", "\<\"m\"\>", ",", "\<\"n\"\>", ",", "\<\"o\"\>", ",", "\<\"p\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig3Tree", "=", 
  RowBox[{"setChildOrder", "[", 
   RowBox[{"fig3Tree", ",", 
    RowBox[{"{", 
     RowBox[{
     "0", ",", "1", ",", "2", ",", "3", ",", "1", ",", "2", ",", "1", ",", 
      "2", ",", "3", ",", "1", ",", "2", ",", "1", ",", "2", ",", "1", ",", 
      "2", ",", "3"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", 
       "o", "p"}, {{{1, 2}, {1, 3}, {1, 4}, {2, 5}, {2, 6}, {4, 7}, {4, 8}, {
       4, 9}, {5, 10}, {5, 11}, {7, 12}, {7, 13}, {11, 14}, {11, 15}, {11, 
       16}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "i" -> {"order" -> 3}, "m" -> {"order" -> 2}, "n" -> {"order" -> 1}, 
         "h" -> {"order" -> 2}, "j" -> {"order" -> 1}, "d" -> {"order" -> 3}, 
         "c" -> {"order" -> 2}, "f" -> {"order" -> 2}, "k" -> {"order" -> 2}, 
         "b" -> {"order" -> 1}, "e" -> {"order" -> 1}, "l" -> {"order" -> 1}, 
         "g" -> {"order" -> 1}, "a" -> {"order" -> 0}, "o" -> {"order" -> 2}, 
         "p" -> {"order" -> 3}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.026831589326088433`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$11", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$12", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$13", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$11", Automatic, Center], 
          DynamicLocation["VertexID$14", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$11", Automatic, Center], 
          DynamicLocation["VertexID$15", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$11", Automatic, Center], 
          DynamicLocation["VertexID$16", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.790476123644314, 3.183068664256558}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.7957671660641396, 2.3873014981924188`}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.591534332128279, 2.3873014981924188`}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.7851850812244887`, 2.3873014981924188`}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.3978835830320698, 1.591534332128279}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.1936507490962094`, 1.591534332128279}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.989417915160349, 1.591534332128279}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.7851850812244887`, 1.591534332128279}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.580952247288628, 1.591534332128279}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.7957671660641394}, 0.03426563702160573], 
          "DynamicName", BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.7957671660641396, 0.7957671660641394}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$11"], 
         InsetBox[
          FormBox["\"k\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$11", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$11"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.591534332128279, 0.7957671660641394}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$12"], 
         InsetBox[
          FormBox["\"l\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$12", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$12"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.3873014981924188`, 0.7957671660641394}, 
           0.03426563702160573], "DynamicName", BoxID -> "VertexID$13"], 
         InsetBox[
          FormBox["\"m\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$13", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$13"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.03426563702160573], "DynamicName", BoxID -> 
          "VertexID$14"], 
         InsetBox[
          FormBox["\"n\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$14", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$14"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7957671660641396, 0.}, 0.03426563702160573], 
          "DynamicName", BoxID -> "VertexID$15"], 
         InsetBox[
          FormBox["\"o\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$15", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$15"], 
       TagBox[{
         TagBox[
          DiskBox[{1.591534332128279, 0.}, 0.03426563702160573], 
          "DynamicName", BoxID -> "VertexID$16"], 
         InsetBox[
          FormBox["\"p\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$16", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$16"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       349.00949846009195`}, {-160.28149846009194`, 144.34236144523993`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Subtrees", "Subsection",
 CellTags->"Section3a"],

Cell["\<\
Before implementing the traversal algorithms, we need a function that \
determines a subtree of a tree. In particular, we want an function that, \
given a tree and a vertex, will return the subtree with the given vertex as \
the root and that includes all of its descendants.\
\>", "Text"],

Cell[TextData[{
 "To produce the subtree, we will use the ",
 StyleBox[ButtonBox["Subgraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subgraph"], "FunctionLink"],
 " function and then copy the \[OpenCurlyDoubleQuote]order\
\[CloseCurlyDoubleQuote] attributes to the new tree from the original. The \
vertices that we want included in the subgraph are the given vertex, which \
will be the root of the subtree, together with all of its descendants. We \
begin by creating a function that finds all of the descendants of the given \
vertex. This function can apply to any rooted tree."
}], "Text"],

Cell[TextData[{
 "The approach is the same as we've used before. We begin with the given \
vertex and create a list consisting of its children, which we obtain with the \
",
 StyleBox["findChildren", "Input"],
 " function. We then begin a loop over the list of descendants. At each step, \
we add all of the children of the current vertex to the list, and then move \
on to the next vertex in the list. This continues until we reach the end of \
the list and there are no more children to add. (Note: this is referred to as \
a level-order traversal.)"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"descendants", "[", 
   RowBox[{
    RowBox[{"T_", "?", "rootedTreeQ"}], ",", "parent_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"dList", ",", "v", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dList", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "parent"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "dList", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"v", "=", 
         RowBox[{"dList", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"dList", "=", 
         RowBox[{"Join", "[", 
          RowBox[{"dList", ",", 
           RowBox[{"findChildren", "[", 
            RowBox[{"T", ",", "v"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "++"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "dList"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Compute the descendants of ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " in the example tree above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"descendants", "[", 
  RowBox[{"fig3Tree", ",", "\"\<e\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"j\"\>", ",", "\<\"k\"\>", ",", "\<\"n\"\>", ",", "\<\"o\"\>", 
   ",", "\<\"p\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To construct the subtree of an ordered rooted tree with a given vertex as \
its root, we need to: find the descendants of the given vertex; use the ",
 StyleBox[ButtonBox["Subgraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subgraph"], "FunctionLink"],
 " function on the given vertex and all its descendants; copy the \
\[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] attribute to the new \
tree; and set the \[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] \
attribute of the root to 0."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"subTree", "[", 
   RowBox[{
    RowBox[{"T_", "?", "orderedRootedTreeQ"}], ",", "newRoot_", ",", 
    "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"vList", ",", "subT", ",", "v"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"vList", "=", 
      RowBox[{"descendants", "[", 
       RowBox[{"T", ",", "newRoot"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"PrependTo", "[", 
      RowBox[{"vList", ",", "newRoot"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"subT", "=", 
      RowBox[{"Subgraph", "[", 
       RowBox[{"T", ",", "vList", ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"subT", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"T", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", "vList"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"subT", ",", "newRoot"}], "}"}], ",", "\"\<order\>\""}], 
       "]"}], "=", "0"}], ";", "\[IndentingNewLine]", "subT"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Let's check this function by finding the subtree with root ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and making sure it satisfies ",
 StyleBox["orderedRootedTreeQ", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subExample", "=", 
  RowBox[{"subTree", "[", 
   RowBox[{"fig3Tree", ",", "\"\<e\>\"", ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "e", "j", "k", "n", "o", "p"}, {{{1, 2}, {1, 3}, {3, 4}, {3, 5}, {3, 
       6}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "n" -> {"order" -> 1}, "j" -> {"order" -> 1}, "k" -> {"order" -> 2}, 
         "e" -> {"order" -> 0}, "o" -> {"order" -> 2}, "p" -> {"order" -> 3}},
        VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Arrowheads[0.0362469412612264], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.4082482904638631, 1.6329931618554523`}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.8164965809277261}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.8164965809277261, 0.8164965809277261}, 
           0.019562323618881772`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"k\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.019562323618881772`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"n\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8164965809277261, 0.}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"o\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{1.6329931618554523`, 0.}, 0.019562323618881772`], 
          "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"p\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.8, 350.7886658983125}, {-178.0606658983113, 
       161.928}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orderedRootedTreeQ", "[", "subExample", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Traversal Algorithms", "Subsection",
 CellTags->"Section3b"],

Cell["\<\
We now implement the three traversal algorithms described in Section 11.3 of \
the text. We begin with the preorder traversal algorithm, which is given as \
Algorithm 1 in the text.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Preorder", "Subsubsection"],

Cell["\<\
Given an ordered rooted tree, the preorder algorithm acts as follows. First, \
it prints the name of the root. Then it calculates the children of the root \
and stores them in order. For each child, in order, the function recursively \
applies itself to the subtree with the given child as root.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"preorder", "[", 
   RowBox[{"T_", "?", "orderedRootedTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"root", ",", "children", ",", "i", ",", "tempSubT"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"root", "=", 
      RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "root", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"children", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "root"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"children", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"children", ",", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "#1"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
           "<", 
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "#2"}], "}"}], ",", "\"\<order\>\""}], 
            "]"}]}], ")"}], "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "children", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tempSubT", "=", 
         RowBox[{"subTree", "[", 
          RowBox[{"T", ",", 
           RowBox[{"children", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"preorder", "[", "tempSubT", "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"preorder", "[", "fig3Tree", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["\<\"a\"\>"], "Print"],

Cell[BoxData["\<\"b\"\>"], "Print"],

Cell[BoxData["\<\"e\"\>"], "Print"],

Cell[BoxData["\<\"j\"\>"], "Print"],

Cell[BoxData["\<\"k\"\>"], "Print"],

Cell[BoxData["\<\"n\"\>"], "Print"],

Cell[BoxData["\<\"o\"\>"], "Print"],

Cell[BoxData["\<\"p\"\>"], "Print"],

Cell[BoxData["\<\"f\"\>"], "Print"],

Cell[BoxData["\<\"c\"\>"], "Print"],

Cell[BoxData["\<\"d\"\>"], "Print"],

Cell[BoxData["\<\"g\"\>"], "Print"],

Cell[BoxData["\<\"l\"\>"], "Print"],

Cell[BoxData["\<\"m\"\>"], "Print"],

Cell[BoxData["\<\"h\"\>"], "Print"],

Cell[BoxData["\<\"i\"\>"], "Print"]
}, Open  ]]
}, Open  ]],

Cell["\<\
You can confirm that this output is consistent with the preorder traversal \
demonstrated in Figure 4 of Section 11.3 of the textbook.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Postorder", "Subsubsection"],

Cell["\<\
Postorder traversal, described in Algorithm 3 of the text, is very similar to \
preorder traversal. The only change needed in the code is that, instead of \
printing the root at the start of the algorithm, the vertex is printed after \
the loop is completed.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"postorder", "[", 
   RowBox[{"T_", "?", "orderedRootedTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"root", ",", "children", ",", "i", ",", "tempSubT"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"root", "=", 
      RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"children", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "root"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"children", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"children", ",", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "#1"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
           "<", 
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "#2"}], "}"}], ",", "\"\<order\>\""}], 
            "]"}]}], ")"}], "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "children", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tempSubT", "=", 
         RowBox[{"subTree", "[", 
          RowBox[{"T", ",", 
           RowBox[{"children", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"postorder", "[", "tempSubT", "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "root", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"postorder", "[", "fig3Tree", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["\<\"j\"\>"], "Print"],

Cell[BoxData["\<\"n\"\>"], "Print"],

Cell[BoxData["\<\"o\"\>"], "Print"],

Cell[BoxData["\<\"p\"\>"], "Print"],

Cell[BoxData["\<\"k\"\>"], "Print"],

Cell[BoxData["\<\"e\"\>"], "Print"],

Cell[BoxData["\<\"f\"\>"], "Print"],

Cell[BoxData["\<\"b\"\>"], "Print"],

Cell[BoxData["\<\"c\"\>"], "Print"],

Cell[BoxData["\<\"l\"\>"], "Print"],

Cell[BoxData["\<\"m\"\>"], "Print"],

Cell[BoxData["\<\"g\"\>"], "Print"],

Cell[BoxData["\<\"h\"\>"], "Print"],

Cell[BoxData["\<\"i\"\>"], "Print"],

Cell[BoxData["\<\"d\"\>"], "Print"],

Cell[BoxData["\<\"a\"\>"], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Inorder", "Subsubsection"],

Cell["\<\
In inorder traversal, the algorithm first applies itself recursively to the \
first child of the vertex, then it prints the vertex, and then it applies \
itself to the remainder of the children, in order.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"inorder", "[", 
   RowBox[{"T_", "?", "orderedRootedTreeQ"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"root", ",", "children", ",", "i", ",", "tempSubT"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"root", "=", 
      RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"children", "=", 
      RowBox[{"findChildren", "[", 
       RowBox[{"T", ",", "root"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"children", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"children", ",", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "#1"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
           "<", 
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"T", ",", "#2"}], "}"}], ",", "\"\<order\>\""}], 
            "]"}]}], ")"}], "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "children", "]"}], "\[NotEqual]", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tempSubT", "=", 
         RowBox[{"subTree", "[", 
          RowBox[{"T", ",", 
           RowBox[{"children", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"inorder", "[", "tempSubT", "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "root", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "2"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "children", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tempSubT", "=", 
         RowBox[{"subTree", "[", 
          RowBox[{"T", ",", 
           RowBox[{"children", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"inorder", "[", "tempSubT", "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"inorder", "[", "fig3Tree", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData["\<\"j\"\>"], "Print"],

Cell[BoxData["\<\"e\"\>"], "Print"],

Cell[BoxData["\<\"n\"\>"], "Print"],

Cell[BoxData["\<\"k\"\>"], "Print"],

Cell[BoxData["\<\"o\"\>"], "Print"],

Cell[BoxData["\<\"p\"\>"], "Print"],

Cell[BoxData["\<\"b\"\>"], "Print"],

Cell[BoxData["\<\"f\"\>"], "Print"],

Cell[BoxData["\<\"a\"\>"], "Print"],

Cell[BoxData["\<\"c\"\>"], "Print"],

Cell[BoxData["\<\"l\"\>"], "Print"],

Cell[BoxData["\<\"g\"\>"], "Print"],

Cell[BoxData["\<\"m\"\>"], "Print"],

Cell[BoxData["\<\"d\"\>"], "Print"],

Cell[BoxData["\<\"h\"\>"], "Print"],

Cell[BoxData["\<\"i\"\>"], "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Infix Notation", "Subsection",
 CellTags->"Section3c"],

Cell[TextData[{
 "In the remainder of this section, we discuss how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to work with the infix, prefix, and postfix forms of arithmetic \
expressions, as described in Section 11.3 of the text. In this subsection, we \
will show how to create a tree representation of an infix expression. In the \
next subsection, we will explore how to evaluate expressions from their \
postfix and prefix forms."
}], "Text"],

Cell[TextData[{
 "Recall that infix notation is the usual notation for basic arithmetic and \
algebraic expressions. We will construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that takes an infix expression and converts it into a tree \
representation. This tree representation can then be traversed using the \
traversals of the previous sections to form various arithmetic representation \
formats."
}], "Text"],

Cell["\<\
The algorithm we use to turn an arithmetic expression in infix notation into \
a tree is recursive. The basis case occurs when the expression consists of a \
single number or variable. In this case, the tree consists of a single vertex.\
\>", "Text"],

Cell["\<\
Otherwise, the expression consists of a left operand, an operator, and a \
right operand. In this case, we (1) apply the algorithm to the left and right \
operands, and (2) combine the resulting trees with the operator as the common \
root. Implementing this will require some preliminary work. \
\>", "Text"],

Cell[TextData[{
 "First, we need to represent arithmetic expressions in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in such a way that we can work with them and ensure that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " won't evaluate them. This can be accomplished by applying ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " to the expression when entering it, as illustrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"2", "+", 
   RowBox[{"3", "*", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"2", "+", 
   RowBox[{"3", " ", "5"}]}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
Second, we need to be able to distinguish the basis case from the recursive \
case.\
\>", "Text"],

Cell[TextData[{
 "Third, the leaves in the tree will be numbers and variables and the \
internal vertices will be operations. In an expression like ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "\[CenterDot]", "4"}], "+", 
    RowBox[{"7", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"3", "+", "x"}], ")"}]}]}], TraditionalForm]]],
 ", we have repetition among the operators and the operands (two 3's, two \
additions, and two multiplications). We need a way to get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to consider each of these to be a distinct object, since ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " insists that the vertices in a graph be distinct, while displaying them \
with the same symbol in the tree."
}], "Text"],

Cell["\<\
Fourth, in the recursive step, we need to be able to identify the operator \
and separate the left and right operands.\
\>", "Text"],

Cell["\<\
And fifth, we will need to implement a function to perform the combination of \
subtrees described in part (2) of the recursive step.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Distinguishing the Basis and Recursive Cases", "Subsubsection"],

Cell["\<\
Any arithmetic expression is either a single integer or variable, or it is \
two expressions joined by an arithmetic operator. \
\>", "Text"],

Cell["\<\
We can determine which kind it is by testing the head of the expression.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "5", "]"}]], "Input"],

Cell[BoxData["Integer"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "x", "]"}]], "Input"],

Cell[BoxData["Symbol"], "Output"]
}, Open  ]],

Cell[TextData[{
 "However, observe that an expression such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "+", "5"}], TraditionalForm]]],
 " also reports having head ",
 StyleBox[ButtonBox["Integer",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Integer"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", 
  RowBox[{"3", "+", "5"}], "]"}]], "Input"],

Cell[BoxData["Integer"], "Output"]
}, Open  ]],

Cell[TextData[{
 "This is because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " evaluates the arithmetic before apply the ",
 StyleBox[ButtonBox["Head",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Head"], "FunctionLink"],
 " function. Recall from above that we will be using ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " to prevent arithmetic evaluation and algebraic simplification. But with ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " in place, ",
 StyleBox[ButtonBox["Head",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Head"], "FunctionLink"],
 " will identify the head of the expression as ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", 
  RowBox[{"Hold", "[", 
   RowBox[{"3", "+", "5"}], "]"}], "]"}]], "Input"],

Cell[BoxData["Hold"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To obtain the correct head of an expression such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "+", "5"}], TraditionalForm]]],
 " while preventing evaluation, we need to use ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 "). Remember that 0 represents the head within a ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 "). "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"5", "[", 
  RowBox[{"[", "0", "]"}], "]"}]], "Input"],

Cell[BoxData["Integer"], "Output"]
}, Open  ]],

Cell[TextData[{
 "For a held expression, ",
 StyleBox["[[1,0]]", "Input"],
 " will give the head of the expression within the ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hold", "[", 
   RowBox[{"3", "+", "5"}], "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "0"}], "]"}], "]"}]], "Input"],

Cell[BoxData["Plus"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Ensuring That Each Occurrence of an Object Is Considered Distinct\
\>", "Subsubsection"],

Cell[TextData[{
 "The tree associated to the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "\[CenterDot]", "4"}], "+", 
    RowBox[{"7", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"3", "+", "x"}], ")"}]}]}], TraditionalForm]]],
 " will have 9 vertices. The internal vertices, the operations, consist of \
two additions and two multiplications. The leaves, the numbers and variables, \
consist of ",
 Cell[BoxData[
  FormBox["4", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["7", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", and two ",
 Cell[BoxData[
  FormBox["3", TraditionalForm]]],
 "s."
}], "Text"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " graphs, each vertex must be unique and distinct from all other vertices. \
In order to make ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " consider two 3s or two additions to be different, we will take advantage \
of the fact that a ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " structure attaches an integer to each variable name in order to ensure it \
is unique. The following reveals the internal symbol associated to a local \
variable."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Module", "[", 
  RowBox[{
   RowBox[{"{", "var", "}"}], ",", 
   RowBox[{"ToString", "[", "var", "]"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"var$2827\"\>"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To use this while building our expression tree, we will define an alternate \
version of ",
 StyleBox["newBinaryTree", "Input"],
 ", which was defined earlier in the chapter. This new function, ",
 StyleBox["newExpressionTree", "Input"],
 ", will declare a local variable and set it equal to the result of applying ",
 StyleBox[ButtonBox["ToString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToString"], "FunctionLink"],
 " to the variable. Since the local variable had not previously been \
assigned, this will store the internal representation of the variable. We \
then use that as the name of the vertex in a new tree. We set the ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " property in order to have the graph displayed with the elements of the \
expression rather than the internal name."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"newExpressionTree", "[", 
   RowBox[{"r_", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"T", ",", "v"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"v", "=", 
      RowBox[{"ToString", "[", "v", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"{", "v", "}"}], ",", 
        RowBox[{"{", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}], "=", 
      "r"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], "=", 
      "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "T", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Now we are able to create trees and we will be assured that each one will \
have distinct vertex names, internally.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Identifying the Operator and Operands", "Subsubsection",
 PageBreakWithin->False],

Cell["\<\
In the recursive case, we must separate a complex expression into its \
operator and the left and right operands. Consider the following example.\
\>", "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleExpr", "=", 
  RowBox[{"Hold", "[", 
   RowBox[{
    RowBox[{"3", "*", "4"}], "+", 
    RowBox[{"7", "*", 
     RowBox[{"(", 
      RowBox[{"3", "+", "x"}], ")"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{
   RowBox[{"3", " ", "4"}], "+", 
   RowBox[{"7", " ", 
    RowBox[{"(", 
     RowBox[{"3", "+", "x"}], ")"}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This expression, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "\[CenterDot]", "4"}], "+", 
    RowBox[{"7", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"3", "+", "x"}], ")"}]}]}], TraditionalForm]]],
 " consists of the sum of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "\[CenterDot]", "4"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", "\[CenterDot]", 
    RowBox[{"(", 
     RowBox[{"3", "+", "x"}], ")"}]}], TraditionalForm]]],
 ". In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we will obtain these two operands by applying ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " is similar to ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") or ",
 StyleBox[ButtonBox["Take",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Take"], "FunctionLink"],
 " in that it returns a part of an expression. However, it allows an optional \
third argument to apply a head to the part of the expression before it is \
returned. This is important here because we need to prevent ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from simplifying the operands."
}], "Text"],

Cell[TextData[{
 "The arguments to ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " will be the expression, the part specification, and the head ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 ". To see what the part specification should be, look at the full form of ",
 StyleBox["exampleExpr", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "exampleExpr", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Hold", "[", 
    RowBox[{"Plus", "[", 
     RowBox[{
      RowBox[{"Times", "[", 
       RowBox[{"3", ",", "4"}], "]"}], ",", 
      RowBox[{"Times", "[", 
       RowBox[{"7", ",", 
        RowBox[{"Plus", "[", 
         RowBox[{"3", ",", "x"}], "]"}]}], "]"}]}], "]"}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We see that the expression is entirely enclosed in ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 ", so part specification ",
 StyleBox["{1}", "Input"],
 " will refer to the entire algebraic expression, which has head ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 ". The arguments of the top ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 " are then referenced as ",
 StyleBox["{1,1}", "Input"],
 " and ",
 StyleBox["{1,2}", "Input"],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Extract", "[", 
  RowBox[{"exampleExpr", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", "Hold"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"3", " ", "4"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Extract", "[", 
  RowBox[{"exampleExpr", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"7", " ", 
   RowBox[{"(", 
    RowBox[{"3", "+", "x"}], ")"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that we can use ",
 StyleBox["{1,0}", "Input"],
 " to obtain the head of the expression, that is, the operator. This is \
equivalent to the use of ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") described above for the same purpose."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Extract", "[", 
  RowBox[{"exampleExpr", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["Plus"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleExpr", "[", 
  RowBox[{"[", 
   RowBox[{"1", ",", "0"}], "]"}], "]"}]], "Input"],

Cell[BoxData["Plus"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combining Subtrees", "Subsubsection"],

Cell[TextData[{
 "As part of Huffman coding in Section 2, we wrote a function, ",
 StyleBox["joinHTrees", "Input"],
 ", for joining two existing trees at a new root. We recreate that function \
here, with the weights removed."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"joinTrees", "[", 
   RowBox[{"newR_", ",", 
    RowBox[{"A_", "?", "binaryTreeQ"}], ",", 
    RowBox[{"B_", "?", "binaryTreeQ"}], ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "newRI", ",", "newT", ",", "newVerts", ",", "Aroot", ",", "Broot", ",", 
      "newEdges", ",", "v", ",", "e", ",", "p", ",", "w"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newRI", "=", 
      RowBox[{"ToString", "[", "newRI", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newVerts", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "newRI", "}"}], ",", 
        RowBox[{"VertexList", "[", "A", "]"}], ",", 
        RowBox[{"VertexList", "[", "B", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Aroot", "=", 
      RowBox[{"findRoot", "[", "A", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Broot", "=", 
      RowBox[{"findRoot", "[", "B", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newEdges", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"EdgeList", "[", "A", "]"}], ",", 
        RowBox[{"EdgeList", "[", "B", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"DirectedEdge", "[", 
           RowBox[{"newRI", ",", "Aroot"}], "]"}], ",", 
          RowBox[{"DirectedEdge", "[", 
           RowBox[{"newRI", ",", "Broot"}], "]"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{"newVerts", ",", "newEdges", ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"A", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "A", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"B", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "B", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "Aroot"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "Broot"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newRI"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"A", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "A", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"B", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "B", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newRI"}], "}"}], ",", "VertexLabels"}], "]"}], 
      "=", "newR"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "newT", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Main Function", "Subsubsection"],

Cell[TextData[{
 "With ",
 StyleBox["joinTrees", "Input"],
 " and ",
 StyleBox["newExpressionTree", "Input"],
 " from above, we are ready to write the function for turning infix \
expression into binary trees."
}], "Text"],

Cell[TextData[{
 "The function accepts a single argument, ",
 StyleBox["expr", "Input"],
 ", the expression. We will give the function the ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " attribute, so that the expression will be held without our having to \
explicitly type ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " when executing the function. This holds the argument only until it is \
first evaluated, so we must immediately place a more permanent ",
 StyleBox[ButtonBox["Hold",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Hold"], "FunctionLink"],
 " on it, provided it was not already explicitly held, and assign it to ",
 StyleBox["e", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "We first test the type of ",
 StyleBox["e", "Input"],
 ". If it is an integer or a symbol, then we use ",
 StyleBox["newExpressionTree", "Input"],
 " to create a new binary tree with ",
 StyleBox["e", "Input"],
 " as the only vertex. "
}], "Text"],

Cell[TextData[{
 "Otherwise, we're in the recursive case. We use ",
 StyleBox[ButtonBox["Extract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Extract"], "FunctionLink"],
 " to determine the left and right operands and the operator. After recursive \
calls to the function to create the trees for the two operands, the subtrees \
are joined at the operator into the result tree."
}], "Text"],

Cell["Here is the function.", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"expressionTree", ",", "HoldFirst"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"expressionTree", "[", 
   RowBox[{"expr_", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "e", ",", "lhs", ",", "rhs", ",", "operator", ",", "lhsTree", ",", 
      "rhsTree", ",", "result"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"expr", "[", 
         RowBox[{"[", "0", "]"}], "]"}], "===", "Hold"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"e", "=", "expr"}], ",", "\[IndentingNewLine]", 
       RowBox[{"e", "=", 
        RowBox[{"Hold", "[", "expr", "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"e", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "0"}], "]"}], "]"}], "===", "Integer"}], "||", 
        RowBox[{
         RowBox[{"e", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "0"}], "]"}], "]"}], "===", "Symbol"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"result", "=", 
        RowBox[{"newExpressionTree", "[", 
         RowBox[{
          RowBox[{"ReleaseHold", "[", "e", "]"}], ",", "opts"}], "]"}]}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"lhs", "=", 
         RowBox[{"Extract", "[", 
          RowBox[{"e", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "1"}], "}"}], ",", "Hold"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"operator", "=", 
         RowBox[{"Extract", "[", 
          RowBox[{"e", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"rhs", "=", 
         RowBox[{"Extract", "[", 
          RowBox[{"e", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"lhsTree", "=", 
         RowBox[{"expressionTree", "[", "lhs", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"rhsTree", "=", 
         RowBox[{"expressionTree", "[", "rhs", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"result", "=", 
         RowBox[{"joinTrees", "[", 
          RowBox[{"operator", ",", "lhsTree", ",", "rhsTree", ",", "opts"}], 
          "]"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "result", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 InitializationCell->True],

Cell["We test the function on the example expression.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleTree", "=", 
  RowBox[{"expressionTree", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"3", "*", "4"}], "+", 
     RowBox[{"7", "*", 
      RowBox[{"(", 
       RowBox[{"3", "+", "x"}], ")"}]}]}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "newRI$3157", "newRI$2868", "v$2831", "v$2848", "newRI$3041", "v$2926", 
       "newRI$2981", "v$2944", "v$2961"}, {{{2, 3}, {2, 4}, {7, 8}, {7, 9}, {
       5, 6}, {5, 7}, {1, 2}, {1, 5}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "newRI$3041" -> {"order" -> 2}, "v$2831" -> {"order" -> 1}, 
         "v$2848" -> {"order" -> 2}, "v$2944" -> {"order" -> 1}, 
         "newRI$3157" -> {"order" -> 0}, "newRI$2981" -> {"order" -> 2}, 
         "newRI$2868" -> {"order" -> 1}, "v$2961" -> {"order" -> 2}, 
         "v$2926" -> {"order" -> 1}}, VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[1, 4], 
          Rational[2, 3]}, {
          Rational[1, 8], 
          Rational[1, 3]}, {
          Rational[3, 8], 
          Rational[1, 3]}, {
          Rational[3, 4], 
          Rational[2, 3]}, {
          Rational[5, 8], 
          Rational[1, 3]}, {
          Rational[7, 8], 
          Rational[1, 3]}, {
          Rational[13, 16], 0}, {
          Rational[15, 16], 0}}, 
       VertexLabels -> {
        "v$2831" -> 3, "newRI$3041" -> Times, "newRI$2868" -> Times, "v$2848" -> 
         4, "v$2944" -> 3, "newRI$3157" -> Plus, "v$2926" -> 7, 
         "v$2961" -> $CellContext`x, "newRI$2981" -> Plus}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["Plus", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["Times", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.6666666666666666}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["Times", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["7", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.3333333333333333}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["Plus", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.9375, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["x", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.79191379163164, 
       350.06028203168836`}, {-213.73626900177285`, 198.47976276003365`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Prefix and Postfix Notation", "Subsection",
 CellTags->"Section3d"],

Cell[TextData[{
 "Suppose we are given a binary tree representation of an arithmetic \
expression. We can express these trees in postfix, infix, or prefix form by \
applying the respective traversal algorithm we designed above. It is left to \
the reader to make the necessary functions to output infix, prefix, and \
postfix expressions. Be careful while doing so to keep in mind that the \
integers, variables, and operators are stored as the ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " property of the vertices of the tree, not as the vertices themselves."
}], "Text"],

Cell["\<\
As a final example in this section, we demonstrate how to evaluate a given \
postfix expression. We will represent the postfix expression as a list of \
symbols, each of which is either a number or one of the arithmetic \
operations' symbols as a string.\
\>", "Text"],

Cell["\<\
Since we're considering postfix expressions, we read the list of symbols from \
left to right. Each time we encounter an operation, that operation is applied \
to the previous two numbers and we update the list by replacing the two \
numbers and the operation symbol by the result of the operation.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"evalPostfix", "[", "expr_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "L"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"L", "=", "expr"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "L", "]"}], ">", "1"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"FreeQ", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
             "\"\<+\>\"", ",", "\"\<-\>\"", ",", "\"\<*\>\"", ",", 
              "\"\</\>\"", ",", "\"\<^\>\""}], "}"}], ",", 
            RowBox[{"L", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
          RowBox[{"i", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Switch", "[", 
         RowBox[{
          RowBox[{"L", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", 
          "\"\<+\>\"", ",", 
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "2"}], "]"}], "]"}], "+", 
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "1"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", "\"\<-\>\"", ",", 
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "2"}], "]"}], "]"}], "-", 
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "1"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", "\"\<*\>\"", ",", 
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "2"}], "]"}], "]"}], "*", 
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "1"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", "\"\</\>\"", ",", 
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "2"}], "]"}], "]"}], "/", 
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "1"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", "\"\<^\>\"", ",", 
          RowBox[{
           RowBox[{"L", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "2"}], "]"}], "]"}], "^", 
            RowBox[{"L", "[", 
             RowBox[{"[", 
              RowBox[{"i", "-", "1"}], "]"}], "]"}]}]}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"L", "=", 
         RowBox[{"Drop", "[", 
          RowBox[{"L", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"i", "-", "2"}], ",", 
             RowBox[{"i", "-", "1"}]}], "}"}]}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"L", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that the ",
 StyleBox[ButtonBox["Drop",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Drop"], "FunctionLink"],
 " function removes a range of elements of the list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"postfixExample", "=", 
  RowBox[{"{", 
   RowBox[{
   "7", ",", "2", ",", "3", ",", "\"\<*\>\"", ",", "\"\<-\>\"", ",", "4", 
    ",", "\"\<^\>\"", ",", "9", ",", "3", ",", "\"\</\>\"", ",", 
    "\"\<+\>\""}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "7", ",", "2", ",", "3", ",", "\<\"*\"\>", ",", "\<\"-\"\>", ",", "4", 
   ",", "\<\"^\"\>", ",", "9", ",", "3", ",", "\<\"/\"\>", ",", "\<\"+\"\>"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"evalPostfix", "[", "postfixExample", "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell["\<\
The reader is left to explore evaluation in the prefix case, which requires \
only a simple modification.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["11.4 Spanning Trees", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "This section explains how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to construct spanning trees for graphs and how to use spanning trees to \
solve many different types of problems. Spanning trees have a myriad of \
applications, including coloring graphs, placing ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " queens on a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Times]", "n"}], TraditionalForm]]],
 " chessboard so that no two of the queens attack each other, and finding a \
subset of a set of numbers with a specified sum. All of these problems, which \
are described in detail in the text, will be explored computationally in this \
section. First we will show how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to form spanning trees using two algorithms: depth-first search and \
breadth-first search. Then we will show how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve the problems just mentioned."
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes two functions, ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " that will perform a search on a graph. Both functions accept a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " and a vertex of the graph as the first two arguments. With these two \
arguments, the functions will return a list of vertices ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["v", "1"], ",", 
     SubscriptBox["v", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["v", "n"]}], "}"}], TraditionalForm]]],
 ". Suppose that the ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 " function returns ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["u", "1"], ",", 
     SubscriptBox["u", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["u", "n"]}], "}"}], TraditionalForm]]],
 ". Then the output of ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " indicates that the tree whose edges are ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "i"], "\[DirectedEdge]", 
    SubscriptBox["u", "i"]}], TraditionalForm]]],
 " is a spanning tree of the graph, with root the vertex such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["u", "i"], "=", 
    SubscriptBox["v", "i"]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "The function ",
 StyleBox[ButtonBox["TreeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraph"], "FunctionLink"],
 " can be used to form a tree from the output of ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["TreeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraph"], "FunctionLink"],
 " can accept as input two lists of vertices, where the first is the list of \
all vertices to appear in the tree and the second is the list each of whose \
elements is the predecessor of the corresponding element of the first list. A \
simple example is shown below of a tree with root 2, vertices 1 and 3 are the \
children of 2, and 4 and 5 are the children of 3, and vertex 6 is the child \
of 1."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2", ",", "3", ",", "3", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{2, 1}, {2, 3}, {3, 4}, {3, 5}, {1, 6}}}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.8687444855261388}, 0.02044525157230459], 
          "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["1", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.6515583641446041, 1.7374889710522776`}, 
           0.02044525157230459], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["2", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3031167282892082`, 0.8687444855261388}, 
           0.02044525157230459], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["3", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.8687444855261388, 0.}, 0.02044525157230459], 
          "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["4", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{1.7374889710522776`, 0.}, 0.02044525157230459], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["5", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02044525157230459], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["6", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       350.80803137831595`}, {-178.0800313783156, 161.92800000000005`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We will illustrate the use of ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " with the graph from Exercise 13 of Section 11.4, which we recreate."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise13", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\"", ",", "\"\<h\>\"", ",", 
      "\"\<i\>\"", ",", "\"\<j\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
      RowBox[{"\"\<f\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<g\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
      RowBox[{"\"\<g\>\"", "\[Rule]", "\"\<j\>\""}], ",", 
      RowBox[{"\"\<h\>\"", "\[Rule]", "\"\<i\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0.5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "0.5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"5", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"5", ",", "0"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "5"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {
      Null, {{1, 2}, {1, 3}, {2, 3}, {3, 4}, {4, 5}, {4, 6}, {5, 6}, {5, 8}, {
       6, 7}, {7, 8}, {7, 10}, {8, 9}}}, {
      ImagePadding -> 5, 
       VertexCoordinates -> {{0, 1}, {0, 0}, {1, 0.5}, {2, 0.5}, {3, 1}, {3, 
         0}, {4, 0}, {4, 1}, {5, 1}, {5, 0}}, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}, {
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}, {
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}, {
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{10.999999999999998`, 
       353.8741580079276}, {-51.15415800792756, 34.47018399747658}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->5]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " and passing the output to ",
 StyleBox[ButtonBox["TreeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreeGraph"], "FunctionLink"],
 " allows us to draw the spanning tree obtained by a depth-first search."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise13DepthFirst", "=", 
  RowBox[{"DepthFirstScan", "[", 
   RowBox[{"exercise13", ",", "\"\<a\>\""}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
   ",", "\<\"d\"\>", ",", "\<\"e\"\>", ",", "\<\"f\"\>", ",", "\<\"g\"\>", 
   ",", "\<\"h\"\>", ",", "\<\"g\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraph", "[", 
  RowBox[{
   RowBox[{"VertexList", "[", "exercise13", "]"}], ",", 
   "exercise13DepthFirst", ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {
      Null, {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 7}, {7, 8}, {8, 9}, {
       7, 10}}}, {ImagePadding -> 10, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}, {
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}, {
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.03493468166720967], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.9203579866168443}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.8407159732336889`}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 2.7610739598505334`}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.6902684899626333, 3.6814319464673777`}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{1.3805369799252667`, 2.7610739598505334`}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3805369799252667`, 1.8407159732336889`}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9203579866168444, 0.9203579866168443}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.9203579866168444, 0.}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.8407159732336889`, 0.9203579866168443}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{13.28431822849494, 
       225.5373569013778}, {-216.4928182057671, 200.8870545006448}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " produces a different tree."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TreeGraph", "[", 
  RowBox[{
   RowBox[{"VertexList", "[", "exercise13", "]"}], ",", 
   RowBox[{"BreadthFirstScan", "[", 
    RowBox[{"exercise13", ",", "\"\<a\>\""}], "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {
      Null, {{1, 2}, {1, 3}, {3, 4}, {4, 5}, {4, 6}, {6, 7}, {5, 8}, {8, 9}, {
       7, 10}}}, {ImagePadding -> 10, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}, {
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}, {
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}, {
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}, {
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.9045340337332908}, 0.02814984257149772], 
          "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02814984257149772], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.8090680674665816`}, 0.02814984257149772], 
          "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9045340337332908, 2.7136021011998723`}, 
           0.02814984257149772], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9045340337332908, 1.8090680674665816`}, 
           0.02814984257149772], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.8090680674665816`, 1.8090680674665816`}, 
           0.02814984257149772], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.8090680674665816`, 0.9045340337332908}, 
           0.02814984257149772], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9045340337332908, 0.9045340337332908}, 
           0.02814984257149772], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.9045340337332908, 0.}, 0.02814984257149772], 
          "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{1.8090680674665816`, 0.}, 0.02814984257149772], 
          "DynamicName", BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{14.628750344705832`, 
       293.3216819769149}, {-215.1093147554163, 199.5963996690824}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that in both cases, the initial vertex, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", is not drawn at the root of the tree. As we described previously, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draws trees to minimize depth, irrespective of the logical root."
}], "Text"],

Cell[TextData[{
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " both accept an optional third argument, which is given as a list of rules. \
The rules in the list identify an \[OpenCurlyDoubleQuote]event\
\[CloseCurlyDoubleQuote] with a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") to be executed when the event occurs. We will illustrate this by \
highlighting the ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " tree within the graph. We make a copy of the graph first, since our \
execution of ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " will modify the graph."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"exercise13Copy", "=", "exercise13"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"BreadthFirstScan", "[", 
   RowBox[{"exercise13Copy", ",", "\"\<a\>\"", ",", 
    RowBox[{"{", 
     RowBox[{"\"\<FrontierEdge\>\"", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"PropertyValue", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"exercise13Copy", ",", "#"}], "}"}], ",", "EdgeStyle"}], 
           "]"}], "=", 
          RowBox[{"Directive", "[", 
           RowBox[{"Thick", ",", "Red"}], "]"}]}], ")"}], "&"}], ")"}]}], 
     "}"}]}], "]"}], ";", " ", "exercise13Copy"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {
      Null, {{1, 2}, {1, 3}, {2, 3}, {3, 4}, {4, 5}, {4, 6}, {5, 6}, {5, 8}, {
       6, 7}, {7, 8}, {7, 10}, {8, 9}}}, {
      EdgeStyle -> {UndirectedEdge["e", "h"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["a", "b"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["d", "e"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["a", "c"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["g", "j"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["h", "i"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["d", "f"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["f", "g"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]], UndirectedEdge["c", "d"] -> Directive[
           Thickness[Large], 
           RGBColor[1, 0, 0]]}, ImagePadding -> 5, 
       VertexCoordinates -> {{0, 1}, {0, 0}, {1, 0.5}, {2, 0.5}, {3, 1}, {3, 
         0}, {4, 0}, {4, 1}, {5, 1}, {5, 0}}, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       LineBox[{
         DynamicLocation["VertexID$2", Automatic, Center], 
         DynamicLocation["VertexID$3", Automatic, Center]}], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       LineBox[{
         DynamicLocation["VertexID$5", Automatic, Center], 
         DynamicLocation["VertexID$6", Automatic, Center]}], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       LineBox[{
         DynamicLocation["VertexID$7", Automatic, Center], 
         DynamicLocation["VertexID$8", Automatic, Center]}], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False], 
       StyleBox[
        LineBox[{
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}], 
        Directive[
         Thickness[Large], 
         RGBColor[1, 0, 0]], StripOnInput -> False]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{10.999999999999998`, 
       353.8741580079276}, {-51.15415800792756, 34.47018399747658}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->5]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We now analyze the function executed above. ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " is called on the graph ",
 StyleBox["exercise13Copy", "Input"],
 " and told to start with vertex ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". The third argument is the list containing a single ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Input"],
 ") identifying the event \[OpenCurlyDoubleQuote]FrontierEdge\
\[CloseCurlyDoubleQuote] with a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 "). A frontier edge refers to an edge between a vertex that is currently \
being visited and a vertex that is newly discovered as a result of visiting \
the current vertex. The pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") associated to the event \[OpenCurlyDoubleQuote]FrontierEdge\
\[CloseCurlyDoubleQuote] is called with a single argument, the edge in \
question. In this example, we use the function to highlight the edges which \
are traversed by the breadth-first exploration. "
}], "Text"],

Cell[TextData[{
 "Alternately, we could use ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " to obtain a list of the edges in the order they are followed. Note that ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " accepts a single argument so it is not necessary to explicitly call it on \
a ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "FunctionLink"],
 " (",
 StyleBox["#", "Input"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise13breadthList", "=", 
  RowBox[{
   RowBox[{"Reap", "[", 
    RowBox[{"BreadthFirstScan", "[", 
     RowBox[{"exercise13", ",", "\"\<a\>\"", ",", 
      RowBox[{"{", 
       RowBox[{"\"\<FrontierEdge\>\"", "\[Rule]", "Sow"}], "}"}]}], "]"}], 
    "]"}], "[", 
   RowBox[{"[", 
    RowBox[{"2", ",", "1"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"b\"\>"}], ",", 
   RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"c\"\>", "\[UndirectedEdge]", "\<\"d\"\>"}], ",", 
   RowBox[{"\<\"d\"\>", "\[UndirectedEdge]", "\<\"e\"\>"}], ",", 
   RowBox[{"\<\"d\"\>", "\[UndirectedEdge]", "\<\"f\"\>"}], ",", 
   RowBox[{"\<\"e\"\>", "\[UndirectedEdge]", "\<\"h\"\>"}], ",", 
   RowBox[{"\<\"f\"\>", "\[UndirectedEdge]", "\<\"g\"\>"}], ",", 
   RowBox[{"\<\"h\"\>", "\[UndirectedEdge]", "\<\"i\"\>"}], ",", 
   RowBox[{"\<\"g\"\>", "\[UndirectedEdge]", "\<\"j\"\>"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["We can now use this list to create an animation.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Animate", "[", 
  RowBox[{
   RowBox[{"HighlightGraph", "[", 
    RowBox[{"exercise13", ",", 
     RowBox[{"exercise13breadthList", "[", 
      RowBox[{"[", 
       RowBox[{"1", ";;", "i"}], "]"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", 
     RowBox[{"Length", "[", "exercise13breadthList", "]"}], ",", "1"}], "}"}],
    ",", 
   RowBox[{"AnimationRunning", "\[Rule]", "False"}], ",", 
   RowBox[{"AnimationRepetitions", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`i$$], 0, 9, 1}}, Typeset`size$$ = {360., {45., 50.}}, 
    Typeset`update$$ = 0, Typeset`initDone$$, Typeset`skipInitDone$$ = 
    True, $CellContext`i$3466$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$$, $CellContext`i$3466$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> 
      HighlightGraph[$CellContext`exercise13, 
        Part[$CellContext`exercise13breadthList, 
         Span[1, $CellContext`i$$]]], 
      "Specifications" :> {{$CellContext`i$$, 0, 9, 1, AnimationRunning -> 
         False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{403., {82., 87.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output"]
}, Open  ]],

Cell["\<\
We leave it to the reader to create a function that generates such an \
animation automatically.\
\>", "Text"],

Cell[TextData[{
 "A second useful event that can be used with ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " is \[OpenCurlyDoubleQuote]DiscoverVertex\[CloseCurlyDoubleQuote]. This \
event occurs when a vertex is discovered for the first time. The associated \
function is called with up to three arguments: the vertex being discovered, \
the vertex currently being visited, and the distance the visited vertex is \
from the starting vertex."
}], "Text"],

Cell[TextData[{
 "The interested reader is encouraged to consult the help pages for",
 " ",
 StyleBox[ButtonBox["DepthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DepthFirstScan"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " for information on other events that can be used."
}], "Text"],

Cell[TextData[{
 "Despite ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s existing functions, we will develop our own using depth-first and \
breadth-first search functions as a way to illustrate these important \
algorithms."
}], "Text"],

Cell[CellGroupData[{

Cell["Depth-First Search", "Subsection",
 CellTags->"Section4a"],

Cell["\<\
We begin by implementing depth-first search. As the name of the algorithm \
suggests, vertices are visited in order of increasing depth of the spanning \
tree. Our implementation is based on Algorithm 1 of Section 11.4 of the \
textbook. \
\>", "Text"],

Cell[TextData[{
 "Recall the terminology defined in the textbook. We say that we are \
\[OpenCurlyDoubleQuote]exploring a vertex\[CloseCurlyDoubleQuote] ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " from the time the vertex is first added to the spanning tree until we have \
backtracked back to ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " for the last time. Note that at any step in the process, we are generally \
exploring multiple vertices. In particular, the root of the spanning tree \
starts being explored at the very beginning of the process and continues \
being explored until it terminates."
}], "Text"],

Cell[TextData[{
 "The function, which we call ",
 StyleBox["depthSearch", "Input"],
 ", will take two arguments: an undirected graph and a vertex in that graph. \
The function operates as follows:"
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "First, we check that the graph is connected using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 " function. If not, there can be no spanning tree and the function returns \
",
 StyleBox[ButtonBox["$Failed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$Failed"], "FunctionLink"],
 "."
}], "Item1Numbered"],

Cell["Next, we initialize the following variables. ", "Item1Numbered"],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["toVisit", "Input"],
 " will be the set of vertices of the graph that have not yet been visited. \
It is initialized to the set of vertices of the graph minus the initial \
vertex."
}], "Item2Numbered"],

Cell[TextData[{
 StyleBox["exploring", "Input"],
 " will be the list of vertices that are currently being explored. As \
vertices are visited, they are added to the end of the ",
 StyleBox["exploring", "Input"],
 " list. When a vertex has been fully explored, i.e., when it has no \
neighbors not already in the tree, then we remove it from the ",
 StyleBox["exploring", "Input"],
 " list. ",
 StyleBox["exploring", "Input"],
 " is initialized to the vertex that is given as the second argument."
}], "Item2Numbered"],

Cell[TextData[{
 StyleBox["T", "Input"],
 " will be the spanning tree that is constructed. It is initialized to the \
graph consisting of all the vertices of the graph, but with no edges. \
Provided that the graph is connected, we know that all the vertices will \
appear in ",
 StyleBox["T", "Input"],
 " and this saves us from adding them one at a time. "
}], "Item2Numbered"]
}, Open  ]],

Cell[TextData[{
 "Following initialization, we begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop which terminates when the ",
 StyleBox["exploring", "Input"],
 " list is empty. The variable ",
 StyleBox["v", "Input"],
 " is set to the last element of the ",
 StyleBox["exploring", "Input"],
 " list. We then compute the intersection, ",
 StyleBox["N", "Input"],
 ", of the neighbors of ",
 StyleBox["v", "Input"],
 " and the ",
 StyleBox["toVisit", "Input"],
 " set of vertices not already contained in the tree. Either,"
}], "Item1Numbered"],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["N", "Input"],
 " is non-empty, in which case, one of its elements is chosen as ",
 StyleBox["w", "Input"],
 ", the next vertex to visit. The edge ",
 StyleBox["v\[DirectedEdge]w", "Input"],
 " is added to the tree ",
 StyleBox["T", "Input"],
 ". Also, ",
 StyleBox["w", "Input"],
 " is removed from the ",
 StyleBox["toVisit", "Input"],
 " set and added to the end of the ",
 StyleBox["exploring", "Input"],
 " list. In the next iteration of the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop, this new vertex will be set to be ",
 StyleBox["v", "Input"],
 "."
}], "Item2Numbered"],

Cell[TextData[{
 StyleBox["N", "Input"],
 " is empty, in which case the vertex ",
 StyleBox["v", "Input"],
 " has been explored completely and so it can be removed from the ",
 StyleBox["exploring", "Input"],
 " list. The next iteration of the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop will set ",
 StyleBox["v", "Input"],
 " to be the vertex one step back in the ",
 StyleBox["exploring", "Input"],
 " list. This is the \[OpenCurlyDoubleQuote]backtracking\
\[CloseCurlyDoubleQuote] step."
}], "Item2Numbered"]
}, Open  ]]
}, Open  ]],

Cell["Here, now, is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"depthSearch", "[", 
   RowBox[{
    RowBox[{"G_", "?", "UndirectedGraphQ"}], ",", "startV_", ",", "opts___"}],
    "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "toVisit", ",", "exploring", ",", "T", ",", "v", ",", "N", ",", "w"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"ConnectedGraphQ", "[", "G", "]"}]}], ",", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"toVisit", "=", 
      RowBox[{"Complement", "[", 
       RowBox[{
        RowBox[{"VertexList", "[", "G", "]"}], ",", 
        RowBox[{"{", "startV", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"exploring", "=", 
      RowBox[{"{", "startV", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"VertexList", "[", "G", "]"}], ",", 
        RowBox[{"{", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "exploring", "]"}], ">", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"v", "=", 
         RowBox[{"exploring", "[", 
          RowBox[{"[", 
           RowBox[{"-", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"N", "=", 
         RowBox[{"Intersection", "[", 
          RowBox[{
           RowBox[{"AdjacencyList", "[", 
            RowBox[{"G", ",", "v"}], "]"}], ",", "toVisit"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "N", "]"}], ">", "0"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"w", "=", 
            RowBox[{"N", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"T", "=", 
            RowBox[{"EdgeAdd", "[", 
             RowBox[{"T", ",", 
              RowBox[{"v", "\[DirectedEdge]", "w"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"toVisit", "=", 
            RowBox[{"Complement", "[", 
             RowBox[{"toVisit", ",", 
              RowBox[{"{", "w", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
           
           RowBox[{"AppendTo", "[", 
            RowBox[{"exploring", ",", "w"}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
          RowBox[{"exploring", "=", 
           RowBox[{"Delete", "[", 
            RowBox[{"exploring", ",", 
             RowBox[{"-", "1"}]}], "]"}]}]}], "\[IndentingNewLine]", 
         "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["Let's test this with our Exercise 13 example from above.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"depthExercise13", "=", 
  RowBox[{"depthSearch", "[", 
   RowBox[{"exercise13", ",", "\"\<a\>\"", ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {{{1, 2}, {2, 3}, {3,
        4}, {4, 5}, {5, 6}, {6, 7}, {7, 8}, {8, 9}, {7, 10}}, Null}, {
      ImagePadding -> 10, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.02650765302197859], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.03493468166720967], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.9203579866168443}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.8407159732336889`}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 2.7610739598505334`}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.6902684899626333, 3.6814319464673777`}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3805369799252667`, 2.7610739598505334`}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.3805369799252667`, 1.8407159732336889`}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.9203579866168444, 0.9203579866168443}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.9203579866168444, 0.}, 0.03493468166720967], 
          "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.8407159732336889`, 0.9203579866168443}, 
           0.03493468166720967], "DynamicName", BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{13.28431822849494, 
       225.5373569013778}, {-216.4928182057671, 200.8870545006448}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can reposition the vertices to match the original with ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PropertyValue", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"depthExercise13", ",", "v"}], "}"}], ",", 
      "VertexCoordinates"}], "]"}], "=", 
    RowBox[{"PropertyValue", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"exercise13", ",", "v"}], "}"}], ",", "VertexCoordinates"}], 
     "]"}]}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"VertexList", "[", "exercise13", "]"}]}], "}"}]}], 
  "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["depthExercise13"], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {{{1, 2}, {2, 3}, {3,
        4}, {4, 5}, {5, 6}, {6, 7}, {7, 8}, {8, 9}, {7, 10}}, Null}, {
      ImagePadding -> 10, 
       VertexCoordinates -> {{0, 1}, {0, 0}, {1, 0.5}, {2, 0.5}, {3, 1}, {3, 
         0}, {4, 0}, {4, 1}, {5, 1}, {5, 0}}, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.023064257028112448`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.799999999999994`, 
       349.0979565443064}, {-50.369956544311705`, 33.22817874040582}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Breadth-First Search", "Subsection",
 CellTags->"Section4b"],

Cell["\<\
We now turn to an implementation of a breadth-first search. Recall that the \
breadth-first algorithm works by examining all vertices at the current depth \
of the spanning tree before moving on to the next level of the graph. Our \
implementation will follow Algorithm 2 of Section 11.4 of the text. \
\>", "Text"],

Cell[TextData[{
 "The function, to be called ",
 StyleBox["breadthSearch", "Input"],
 ", again takes two arguments: an undirected graph and a vertex to act as the \
starting point. It proceeds as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 " First, we check that the graph is connected using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 " function."
}], "Item1Numbered"],

Cell["Next, we initialize the following variables. ", "Item1Numbered"],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["toVisit", "Input"],
 ", as before, will be the set of vertices of the graph not yet visited. It \
is initialized to the set of vertices of the graph with the initial vertex \
excluded."
}], "Item2Numbered"],

Cell[TextData[{
 StyleBox["toProcess", "Input"],
 " will be the list of vertices that have been determined to be incident to a \
vertex in the tree but which have not yet been processed. ",
 StyleBox["toProcess", "Input"],
 " is initialized to the vertex that is given as the second argument to the \
procedure."
}], "Item2Numbered"],

Cell[TextData[{
 StyleBox["T", "Input"],
 " will be the spanning tree that is constructed. Once again, it is \
initialized to the tree consisting of all the vertices of the given graph, \
but with no edges."
}], "Item2Numbered"]
}, Open  ]],

Cell[TextData[{
 "Following initialization, we begin a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop that terminates when the ",
 StyleBox["toProcess", "Input"],
 " list is empty. The variable ",
 StyleBox["v", "Input"],
 " is set to the ",
 StyleBox["first",
  FontSlant->"Italic"],
 " element of the ",
 StyleBox["toProcess", "Input"],
 " list. We then compute the intersection, ",
 StyleBox["N", "Input"],
 ", of the neighbors of ",
 StyleBox["v", "Input"],
 " and the ",
 StyleBox["toVisit", "Input"],
 " set. For each element ",
 StyleBox["w", "Input"],
 " of ",
 StyleBox["N", "Input"],
 ", an edge ",
 StyleBox["v\[DirectedEdge]w", "Input"],
 " is added to ",
 StyleBox["T", "Input"],
 " and ",
 StyleBox["w", "Input"],
 " is added to the end of the ",
 StyleBox["toProcess", "Input"],
 " list and removed from the ",
 StyleBox["toVisit", "Input"],
 " set. Then ",
 StyleBox["v", "Input"],
 " is removed from ",
 StyleBox["toProcess", "Input"],
 "."
}], "Item1Numbered"]
}, Open  ]],

Cell[TextData[{
 "Observe that, since neighbors are added to the end of the ",
 StyleBox["toProcess", "Input"],
 " list and are processed from the beginning of the list, we are assured that \
all vertices on a given level will be processed before any vertex at a lower \
level."
}], "Text"],

Cell["Here is the implementation. ", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"breadthSearch", "[", 
   RowBox[{
    RowBox[{"G_", "?", "UndirectedGraphQ"}], ",", "startV_", ",", "opts___"}],
    "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "toVisit", ",", "toProcess", ",", "T", ",", "v", ",", "N", ",", "w"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"ConnectedGraphQ", "[", "G", "]"}]}], ",", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"toVisit", "=", 
      RowBox[{"Complement", "[", 
       RowBox[{
        RowBox[{"VertexList", "[", "G", "]"}], ",", 
        RowBox[{"{", "startV", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"toProcess", "=", 
      RowBox[{"{", "startV", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"VertexList", "[", "G", "]"}], ",", 
        RowBox[{"{", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "toProcess", "]"}], ">", "0"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"v", "=", 
         RowBox[{"toProcess", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"N", "=", 
         RowBox[{"Intersection", "[", 
          RowBox[{
           RowBox[{"AdjacencyList", "[", 
            RowBox[{"G", ",", "v"}], "]"}], ",", "toVisit"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"T", "=", 
            RowBox[{"EdgeAdd", "[", 
             RowBox[{"T", ",", 
              RowBox[{"v", "\[DirectedEdge]", "w"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"toProcess", ",", "w"}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"toVisit", "=", 
            RowBox[{"Complement", "[", 
             RowBox[{"toVisit", ",", 
              RowBox[{"{", "w", "}"}]}], "]"}]}], ";"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"w", ",", "N"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"toProcess", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"toProcess", ",", "1"}], "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "T"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["Once again, we illustrate using Exercise 13.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"breadthExercise13", "=", 
   RowBox[{"breadthSearch", "[", 
    RowBox[{"exercise13", ",", "\"\<a\>\"", ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
     RowBox[{"ImagePadding", "\[Rule]", "5"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"breadthExercise13", ",", "v"}], "}"}], ",", 
       "VertexCoordinates"}], "]"}], "=", 
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"exercise13", ",", "v"}], "}"}], ",", "VertexCoordinates"}], 
      "]"}]}], "\[IndentingNewLine]", ",", 
    RowBox[{"{", 
     RowBox[{"v", ",", 
      RowBox[{"VertexList", "[", "exercise13", "]"}]}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", "breadthExercise13"}], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {{{1, 2}, {1, 3}, {3,
        4}, {4, 5}, {4, 6}, {5, 8}, {6, 7}, {8, 9}, {7, 10}}, Null}, {
      ImagePadding -> 5, 
       VertexCoordinates -> {{0, 1}, {0, 0}, {1, 0.5}, {2, 0.5}, {3, 1}, {3, 
         0}, {4, 0}, {4, 1}, {5, 1}, {5, 0}}, VertexLabels -> {"Name"}}]], 
    Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.023064257028112448`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$5", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$7", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$8", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.5}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{3., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{4., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 1.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{5., 0.}, 0.043048128342245986`], "DynamicName", BoxID -> 
          "VertexID$10"], 
         InsetBox[
          FormBox["\"j\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{10.999999999999998`, 
       353.8741580079276}, {-51.154158007927336`, 34.47018399747658}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->5]], "Output"]
}, Open  ]],

Cell["\<\
Before moving on to backtracking, let's take a moment to compare the trees \
produced by the two algorithms.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"breadthSearch", "[", 
       RowBox[{"exercise13", ",", "\"\<a\>\""}], "]"}], ",", 
      RowBox[{"depthSearch", "[", 
       RowBox[{"exercise13", ",", "\"\<a\>\""}], "]"}]}], "}"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{
          "a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {{{1, 2}, {1, 
           3}, {3, 4}, {4, 5}, {4, 6}, {5, 8}, {6, 7}, {8, 9}, {7, 10}}, 
           Null}]], Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
           Arrowheads[0.03014992502457375], 
           Directive[
            Opacity[0.7], 
            Hue[0.6, 0.7, 0.5]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$1", Automatic, Center], 
              DynamicLocation["VertexID$2", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$1", Automatic, Center], 
              DynamicLocation["VertexID$3", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$3", Automatic, Center], 
              DynamicLocation["VertexID$4", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$4", Automatic, Center], 
              DynamicLocation["VertexID$5", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$4", Automatic, Center], 
              DynamicLocation["VertexID$6", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$5", Automatic, Center], 
              DynamicLocation["VertexID$8", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$6", Automatic, Center], 
              DynamicLocation["VertexID$7", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$7", Automatic, Center], 
              DynamicLocation["VertexID$10", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$8", Automatic, Center], 
              DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
           Directive[
            Hue[0.6, 0.2, 0.8], 
            EdgeForm[
             Directive[
              GrayLevel[0], 
              Opacity[0.7]]]], 
           TagBox[
            DiskBox[{0., 0.9045340337332908}, 0.02814984257149772], 
            "DynamicName", BoxID -> "VertexID$1"], 
           TagBox[
            DiskBox[{0., 0.}, 0.02814984257149772], "DynamicName", BoxID -> 
            "VertexID$2"], 
           TagBox[
            DiskBox[{0., 1.8090680674665816`}, 0.02814984257149772], 
            "DynamicName", BoxID -> "VertexID$3"], 
           TagBox[
            
            DiskBox[{0.9045340337332908, 2.7136021011998723`}, 
             0.02814984257149772], "DynamicName", BoxID -> "VertexID$4"], 
           TagBox[
            
            DiskBox[{0.9045340337332908, 1.8090680674665816`}, 
             0.02814984257149772], "DynamicName", BoxID -> "VertexID$5"], 
           TagBox[
            
            DiskBox[{1.8090680674665816`, 1.8090680674665816`}, 
             0.02814984257149772], "DynamicName", BoxID -> "VertexID$6"], 
           TagBox[
            
            DiskBox[{1.8090680674665816`, 0.9045340337332908}, 
             0.02814984257149772], "DynamicName", BoxID -> "VertexID$7"], 
           TagBox[
            
            DiskBox[{0.9045340337332908, 0.9045340337332908}, 
             0.02814984257149772], "DynamicName", BoxID -> "VertexID$8"], 
           TagBox[
            DiskBox[{0.9045340337332908, 0.}, 0.02814984257149772], 
            "DynamicName", BoxID -> "VertexID$9"], 
           TagBox[
            DiskBox[{1.8090680674665816`, 0.}, 0.02814984257149772], 
            "DynamicName", BoxID -> "VertexID$10"]}}], $CellContext`flag}, 
        TagBox[
         DynamicBox[GraphComputation`NetworkGraphicsBox[
          3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
          CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
           SynchronousUpdating -> False, 
           TrackedSymbols :> {$CellContext`flag}},
          
          ImageSizeCache->{{1.937333310950855, 
           144.9293322365919}, {-108.18016002148713`, 103.18016002148718`}}],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False,
        UnsavedVariables:>{$CellContext`flag}]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{
          "a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {{{1, 2}, {2, 
           3}, {3, 4}, {4, 5}, {5, 6}, {6, 7}, {7, 8}, {8, 9}, {7, 10}}, 
           Null}]], Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
           Arrowheads[0.02650765302197859], 
           Directive[
            Opacity[0.7], 
            Hue[0.6, 0.7, 0.5]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$1", Automatic, Center], 
              DynamicLocation["VertexID$2", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$2", Automatic, Center], 
              DynamicLocation["VertexID$3", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$3", Automatic, Center], 
              DynamicLocation["VertexID$4", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$4", Automatic, Center], 
              DynamicLocation["VertexID$5", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$5", Automatic, Center], 
              DynamicLocation["VertexID$6", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$6", Automatic, Center], 
              DynamicLocation["VertexID$7", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$7", Automatic, Center], 
              DynamicLocation["VertexID$8", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$7", Automatic, Center], 
              DynamicLocation["VertexID$10", Automatic, Center]}]], 
           ArrowBox[
            LineBox[{
              DynamicLocation["VertexID$8", Automatic, Center], 
              DynamicLocation["VertexID$9", Automatic, Center]}]]}, {
           Directive[
            Hue[0.6, 0.2, 0.8], 
            EdgeForm[
             Directive[
              GrayLevel[0], 
              Opacity[0.7]]]], 
           TagBox[
            DiskBox[{0., 0.}, 0.03493468166720967], "DynamicName", BoxID -> 
            "VertexID$1"], 
           TagBox[
            DiskBox[{0., 0.9203579866168443}, 0.03493468166720967], 
            "DynamicName", BoxID -> "VertexID$2"], 
           TagBox[
            DiskBox[{0., 1.8407159732336889`}, 0.03493468166720967], 
            "DynamicName", BoxID -> "VertexID$3"], 
           TagBox[
            DiskBox[{0., 2.7610739598505334`}, 0.03493468166720967], 
            "DynamicName", BoxID -> "VertexID$4"], 
           TagBox[
            
            DiskBox[{0.6902684899626333, 3.6814319464673777`}, 
             0.03493468166720967], "DynamicName", BoxID -> "VertexID$5"], 
           TagBox[
            
            DiskBox[{1.3805369799252667`, 2.7610739598505334`}, 
             0.03493468166720967], "DynamicName", BoxID -> "VertexID$6"], 
           TagBox[
            
            DiskBox[{1.3805369799252667`, 1.8407159732336889`}, 
             0.03493468166720967], "DynamicName", BoxID -> "VertexID$7"], 
           TagBox[
            
            DiskBox[{0.9203579866168444, 0.9203579866168443}, 
             0.03493468166720967], "DynamicName", BoxID -> "VertexID$8"], 
           TagBox[
            DiskBox[{0.9203579866168444, 0.}, 0.03493468166720967], 
            "DynamicName", BoxID -> "VertexID$9"], 
           TagBox[
            
            DiskBox[{1.8407159732336889`, 0.9203579866168443}, 
             0.03493468166720967], "DynamicName", BoxID -> 
            "VertexID$10"]}}], $CellContext`flag}, 
        TagBox[
         DynamicBox[GraphComputation`NetworkGraphicsBox[
          3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
          CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
           SynchronousUpdating -> False, 
           TrackedSymbols :> {$CellContext`flag}},
          
          ImageSizeCache->{{1.2357485901126513`, 
           110.55168091551981`}, {-108.85368135349185`, 103.85368135349185`}}],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False,
        UnsavedVariables:>{$CellContext`flag}]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FrameTicks->None]}
   },
   AutoDelete->False,
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Notice that the two spanning trees are quite different, even though ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " roots them to be as balanced as possible. In particular, the depth-first \
search has a deep and thin structure, whereas the breadth-first search is \
shorter and wider appearing."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Graph Coloring via Backtracking", "Subsection",
 CellTags->"Section4c"],

Cell[TextData[{
 "Backtracking is a method that can be used to find solutions to problems \
that might be impractical to solve using exhaustive search techniques. \
Backtracking is based on the systematic search for a solution to a problem \
using a decision tree. (See the text for a complete discussion.) Here we show \
how to use backtracking to solve several different problems, including \
coloring a graph, the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-queens problem, and the subset sum problem. "
}], "Text"],

Cell[TextData[{
 "The first problem we will attack via a backtracking procedure is the \
problem of coloring a graph using ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " colors, where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a positive integer. Given a graph, we will attempt to color it using ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " colors using the method described in Example 6 of Section 11.4."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Fix an order on the vertices of the graph, say ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["v", "1"], ",", 
    SubscriptBox["v", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["v", "m"]}], TraditionalForm]]],
 " and fix an ordering of the colors as color 1, color 2, ..., color ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". We will use the ordering of the vertices that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " automatically imposes. For the colors, we will require an ordered list of \
colors as one of the arguments to the function."
}], "Item1Numbered"],

Cell[TextData[{
 "We store the current state of the coloring in a list we will call ",
 StyleBox["coloring", "Input"],
 ". The ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th entry in this list will correspond to the color of the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th vertex. For example, ",
 StyleBox["coloring = {1,2,1}", "Input"],
 ", corresponds to vertex ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "1"], TraditionalForm]]],
 " assigned color 1, vertex ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "2"], TraditionalForm]]],
 " assigned color 2, and vertex ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "3"], TraditionalForm]]],
 " assigned color 1. This ",
 StyleBox["coloring", "Input"],
 " list is similar to the ",
 StyleBox["exploring", "Input"],
 " list from ",
 StyleBox["depthSearch", "Input"],
 ". In both cases, you can think of the list as storing the path from the \
root of the tree to the current vertex. In this case, the level, which \
corresponds to the position in the list, carries additional information. \
Specifically, level ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " in the decision tree (i.e., position ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " in the ",
 StyleBox["coloring", "Input"],
 " list) corresponds to deciding the color of vertex ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["v", "k"], TraditionalForm]]],
 "."
}], "Item1Numbered"],

Cell[TextData[{
 "We initialize ",
 StyleBox["coloring", "Input"],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "1", "}"}], TraditionalForm]]],
 " and set a counter variable ",
 StyleBox["i", "Input"],
 " to 2. The variable ",
 StyleBox["i", "Input"],
 " will indicate the vertex that requires a decision."
}], "Item1Numbered"],

Cell[TextData[{
 "Set ",
 StyleBox["N", "Input"],
 " equal to the neighbors of the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th vertex of the graph, and then construct a set, ",
 StyleBox["used", "Input"],
 ", consisting of the indices of the colors assigned to the neighbors. The ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th vertex will be assigned the color with the smallest index not in ",
 StyleBox["used", "Input"],
 ", assuming there are any remaining colors."
}], "Item1Numbered"],

Cell[TextData[{
 "If there are no possible colors for the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th vertex, then we must backtrack. We decrease ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " by one. To ensure that we do not repeat a choice already made when we \
revisit a vertex, we make the following modification to how colors are \
chosen. If the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th position of ",
 StyleBox["coloring", "Input"],
 " has already been set, then we know we're in the process of backtracking. \
We insist that the new choice for the color of vertex ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " is the smallest possible color greater than the current color."
}], "Item1Numbered"],

Cell[TextData[{
 "The function terminates in one of two cases. If ",
 StyleBox["i", "Input"],
 " is set to a value greater then the number of vertices, then we know that \
",
 StyleBox["coloring", "Input"],
 " contains a valid assignment for all vertices. On the other hand, if ",
 StyleBox["i", "Input"],
 " is ever set to 1, then we know that we have backtracked all the way to the \
root. Since the color of the first vertex doesn't affect the validity of the \
coloring, this indicates that we have exhausted all possible colorings and \
that the graph cannot be colored with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " colors."
}], "Item1Numbered"]
}, Open  ]],

Cell[TextData[{
 "Our function will be called ",
 StyleBox["backColor", "Input"],
 ". It will accept two arguments: the graph to be colored and a list of \
colors. If it is successful, it will display the graph with the vertices \
colored. If it determines that there is no ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-coloring of the graph, it will return ",
 StyleBox[ButtonBox["$Failed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$Failed"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"backColor", "[", 
   RowBox[{"G_Graph", ",", 
    RowBox[{"C", ":", 
     RowBox[{"{", "__RGBColor", "}"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "verts", ",", "numverts", ",", "allcolorsL", ",", "k", ",", "coloring", 
      ",", "i", ",", "N", ",", "j", ",", "used", ",", "available", ",", "g"}],
      "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"verts", "=", 
      RowBox[{"VertexList", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"numverts", "=", 
      RowBox[{"Length", "[", "verts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"allcolorsL", "=", 
      RowBox[{"Range", "[", 
       RowBox[{"Length", "[", "C", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"coloring", "=", 
      RowBox[{"{", "1", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"i", ">", "1"}], "&&", 
        RowBox[{"i", "\[LessEqual]", "numverts"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"N", "=", 
         RowBox[{"VertexList", "[", 
          RowBox[{"NeighborhoodGraph", "[", 
           RowBox[{"G", ",", 
            RowBox[{"verts", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"used", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"j", "=", "1"}], ",", 
          RowBox[{"j", "\[LessEqual]", 
           RowBox[{"i", "-", "1"}]}], ",", 
          RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"MemberQ", "[", 
              RowBox[{"N", ",", 
               RowBox[{"verts", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], "]"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"used", ",", 
               RowBox[{"coloring", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], "]"}]}], "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "coloring", "]"}], "\[GreaterEqual]", "i"}],
           ",", 
          RowBox[{"used", "=", 
           RowBox[{"Union", "[", 
            RowBox[{"used", ",", 
             RowBox[{"Range", "[", 
              RowBox[{"coloring", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"available", "=", 
         RowBox[{"Complement", "[", 
          RowBox[{"allcolorsL", ",", "used"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "available", "]"}], ">", "0"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"coloring", "=", 
            RowBox[{"Append", "[", 
             RowBox[{
              RowBox[{"coloring", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"i", "-", "1"}]}], "]"}], "]"}], ",", 
              RowBox[{"First", "[", "available", "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "coloring", "]"}], "\[GreaterEqual]", 
              "i"}], ",", 
             RowBox[{"coloring", "=", 
              RowBox[{"coloring", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"i", "-", "1"}]}], "]"}], "]"}]}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "--"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"i", ">", "numverts"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"g", "=", "G"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"k", "=", "1"}], ",", 
          RowBox[{"k", "\[LessEqual]", "numverts"}], ",", 
          RowBox[{"k", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"g", ",", 
               RowBox[{"verts", "[", 
                RowBox[{"[", "k", "]"}], "]"}]}], "}"}], ",", "VertexStyle"}],
             "]"}], "=", 
           RowBox[{"C", "[", 
            RowBox[{"[", 
             RowBox[{"coloring", "[", 
              RowBox[{"[", "k", "]"}], "]"}], "]"}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", "g", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We test our function on the example given in Figure 11 of Section 11.4 of the \
text.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure11Graph", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e"}, {
      Null, {{1, 2}, {1, 5}, {2, 3}, {2, 4}, {2, 5}, {3, 4}, {4, 5}}}, {
      ImagePadding -> 10, 
       VertexCoordinates -> {{0, 0}, {1, 0}, {2, 0}, {2, 1}, {0, 1}}, 
       VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.799999999999997`, 
       350.85017364352365`}, {-98.12217364352716, 82.33659545312992}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"backColor", "[", 
  RowBox[{"figure11Graph", ",", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Blue", ",", "Green"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e"}, {
      Null, {{1, 2}, {1, 5}, {2, 3}, {2, 4}, {2, 5}, {3, 4}, {4, 5}}}, {
      ImagePadding -> 10, 
       VertexCoordinates -> {{0, 0}, {1, 0}, {2, 0}, {2, 1}, {0, 1}}, 
       VertexLabels -> {"Name"}, 
       VertexStyle -> {
        "a" -> RGBColor[1, 0, 0], "b" -> RGBColor[0, 0, 1], "e" -> 
         RGBColor[0, 1, 0], "c" -> RGBColor[0, 1, 0], "d" -> 
         RGBColor[1, 0, 0]}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          StyleBox[
           DiskBox[{0., 0.}, 0.02261146496815286], 
           RGBColor[1, 0, 0], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          StyleBox[
           DiskBox[{1., 0.}, 0.02261146496815286], 
           RGBColor[0, 0, 1], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          StyleBox[
           DiskBox[{2., 0.}, 0.02261146496815286], 
           RGBColor[0, 1, 0], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          StyleBox[
           DiskBox[{2., 1.}, 0.02261146496815286], 
           RGBColor[1, 0, 0], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          StyleBox[
           DiskBox[{0., 1.}, 0.02261146496815286], 
           RGBColor[0, 1, 0], StripOnInput -> False], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.799999999999997`, 
       350.85017364352365`}, {-98.12217364352264, 82.33659545312989}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell["\<\
On the other hand, the complete graph on 5 vertices cannot be 3-colored or \
4-colored.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"K5", "=", 
   RowBox[{"CompleteGraph", "[", "5", "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"backColor", "[", 
  RowBox[{"K5", ",", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Blue", ",", "Green"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["$Failed"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"backColor", "[", 
  RowBox[{"K5", ",", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Blue", ",", "Green", ",", "Yellow"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData["$Failed"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"backColor", "[", 
  RowBox[{"K5", ",", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Blue", ",", "Green", ",", "Yellow", ",", "Brown"}], 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 4, 8, 12, 16, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {5}, {
          1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {
      GraphLayout -> "CircularEmbedding", 
       VertexStyle -> {
        2 -> RGBColor[0, 0, 1], 1 -> RGBColor[1, 0, 0], 5 -> 
         RGBColor[0.6, 0.4, 0.2], 4 -> RGBColor[1, 1, 0], 3 -> 
         RGBColor[0, 1, 0]}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       LineBox[{{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}, {
          DynamicLocation["VertexID$3", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}, {
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}}]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[
        StyleBox[
         DiskBox[{-0.9510565162951538, 0.3090169943749484}, 
          0.021812234931106983`], 
         RGBColor[1, 0, 0], StripOnInput -> False], "DynamicName", BoxID -> 
        "VertexID$1"], 
       TagBox[
        StyleBox[
         DiskBox[{-0.5877852522924726, -0.8090169943749468}, 
          0.021812234931106983`], 
         RGBColor[0, 0, 1], StripOnInput -> False], "DynamicName", BoxID -> 
        "VertexID$2"], 
       TagBox[
        StyleBox[
         DiskBox[{0.5877852522924738, -0.8090169943749481}, 
          0.021812234931106983`], 
         RGBColor[0, 1, 0], StripOnInput -> False], "DynamicName", BoxID -> 
        "VertexID$3"], 
       TagBox[
        StyleBox[
         DiskBox[{0.9510565162951533, 0.30901699437494634`}, 
          0.021812234931106983`], 
         RGBColor[1, 1, 0], StripOnInput -> False], "DynamicName", BoxID -> 
        "VertexID$4"], 
       TagBox[
        StyleBox[
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.021812234931106983`], 
         RGBColor[0.6, 0.4, 0.2], StripOnInput -> False], "DynamicName", 
        BoxID -> "VertexID$5"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{6.18000000000001, 352.82}, {-168.10719999999998`, 
       162.04310466848688`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Before moving on to the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-queens problem, we illustrate how we can modify our backtracking function \
to record and display the decision tree. Instead of displaying the graph, our \
modified algorithm will produce the decision tree. We do this by building a \
list of the decisions. Each time a color is assigned, we create a new vertex \
by adding the current state of the coloring list (converted to a string) and \
connecting it to the previous state. Note that we use ",
 StyleBox[ButtonBox["TreePlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TreePlot"], "FunctionLink"],
 " here, rather than a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object, so as to be able to draw the root at the top of the image."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"backColorDT", "[", 
   RowBox[{"G_Graph", ",", 
    RowBox[{"C", ":", 
     RowBox[{"{", "__RGBColor", "}"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "verts", ",", "numverts", ",", "allcolorsL", ",", "k", ",", "coloring", 
      ",", "i", ",", "N", ",", "j", ",", "used", ",", "available", ",", 
      "DTList", ",", "parentV", ",", "newV"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"verts", "=", 
      RowBox[{"VertexList", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"numverts", "=", 
      RowBox[{"Length", "[", "verts", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"allcolorsL", "=", 
      RowBox[{"Range", "[", 
       RowBox[{"Length", "[", "C", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"coloring", "=", 
      RowBox[{"{", "1", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newV", "=", 
      RowBox[{"ToString", "[", "coloring", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"DTList", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"i", ">", "1"}], "&&", 
        RowBox[{"i", "\[LessEqual]", "numverts"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"N", "=", 
         RowBox[{"VertexList", "[", 
          RowBox[{"NeighborhoodGraph", "[", 
           RowBox[{"G", ",", 
            RowBox[{"verts", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"used", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"j", "=", "1"}], ",", 
          RowBox[{"j", "\[LessEqual]", 
           RowBox[{"i", "-", "1"}]}], ",", 
          RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"MemberQ", "[", 
              RowBox[{"N", ",", 
               RowBox[{"verts", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], "]"}], ",", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"used", ",", 
               RowBox[{"coloring", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}], "]"}]}], "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "coloring", "]"}], "\[GreaterEqual]", "i"}],
           ",", 
          RowBox[{"used", "=", 
           RowBox[{"Union", "[", 
            RowBox[{"used", ",", 
             RowBox[{"Range", "[", 
              RowBox[{"coloring", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"available", "=", 
         RowBox[{"Complement", "[", 
          RowBox[{"allcolorsL", ",", "used"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "available", "]"}], ">", "0"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"parentV", "=", 
            RowBox[{"ToString", "[", 
             RowBox[{"coloring", "[", 
              RowBox[{"[", 
               RowBox[{"1", ";;", 
                RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"coloring", "=", 
            RowBox[{"Append", "[", 
             RowBox[{
              RowBox[{"coloring", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"i", "-", "1"}]}], "]"}], "]"}], ",", 
              RowBox[{"First", "[", "available", "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"newV", "=", 
            RowBox[{"ToString", "[", "coloring", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"DTList", ",", 
             RowBox[{"parentV", "\[Rule]", "newV"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "coloring", "]"}], "\[GreaterEqual]", 
              "i"}], ",", 
             RowBox[{"coloring", "=", 
              RowBox[{"coloring", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"i", "-", "1"}]}], "]"}], "]"}]}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "--"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"TreePlot", "[", 
      RowBox[{"DTList", ",", "Top", ",", "\"\<{1}\>\"", ",", 
       RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"backColorDT", "[", 
  RowBox[{"figure11Graph", ",", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Blue", ",", "Green"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0.4803844614152614, 3.8430756913220914`}, {
    0.4803844614152614, 2.8823067684915684`}, {0., 1.9215378456610457`}, {0., 
    0.960768922830523}, {0.9607689228305228, 1.9215378456610457`}, {
    0.9607689228305228, 0.960768922830523}, {0.9607689228305228, 0.}}, {
     {RGBColor[0.5, 0., 0.], 
      LineBox[{{1, 2}, {2, 3}, {2, 5}, {3, 4}, {5, 6}, {6, 7}}]}, {InsetBox[
       FrameBox["\<\"{1}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["\<\"{1, 2}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["\<\"{1, 2, 1}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["\<\"{1, 2, 1, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4], InsetBox[
       FrameBox["\<\"{1, 2, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 5], InsetBox[
       FrameBox["\<\"{1, 2, 3, 1}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 6], InsetBox[
       FrameBox["\<\"{1, 2, 3, 1, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 7]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{0.4803844614152614, 3.8430756913220914`}, {
     0.4803844614152614, 2.8823067684915684`}, {0., 1.9215378456610457`}, {0.,
      0.960768922830523}, {0.9607689228305228, 1.9215378456610457`}, {
     0.9607689228305228, 0.960768922830523}, {0.9607689228305228, 0.}}]& ],
  AspectRatio->2.,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output"]
}, Open  ]],

Cell["", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"backColorDT", "[", 
  RowBox[{"K5", ",", 
   RowBox[{"{", 
    RowBox[{"Red", ",", "Blue", ",", "Green", ",", "Yellow"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{1.956151991089879, 2.347382389307855}, {
    0.39123039821797584`, 1.5649215928719031`}, {0., 0.7824607964359516}, {0.,
     0.}, {0.7824607964359517, 0.7824607964359516}, {0.7824607964359517, 
    0.}, {1.956151991089879, 1.5649215928719031`}, {1.5649215928719034`, 
    0.7824607964359516}, {1.5649215928719034`, 0.}, {2.347382389307855, 
    0.7824607964359516}, {2.347382389307855, 0.}, {3.5210735839617824`, 
    1.5649215928719031`}, {3.1298431857438067`, 0.7824607964359516}, {
    3.1298431857438067`, 0.}, {3.912303982179758, 0.7824607964359516}, {
    3.912303982179758, 0.}}, {
     {RGBColor[0.5, 0., 0.], 
      LineBox[{{1, 2}, {1, 7}, {1, 12}, {2, 3}, {2, 5}, {3, 4}, {5, 6}, {7, 
       8}, {7, 10}, {8, 9}, {10, 11}, {12, 13}, {12, 15}, {13, 14}, {15, 
       16}}]}, {InsetBox[
       FrameBox["\<\"{1}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["\<\"{1, 2}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["\<\"{1, 2, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["\<\"{1, 2, 3, 4}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4], InsetBox[
       FrameBox["\<\"{1, 2, 4}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 5], InsetBox[
       FrameBox["\<\"{1, 2, 4, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 6], InsetBox[
       FrameBox["\<\"{1, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 7], InsetBox[
       FrameBox["\<\"{1, 3, 2}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 8], InsetBox[
       FrameBox["\<\"{1, 3, 2, 4}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 9], InsetBox[
       FrameBox["\<\"{1, 3, 4}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 10], InsetBox[
       FrameBox["\<\"{1, 3, 4, 2}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 11], InsetBox[
       FrameBox["\<\"{1, 4}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 12], InsetBox[
       FrameBox["\<\"{1, 4, 2}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 13], InsetBox[
       FrameBox["\<\"{1, 4, 2, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 14], InsetBox[
       FrameBox["\<\"{1, 4, 3}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 15], InsetBox[
       FrameBox["\<\"{1, 4, 3, 2}\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 16]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{1.956151991089879, 2.347382389307855}, {
     0.39123039821797584`, 1.5649215928719031`}, {0., 0.7824607964359516}, {
     0., 0.}, {0.7824607964359517, 0.7824607964359516}, {0.7824607964359517, 
     0.}, {1.956151991089879, 1.5649215928719031`}, {1.5649215928719034`, 
     0.7824607964359516}, {1.5649215928719034`, 0.}, {2.347382389307855, 
     0.7824607964359516}, {2.347382389307855, 0.}, {3.5210735839617824`, 
     1.5649215928719031`}, {3.1298431857438067`, 0.7824607964359516}, {
     3.1298431857438067`, 0.}, {3.912303982179758, 0.7824607964359516}, {
     3.912303982179758, 0.}}]& ],
  AspectRatio->0.7745966692414834,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["n-Queens Problem via Backtracking", "Subsection",
 CellTags->"Section4d"],

Cell[TextData[{
 "Another problem with an elegant backtracking solution is the problem of \
placing ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " queens on an ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Times]", "n"}], TraditionalForm]]],
 " chessboard so that no queen can attack another. This means that no two \
queens can be placed on the same horizontal, vertical, or diagonal line. We \
will solve this problem with a backtracking algorithm. The solution we \
present here is based on the solution given in Example 7 in Section 11.4. We \
place queens in a greedy fashion on the chessboard until either all the \
queens are placed or there is no available position for a queen to be placed \
without coming under attack from a queen already on the board."
}], "Text"],

Cell[TextData[{
 "Following the textbook, the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th step in the backtracking algorithm will be to place a queen in the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th column (or file, in chess terms). Like the ",
 StyleBox["coloring", "Input"],
 " list and ",
 StyleBox["exploring", "Input"],
 " list, the algorithm will build a ",
 StyleBox["queens", "Input"],
 " list. In this case, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"queens", "[", 
    RowBox[{"[", "i", "]"}], "]"}], "=", "j"}]], "myCode"],
 " will indicate that a queen is placed in the ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 "th row (rank) in the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th column (file). We will build a helper function, ",
 StyleBox["validQueens", "Input"],
 ", that, given the dimension of the board and the current ",
 StyleBox["queens", "Input"],
 " list, will determine the possible locations for a queen in the next column."
}], "Text"],

Cell[TextData[{
 "To implement ",
 StyleBox["validQueens", "Input"],
 ", we will need a representation of the status of the board; specifically, \
for each square on the board, whether it is safe or under attack. It is \
natural to represent the board as a matrix with an entry 1 indicating that \
the corresponding square is safe and 0 that it is under attack. We create a \
square matrix with all entries initialized to 1 by issuing the ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " function with two arguments: the common value 1 and the dimension of the \
matrix in a list. For example, the following initializes the matrix \
representing the board of dimension 5 on which no queens have been placed."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ConstantArray", "[", 
   RowBox[{"1", ",", 
    RowBox[{"{", 
     RowBox[{"5", ",", "5"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", "1", "1", "1"},
     {"1", "1", "1", "1", "1"},
     {"1", "1", "1", "1", "1"},
     {"1", "1", "1", "1", "1"},
     {"1", "1", "1", "1", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We now build a ",
 StyleBox["boardStatus", "Input"],
 " function that, given the current list of queen locations and the dimension \
of the board will return a matrix representing the board with that \
configuration. The matrix will contain 1 in positions not under attack, 0 in \
positions under attack, and will represent the location of queens by the \
symbol Q."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"boardStatus", "[", 
   RowBox[{"queenList_List", ",", "dim_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "board", ",", "i", ",", "dif", ",", "qRank", ",", "qFile", ",", 
      "vQueens"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"board", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"1", ",", 
        RowBox[{"{", 
         RowBox[{"dim", ",", "dim"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"qFile", "=", "1"}], ",", 
       RowBox[{"qFile", "\[LessEqual]", 
        RowBox[{"Length", "[", "queenList", "]"}]}], ",", 
       RowBox[{"qFile", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"qRank", "=", 
         RowBox[{"queenList", "[", 
          RowBox[{"[", "qFile", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"i", "=", "1"}], ",", 
          RowBox[{"i", "\[LessEqual]", "dim"}], ",", 
          RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"board", "[", 
             RowBox[{"[", 
              RowBox[{"qRank", ",", "i"}], "]"}], "]"}], "=", "0"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"board", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "qFile"}], "]"}], "]"}], "=", "0"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"dif", "=", 
            RowBox[{"i", "-", "qFile"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"qRank", "+", "dif"}], "\[LessEqual]", "dim"}], "&&", 
              RowBox[{
               RowBox[{"qRank", "+", "dif"}], "\[GreaterEqual]", "1"}]}], ",", 
             RowBox[{
              RowBox[{"board", "[", 
               RowBox[{"[", 
                RowBox[{
                 RowBox[{"qRank", "+", "dif"}], ",", "i"}], "]"}], "]"}], "=",
               "0"}]}], "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"qRank", "-", "dif"}], "\[LessEqual]", "dim"}], "&&", 
              RowBox[{
               RowBox[{"qRank", "-", "dif"}], "\[GreaterEqual]", "1"}]}], ",", 
             RowBox[{
              RowBox[{"board", "[", 
               RowBox[{"[", 
                RowBox[{
                 RowBox[{"qRank", "-", "dif"}], ",", "i"}], "]"}], "]"}], "=",
               "0"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"qFile", "=", "1"}], ",", 
       RowBox[{"qFile", "\[LessEqual]", 
        RowBox[{"Length", "[", "queenList", "]"}]}], ",", 
       RowBox[{"qFile", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"board", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"queenList", "[", 
            RowBox[{"[", "qFile", "]"}], "]"}], ",", "qFile"}], "]"}], "]"}], 
        "=", "\"\<Q\>\""}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "board"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For example, on a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"10", "\[Times]", "10"}], TraditionalForm]]],
 " board, with the first queen in the second row and the second queen in the \
seventh row, the board looks as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"boardStatus", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "7"}], "}"}], ",", "10"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "1", "1", "1", "1", "1", "0", "1", "1"},
     {"\<\"Q\"\>", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "1", "1", "1", "0", "1", "1", "1", "1"},
     {"0", "0", "0", "1", "0", "1", "1", "1", "1", "1"},
     {"0", "0", "1", "0", "1", "1", "1", "1", "1", "1"},
     {"0", "0", "0", "1", "0", "1", "1", "1", "1", "1"},
     {"0", "\<\"Q\"\>", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "1", "1", "1", "0", "1", "1", "1"},
     {"0", "0", "1", "0", "1", "1", "1", "0", "1", "1"},
     {"0", "0", "1", "1", "0", "1", "1", "1", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["validQueens", "Input"],
 " function will take the same arguments, the list of queen locations and \
dimension of the board, pass them to ",
 StyleBox["boardStatus", "Input"],
 ", and use the resulting matrix to determine available positions in the next \
column. We could omit the ",
 StyleBox["boardStatus", "Input"],
 " function and instead create ",
 StyleBox["validQueens", "Input"],
 " independently, but, as you see above, the ",
 StyleBox["boardStatus", "Input"],
 " function provides a useful visualization."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"validQueens", "[", 
   RowBox[{"queenList_List", ",", "dim_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"board", ",", "file", ",", "i", ",", "freeSet"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"board", "=", 
      RowBox[{"boardStatus", "[", 
       RowBox[{"queenList", ",", "dim"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"file", "=", 
      RowBox[{
       RowBox[{"Length", "[", "queenList", "]"}], "+", "1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"freeSet", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "dim"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"board", "[", 
           RowBox[{"[", 
            RowBox[{"i", ",", "file"}], "]"}], "]"}], "\[Equal]", "1"}], ",", 
         
         RowBox[{"AppendTo", "[", 
          RowBox[{"freeSet", ",", "i"}], "]"}]}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "freeSet"}]}],
    "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "With this preliminary work out of the way, we are ready to write the main \
program, ",
 StyleBox["nQueens", "Input"],
 ". It will work in much the same way as our previous examples."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "We keep a ",
 StyleBox["queens", "Input"],
 " list, initialized to the empty list, that records the locations of \
queens."
}], "Item1Numbered"],

Cell[TextData[{
 "We initialize a counter ",
 StyleBox["file", "Input"],
 " to 1. This indicates the column in which we need to place a queen. Notice \
that, in the ",
 StyleBox["backColor", "Input"],
 " algorithm, we initialized the counter to 2. The reason for the difference \
is that, in the coloring algorithm, the color of the first vertex was \
arbitrary and changing it from color 1 to a different color could not \
possibly affect the outcome. In this case, it may be the case that there is \
no solution with the first queen in file 1, rank 1, but there is a solution \
if the file 1 queen is in a different rank."
}], "Item1Numbered"],

Cell[TextData[{
 "Apply the ",
 StyleBox["validQueens", "Input"],
 " function with the ",
 StyleBox["queens", "Input"],
 " list and the board dimension. Store the resulting set as ",
 StyleBox["open", "Input"],
 "."
}], "Item1Numbered"],

Cell[TextData[{
 "As with the ",
 StyleBox["backColor", "Input"],
 " algorithm, we determine if the current assignment is a new assignment or a \
result of backtracking. If it is a backtracking step, we remove from ",
 StyleBox["open", "Input"],
 " the positions equal to or smaller than the previous attempt."
}], "Item1Numbered"],

Cell[TextData[{
 "We terminate when ",
 StyleBox["file", "Input"],
 " either exceeds the board dimension, in which case we have found a \
solution, or when it is backtracked to 0, in which case we have exhausted all \
possibilities."
}], "Item1Numbered"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nQueens", "[", "boardDim_Integer", "]"}], "/;", 
   RowBox[{"boardDim", ">", "0"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"queens", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"file", "=", "1"}], ",", "open", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"file", ">", "0"}], "&&", 
        RowBox[{"file", "\[LessEqual]", "boardDim"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"open", "=", 
         RowBox[{"validQueens", "[", 
          RowBox[{
           RowBox[{"queens", "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", 
              RowBox[{"file", "-", "1"}]}], "]"}], "]"}], ",", "boardDim"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "queens", "]"}], "\[GreaterEqual]", 
           "file"}], ",", "\[IndentingNewLine]", 
          RowBox[{"open", "=", 
           RowBox[{"Complement", "[", 
            RowBox[{"open", ",", 
             RowBox[{"Range", "[", 
              RowBox[{"queens", "[", 
               RowBox[{"[", "file", "]"}], "]"}], "]"}]}], "]"}]}]}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"open", "\[NotEqual]", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"queens", "=", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"queens", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"file", "-", "1"}]}], "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"open", "[", 
                RowBox[{"[", "1", "]"}], "]"}], "}"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"file", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", 
           RowBox[{"else", " ", "backtrack"}], "*)"}], "\[IndentingNewLine]", 
          
          RowBox[{
           RowBox[{"queens", "=", 
            RowBox[{"queens", "[", 
             RowBox[{"[", 
              RowBox[{"1", ";;", 
               RowBox[{"file", "-", "1"}]}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"file", "--"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"file", ">", "boardDim"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"MatrixForm", "[", 
         RowBox[{"boardStatus", "[", 
          RowBox[{"queens", ",", "boardDim"}], "]"}], "]"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can use this to find one solution to the ",
 Cell[BoxData[
  FormBox["8", TraditionalForm]]],
 "-queens problem (",
 Cell[BoxData[
  FormBox[
   RowBox[{"8", "\[Times]", "8"}], TraditionalForm]]],
 " is the size of the standard board)."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nQueens", "[", "8", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\<\"Q\"\>", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "\<\"Q\"\>", "0"},
     {"0", "0", "0", "0", "\<\"Q\"\>", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "\<\"Q\"\>"},
     {"0", "\<\"Q\"\>", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "\<\"Q\"\>", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "\<\"Q\"\>", "0", "0"},
     {"0", "0", "\<\"Q\"\>", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Subset Sum Problem via Backtracking", "Subsection",
 CellTags->"Section4e"],

Cell[TextData[{
 "Finally, we consider the subset sum problem. Given a set of integers ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " and a value ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ", we want to find a subset ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " whose sum is ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ". To use backtracking on this problem, we first impose an ordering on the \
set ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ". We successively select integers from ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " to include in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " until the sum of the elements of ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " equals or exceeds ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 ", and backtrack when the sum exceeds ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Before we get to the main algorithm, it is worth reviewing two items of \
syntax. First, given a list of values, we can compute their sum with the ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 " function by using ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") to replace the ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 " head with the ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 " head, which then results in the sum being computed."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listofvalues", "=", 
  RowBox[{"{", 
   RowBox[{"3", ",", "7", ",", "11", ",", "15", ",", 
    RowBox[{"-", "4"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "7", ",", "11", ",", "15", ",", 
   RowBox[{"-", "4"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plus", "@@", "listofvalues"}]], "Input"],

Cell[BoxData["32"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Second, given a list of values and a second list consisting of indices into \
the first list, we can obtain the sublist of values corresponding to the \
positions described by the second list by using the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator with the list of indices."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listofstuff", "=", 
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
    "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\""}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"e\"\>", ",", "\<\"f\"\>", ",", "\<\"g\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listofindices", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "3", ",", "4", ",", "7"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "4", ",", "7"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listofstuff", "[", 
  RowBox[{"[", "listofindices", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", ",", "\<\"g\"\>"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"listofstuff", "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "6"}], "}"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"c\"\>", ",", "\<\"d\"\>", ",", "\<\"f\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
As the general pattern of backtracking algorithms should be clear by this \
point, we omit a detailed description of the function.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"subsetSum", "[", 
   RowBox[{
    RowBox[{"S", ":", 
     RowBox[{"{", "__Integer", "}"}]}], ",", "M_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "bIndices", ",", "allIndices", ",", "i", ",", "availIndices", ",", "k", 
      ",", "currentSum"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"bIndices", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"allIndices", "=", 
      RowBox[{"Range", "[", 
       RowBox[{"Length", "[", "S", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"currentSum", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"i", ">", "0"}], "&&", 
        RowBox[{"currentSum", "\[NotEqual]", "M"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"availIndices", "=", 
         RowBox[{"Complement", "[", 
          RowBox[{"allIndices", ",", "bIndices"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "bIndices", "]"}], "\[GreaterEqual]", "i"}],
           ",", 
          RowBox[{"availIndices", "=", 
           RowBox[{"Complement", "[", 
            RowBox[{"availIndices", ",", 
             RowBox[{"Range", "[", 
              RowBox[{"bIndices", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}]}]}], "]"}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"currentSum", "+", 
              RowBox[{"S", "[", 
               RowBox[{"[", "k", "]"}], "]"}]}], ">", "M"}], ",", 
            RowBox[{"availIndices", "=", 
             RowBox[{"Complement", "[", 
              RowBox[{"availIndices", ",", 
               RowBox[{"{", "k", "}"}]}], "]"}]}]}], "]"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"k", ",", "availIndices"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"availIndices", "\[NotEqual]", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"bIndices", "=", 
            RowBox[{"Append", "[", 
             RowBox[{
              RowBox[{"bIndices", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"i", "-", "1"}]}], "]"}], "]"}], ",", 
              RowBox[{"availIndices", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "++"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "bIndices", "]"}], "\[GreaterEqual]", 
              "i"}], ",", 
             RowBox[{"bIndices", "=", 
              RowBox[{"bIndices", "[", 
               RowBox[{"[", 
                RowBox[{"1", ";;", 
                 RowBox[{"i", "-", "1"}]}], "]"}], "]"}]}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "--"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"currentSum", "=", 
         RowBox[{"Plus", "@@", 
          RowBox[{"S", "[", 
           RowBox[{"[", 
            RowBox[{"bIndices", "[", 
             RowBox[{"[", 
              RowBox[{"1", ";;", 
               RowBox[{"i", "-", "1"}]}], "]"}], "]"}], "]"}], "]"}]}]}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"i", "\[Equal]", "0"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "$Failed", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", 
        RowBox[{"S", "[", 
         RowBox[{"[", "bIndices", "]"}], "]"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetSum", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"31", ",", "27", ",", "15", ",", "11", ",", "7", ",", "5"}], 
    "}"}], ",", "39"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"27", ",", "7", ",", "5"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetSum", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"31", ",", "27", ",", "15", ",", "11", ",", "7", ",", "5"}], 
    "}"}], ",", "40"}], "]"}]], "Input"],

Cell[BoxData["$Failed"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["11.5 Minimum Spanning Trees", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "This section explains how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the minimum spanning tree of a weighted graph. Recall that a \
minimum spanning tree ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " of a weighted graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " is a spanning tree of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " with the minimum weight of all spanning trees of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". The two best known algorithms for constructing minimum spanning trees are \
called Prim's algorithm and Kruskal's algorithm. In this section, we will \
implement these algorithms, as this is another case in which understanding \
the implementation can help you better understand the algorithms. "
}], "Text"],

Cell[TextData[{
 "Before implementing the algorithms and finding minimum spanning trees, \
though, we will describe two built-in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " commands, ",
 StyleBox[ButtonBox["GraphDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistance"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["GraphDistanceMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistanceMatrix"], "FunctionLink"],
 ", that calculate the minimum distance between two vertices in a graph. "
}], "Text"],

Cell[TextData[{
 "First, we construct a graph to use as an example. We will recreate Exercise \
3 from Section 11.5. To define undirected, weighted edges, we will give ",
 StyleBox[ButtonBox["EdgeWeight",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeWeight"], "FunctionLink"],
 " as an option to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ", associating it with the list of the edge weights in the same order as the \
edges are given. Recall that to get the edge weights to appear as labels on \
the graph, we need to explicitly set the edge labels, which we do by \
constructing a loop setting the label of each each to its weight."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"exercise3", "=", 
   RowBox[{"Graph", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
       "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\"", ",", "\"\<h\>\"", 
       ",", "\"\<i\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
       RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
       RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
       RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
       RowBox[{"\"\<f\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
       RowBox[{"\"\<f\>\"", "\[Rule]", "\"\<i\>\""}], ",", 
       RowBox[{"\"\<g\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
       RowBox[{"\"\<h\>\"", "\[Rule]", "\"\<i\>\""}]}], "}"}], ",", 
     RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
     RowBox[{"EdgeWeight", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "5", ",", "2", ",", "4", ",", "3", ",", "5", ",", "6", ",", "3", ",", 
        "7", ",", "6", ",", "8", ",", "1", ",", "3", ",", "4", ",", "4", ",", 
        "4", ",", "2"}], "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"VertexCoordinates", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"2", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"2", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"2", ",", "0"}], "}"}]}], "}"}]}], ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
     RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Do", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"exercise3", ",", "e"}], "}"}], ",", "EdgeLabels"}], "]"}], 
     "=", 
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"exercise3", ",", "e"}], "}"}], ",", "EdgeWeight"}], "]"}]}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"e", ",", 
      RowBox[{"EdgeList", "[", "exercise3", "]"}]}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", "exercise3"}], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
      Null, {{1, 2}, {1, 4}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {3, 6}, {4, 5}, {
       4, 7}, {4, 8}, {5, 6}, {5, 8}, {6, 8}, {6, 9}, {7, 8}, {8, 9}}}, {
      EdgeLabels -> {
        UndirectedEdge["g", "h"] -> 4, UndirectedEdge["b", "d"] -> 3, 
         UndirectedEdge["d", "g"] -> 6, UndirectedEdge["e", "h"] -> 3, 
         UndirectedEdge["a", "b"] -> 5, UndirectedEdge["c", "f"] -> 3, 
         UndirectedEdge["b", "c"] -> 4, UndirectedEdge["f", "i"] -> 4, 
         UndirectedEdge["f", "h"] -> 4, UndirectedEdge["h", "i"] -> 2, 
         UndirectedEdge["a", "d"] -> 2, UndirectedEdge["e", "f"] -> 1, 
         UndirectedEdge["d", "e"] -> 7, UndirectedEdge["b", "f"] -> 6, 
         UndirectedEdge["d", "h"] -> 8, UndirectedEdge["b", "e"] -> 5}, 
       EdgeWeight -> {5, 2, 4, 3, 5, 6, 3, 7, 6, 8, 1, 3, 4, 4, 4, 2}, 
       ImagePadding -> 10, 
       VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {2, 1}, {
        0, 0}, {1, 0}, {2, 0}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$1", Automatic, Center], 
           DynamicLocation["VertexID$2", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["5", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$1", Automatic, Center], 
           DynamicLocation["VertexID$4", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["2", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$3", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$3"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$3", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$4", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$4"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$4", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$5", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$5"], 
        InsetBox[
         FormBox["5", TraditionalForm], 
         DynamicLocation["EdgeLabelID$5", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$6", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$6"], 
        InsetBox[
         FormBox["6", TraditionalForm], 
         DynamicLocation["EdgeLabelID$6", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$3", Automatic, Center], 
           DynamicLocation["VertexID$6", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$7"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$7", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$4", Automatic, Center], 
           DynamicLocation["VertexID$5", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$8"], 
        InsetBox[
         FormBox["7", TraditionalForm], 
         DynamicLocation["EdgeLabelID$8", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$4", Automatic, Center], 
           DynamicLocation["VertexID$7", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$9"], 
        InsetBox[
         FormBox["6", TraditionalForm], 
         DynamicLocation["EdgeLabelID$9", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$4", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$10"], 
        InsetBox[
         FormBox["8", TraditionalForm], 
         DynamicLocation["EdgeLabelID$10", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$5", Automatic, Center], 
           DynamicLocation["VertexID$6", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$11"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$11", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$5", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$12"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$12", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$6", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$13"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$13", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$6", Automatic, Center], 
           DynamicLocation["VertexID$9", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$14"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$14", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$7", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$15"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$15", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$8", Automatic, Center], 
           DynamicLocation["VertexID$9", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$16"], 
        InsetBox[
         FormBox["2", TraditionalForm], 
         DynamicLocation["EdgeLabelID$16", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.385786437626887`, 
       350.26438720589675`}, {-178.53638720589697`, 163.73361810924303`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["GraphDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistance"], "FunctionLink"],
 " function accepts two or three arguments. Given a graph and two vertices in \
the graph, it returns the distance between the two vertices. For example, in \
the graph above, we compute the distance from ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphDistance", "[", 
  RowBox[{"exercise3", ",", "\"\<c\>\"", ",", "\"\<e\>\""}], "]"}]], "Input"],

Cell[BoxData["4.`"], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also apply ",
 StyleBox[ButtonBox["GraphDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistance"], "FunctionLink"],
 " to a graph and just one of its vertices. In this case, the output will be \
a list of the shortest distances from the given vertex to each of the \
vertices in the graph. For example, the following shows the distances from \
the central vertex ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " to each of the vertices in the graph. The 0 in the output corresponds to \
the distance from the vertex to itself."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphDistance", "[", 
  RowBox[{"exercise3", ",", "\"\<e\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "9.`", ",", "5.`", ",", "4.`", ",", "7.`", ",", "0", ",", "1.`", ",", "7.`",
    ",", "3.`", ",", "5.`"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["GraphDistanceMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistanceMatrix"], "FunctionLink"],
 " function applied to a graph produces the matrix whose entries are the \
distances between corresponding vertices. The rows and columns are ordered \
according to the order of vertices in the graph, agreeing with the output of ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GraphDistanceMatrix", "[", "exercise3", "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.`", "5.`", "9.`", "2.`", "9.`", "10.`", "8.`", "10.`", "12.`"},
     {"5.`", "0.`", "4.`", "3.`", "5.`", "6.`", "9.`", "8.`", "10.`"},
     {"9.`", "4.`", "0.`", "7.`", "4.`", "3.`", "11.`", "7.`", "7.`"},
     {"2.`", "3.`", "7.`", "0.`", "7.`", "8.`", "6.`", "8.`", "10.`"},
     {"9.`", "5.`", "4.`", "7.`", "0.`", "1.`", "7.`", "3.`", "5.`"},
     {"10.`", "6.`", "3.`", "8.`", "1.`", "0.`", "8.`", "4.`", "4.`"},
     {"8.`", "9.`", "11.`", "6.`", "7.`", "8.`", "0.`", "4.`", "6.`"},
     {"10.`", "8.`", "7.`", "8.`", "3.`", "4.`", "4.`", "0.`", "2.`"},
     {"12.`", "10.`", "7.`", "10.`", "5.`", "4.`", "6.`", "2.`", "0.`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " and the ",
 StyleBox[ButtonBox["TableHeadings",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableHeadings"], "FunctionLink"],
 " option, we can easily create a table of the distances with row and column \
headings making the vertices explicit. ",
 StyleBox[ButtonBox["TableHeadings",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableHeadings"], "FunctionLink"],
 " is associated with a list of two lists in order to label the rows and \
columns."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{
   RowBox[{"GraphDistanceMatrix", "[", "exercise3", "]"}], ",", 
   RowBox[{"TableHeadings", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"VertexList", "[", "exercise3", "]"}], ",", 
      RowBox[{"VertexList", "[", "exercise3", "]"}]}], "}"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      StyleBox["\[Null]",
       ShowStringCharacters->False], 
      TagBox["\<\"a\"\>",
       HoldForm], 
      TagBox["\<\"b\"\>",
       HoldForm], 
      TagBox["\<\"c\"\>",
       HoldForm], 
      TagBox["\<\"d\"\>",
       HoldForm], 
      TagBox["\<\"e\"\>",
       HoldForm], 
      TagBox["\<\"f\"\>",
       HoldForm], 
      TagBox["\<\"g\"\>",
       HoldForm], 
      TagBox["\<\"h\"\>",
       HoldForm], 
      TagBox["\<\"i\"\>",
       HoldForm]},
     {
      TagBox["\<\"a\"\>",
       HoldForm], "0.`", "5.`", "9.`", "2.`", "9.`", "10.`", "8.`", "10.`", 
      "12.`"},
     {
      TagBox["\<\"b\"\>",
       HoldForm], "5.`", "0.`", "4.`", "3.`", "5.`", "6.`", "9.`", "8.`", 
      "10.`"},
     {
      TagBox["\<\"c\"\>",
       HoldForm], "9.`", "4.`", "0.`", "7.`", "4.`", "3.`", "11.`", "7.`", 
      "7.`"},
     {
      TagBox["\<\"d\"\>",
       HoldForm], "2.`", "3.`", "7.`", "0.`", "7.`", "8.`", "6.`", "8.`", 
      "10.`"},
     {
      TagBox["\<\"e\"\>",
       HoldForm], "9.`", "5.`", "4.`", "7.`", "0.`", "1.`", "7.`", "3.`", 
      "5.`"},
     {
      TagBox["\<\"f\"\>",
       HoldForm], "10.`", "6.`", "3.`", "8.`", "1.`", "0.`", "8.`", "4.`", 
      "4.`"},
     {
      TagBox["\<\"g\"\>",
       HoldForm], "8.`", "9.`", "11.`", "6.`", "7.`", "8.`", "0.`", "4.`", 
      "6.`"},
     {
      TagBox["\<\"h\"\>",
       HoldForm], "10.`", "8.`", "7.`", "8.`", "3.`", "4.`", "4.`", "0.`", 
      "2.`"},
     {
      TagBox["\<\"i\"\>",
       HoldForm], "12.`", "10.`", "7.`", "10.`", "5.`", "4.`", "6.`", "2.`", 
      "0.`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxDividers->{
     "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
      "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[2.0999999999999996`]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   {OutputFormsDump`HeadedRows, OutputFormsDump`HeadedColumns}],
  Function[BoxForm`e$, 
   TableForm[
   BoxForm`e$, 
    TableHeadings -> {{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
      "a", "b", "c", "d", "e", "f", "g", "h", "i"}}]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prim\[CloseCurlyQuote]s Algorithm", "Subsection",
 CellTags->"Section5a"],

Cell["\<\
We will now build functions to implement Prim\[CloseCurlyQuote]s algorithm. \
We will also see how to create animations that illustrate the process of \
building the spanning tree.\
\>", "Text"],

Cell[TextData[{
 "Since both Prim\[CloseCurlyQuote]s algorithm and Kruskal\[CloseCurlyQuote]s \
depend on choosing an edge of smallest weight, it will be useful to be able \
to apply ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " to a list of edges. Recall that ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " accepts an optional second argument: a function that takes 2 arguments and \
returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the first argument is less than the second. Since this function needs \
to also depend on the graph, as well as the two edges, we will create a \
function that takes a graph as the argument and produces a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") that requires only the two edges as arguments."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"edgeCompare", "[", "G_", "]"}], ":=", 
  RowBox[{"Function", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"e1", ",", "e2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"G", ",", "e1"}], "}"}], ",", "EdgeWeight"}], "]"}], "<", 
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"G", ",", "e2"}], "}"}], ",", "EdgeWeight"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The list we provided to ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " as its first argument specifies that the symbols ",
 StyleBox["e1", "Input"],
 " and ",
 StyleBox["e2", "Input"],
 " will be interpreted as parameters, in place of the generic ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "FunctionLink"],
 " (",
 StyleBox["#", "Input"],
 ")."
}], "Text"],

Cell[TextData[{
 "Applying ",
 StyleBox["edgeCompare", "Input"],
 " to a graph will output a function. Assigning the result to a symbol means \
that symbol can then be used as a function. The expression below will cause \
",
 StyleBox["testComp", "Input"],
 " to act as a function on two argument, edges in ",
 StyleBox["exercise3", "Input"],
 ", and return ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the weight of the first is less than or equal to the weight of the \
second. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"testComp", "=", 
  RowBox[{"edgeCompare", "[", "exercise3", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Function", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"e1$", ",", "e2$"}], "}"}], ",", 
   RowBox[{
    RowBox[{"PropertyValue", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        GraphicsBox[
         NamespaceBox["NetworkGraphics",
          DynamicModuleBox[{Typeset`graph = HoldComplete[
            
            Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
             Null, {{1, 2}, {1, 4}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {3, 6}, {
              4, 5}, {4, 7}, {4, 8}, {5, 6}, {5, 8}, {6, 8}, {6, 9}, {7, 8}, {
              8, 9}}}, {
             EdgeLabels -> {
               UndirectedEdge["g", "h"] -> 4, UndirectedEdge["b", "d"] -> 3, 
                UndirectedEdge["d", "g"] -> 6, UndirectedEdge["e", "h"] -> 3, 
                UndirectedEdge["a", "b"] -> 5, UndirectedEdge["c", "f"] -> 3, 
                UndirectedEdge["b", "c"] -> 4, UndirectedEdge["f", "i"] -> 4, 
                UndirectedEdge["f", "h"] -> 4, UndirectedEdge["h", "i"] -> 2, 
                UndirectedEdge["a", "d"] -> 2, UndirectedEdge["e", "f"] -> 1, 
                UndirectedEdge["d", "e"] -> 7, UndirectedEdge["b", "f"] -> 6, 
                UndirectedEdge["d", "h"] -> 8, UndirectedEdge["b", "e"] -> 5},
               EdgeWeight -> {5, 2, 4, 3, 5, 6, 3, 7, 6, 8, 1, 3, 4, 4, 4, 2},
               ImagePadding -> 10, 
              VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {
               2, 1}, {0, 0}, {1, 0}, {2, 0}}, VertexLabels -> {"Name"}}]], 
           Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
              Directive[
               Opacity[0.7], 
               Hue[0.6, 0.7, 0.5]], {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$1", Automatic, Center], 
                  DynamicLocation["VertexID$2", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$1"], 
               InsetBox[
                FormBox["5", TraditionalForm], 
                DynamicLocation["EdgeLabelID$1", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$1", Automatic, Center], 
                  DynamicLocation["VertexID$4", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$2"], 
               InsetBox[
                FormBox["2", TraditionalForm], 
                DynamicLocation["EdgeLabelID$2", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$3", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$3"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$3", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$4", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$4"], 
               InsetBox[
                FormBox["3", TraditionalForm], 
                DynamicLocation["EdgeLabelID$4", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$5", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$5"], 
               InsetBox[
                FormBox["5", TraditionalForm], 
                DynamicLocation["EdgeLabelID$5", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$6", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$6"], 
               InsetBox[
                FormBox["6", TraditionalForm], 
                DynamicLocation["EdgeLabelID$6", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$3", Automatic, Center], 
                  DynamicLocation["VertexID$6", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$7"], 
               InsetBox[
                FormBox["3", TraditionalForm], 
                DynamicLocation["EdgeLabelID$7", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$4", Automatic, Center], 
                  DynamicLocation["VertexID$5", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$8"], 
               InsetBox[
                FormBox["7", TraditionalForm], 
                DynamicLocation["EdgeLabelID$8", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$4", Automatic, Center], 
                  DynamicLocation["VertexID$7", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$9"], 
               InsetBox[
                FormBox["6", TraditionalForm], 
                DynamicLocation["EdgeLabelID$9", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$4", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$10"], 
               InsetBox[
                FormBox["8", TraditionalForm], 
                DynamicLocation["EdgeLabelID$10", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$5", Automatic, Center], 
                  DynamicLocation["VertexID$6", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$11"], 
               InsetBox[
                FormBox["1", TraditionalForm], 
                DynamicLocation["EdgeLabelID$11", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$5", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$12"], 
               InsetBox[
                FormBox["3", TraditionalForm], 
                DynamicLocation["EdgeLabelID$12", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$6", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$13"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$13", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$6", Automatic, Center], 
                  DynamicLocation["VertexID$9", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$14"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$14", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$7", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$15"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$15", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$8", Automatic, Center], 
                  DynamicLocation["VertexID$9", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$16"], 
               InsetBox[
                FormBox["2", TraditionalForm], 
                DynamicLocation["EdgeLabelID$16", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
              Directive[
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[
                Directive[
                 GrayLevel[0], 
                 Opacity[0.7]]]], 
              TagBox[{
                TagBox[
                 DiskBox[{0., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$1"], 
                InsetBox[
                 FormBox["\"a\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$1"], 
              TagBox[{
                TagBox[
                 DiskBox[{1., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$2"], 
                InsetBox[
                 FormBox["\"b\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$2"], 
              TagBox[{
                TagBox[
                 DiskBox[{2., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$3"], 
                InsetBox[
                 FormBox["\"c\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$3"], 
              TagBox[{
                TagBox[
                 DiskBox[{0., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$4"], 
                InsetBox[
                 FormBox["\"d\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$4"], 
              TagBox[{
                TagBox[
                 DiskBox[{1., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$5"], 
                InsetBox[
                 FormBox["\"e\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$5"], 
              TagBox[{
                TagBox[
                 DiskBox[{2., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$6"], 
                InsetBox[
                 FormBox["\"f\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$6"], 
              TagBox[{
                TagBox[
                 DiskBox[{0., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$7"], 
                InsetBox[
                 FormBox["\"g\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$7"], 
              TagBox[{
                TagBox[
                 DiskBox[{1., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$8"], 
                InsetBox[
                 FormBox["\"h\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$8"], 
              TagBox[{
                TagBox[
                 DiskBox[{2., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$9"], 
                InsetBox[
                 FormBox["\"i\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> 
               "VertexLabelID$9"]}}], $CellContext`flag}, 
           TagBox[
            
            DynamicBox[GraphComputation`NetworkGraphicsBox[
             3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
             CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
              SynchronousUpdating -> False, 
              TrackedSymbols :> {$CellContext`flag}},
             
             ImageSizeCache->{{8.228732560330084, 
              86.50371646227612}, {-49.55971646226848, 37.215267439677376`}}],
            
            MouseAppearanceTag["NetworkGraphics"]],
           AllowKernelInitialization->False,
           UnsavedVariables:>{$CellContext`flag}]],
         DefaultBaseStyle->{
          "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
           Hue[0.8, 1., 0.6]},
         FrameTicks->None,
         ImagePadding->10], ",", "e1$"}], "}"}], ",", "EdgeWeight"}], "]"}], 
    "<", 
    RowBox[{"PropertyValue", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        GraphicsBox[
         NamespaceBox["NetworkGraphics",
          DynamicModuleBox[{Typeset`graph = HoldComplete[
            
            Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
             Null, {{1, 2}, {1, 4}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {3, 6}, {
              4, 5}, {4, 7}, {4, 8}, {5, 6}, {5, 8}, {6, 8}, {6, 9}, {7, 8}, {
              8, 9}}}, {
             EdgeLabels -> {
               UndirectedEdge["g", "h"] -> 4, UndirectedEdge["b", "d"] -> 3, 
                UndirectedEdge["d", "g"] -> 6, UndirectedEdge["e", "h"] -> 3, 
                UndirectedEdge["a", "b"] -> 5, UndirectedEdge["c", "f"] -> 3, 
                UndirectedEdge["b", "c"] -> 4, UndirectedEdge["f", "i"] -> 4, 
                UndirectedEdge["f", "h"] -> 4, UndirectedEdge["h", "i"] -> 2, 
                UndirectedEdge["a", "d"] -> 2, UndirectedEdge["e", "f"] -> 1, 
                UndirectedEdge["d", "e"] -> 7, UndirectedEdge["b", "f"] -> 6, 
                UndirectedEdge["d", "h"] -> 8, UndirectedEdge["b", "e"] -> 5},
               EdgeWeight -> {5, 2, 4, 3, 5, 6, 3, 7, 6, 8, 1, 3, 4, 4, 4, 2},
               ImagePadding -> 10, 
              VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {
               2, 1}, {0, 0}, {1, 0}, {2, 0}}, VertexLabels -> {"Name"}}]], 
           Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
              Directive[
               Opacity[0.7], 
               Hue[0.6, 0.7, 0.5]], {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$1", Automatic, Center], 
                  DynamicLocation["VertexID$2", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$1"], 
               InsetBox[
                FormBox["5", TraditionalForm], 
                DynamicLocation["EdgeLabelID$1", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$1", Automatic, Center], 
                  DynamicLocation["VertexID$4", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$2"], 
               InsetBox[
                FormBox["2", TraditionalForm], 
                DynamicLocation["EdgeLabelID$2", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$3", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$3"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$3", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$4", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$4"], 
               InsetBox[
                FormBox["3", TraditionalForm], 
                DynamicLocation["EdgeLabelID$4", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$5", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$5"], 
               InsetBox[
                FormBox["5", TraditionalForm], 
                DynamicLocation["EdgeLabelID$5", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$2", Automatic, Center], 
                  DynamicLocation["VertexID$6", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$6"], 
               InsetBox[
                FormBox["6", TraditionalForm], 
                DynamicLocation["EdgeLabelID$6", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$3", Automatic, Center], 
                  DynamicLocation["VertexID$6", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$7"], 
               InsetBox[
                FormBox["3", TraditionalForm], 
                DynamicLocation["EdgeLabelID$7", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$4", Automatic, Center], 
                  DynamicLocation["VertexID$5", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$8"], 
               InsetBox[
                FormBox["7", TraditionalForm], 
                DynamicLocation["EdgeLabelID$8", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$4", Automatic, Center], 
                  DynamicLocation["VertexID$7", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$9"], 
               InsetBox[
                FormBox["6", TraditionalForm], 
                DynamicLocation["EdgeLabelID$9", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$4", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$10"], 
               InsetBox[
                FormBox["8", TraditionalForm], 
                DynamicLocation["EdgeLabelID$10", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$5", Automatic, Center], 
                  DynamicLocation["VertexID$6", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$11"], 
               InsetBox[
                FormBox["1", TraditionalForm], 
                DynamicLocation["EdgeLabelID$11", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$5", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$12"], 
               InsetBox[
                FormBox["3", TraditionalForm], 
                DynamicLocation["EdgeLabelID$12", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$6", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$13"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$13", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$6", Automatic, Center], 
                  DynamicLocation["VertexID$9", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$14"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$14", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$7", Automatic, Center], 
                  DynamicLocation["VertexID$8", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$15"], 
               InsetBox[
                FormBox["4", TraditionalForm], 
                DynamicLocation["EdgeLabelID$15", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
               TagBox[
                LineBox[{
                  DynamicLocation["VertexID$8", Automatic, Center], 
                  DynamicLocation["VertexID$9", Automatic, Center]}], 
                "DynamicName", BoxID -> "EdgeLabelID$16"], 
               InsetBox[
                FormBox["2", TraditionalForm], 
                DynamicLocation["EdgeLabelID$16", Automatic, 
                 Scaled[0.5]], 
                ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
              Directive[
               Hue[0.6, 0.2, 0.8], 
               EdgeForm[
                Directive[
                 GrayLevel[0], 
                 Opacity[0.7]]]], 
              TagBox[{
                TagBox[
                 DiskBox[{0., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$1"], 
                InsetBox[
                 FormBox["\"a\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$1"], 
              TagBox[{
                TagBox[
                 DiskBox[{1., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$2"], 
                InsetBox[
                 FormBox["\"b\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$2"], 
              TagBox[{
                TagBox[
                 DiskBox[{2., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$3"], 
                InsetBox[
                 FormBox["\"c\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$3"], 
              TagBox[{
                TagBox[
                 DiskBox[{0., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$4"], 
                InsetBox[
                 FormBox["\"d\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$4"], 
              TagBox[{
                TagBox[
                 DiskBox[{1., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$5"], 
                InsetBox[
                 FormBox["\"e\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$5"], 
              TagBox[{
                TagBox[
                 DiskBox[{2., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$6"], 
                InsetBox[
                 FormBox["\"f\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$6"], 
              TagBox[{
                TagBox[
                 DiskBox[{0., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$7"], 
                InsetBox[
                 FormBox["\"g\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$7"], 
              TagBox[{
                TagBox[
                 DiskBox[{1., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$8"], 
                InsetBox[
                 FormBox["\"h\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> "VertexLabelID$8"], 
              TagBox[{
                TagBox[
                 DiskBox[{2., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
                 "VertexID$9"], 
                InsetBox[
                 FormBox["\"i\"", TraditionalForm], 
                 Offset[{2, 2}, 
                  DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
                 ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, 
               "DynamicName", BoxID -> 
               "VertexLabelID$9"]}}], $CellContext`flag}, 
           TagBox[
            
            DynamicBox[GraphComputation`NetworkGraphicsBox[
             3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
             CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
              SynchronousUpdating -> False, 
              TrackedSymbols :> {$CellContext`flag}},
             
             ImageSizeCache->{{8.228732560330084, 
              86.50371646227612}, {-49.55971646227596, 37.2152674396699}}],
            MouseAppearanceTag["NetworkGraphics"]],
           AllowKernelInitialization->False,
           UnsavedVariables:>{$CellContext`flag}]],
         DefaultBaseStyle->{
          "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
           Hue[0.8, 1., 0.6]},
         FrameTicks->None,
         ImagePadding->10], ",", "e2$"}], "}"}], ",", "EdgeWeight"}], 
     "]"}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"testComp", "[", 
  RowBox[{
   RowBox[{"\"\<a\>\"", "\[UndirectedEdge]", "\"\<d\>\""}], ",", 
   RowBox[{"\"\<a\>\"", "\[UndirectedEdge]", "\"\<b\>\""}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
Prim\[CloseCurlyQuote]s algorithm is given as Algorithm 1 in Section 11.5 of \
the textbook. It constructs a minimum spanning tree by successively selecting \
an edge of smallest weight that extends the tree without creating any loops. \
\
\>", "Text"],

Cell[TextData[{
 "To simplify our implementation of Prim's algorithm, we will create a \
function ",
 StyleBox["minEdge", "Input"],
 ". Given the original graph and the list of vertices already included in the \
spanning tree, ",
 StyleBox["minEdge", "Input"],
 " determines which edge of the graph should be added next."
}], "Text"],

Cell[TextData[{
 StyleBox["minEdge", "Input"],
 " first needs to determine the set of edges that are incident with a vertex \
currently in the tree. It will do this by applying ",
 StyleBox[ButtonBox["NeighborhoodGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NeighborhoodGraph"], "FunctionLink"],
 " to the graph and the list of vertices in the tree. The ",
 StyleBox["minEdge", "Input"],
 " function then eliminates any edge with both ends already in the spanning \
tree or neither end in the spanning tree. (This is equivalent to the \
condition that the edge not introduce a simple circuit but that the new graph \
will be connected.) Once it has determined the valid candidates, the ",
 StyleBox["minEdge", "Input"],
 " function returns the edge with smallest weight. We also include the \
special case that the spanning tree has not yet been started, in which case \
we call the function with the empty list as the second argument."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"minEdge", "[", 
   RowBox[{"G_Graph", ",", "V_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"possibleEdges", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"V", "\[Equal]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"empty", " ", "tree", " ", "case"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"possibleEdges", "=", 
        RowBox[{"EdgeList", "[", "G", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"nonempty", " ", "case"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"possibleEdges", "=", 
         RowBox[{"EdgeList", "[", 
          RowBox[{"NeighborhoodGraph", "[", 
           RowBox[{"G", ",", "V"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"i", "\[LessEqual]", 
           RowBox[{"Length", "[", "possibleEdges", "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"MemberQ", "[", 
              RowBox[{"V", ",", 
               RowBox[{
                RowBox[{"possibleEdges", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "\[Equal]", 
             RowBox[{"MemberQ", "[", 
              RowBox[{"V", ",", 
               RowBox[{
                RowBox[{"possibleEdges", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"possibleEdges", "=", 
             RowBox[{"Delete", "[", 
              RowBox[{"possibleEdges", ",", "i"}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"i", "++"}]}], "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"possibleEdges", "\[Equal]", 
        RowBox[{"{", "}"}]}], ",", 
       RowBox[{"Return", "[", "Null", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"possibleEdges", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"possibleEdges", ",", 
        RowBox[{"edgeCompare", "[", "G", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"possibleEdges", "[", 
      RowBox[{"[", "1", "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
With this function in place, Prim's algorithm is fairly straightforward to \
implement.\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Begin building the spanning tree by finding the edge of minimum weight by \
calling the function ",
 StyleBox["minEdge", "Input"],
 " on the graph and the empty list."
}], "Item1Numbered"],

Cell[TextData[{
 "Continue building the spanning tree one edge at a time by adding the edge \
returned by ",
 StyleBox["minEdge", "Input"],
 ". (Note that we must add the new vertex before the edge, since ",
 StyleBox[ButtonBox["EdgeAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeAdd"], "FunctionLink"],
 " expects both endpoints of the edge to be added to already be in the graph.)"
}], "Item1Numbered"],

Cell[TextData[{
 "After ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "2"}], TraditionalForm]]],
 " repetitions of step 2, where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the number of vertices in the graph, the spanning tree is complete."
}], "Item1Numbered"],

Cell["\<\
For the sake of displaying the resulting tree, we conclude the function by \
copying the vertex positions from the original graph to the tree. \
\>", "Item1Numbered"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"prim", "[", 
   RowBox[{"G_Graph", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"newEdge", ",", "T", ",", "n", ",", "i", ",", "v"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newEdge", "=", 
      RowBox[{"minEdge", "[", 
       RowBox[{"G", ",", 
        RowBox[{"{", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"{", "newEdge", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], "]"}], 
      "=", 
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"G", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", "newEdge"}], "}"}], ",", "EdgeLabels"}], "]"}], 
      "=", 
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"G", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"n", "=", 
      RowBox[{"VertexCount", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"n", "-", "2"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"newEdge", "=", 
         RowBox[{"minEdge", "[", 
          RowBox[{"G", ",", 
           RowBox[{"VertexList", "[", "T", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"VertexQ", "[", 
           RowBox[{"T", ",", 
            RowBox[{"newEdge", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"T", "=", 
           RowBox[{"VertexAdd", "[", 
            RowBox[{"T", ",", 
             RowBox[{"newEdge", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"T", "=", 
           RowBox[{"VertexAdd", "[", 
            RowBox[{"T", ",", 
             RowBox[{"newEdge", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"T", "=", 
         RowBox[{"EdgeAdd", "[", 
          RowBox[{"T", ",", "newEdge"}], "]"}]}], ";", "\[IndentingNewLine]", 
        
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], "]"}], 
         "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"G", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "newEdge"}], "}"}], ",", "EdgeLabels"}], "]"}], 
         "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"G", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], 
          "]"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "v"}], "}"}], ",", "VertexCoordinates"}], "]"}],
          "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"G", ",", "v"}], "}"}], ",", "VertexCoordinates"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}], "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"G", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}]}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "T", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell["\<\
We can now use this algorithm to find the minimum spanning tree for Exercise \
3.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"primExercise3", "=", 
  RowBox[{"prim", "[", 
   RowBox[{"exercise3", ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"e", "f", "c", "h", "i", "g", "b", "d", "a"}, {
      Null, {{1, 2}, {3, 2}, {1, 4}, {4, 5}, {6, 4}, {7, 3}, {7, 8}, {9, 
       8}}}, {EdgeLabels -> {
        UndirectedEdge["a", "d"] -> 2, UndirectedEdge["h", "i"] -> 2, 
         UndirectedEdge["g", "h"] -> 4, UndirectedEdge["b", "c"] -> 4, 
         UndirectedEdge["c", "f"] -> 3, UndirectedEdge["b", "d"] -> 3, 
         UndirectedEdge["e", "f"] -> 1, UndirectedEdge["e", "h"] -> 3}, 
       EdgeWeight -> {1, 1, 1, 1, 1, 1, 1, 2}, ImagePadding -> 10, 
       VertexCoordinates -> {{1, 1}, {2, 1}, {2, 2}, {1, 0}, {2, 0}, {0, 0}, {
        1, 2}, {0, 1}, {0, 2}}, 
       VertexLabels -> {
        "d" -> "Name", "b" -> "Name", "c" -> "Name", "i" -> "Name", "a" -> 
         "Name", "g" -> "Name", "e" -> "Name", "h" -> "Name", "f" -> 
         "Name"}}]], Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$1", Automatic, Center], 
           DynamicLocation["VertexID$2", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$1", Automatic, Center], 
           DynamicLocation["VertexID$4", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$3", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$3"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$3", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$3", Automatic, Center], 
           DynamicLocation["VertexID$7", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$4"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$4", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$4", Automatic, Center], 
           DynamicLocation["VertexID$5", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$5"], 
        InsetBox[
         FormBox["2", TraditionalForm], 
         DynamicLocation["EdgeLabelID$5", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$4", Automatic, Center], 
           DynamicLocation["VertexID$6", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$6"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$6", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$7", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$7"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$7", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$8", Automatic, Center], 
           DynamicLocation["VertexID$9", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$8"], 
        InsetBox[
         FormBox["2", TraditionalForm], 
         DynamicLocation["EdgeLabelID$8", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{1., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.385786437626887`, 
       350.26438720589675`}, {-178.53638720589583`, 163.7336181092432}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Before moving on to Kruskal\[CloseCurlyQuote]s algorithm, we\
\[CloseCurlyQuote]ll create a function to produce an animation demonstrating \
Prim\[CloseCurlyQuote]s algorithm in action. First, we\[CloseCurlyQuote]ll \
modify the ",
 StyleBox["prim", "Input"],
 " function to record the list of edges that form the tree and return this \
list of edges rather than the tree."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"primEdges", "[", "G_Graph", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "newEdge", ",", "T", ",", "edgeList", ",", "n", ",", "i", ",", "v"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newEdge", "=", 
      RowBox[{"minEdge", "[", 
       RowBox[{"G", ",", 
        RowBox[{"{", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{"{", "newEdge", "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"edgeList", "=", 
      RowBox[{"{", "newEdge", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", "=", 
      RowBox[{"VertexCount", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"n", "-", "2"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"newEdge", "=", 
         RowBox[{"minEdge", "[", 
          RowBox[{"G", ",", 
           RowBox[{"VertexList", "[", "T", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"VertexQ", "[", 
           RowBox[{"T", ",", 
            RowBox[{"newEdge", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"T", "=", 
           RowBox[{"VertexAdd", "[", 
            RowBox[{"T", ",", 
             RowBox[{"newEdge", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"T", "=", 
           RowBox[{"VertexAdd", "[", 
            RowBox[{"T", ",", 
             RowBox[{"newEdge", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"T", "=", 
         RowBox[{"EdgeAdd", "[", 
          RowBox[{"T", ",", "newEdge"}], "]"}]}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"AppendTo", "[", 
         RowBox[{"edgeList", ",", "newEdge"}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "edgeList"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"primEdges", "[", "exercise3", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"e\"\>", "\[UndirectedEdge]", "\<\"f\"\>"}], ",", 
   RowBox[{"\<\"c\"\>", "\[UndirectedEdge]", "\<\"f\"\>"}], ",", 
   RowBox[{"\<\"e\"\>", "\[UndirectedEdge]", "\<\"h\"\>"}], ",", 
   RowBox[{"\<\"h\"\>", "\[UndirectedEdge]", "\<\"i\"\>"}], ",", 
   RowBox[{"\<\"g\"\>", "\[UndirectedEdge]", "\<\"h\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"d\"\>"}], ",", 
   RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"d\"\>"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We now write the function that will produce the animation. Since this is \
nearly identical to the ",
 StyleBox["animatePath", "Input"],
 " function from Chapter 10, we refer the reader to Section 10.5 of this \
manual for a detailed explanation."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"animateTree", "[", 
   RowBox[{"G_Graph", ",", "T_List", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "len"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"len", "=", 
      RowBox[{"Length", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Animate", "[", 
      RowBox[{
       RowBox[{"HighlightGraph", "[", 
        RowBox[{"G", ",", 
         RowBox[{"T", "[", 
          RowBox[{"[", 
           RowBox[{"1", ";;", "i"}], "]"}], "]"}], ",", "opts"}], "]"}], ",", 
       
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"i", ",", "0", ",", "\"\<step\>\""}], "}"}], ",", "0", ",", 
         "len", ",", "1"}], "}"}], ",", 
       RowBox[{"AnimationRunning", "\[Rule]", "False"}], ",", 
       RowBox[{"AnimationRepetitions", "\[Rule]", "1"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"animateTree", "[", 
  RowBox[{"exercise3", ",", 
   RowBox[{"primEdges", "[", "exercise3", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$4280$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`i$4280$$], 0, "step"}, 0, 8, 1}}, Typeset`size$$ = {
    360., {177., 182.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = True, $CellContext`i$4280$4281$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$4280$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$4280$$, $CellContext`i$4280$4281$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> HighlightGraph[
        Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
         Null, {{1, 2}, {1, 4}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {3, 6}, {4, 
          5}, {4, 7}, {4, 8}, {5, 6}, {5, 8}, {6, 8}, {6, 9}, {7, 8}, {8, 
          9}}}, {EdgeLabels -> {
           UndirectedEdge["g", "h"] -> 4, UndirectedEdge["b", "d"] -> 3, 
            UndirectedEdge["d", "g"] -> 6, UndirectedEdge["e", "h"] -> 3, 
            UndirectedEdge["a", "b"] -> 5, UndirectedEdge["c", "f"] -> 3, 
            UndirectedEdge["b", "c"] -> 4, UndirectedEdge["f", "i"] -> 4, 
            UndirectedEdge["f", "h"] -> 4, UndirectedEdge["h", "i"] -> 2, 
            UndirectedEdge["a", "d"] -> 2, UndirectedEdge["e", "f"] -> 1, 
            UndirectedEdge["d", "e"] -> 7, UndirectedEdge["b", "f"] -> 6, 
            UndirectedEdge["d", "h"] -> 8, UndirectedEdge["b", "e"] -> 5}, 
          EdgeWeight -> {5, 2, 4, 3, 5, 6, 3, 7, 6, 8, 1, 3, 4, 4, 4, 2}, 
          ImagePadding -> 10, 
          VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {2, 
           1}, {0, 0}, {1, 0}, {2, 0}}, VertexLabels -> {"Name"}}], 
        Part[{
          UndirectedEdge["e", "f"], 
          UndirectedEdge["c", "f"], 
          UndirectedEdge["e", "h"], 
          UndirectedEdge["h", "i"], 
          UndirectedEdge["g", "h"], 
          UndirectedEdge["b", "c"], 
          UndirectedEdge["b", "d"], 
          UndirectedEdge["a", "d"]}, 
         Span[1, $CellContext`i$4280$$]]], 
      "Specifications" :> {{{$CellContext`i$4280$$, 0, "step"}, 0, 8, 1, 
         AnimationRunning -> False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{403., {214., 219.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Kruskal\[CloseCurlyQuote]s Algorithm", "Subsection",
 CellTags->"Section5b"],

Cell["\<\
Recall that Kruskal's algorithm, Algorithm 2 in Section 11.5, begins in the \
same way as Prim's algorithm, with the edge of smallest weight. The \
difference is that at each step, Kruskal's algorithm adds whatever edge is of \
least weight which does not create a simple circuit, regardless of whether it \
is incident to an edge already in the graph.\
\>", "Text"],

Cell[TextData[{
 "We begin with a function to test whether or not a given edge will create a \
simple circuit. Note that, during the steps of Kruskal's algorithm, we have a \
forest of trees. An edge will create a simple circuit if and only if both of \
its endpoints are in the same tree within the forest. We use the ",
 StyleBox[ButtonBox["ConnectedComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedComponents"], "FunctionLink"],
 " function to find the trees. The ",
 StyleBox[ButtonBox["ConnectedComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedComponents"], "FunctionLink"],
 " function returns a list of lists, where each inner list is the vertices \
within one of the connected components of the graph. We test an edge by \
looping through each of the connected components and making sure that both \
ends do not appear in the same component."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"noCircuitQ", "[", 
   RowBox[{"G_Graph", ",", "edge_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"components", ",", "C"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"components", "=", 
      RowBox[{"ConnectedComponents", "[", "G", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{"C", ",", 
              RowBox[{"edge", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "&&", 
            RowBox[{"MemberQ", "[", 
             RowBox[{"C", ",", 
              RowBox[{"edge", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ",", 
           RowBox[{"Throw", "[", "False", "]"}]}], "]"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"C", ",", "components"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Now we implement Kruskal's algorithm. The function is as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Initialize ",
 StyleBox["edges", "Input"],
 " to the list of edges of the given graph and sort this list using the ",
 StyleBox["edgeCompare", "Input"],
 " function created above."
}], "Item1Numbered"],

Cell[TextData[{
 "Initialize ",
 StyleBox["T", "Input"],
 " to the graph with all the vertices of ",
 StyleBox["G", "Input"],
 " and no edges.."
}], "Item1Numbered"],

Cell[TextData[{
 "Consider the first edge in the ",
 StyleBox["edges", "Input"],
 " list. Use ",
 StyleBox["noCircuitQ", "Input"],
 " to determine if it is safe to add to the tree. If we can add it to the \
tree, add one or both of its ends as new vertices to ",
 StyleBox["T", "Input"],
 " as needed and add the edge."
}], "Item1Numbered"],

Cell[TextData[{
 "Regardless of whether ",
 StyleBox["noCircuitQ", "Input"],
 " approves the addition of the first edge in ",
 StyleBox["edges", "Input"],
 " to the tree, remove the edge from ",
 StyleBox["edges", "Input"],
 " \[LongDash] either the edge is now used in the tree and so won't be used \
again, or its addition would create a circuit (a fact which won't change \
later)."
}], "Item1Numbered"],

Cell[TextData[{
 "Repeat steps 3 and 4 until ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " edges have been added, where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the number of vertices."
}], "Item1Numbered"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"kruskal", "[", 
   RowBox[{"G_Graph", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "edges", ",", "T", ",", "n", ",", "i", ",", "newEdge", ",", "v"}], "}"}],
     ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"edges", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{
        RowBox[{"EdgeList", "[", "G", "]"}], ",", 
        RowBox[{"edgeCompare", "[", "G", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"VertexList", "[", "G", "]"}], ",", 
        RowBox[{"{", "}"}], ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"n", "=", 
      RowBox[{"VertexCount", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"n", "-", "1"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"newEdge", "=", 
         RowBox[{"edges", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"noCircuitQ", "[", 
           RowBox[{"T", ",", "newEdge"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"T", "=", 
            RowBox[{"EdgeAdd", "[", 
             RowBox[{"T", ",", "newEdge"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"PropertyValue", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"T", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], 
             "]"}], "=", 
            RowBox[{"PropertyValue", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"G", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"PropertyValue", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"T", ",", "newEdge"}], "}"}], ",", "EdgeLabels"}], 
             "]"}], "=", 
            RowBox[{"PropertyValue", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"G", ",", "newEdge"}], "}"}], ",", "EdgeWeight"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"i", "++"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"edges", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"edges", ",", "1"}], "]"}]}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "v"}], "}"}], ",", "VertexCoordinates"}], "]"}],
          "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"G", ",", "v"}], "}"}], ",", "VertexCoordinates"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}], "=", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"G", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}]}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "T", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell["\<\
Note that Kruskal's algorithm produces the same minimum spanning tree for \
Exercise 3 as did our implementation of Prim's algorithm (in fact, this graph \
has a unique minimum spanning tree).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kruskalExercise3", "=", 
  RowBox[{"kruskal", "[", 
   RowBox[{"exercise3", ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
      Null, {{5, 6}, {8, 9}, {1, 4}, {5, 8}, {3, 6}, {2, 4}, {7, 8}, {2, 
       3}}}, {EdgeLabels -> {
        UndirectedEdge["a", "d"] -> 2, UndirectedEdge["g", "h"] -> 4, 
         UndirectedEdge["h", "i"] -> 2, UndirectedEdge["b", "c"] -> 4, 
         UndirectedEdge["c", "f"] -> 3, UndirectedEdge["b", "d"] -> 3, 
         UndirectedEdge["e", "f"] -> 1, UndirectedEdge["e", "h"] -> 3}, 
       EdgeWeight -> {1, 1, 1, 1, 1, 1, 1, 4}, ImagePadding -> 10, 
       VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {2, 1}, {
        0, 0}, {1, 0}, {2, 0}}, 
       VertexLabels -> {
        "d" -> "Name", "c" -> "Name", "b" -> "Name", "i" -> "Name", "g" -> 
         "Name", "a" -> "Name", "e" -> "Name", "h" -> "Name", "f" -> 
         "Name"}}]], Typeset`boxes, Typeset`boxes$s2d = GraphicsGroupBox[{{
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$1", Automatic, Center], 
           DynamicLocation["VertexID$4", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["2", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$3", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$2", Automatic, Center], 
           DynamicLocation["VertexID$4", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$3"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$3", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$3", Automatic, Center], 
           DynamicLocation["VertexID$6", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$4"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$4", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$5", Automatic, Center], 
           DynamicLocation["VertexID$6", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$5"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$5", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$5", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$6"], 
        InsetBox[
         FormBox["3", TraditionalForm], 
         DynamicLocation["EdgeLabelID$6", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$7", Automatic, Center], 
           DynamicLocation["VertexID$8", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$7"], 
        InsetBox[
         FormBox["4", TraditionalForm], 
         DynamicLocation["EdgeLabelID$7", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         LineBox[{
           DynamicLocation["VertexID$8", Automatic, Center], 
           DynamicLocation["VertexID$9", Automatic, Center]}], "DynamicName", 
         BoxID -> "EdgeLabelID$8"], 
        InsetBox[
         FormBox["2", TraditionalForm], 
         DynamicLocation["EdgeLabelID$8", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$1"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 2.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$4"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$5"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 1.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$6"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$7"], 
         InsetBox[
          FormBox["\"g\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{1., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{2., 0.}, 0.02261146496815286], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.385786437626887`, 
       350.26438720589675`}, {-178.53638720589583`, 163.7336181092432}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can produce an animation, like we did for Prim's algorithm, by modifying \
the function to produce the list of edges in the order they are added and \
then using the ",
 StyleBox["animateTree", "Input"],
 " function once again."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"kruskalEdges", "[", "G_Graph", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "edges", ",", "edgeList", ",", "T", ",", "n", ",", "i", ",", "newEdge", 
      ",", "v"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"edges", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{
        RowBox[{"EdgeList", "[", "G", "]"}], ",", 
        RowBox[{"edgeCompare", "[", "G", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"edgeList", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"VertexList", "[", "G", "]"}], ",", 
        RowBox[{"{", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", "=", 
      RowBox[{"VertexCount", "[", "G", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"n", "-", "1"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"newEdge", "=", 
         RowBox[{"edges", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"noCircuitQ", "[", 
           RowBox[{"T", ",", "newEdge"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"T", "=", 
            RowBox[{"EdgeAdd", "[", 
             RowBox[{"T", ",", "newEdge"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"edgeList", ",", "newEdge"}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"i", "++"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"edges", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"edges", ",", "1"}], "]"}]}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "edgeList"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"animateTree", "[", 
  RowBox[{"exercise3", ",", 
   RowBox[{"kruskalEdges", "[", "exercise3", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$4370$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`i$4370$$], 0, "step"}, 0, 8, 1}}, Typeset`size$$ = {
    360., {177., 182.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = True, $CellContext`i$4370$4371$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$4370$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$4370$$, $CellContext`i$4370$4371$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> HighlightGraph[
        Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i"}, {
         Null, {{1, 2}, {1, 4}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {3, 6}, {4, 
          5}, {4, 7}, {4, 8}, {5, 6}, {5, 8}, {6, 8}, {6, 9}, {7, 8}, {8, 
          9}}}, {EdgeLabels -> {
           UndirectedEdge["g", "h"] -> 4, UndirectedEdge["b", "d"] -> 3, 
            UndirectedEdge["d", "g"] -> 6, UndirectedEdge["e", "h"] -> 3, 
            UndirectedEdge["a", "b"] -> 5, UndirectedEdge["c", "f"] -> 3, 
            UndirectedEdge["b", "c"] -> 4, UndirectedEdge["f", "i"] -> 4, 
            UndirectedEdge["f", "h"] -> 4, UndirectedEdge["h", "i"] -> 2, 
            UndirectedEdge["a", "d"] -> 2, UndirectedEdge["e", "f"] -> 1, 
            UndirectedEdge["d", "e"] -> 7, UndirectedEdge["b", "f"] -> 6, 
            UndirectedEdge["d", "h"] -> 8, UndirectedEdge["b", "e"] -> 5}, 
          EdgeWeight -> {5, 2, 4, 3, 5, 6, 3, 7, 6, 8, 1, 3, 4, 4, 4, 2}, 
          ImagePadding -> 10, 
          VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {2, 
           1}, {0, 0}, {1, 0}, {2, 0}}, VertexLabels -> {"Name"}}], 
        Part[{
          UndirectedEdge["e", "f"], 
          UndirectedEdge["h", "i"], 
          UndirectedEdge["a", "d"], 
          UndirectedEdge["e", "h"], 
          UndirectedEdge["c", "f"], 
          UndirectedEdge["b", "d"], 
          UndirectedEdge["g", "h"], 
          UndirectedEdge["b", "c"]}, 
         Span[1, $CellContext`i$4370$$]]], 
      "Specifications" :> {{{$CellContext`i$4370$$, 0, "step"}, 0, 8, 1, 
         AnimationRunning -> False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{403., {214., 219.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output"]
}, Open  ]],

Cell["\<\
By comparing the two animations, you can see how the two algorithms provide \
different routes to the same end result.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 6", "Subsection",
 CellTags->"SolutionsCP6"],

Cell["\<\
Given the ordered list of edges of an ordered rooted tree, find the universal \
addresses of its vertices.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " Recall that the universal address of a vertex in an ordered rooted tree is \
defined as follows. The root has address 0 and its children have addresses 1, \
2, 3, etc., in order. The address of every other vertex is defined \
recursively as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", ".", "n"}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is the address of the vertex's parent and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is 1 if the vertex is the first child of its parent, 2 if it is the second \
child, etc."
}], "Text"],

Cell["\<\
Before solving this problem, we will make the following assumption on the \
input: the edges are sorted according to the lexicographical order of the \
universal address of their terminal vertex. That is to say, the edges are \
listed in the order of their appearance from left to right and top to bottom \
when the tree is drawn in the usual way. \
\>", "Text"],

Cell[TextData[{
 "We build the ordered rooted tree (satisfying ",
 StyleBox["orderedRootedTreeQ", "Input"],
 ") determined by the list of edges and add a \
\[OpenCurlyDoubleQuote]univ-address\[CloseCurlyDoubleQuote] property to each \
vertex containing the vertex's universal address. First, here is an ordered \
list of edges for an ordered rooted tree."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cp6Example", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\"\<D\>\"", "\[Rule]", "\"\<E\>\""}], ",", 
    RowBox[{"\"\<D\>\"", "\[Rule]", "\"\<C\>\""}], ",", 
    RowBox[{"\"\<D\>\"", "\[Rule]", "\"\<I\>\""}], ",", 
    RowBox[{"\"\<E\>\"", "\[Rule]", "\"\<L\>\""}], ",", 
    RowBox[{"\"\<E\>\"", "\[Rule]", "\"\<F\>\""}], ",", 
    RowBox[{"\"\<I\>\"", "\[Rule]", "\"\<B\>\""}], ",", 
    RowBox[{"\"\<I\>\"", "\[Rule]", "\"\<K\>\""}], ",", 
    RowBox[{"\"\<I\>\"", "\[Rule]", "\"\<H\>\""}], ",", 
    RowBox[{"\"\<F\>\"", "\[Rule]", "\"\<J\>\""}], ",", 
    RowBox[{"\"\<F\>\"", "\[Rule]", "\"\<G\>\""}], ",", 
    RowBox[{"\"\<F\>\"", "\[Rule]", "\"\<A\>\""}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"D\"\>", "\[Rule]", "\<\"E\"\>"}], ",", 
   RowBox[{"\<\"D\"\>", "\[Rule]", "\<\"C\"\>"}], ",", 
   RowBox[{"\<\"D\"\>", "\[Rule]", "\<\"I\"\>"}], ",", 
   RowBox[{"\<\"E\"\>", "\[Rule]", "\<\"L\"\>"}], ",", 
   RowBox[{"\<\"E\"\>", "\[Rule]", "\<\"F\"\>"}], ",", 
   RowBox[{"\<\"I\"\>", "\[Rule]", "\<\"B\"\>"}], ",", 
   RowBox[{"\<\"I\"\>", "\[Rule]", "\<\"K\"\>"}], ",", 
   RowBox[{"\<\"I\"\>", "\[Rule]", "\<\"H\"\>"}], ",", 
   RowBox[{"\<\"F\"\>", "\[Rule]", "\<\"J\"\>"}], ",", 
   RowBox[{"\<\"F\"\>", "\[Rule]", "\<\"G\"\>"}], ",", 
   RowBox[{"\<\"F\"\>", "\[Rule]", "\<\"A\"\>"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We have purposefully chosen vertex names that are out of order with respect \
to the ordering of the edges so that our construction is sure to rely only on \
the ordering of the edges and not the vertex labels.\
\>", "Text"],

Cell[TextData[{
 "Note that creating a graph with these edges only requires passing it to the \
",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"cp6Graph", "=", 
   RowBox[{"Graph", "[", "cp6Example", "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
Our first task is to turn this into an ordered rooted tree. Since we provided \
directed edges, it is already a rooted tree.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rootedTreeQ", "[", "cp6Graph", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
To make this graph an ordered rooted tree, we need to set the \
\[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] property for each \
vertex. For the root, we just set the root's \[OpenCurlyDoubleQuote]order\
\[CloseCurlyDoubleQuote] to 0.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PropertyValue", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"cp6Graph", ",", 
      RowBox[{"findRoot", "[", "cp6Graph", "]"}]}], "}"}], ",", 
    "\"\<order\>\""}], "]"}], "=", "0"}]], "Input",
 InitializationCell->True],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
 "For the rest of the vertices, we need to do some more work. Our approach \
will be as follows. Loop through all of the edges in the original edge list, \
in order, keeping track of two variables, ",
 StyleBox["curParent", "Input"],
 ", the current parent, and the ",
 StyleBox["childOrder", "Input"],
 ". The ",
 StyleBox["curParent", "Input"],
 " will initially be set to the root of the tree and ",
 StyleBox["childOrder", "Input"],
 " will be initialized to 1. For the first edge in the list, we assign the \
child vertex (the terminal vertex of the ordered edge) \
\[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] equal to ",
 StyleBox["childOrder", "Input"],
 ". We then go to the next edge. If the ",
 StyleBox["curParent", "Input"],
 " is the same as the parent vertex in this edge, then ",
 StyleBox["childOrder", "Input"],
 " is incremented and we assign the child vertex of this edge an order equal \
to the new ",
 StyleBox["childOrder", "Input"],
 " value. Otherwise, the parent vertex of this new edge is different from ",
 StyleBox["curParent", "Input"],
 ". This indicates that we have moved on to a new parent with a new set of \
children, so we set ",
 StyleBox["curParent", "Input"],
 " to this new parent and reset ",
 StyleBox["childOrder", "Input"],
 " to 1. Here is the code for this step in the process."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"curParent", "=", 
   RowBox[{"findRoot", "[", "cp6Graph", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"childOrder", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"thisEdge", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "curParent"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"curParent", "=", 
        RowBox[{"thisEdge", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"childOrder", "=", "1"}]}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"cp6Graph", ",", 
         RowBox[{"thisEdge", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", "\"\<order\>\""}], 
      "]"}], "=", "childOrder"}], ";", "\[IndentingNewLine]", 
    RowBox[{"childOrder", "++"}]}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"thisEdge", ",", "cp6Example"}], "}"}]}], "]"}]}], "Input",
 InitializationCell->True],

Cell["\<\
Now that the order attributes are set, our graph is an ordered rooted tree. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"orderedRootedTreeQ", "[", "cp6Graph", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"plotORTree", "[", 
  RowBox[{"cp6Graph", ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "D", "E", "C", "I", "L", "F", "B", "K", "H", "J", "G", "A"}, {{{1, 2}, {
       1, 3}, {1, 4}, {2, 5}, {2, 6}, {4, 7}, {4, 8}, {4, 9}, {6, 10}, {6, 
       11}, {6, 12}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "J" -> {"order" -> 1}, "L" -> {"order" -> 1}, "G" -> {"order" -> 2}, 
         "E" -> {"order" -> 1}, "F" -> {"order" -> 2}, "A" -> {"order" -> 3}, 
         "B" -> {"order" -> 1}, "H" -> {"order" -> 3}, "D" -> {"order" -> 0}, 
         "I" -> {"order" -> 3}, "K" -> {"order" -> 2}, "C" -> {"order" -> 2}},
        VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Arrowheads[0.028665006973120768`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$11", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$12", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.34263754799435, 2.3016643679903144`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"D\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.38361072799838575`, 1.5344429119935428`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"E\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.1508321839951572`, 1.5344429119935428`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"C\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.3016643679903144`, 1.5344429119935428`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"I\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.7672214559967714}, 0.030731491585499593`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"L\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.7672214559967715, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"F\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.534442911993543, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"B\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.3016643679903144`, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"K\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.068885823987086, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"H\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.030731491585499593`], "DynamicName", BoxID -> 
          "VertexID$10"], 
         InsetBox[
          FormBox["\"J\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7672214559967715, 0.}, 0.030731491585499593`], 
          "DynamicName", BoxID -> "VertexID$11"], 
         InsetBox[
          FormBox["\"G\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$11", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$11"], 
       TagBox[{
         TagBox[
          DiskBox[{1.534442911993543, 0.}, 0.030731491585499593`], 
          "DynamicName", BoxID -> "VertexID$12"], 
         InsetBox[
          FormBox["\"A\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$12", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$12"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       352.96857199521406`}, {-138.24057199521832`, 121.93017909751302`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]],

Cell["We use the approach above to define a function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"treeFromList", "[", 
   RowBox[{"L_List", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "T", ",", "e", ",", "curParent", ",", "childOrder", ",", "thisEdge"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"T", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{"L", ",", "opts"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"T", ",", 
          RowBox[{"findRoot", "[", "T", "]"}]}], "}"}], ",", 
        "\"\<order\>\""}], "]"}], "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"curParent", "=", 
      RowBox[{"findRoot", "[", "T", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"childOrder", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"thisEdge", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "\[NotEqual]", "curParent"}], ",",
           "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"curParent", "=", 
            RowBox[{"thisEdge", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"childOrder", "=", "1"}]}]}], "\[IndentingNewLine]", "]"}],
         ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"T", ",", 
             RowBox[{"thisEdge", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", "\"\<order\>\""}],
           "]"}], "=", "childOrder"}], ";", "\[IndentingNewLine]", 
        RowBox[{"childOrder", "++"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"thisEdge", ",", "L"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "We could now create a function that uses the pattern of a breadth first \
search to assign the universal addresses. But instead we will take this \
opportunity to use the built-in function ",
 StyleBox[ButtonBox["BreadthFirstScan",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BreadthFirstScan"], "FunctionLink"],
 " instead. The first two arguments will be the graph and the root of the \
graph, which explicitly tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " where to begin the breadth first search. The third argument will be a list \
specifying a function to use when the event ",
 StyleBox["\[OpenCurlyDoubleQuote]DiscoverVertex\[CloseCurlyDoubleQuote]", 
  "Input"],
 " occurs. The function is applied to two arguments, ",
 StyleBox["#1", "Input"],
 " refers to the vertex being discovered, and ",
 StyleBox["#2", "Input"],
 " refers to the vertex from which it is discovered, that is, its parent. The \
body of our function will check if the parent has \
\[OpenCurlyDoubleQuote]order\[CloseCurlyDoubleQuote] property 0, indicating \
that the discovered vertex is either the root or a child of the root. In this \
case, the \[OpenCurlyDoubleQuote]univ-address\[CloseCurlyDoubleQuote] \
property will be set to the vertex\[CloseCurlyQuote]s order. Otherwise, the \
\[OpenCurlyDoubleQuote]univ-address\[CloseCurlyDoubleQuote] will be set to \
the concatenation of the parent\[CloseCurlyQuote]s address and the vertex\
\[CloseCurlyQuote]s order."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"universalAddress", "[", 
   RowBox[{"edgeList_List", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "G", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"G", "=", 
      RowBox[{"treeFromList", "[", 
       RowBox[{"edgeList", ",", "opts"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"BreadthFirstScan", "[", 
      RowBox[{"G", ",", 
       RowBox[{"findRoot", "[", "G", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<DiscoverVertex\>\"", "\[Rule]", "\[IndentingNewLine]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"PropertyValue", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"G", ",", "#2"}], "}"}], ",", "\"\<order\>\""}], 
                 "]"}], "\[Equal]", "0"}], ",", "\[IndentingNewLine]", 
               RowBox[{"(*", " ", 
                RowBox[{"root", " ", "or", " ", 
                 RowBox[{"root", "'"}], "s", " ", "child"}], " ", "*)"}], 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"PropertyValue", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"G", ",", "#1"}], "}"}], ",", 
                  "\"\<univ-address\>\""}], "]"}], "=", 
                RowBox[{"ToString", "[", 
                 RowBox[{"PropertyValue", "[", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"G", ",", "#1"}], "}"}], ",", "\"\<order\>\""}], 
                  "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
               RowBox[{"(*", " ", 
                RowBox[{"descendant", " ", "of", " ", "a", " ", 
                 RowBox[{"root", "'"}], "s", " ", "child"}], " ", "*)"}], 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"PropertyValue", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"G", ",", "#1"}], "}"}], ",", 
                  "\"\<univ-address\>\""}], "]"}], "=", 
                RowBox[{
                 RowBox[{"PropertyValue", "[", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"G", ",", "#2"}], "}"}], ",", 
                   "\"\<univ-address\>\""}], "]"}], "<>", "\"\<.\>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"PropertyValue", "[", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"G", ",", "#1"}], "}"}], ",", "\"\<order\>\""}], 
                   "]"}], "]"}]}]}]}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "set", " ", "address", " ", "as", " ", "vertex", " ", "label"}],
               " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"PropertyValue", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"G", ",", "#1"}], "}"}], ",", "VertexLabels"}], 
               "]"}], "=", 
              RowBox[{"PropertyValue", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"G", ",", "#1"}], "}"}], ",", 
                "\"\<univ-address\>\""}], "]"}]}]}], "\[IndentingNewLine]", 
            ")"}], "&"}], ")"}]}], "}"}]}], "]"}], ";", "\[IndentingNewLine]",
      "G"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"universalAddress", "[", 
  RowBox[{"cp6Example", ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "D", "E", "C", "I", "L", "F", "B", "K", "H", "J", "G", "A"}, {{{1, 2}, {
       1, 3}, {1, 4}, {2, 5}, {2, 6}, {4, 7}, {4, 8}, {4, 9}, {6, 10}, {6, 
       11}, {6, 12}}, Null}, {
      ImagePadding -> 10, 
       Properties -> {
        "J" -> {"order" -> 1, "univ-address" -> "1.2.1"}, 
         "L" -> {"order" -> 1, "univ-address" -> "1.1"}, 
         "G" -> {"order" -> 2, "univ-address" -> "1.2.2"}, 
         "E" -> {"order" -> 1, "univ-address" -> "1"}, 
         "F" -> {"order" -> 2, "univ-address" -> "1.2"}, 
         "A" -> {"order" -> 3, "univ-address" -> "1.2.3"}, 
         "B" -> {"order" -> 1, "univ-address" -> "3.1"}, 
         "H" -> {"order" -> 3, "univ-address" -> "3.3"}, 
         "D" -> {"order" -> 0, "univ-address" -> "0"}, 
         "I" -> {"order" -> 3, "univ-address" -> "3"}, 
         "K" -> {"order" -> 2, "univ-address" -> "3.2"}, 
         "C" -> {"order" -> 2, "univ-address" -> "2"}}, 
       VertexLabels -> {
        "H" -> "3.3", "L" -> "1.1", "C" -> "2", "I" -> "3", "K" -> "3.2", "D" -> 
         "0", "F" -> "1.2", "A" -> "1.2.3", "E" -> "1", "G" -> "1.2.2", "J" -> 
         "1.2.1", "B" -> "3.1"}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Arrowheads[0.028665006973120768`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$2", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$3", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$1", Automatic, Center], 
          DynamicLocation["VertexID$4", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$5", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$2", Automatic, Center], 
          DynamicLocation["VertexID$6", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$7", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$8", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$4", Automatic, Center], 
          DynamicLocation["VertexID$9", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$10", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$11", Automatic, Center]}]], 
       ArrowBox[
        LineBox[{
          DynamicLocation["VertexID$6", Automatic, Center], 
          DynamicLocation["VertexID$12", Automatic, Center]}]]}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.34263754799435, 2.3016643679903144`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"0\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.38361072799838575`, 1.5344429119935428`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.1508321839951572`, 1.5344429119935428`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.3016643679903144`, 1.5344429119935428`}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"3\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.7672214559967714}, 0.030731491585499593`], 
          "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"1.1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.7672214559967715, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"1.2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.534442911993543, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"3.1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.3016643679903144`, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"3.2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.068885823987086, 0.7672214559967714}, 
           0.030731491585499593`], "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"3.3\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 0.}, 0.030731491585499593`], "DynamicName", BoxID -> 
          "VertexID$10"], 
         InsetBox[
          FormBox["\"1.2.1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"], 
       TagBox[{
         TagBox[
          DiskBox[{0.7672214559967715, 0.}, 0.030731491585499593`], 
          "DynamicName", BoxID -> "VertexID$11"], 
         InsetBox[
          FormBox["\"1.2.2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$11", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$11"], 
       TagBox[{
         TagBox[
          DiskBox[{1.534442911993543, 0.}, 0.030731491585499593`], 
          "DynamicName", BoxID -> "VertexID$12"], 
         InsetBox[
          FormBox["\"1.2.3\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$12", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$12"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{15.800000000000004`, 
       358.96857199521406`}, {-138.24057199521457`, 121.93017909751302`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->10]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 1", "Subsection",
 CellTags->"SolutionsCE1"],

Cell["Display all trees with six vertices.", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To solve this problem, we make use of a recursive definition of tress. The \
empty graph is a tree, the graph with a single vertex is also a tree, and the \
graph with two vertices with an edge between them is a tree. Given any tree, \
we can form a new tree with one additional vertex by adding the new vertex as \
a leaf connected to any one of the original vertices. (The reader can verify \
that this indeed creates all trees with one more vertex.)"
}], "Text"],

Cell[TextData[{
 "We shall create a function, called ",
 StyleBox["extendTrees", "Input"],
 ", that accepts as input a list of trees on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " vertices and returns the resulting list of trees on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " vertices. For each of the original trees, we consider each vertex of the \
tree and create a new tree by adding a leaf to it."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"extendTrees", "[", "trees_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"newTrees", ",", "newV", ",", "T", ",", "v", ",", "newT"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newTrees", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newV", "=", 
      RowBox[{
       RowBox[{"VertexCount", "[", 
        RowBox[{"trees", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "+", "1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"newT", "=", 
           RowBox[{"VertexAdd", "[", 
            RowBox[{"T", ",", "newV"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"newT", "=", 
           RowBox[{"EdgeAdd", "[", 
            RowBox[{"newT", ",", 
             RowBox[{"v", "\[Rule]", "newV"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"newTrees", ",", "newT"}], "]"}]}], "\[IndentingNewLine]", 
         ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", 
           RowBox[{"VertexList", "[", "T", "]"}]}], "}"}]}], "]"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"T", ",", "trees"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "newTrees"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We can now use this procedure to determine all trees on four vertices. \
Finding all the trees of larger sizes is left to the reader. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allTrees", "=", 
  RowBox[{"{", 
   RowBox[{"Graph", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", "\[Rule]", "2"}], "}"}], ",", 
     RowBox[{"DirectedEdges", "\[Rule]", "True"}]}], "]"}], "}"}]}]], "Input",\

 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  GraphicsBox[
   NamespaceBox["NetworkGraphics",
    DynamicModuleBox[{Typeset`graph = HoldComplete[
      Graph[{1, 2}, {{{1, 2}}, Null}]], Typeset`boxes, Typeset`boxes$s2d = 
     GraphicsGroupBox[{{
        Arrowheads[0.041582132564841494`], 
        Directive[
         Opacity[0.7], 
         Hue[0.6, 0.7, 0.5]], 
        ArrowBox[
         LineBox[{
           DynamicLocation["VertexID$1", Automatic, Center], 
           DynamicLocation["VertexID$2", Automatic, Center]}]]}, {
        Directive[
         Hue[0.6, 0.2, 0.8], 
         EdgeForm[
          Directive[
           GrayLevel[0], 
           Opacity[0.7]]]], 
        TagBox[
         DiskBox[{1., 0.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
        TagBox[
         DiskBox[{0., 1.2246467991473532`*^-16}, 0.01273], "DynamicName", 
         BoxID -> "VertexID$2"]}}], $CellContext`flag}, 
     TagBox[
      DynamicBox[GraphComputation`NetworkGraphicsBox[
       3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
       CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
        SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
       ImageSizeCache->{{2.5800000000000005`, 176.42000000000002`}, {-5.5632, 
        0.7032232637060645}}],
      MouseAppearanceTag["NetworkGraphics"]],
     AllowKernelInitialization->False,
     UnsavedVariables:>{$CellContext`flag}]],
   DefaultBaseStyle->{
    "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
   FrameTicks->None], "}"}]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "3"}], ",", 
    RowBox[{"i", "\[LessEqual]", "4"}], ",", 
    RowBox[{"i", "++"}], ",", 
    RowBox[{"allTrees", "=", 
     RowBox[{"extendTrees", "[", "allTrees", "]"}]}]}], "]"}], ";"}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "We now display the trees on four vertices. We ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " ",
 StyleBox["rootedPlot", "Input"],
 " over the list in order to display the trees properly."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{
   RowBox[{"Partition", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"rootedPlot", "[", 
         RowBox[{"#", ",", 
          RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}], "&"}], ",", 
       "allTrees"}], "]"}], ",", "3"}], "]"}], ",", 
   RowBox[{"Frame", "\[Rule]", "All"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[
      TagBox[
       GraphicsComplexBox[{{0.7745966692414834, 0.7745966692414834}, {0., 
        0.}, {0.7745966692414834, 0.}, {1.5491933384829668`, 0.}}, {
         {RGBColor[0.5, 0., 0.], Arrowheads[{{0.03685673738335107, 0.7}}], 
          ArrowBox[{1, 2}], ArrowBox[{1, 3}], ArrowBox[{1, 4}]}, {InsetBox[
           FrameBox["1",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 1], InsetBox[
           FrameBox["2",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 2], InsetBox[
           FrameBox["3",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 3], InsetBox[
           FrameBox["4",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 4]}}],
       Annotation[#, 
        VertexCoordinateRules -> {{0.7745966692414834, 0.7745966692414834}, {
         0., 0.}, {0.7745966692414834, 0.}, {1.5491933384829668`, 0.}}]& ],
      AspectRatio->0.7071067811865476,
      FrameTicks->None,
      PlotRange->All,
      PlotRangePadding->Scaled[0.1]], 
     GraphicsBox[
      TagBox[
       GraphicsComplexBox[{{0.4629100498862757, 1.8516401995451028`}, {0., 
        0.9258200997725514}, {0.9258200997725514, 0.9258200997725514}, {0., 
        0.}}, {
         {RGBColor[0.5, 0., 0.], Arrowheads[{{0.03476723522995534, 0.7}}], 
          ArrowBox[{1, 2}], ArrowBox[{1, 3}], ArrowBox[{2, 4}]}, {InsetBox[
           FrameBox["1",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 1], InsetBox[
           FrameBox["2",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 2], InsetBox[
           FrameBox["3",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 3], InsetBox[
           FrameBox["4",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 4]}}],
       Annotation[#, 
        VertexCoordinateRules -> {{0.4629100498862757, 1.8516401995451028`}, {
         0., 0.9258200997725514}, {0.9258200997725514, 0.9258200997725514}, {
         0., 0.}}]& ],
      AspectRatio->1.4142135623730951`,
      FrameTicks->None,
      PlotRange->All,
      PlotRangePadding->Scaled[0.1]], 
     GraphicsBox[
      TagBox[
       GraphicsComplexBox[{{0.4629100498862757, 1.8516401995451028`}, {0., 
        0.9258200997725514}, {0.9258200997725514, 0.9258200997725514}, {
        0.9258200997725514, 0.}}, {
         {RGBColor[0.5, 0., 0.], Arrowheads[{{0.03476723522995534, 0.7}}], 
          ArrowBox[{1, 2}], ArrowBox[{1, 3}], ArrowBox[{3, 4}]}, {InsetBox[
           FrameBox["1",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 1], InsetBox[
           FrameBox["2",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 2], InsetBox[
           FrameBox["3",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 3], InsetBox[
           FrameBox["4",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 4]}}],
       Annotation[#, 
        VertexCoordinateRules -> {{0.4629100498862757, 1.8516401995451028`}, {
         0., 0.9258200997725514}, {0.9258200997725514, 0.9258200997725514}, {
         0.9258200997725514, 0.}}]& ],
      AspectRatio->1.4142135623730951`,
      FrameTicks->None,
      PlotRange->All,
      PlotRangePadding->Scaled[0.1]]},
    {
     GraphicsBox[
      TagBox[
       GraphicsComplexBox[{{0.4629100498862757, 1.8516401995451028`}, {0., 
        0.9258200997725514}, {0., 0.}, {0.9258200997725514, 
        0.9258200997725514}}, {
         {RGBColor[0.5, 0., 0.], Arrowheads[{{0.03476723522995534, 0.7}}], 
          ArrowBox[{1, 2}], ArrowBox[{1, 4}], ArrowBox[{2, 3}]}, {InsetBox[
           FrameBox["1",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 1], InsetBox[
           FrameBox["2",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 2], InsetBox[
           FrameBox["3",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 3], InsetBox[
           FrameBox["4",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 4]}}],
       Annotation[#, 
        VertexCoordinateRules -> {{0.4629100498862757, 1.8516401995451028`}, {
         0., 0.9258200997725514}, {0., 0.}, {0.9258200997725514, 
         0.9258200997725514}}]& ],
      AspectRatio->1.4142135623730951`,
      FrameTicks->None,
      PlotRange->All,
      PlotRangePadding->Scaled[0.1]], 
     GraphicsBox[
      TagBox[
       GraphicsComplexBox[{{0.4629100498862757, 1.8516401995451028`}, {
        0.4629100498862757, 0.9258200997725514}, {0., 0.}, {
        0.9258200997725514, 0.}}, {
         {RGBColor[0.5, 0., 0.], Arrowheads[{{0.03476723522995534, 0.7}}], 
          ArrowBox[{1, 2}], ArrowBox[{2, 3}], ArrowBox[{2, 4}]}, {InsetBox[
           FrameBox["1",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 1], InsetBox[
           FrameBox["2",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 2], InsetBox[
           FrameBox["3",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 3], InsetBox[
           FrameBox["4",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 4]}}],
       Annotation[#, 
        VertexCoordinateRules -> {{0.4629100498862757, 1.8516401995451028`}, {
         0.4629100498862757, 0.9258200997725514}, {0., 0.}, {
         0.9258200997725514, 0.}}]& ],
      AspectRatio->1.4142135623730951`,
      FrameTicks->None,
      PlotRange->All,
      PlotRangePadding->Scaled[0.1]], 
     GraphicsBox[
      TagBox[GraphicsComplexBox[{{0., 3.}, {0., 2.}, {0., 1.}, {0., 0.}}, {
         {RGBColor[0.5, 0., 0.], Arrowheads[{{0.028937842778793414`, 0.7}}], 
          ArrowBox[{1, 2}], ArrowBox[{2, 3}], ArrowBox[{3, 4}]}, {InsetBox[
           FrameBox["1",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 1], InsetBox[
           FrameBox["2",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 2], InsetBox[
           FrameBox["3",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 3], InsetBox[
           FrameBox["4",
            Background->RGBColor[1, 1, 0.8],
            FrameStyle->RGBColor[0.94, 0.85, 0.36],
            StripOnInput->False], 4]}}],
       Annotation[#, 
        VertexCoordinateRules -> {{0., 3.}, {0., 2.}, {0., 1.}, {0., 0.}}]& ],
      
      AspectRatio->Automatic,
      FrameTicks->None,
      PlotRange->All,
      PlotRangePadding->Scaled[0.1]]}
   },
   AutoDelete->False,
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 3", "Subsection",
 CellTags->"SolutionsCE3"],

Cell["\<\
Construct a Huffman code for the symbols with ASCII codes given the frequency \
of their occurrence in representative input. \
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " ASCII, which stands for American Standard Code for Information \
Interchange, includes 128 characters, including 33 non-printing characters. \
Most of the non-printing characters, with the exception of the space and the \
carriage return or newline character, however, are rarely used. We will focus \
on the standard characters of English and the newline character."
}], "Text"],

Cell[TextData[{
 "Since we've already created the function, ",
 StyleBox["huffmanCode", "Input"],
 ", which creates a Huffman code based on a list of character/weight pairs, \
the main work we need to do is determine the frequencies of characters in a \
sample input. We use the following input, which contains letters, \
punctuation, and newline characters. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"inputText", "=", 
  "\"\<The quick brown fox said,\n\\\"How do you do, my friend?\\\"\nThen he \
ran very quickly off into the sunset.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"The quick brown fox said,\\n\\\"How do you do, my friend?\\\
\"\\nThen he ran very quickly off into the sunset.\"\>"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can turn this string into a list by applying the ",
 StyleBox[ButtonBox["Characters",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Characters"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"textList", "=", 
  RowBox[{"Characters", "[", "inputText", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"T\"\>", ",", "\<\"h\"\>", ",", "\<\"e\"\>", ",", "\<\" \"\>", 
   ",", "\<\"q\"\>", ",", "\<\"u\"\>", ",", "\<\"i\"\>", ",", "\<\"c\"\>", 
   ",", "\<\"k\"\>", ",", "\<\" \"\>", ",", "\<\"b\"\>", ",", "\<\"r\"\>", 
   ",", "\<\"o\"\>", ",", "\<\"w\"\>", ",", "\<\"n\"\>", ",", "\<\" \"\>", 
   ",", "\<\"f\"\>", ",", "\<\"o\"\>", ",", "\<\"x\"\>", ",", "\<\" \"\>", 
   ",", "\<\"s\"\>", ",", "\<\"a\"\>", ",", "\<\"i\"\>", ",", "\<\"d\"\>", 
   ",", "\<\",\"\>", ",", "\<\"\\n\"\>", ",", "\<\"\\\"\"\>", 
   ",", "\<\"H\"\>", ",", "\<\"o\"\>", ",", "\<\"w\"\>", ",", "\<\" \"\>", 
   ",", "\<\"d\"\>", ",", "\<\"o\"\>", ",", "\<\" \"\>", ",", "\<\"y\"\>", 
   ",", "\<\"o\"\>", ",", "\<\"u\"\>", ",", "\<\" \"\>", ",", "\<\"d\"\>", 
   ",", "\<\"o\"\>", ",", "\<\",\"\>", ",", "\<\" \"\>", ",", "\<\"m\"\>", 
   ",", "\<\"y\"\>", ",", "\<\" \"\>", ",", "\<\"f\"\>", ",", "\<\"r\"\>", 
   ",", "\<\"i\"\>", ",", "\<\"e\"\>", ",", "\<\"n\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"?\"\>", ",", "\<\"\\\"\"\>", ",", "\<\"\\n\"\>", 
   ",", "\<\"T\"\>", ",", "\<\"h\"\>", ",", "\<\"e\"\>", ",", "\<\"n\"\>", 
   ",", "\<\" \"\>", ",", "\<\"h\"\>", ",", "\<\"e\"\>", ",", "\<\" \"\>", 
   ",", "\<\"r\"\>", ",", "\<\"a\"\>", ",", "\<\"n\"\>", ",", "\<\" \"\>", 
   ",", "\<\"v\"\>", ",", "\<\"e\"\>", ",", "\<\"r\"\>", ",", "\<\"y\"\>", 
   ",", "\<\" \"\>", ",", "\<\"q\"\>", ",", "\<\"u\"\>", ",", "\<\"i\"\>", 
   ",", "\<\"c\"\>", ",", "\<\"k\"\>", ",", "\<\"l\"\>", ",", "\<\"y\"\>", 
   ",", "\<\" \"\>", ",", "\<\"o\"\>", ",", "\<\"f\"\>", ",", "\<\"f\"\>", 
   ",", "\<\" \"\>", ",", "\<\"i\"\>", ",", "\<\"n\"\>", ",", "\<\"t\"\>", 
   ",", "\<\"o\"\>", ",", "\<\" \"\>", ",", "\<\"t\"\>", ",", "\<\"h\"\>", 
   ",", "\<\"e\"\>", ",", "\<\" \"\>", ",", "\<\"s\"\>", ",", "\<\"u\"\>", 
   ",", "\<\"n\"\>", ",", "\<\"s\"\>", ",", "\<\"e\"\>", ",", "\<\"t\"\>", 
   ",", "\<\".\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 ", we can have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " display the newline characters for us explicitly. Also, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " differentiates between a newline character and a space character."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"textList", "[", 
   RowBox[{"[", "26", "]"}], "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox["\"\<\\n\>\"",
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"textList", "[", 
   RowBox[{"[", "4", "]"}], "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox["\"\< \>\"",
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"textList", "[", 
   RowBox[{"[", "26", "]"}], "]"}], "\[Equal]", 
  RowBox[{"textList", "[", 
   RowBox[{"[", "4", "]"}], "]"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We will calculate the frequencies of the characters in our text by applying \
the ",
 StyleBox[ButtonBox["Tally",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tally"], "FunctionLink"],
 " function. When applied to a list, ",
 StyleBox[ButtonBox["Tally",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tally"], "FunctionLink"],
 " returns a list of pairs consisting of the unique elements of the original \
list and the number of times each appears."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tally", "[", 
  RowBox[{"Characters", "[", "inputText", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"T\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"h\"\>", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"e\"\>", ",", "7"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\" \"\>", ",", "17"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"q\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"u\"\>", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"i\"\>", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"k\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"r\"\>", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"o\"\>", ",", "8"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"w\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"n\"\>", ",", "6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"f\"\>", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"x\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"s\"\>", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"d\"\>", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\",\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"\\n\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"\\\"\"\>", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"H\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"y\"\>", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"m\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"?\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"v\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"l\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"t\"\>", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\".\"\>", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The Huffman function expects the input to consist of the characters and \
their relative frequency. So we need to divide the counts from the ",
 StyleBox[ButtonBox["Tally",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tally"], "FunctionLink"],
 " output by the number of characters."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"Characters", "[", "inputText", "]"}], "]"}]], "Input"],

Cell[BoxData["99"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"foxList", "=", 
  RowBox[{
   RowBox[{"Tally", "[", 
    RowBox[{"Characters", "[", "inputText", "]"}], "]"}], "/.", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"c_", ",", "n_"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"c", ",", 
      RowBox[{"n", "/", "99"}]}], "}"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"T\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"h\"\>", ",", 
     FractionBox["4", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"e\"\>", ",", 
     FractionBox["7", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\" \"\>", ",", 
     FractionBox["17", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"q\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"u\"\>", ",", 
     FractionBox["4", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"i\"\>", ",", 
     FractionBox["5", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"k\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"r\"\>", ",", 
     FractionBox["4", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"o\"\>", ",", 
     FractionBox["8", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"w\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"n\"\>", ",", 
     FractionBox["2", "33"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"f\"\>", ",", 
     FractionBox["4", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"x\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"s\"\>", ",", 
     FractionBox["1", "33"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"d\"\>", ",", 
     FractionBox["4", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\",\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"\\n\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"\\\"\"\>", ",", 
     FractionBox["2", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"H\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"y\"\>", ",", 
     FractionBox["4", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"m\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"?\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"v\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"l\"\>", ",", 
     FractionBox["1", "99"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"t\"\>", ",", 
     FractionBox["1", "33"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\".\"\>", ",", 
     FractionBox["1", "99"]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Finally, we're able to apply the ",
 StyleBox["huffmanCode", "Input"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"foxCode", "=", 
  RowBox[{"huffmanCode", "[", 
   RowBox[{"foxList", ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"ImagePadding", "\[Rule]", "5"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "I29", "I28", "I26", "I21", "I14", "I5", "I2", "x", "b", "I4", "m", "H",
        "I7", "\n", "\"", "I16", "u", "h", "I23", "I13", "I8", "q", "T", 
       "I10", "k", "c", "o", "I25", "I22", "I15", "y", "I6", "I1", "l", ".", 
       "I3", "?", "v", "I17", "f", "d", "I20", "I12", "I9", "a", ",", "s", 
       "e", "I27", "I24", "I19", "n", "i", "I18", "I11", "t", "w", "r", 
       " "}, {{{7, 8}, {7, 9}, {10, 11}, {10, 12}, {6, 7}, {6, 10}, {13, 
       14}, {13, 15}, {5, 6}, {5, 13}, {16, 17}, {16, 18}, {4, 5}, {4, 16}, {
       21, 22}, {21, 23}, {24, 25}, {24, 26}, {20, 21}, {20, 24}, {19, 20}, {
       19, 27}, {3, 4}, {3, 19}, {33, 34}, {33, 35}, {36, 37}, {36, 38}, {32, 
       33}, {32, 36}, {30, 31}, {30, 32}, {39, 40}, {39, 41}, {29, 30}, {29, 
       39}, {44, 45}, {44, 46}, {43, 44}, {43, 47}, {42, 43}, {42, 48}, {28, 
       29}, {28, 42}, {2, 3}, {2, 28}, {51, 52}, {51, 53}, {55, 56}, {55, 
       57}, {54, 55}, {54, 58}, {50, 51}, {50, 54}, {49, 50}, {49, 59}, {1, 
       2}, {1, 49}}, Null}, {
      EdgeLabels -> {
        DirectedEdge["I1", "l"] -> 0, DirectedEdge["I14", "I7"] -> 1, 
         DirectedEdge["I4", "H"] -> 1, DirectedEdge["I18", "r"] -> 1, 
         DirectedEdge["I12", "I9"] -> 0, DirectedEdge["I6", "I3"] -> 1, 
         DirectedEdge["I15", "I6"] -> 1, DirectedEdge["I22", "I15"] -> 0, 
         DirectedEdge["I28", "I25"] -> 1, DirectedEdge["I3", "?"] -> 0, 
         DirectedEdge["I27", " "] -> 1, DirectedEdge["I6", "I1"] -> 0, 
         DirectedEdge["I8", "T"] -> 1, DirectedEdge["I26", "I23"] -> 1, 
         DirectedEdge["I26", "I21"] -> 0, DirectedEdge["I7", "\n"] -> 0, 
         DirectedEdge["I10", "k"] -> 0, DirectedEdge["I4", "m"] -> 0, 
         DirectedEdge["I2", "x"] -> 0, DirectedEdge["I14", "I5"] -> 0, 
         DirectedEdge["I29", "I27"] -> 1, DirectedEdge["I16", "h"] -> 1, 
         DirectedEdge["I17", "d"] -> 1, DirectedEdge["I20", "e"] -> 1, 
         DirectedEdge["I28", "I26"] -> 0, DirectedEdge["I9", "a"] -> 0, 
         DirectedEdge["I17", "f"] -> 0, DirectedEdge["I15", "y"] -> 0, 
         DirectedEdge["I23", "o"] -> 1, DirectedEdge["I25", "I20"] -> 1, 
         DirectedEdge["I19", "i"] -> 1, DirectedEdge["I24", "I18"] -> 1, 
         DirectedEdge["I5", "I2"] -> 0, DirectedEdge["I21", "I16"] -> 1, 
         DirectedEdge["I3", "v"] -> 1, DirectedEdge["I8", "q"] -> 0, 
         DirectedEdge["I23", "I13"] -> 0, DirectedEdge["I27", "I24"] -> 0, 
         DirectedEdge["I11", "w"] -> 1, DirectedEdge["I5", "I4"] -> 1, 
         DirectedEdge["I10", "c"] -> 1, DirectedEdge["I9", ","] -> 1, 
         DirectedEdge["I13", "I10"] -> 1, DirectedEdge["I12", "s"] -> 1, 
         DirectedEdge["I11", "t"] -> 0, DirectedEdge["I2", "b"] -> 1, 
         DirectedEdge["I22", "I17"] -> 1, DirectedEdge["I16", "u"] -> 0, 
         DirectedEdge["I21", "I14"] -> 0, DirectedEdge["I25", "I22"] -> 0, 
         DirectedEdge["I18", "I11"] -> 0, DirectedEdge["I24", "I19"] -> 0, 
         DirectedEdge["I13", "I8"] -> 0, DirectedEdge["I29", "I28"] -> 0, 
         DirectedEdge["I1", "."] -> 1, DirectedEdge["I20", "I12"] -> 0, 
         DirectedEdge["I19", "n"] -> 0, DirectedEdge["I7", "\""] -> 1}, 
       ImagePadding -> 5, 
       Properties -> {
        "I12" -> {"order" -> 1}, "r" -> {"order" -> 2}, "w" -> {"order" -> 2},
          "i" -> {"order" -> 2}, "I14" -> {"order" -> 1}, 
         "I11" -> {"order" -> 1}, "I29" -> {"order" -> 0}, 
         "m" -> {"order" -> 1}, "I16" -> {"order" -> 2}, 
         "I13" -> {"order" -> 1}, "I18" -> {"order" -> 2}, 
         "I24" -> {"order" -> 1}, "?" -> {"order" -> 1}, 
         "I26" -> {"order" -> 1}, "I22" -> {"order" -> 1}, 
         "u" -> {"order" -> 1}, "I28" -> {"order" -> 1}, 
         "\n" -> {"order" -> 1}, "n" -> {"order" -> 1}, " " -> {"order" -> 2},
          "h" -> {"order" -> 2}, "," -> {"order" -> 2}, 
         "GraphProperties" -> {"weight" -> 1}, "d" -> {"order" -> 2}, 
         "I3" -> {"order" -> 2}, "t" -> {"order" -> 1}, "c" -> {"order" -> 2},
          "I25" -> {"order" -> 2}, "I8" -> {"order" -> 1}, 
         "T" -> {"order" -> 2}, "I1" -> {"order" -> 1}, "f" -> {"order" -> 1},
          "I27" -> {"order" -> 2}, "I10" -> {"order" -> 2}, 
         "I4" -> {"order" -> 2}, "I9" -> {"order" -> 1}, 
         "I19" -> {"order" -> 1}, "k" -> {"order" -> 1}, 
         "b" -> {"order" -> 2}, "e" -> {"order" -> 2}, 
         "I15" -> {"order" -> 1}, "." -> {"order" -> 2}, 
         "y" -> {"order" -> 1}, "I6" -> {"order" -> 2}, 
         "I2" -> {"order" -> 1}, "s" -> {"order" -> 2}, "H" -> {"order" -> 2},
          "I7" -> {"order" -> 2}, "l" -> {"order" -> 1}, 
         "I17" -> {"order" -> 2}, "q" -> {"order" -> 1}, 
         "I21" -> {"order" -> 1}, "I5" -> {"order" -> 1}, 
         "I23" -> {"order" -> 2}, "\"" -> {"order" -> 2}, 
         "a" -> {"order" -> 1}, "I20" -> {"order" -> 2}, 
         "v" -> {"order" -> 2}, "x" -> {"order" -> 1}, "o" -> {"order" -> 2}},
        VertexLabels -> {"Name"}}]], Typeset`boxes, Typeset`boxes$s2d = 
    GraphicsGroupBox[{{
       Arrowheads[0.013430051770049637`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$2", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$49", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$2", Automatic, Center], 
            DynamicLocation["VertexID$3", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$3"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$3", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$2", Automatic, Center], 
            DynamicLocation["VertexID$28", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$4"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$4", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$3", Automatic, Center], 
            DynamicLocation["VertexID$4", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$5"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$5", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$3", Automatic, Center], 
            DynamicLocation["VertexID$19", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$6"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$6", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$4", Automatic, Center], 
            DynamicLocation["VertexID$5", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$7"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$7", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$4", Automatic, Center], 
            DynamicLocation["VertexID$16", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$8"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$8", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$5", Automatic, Center], 
            DynamicLocation["VertexID$6", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$9"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$9", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$5", Automatic, Center], 
            DynamicLocation["VertexID$13", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$10"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$10", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$6", Automatic, Center], 
            DynamicLocation["VertexID$7", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$11"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$11", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$6", Automatic, Center], 
            DynamicLocation["VertexID$10", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$12"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$12", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$7", Automatic, Center], 
            DynamicLocation["VertexID$8", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$13"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$13", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$7", Automatic, Center], 
            DynamicLocation["VertexID$9", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$14"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$14", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$10", Automatic, Center], 
            DynamicLocation["VertexID$11", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$15"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$15", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$10", Automatic, Center], 
            DynamicLocation["VertexID$12", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$16"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$16", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$13", Automatic, Center], 
            DynamicLocation["VertexID$14", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$17"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$17", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$13", Automatic, Center], 
            DynamicLocation["VertexID$15", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$18"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$18", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$16", Automatic, Center], 
            DynamicLocation["VertexID$17", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$19"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$19", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$16", Automatic, Center], 
            DynamicLocation["VertexID$18", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$20"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$20", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$19", Automatic, Center], 
            DynamicLocation["VertexID$20", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$21"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$21", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$19", Automatic, Center], 
            DynamicLocation["VertexID$27", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$22"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$22", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$20", Automatic, Center], 
            DynamicLocation["VertexID$21", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$23"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$23", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$20", Automatic, Center], 
            DynamicLocation["VertexID$24", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$24"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$24", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$21", Automatic, Center], 
            DynamicLocation["VertexID$22", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$25"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$25", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$21", Automatic, Center], 
            DynamicLocation["VertexID$23", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$26"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$26", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$24", Automatic, Center], 
            DynamicLocation["VertexID$25", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$27"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$27", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$24", Automatic, Center], 
            DynamicLocation["VertexID$26", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$28"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$28", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$28", Automatic, Center], 
            DynamicLocation["VertexID$29", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$29"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$29", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$28", Automatic, Center], 
            DynamicLocation["VertexID$42", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$30"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$30", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$29", Automatic, Center], 
            DynamicLocation["VertexID$30", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$31"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$31", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$29", Automatic, Center], 
            DynamicLocation["VertexID$39", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$32"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$32", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$30", Automatic, Center], 
            DynamicLocation["VertexID$31", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$33"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$33", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$30", Automatic, Center], 
            DynamicLocation["VertexID$32", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$34"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$34", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$32", Automatic, Center], 
            DynamicLocation["VertexID$33", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$35"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$35", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$32", Automatic, Center], 
            DynamicLocation["VertexID$36", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$36"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$36", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$33", Automatic, Center], 
            DynamicLocation["VertexID$34", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$37"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$37", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$33", Automatic, Center], 
            DynamicLocation["VertexID$35", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$38"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$38", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$36", Automatic, Center], 
            DynamicLocation["VertexID$37", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$39"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$39", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$36", Automatic, Center], 
            DynamicLocation["VertexID$38", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$40"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$40", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$39", Automatic, Center], 
            DynamicLocation["VertexID$40", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$41"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$41", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$39", Automatic, Center], 
            DynamicLocation["VertexID$41", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$42"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$42", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$42", Automatic, Center], 
            DynamicLocation["VertexID$43", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$43"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$43", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$42", Automatic, Center], 
            DynamicLocation["VertexID$48", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$44"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$44", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$43", Automatic, Center], 
            DynamicLocation["VertexID$44", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$45"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$45", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$43", Automatic, Center], 
            DynamicLocation["VertexID$47", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$46"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$46", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$44", Automatic, Center], 
            DynamicLocation["VertexID$45", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$47"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$47", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$44", Automatic, Center], 
            DynamicLocation["VertexID$46", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$48"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$48", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$49", Automatic, Center], 
            DynamicLocation["VertexID$50", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$49"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$49", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$49", Automatic, Center], 
            DynamicLocation["VertexID$59", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$50"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$50", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$50", Automatic, Center], 
            DynamicLocation["VertexID$51", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$51"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$51", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$50", Automatic, Center], 
            DynamicLocation["VertexID$54", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$52"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$52", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$51", Automatic, Center], 
            DynamicLocation["VertexID$52", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$53"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$53", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$51", Automatic, Center], 
            DynamicLocation["VertexID$53", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$54"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$54", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$54", Automatic, Center], 
            DynamicLocation["VertexID$55", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$55"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$55", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$54", Automatic, Center], 
            DynamicLocation["VertexID$58", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$56"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$56", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$55", Automatic, Center], 
            DynamicLocation["VertexID$56", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$57"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$57", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$55", Automatic, Center], 
            DynamicLocation["VertexID$57", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$58"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$58", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.9001357335040145`, 5.794243861067578}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$1"], 
         InsetBox[
          FormBox["\"I29\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.993529660626576, 7.318039813248356}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$2"], 
         InsetBox[
          FormBox["\"I28\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          
          DiskBox[{9.810439381393632, 7.993695238239089}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$3"], 
         InsetBox[
          FormBox["\"I26\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          
          DiskBox[{11.52462777660659, 7.691058593782079}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"I21\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          
          DiskBox[{13.123323153854393`, 7.294245086986095}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"I14\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          
          DiskBox[{14.579936331819617`, 7.263967373230148}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"I5\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          
          DiskBox[{15.522077720773565`, 7.6569046415790725`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"I2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          
          DiskBox[{16.01565574121019, 8.074782325605376}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$8"], 
         InsetBox[
          FormBox["\"x\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          
          DiskBox[{16.192369183238107`, 7.710242913829239}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$9"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          
          DiskBox[{15.512765828423625`, 6.8051688468896545`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["\"I4\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"], 
       TagBox[{
         TagBox[
          
          DiskBox[{15.982955876011754`, 6.346627169034889}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$11"], 
         InsetBox[
          FormBox["\"m\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$11", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$11"], 
       TagBox[{
         TagBox[
          
          DiskBox[{16.183365220731282`, 6.700375935734141}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$12"], 
         InsetBox[
          FormBox["\"H\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$12", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$12"], 
       TagBox[{
         TagBox[
          
          DiskBox[{13.592015344374666`, 6.6018435313202275`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$13"], 
         InsetBox[
          FormBox["\"I7\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$13", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$13"], 
       TagBox[{
         TagBox[
          
          DiskBox[{13.68728134886893, 6.0357335701298735`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$14"], 
         InsetBox[
          FormBox["\"\\n\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$14", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$14"], 
       TagBox[{
         TagBox[
          
          DiskBox[{14.064810474878879`, 6.2752927945341215`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$15"], 
         InsetBox[
          FormBox["\"\\\"\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$15", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$15"], 
       TagBox[{
         TagBox[
          
          DiskBox[{12.111348090800163`, 8.012180522383613}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$16"], 
         InsetBox[
          FormBox["\"I16\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$16", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$16"], 
       TagBox[{
         TagBox[
          
          DiskBox[{12.597023295877367`, 8.013982227464616}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$17"], 
         InsetBox[
          FormBox["\"u\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$17", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$17"], 
       TagBox[{
         TagBox[
          
          DiskBox[{12.396666707849644`, 8.450906714477421}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$18"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$18", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$18"], 
       TagBox[{
         TagBox[
          
          DiskBox[{10.204187806417238`, 9.336979533079226}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$19"], 
         InsetBox[
          FormBox["\"I23\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$19", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$19"], 
       TagBox[{
         TagBox[
          
          DiskBox[{10.60958906058619, 10.630030298988661`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$20"], 
         InsetBox[
          FormBox["\"I13\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$20", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$20"], 
       TagBox[{
         TagBox[
          
          DiskBox[{11.253899955222671`, 11.381788329726387`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$21"], 
         InsetBox[
          FormBox["\"I8\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$21", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$21"], 
       TagBox[{
         TagBox[
          
          DiskBox[{11.7940553068796, 11.721640503053187`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$22"], 
         InsetBox[
          FormBox["\"q\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$22", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$22"], 
       TagBox[{
         TagBox[
          
          DiskBox[{11.488214544408727`, 11.99866147656352}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$23"], 
         InsetBox[
          FormBox["\"T\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$23", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$23"], 
       TagBox[{
         TagBox[
          
          DiskBox[{10.48666148285137, 11.574952640361253`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$24"], 
         InsetBox[
          FormBox["\"I10\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$24", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$24"], 
       TagBox[{
         TagBox[
          
          DiskBox[{10.233177447572036`, 12.13151036667604}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$25"], 
         InsetBox[
          FormBox["\"k\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$25", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$25"], 
       TagBox[{
         TagBox[
          
          DiskBox[{10.611465355711871`, 12.208000000734103`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$26"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$26", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$26"], 
       TagBox[{
         TagBox[
          
          DiskBox[{10.167276363039733`, 9.744180630747659}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$27"], 
         InsetBox[
          FormBox["\"o\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$27", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$27"], 
       TagBox[{
         TagBox[
          
          DiskBox[{6.197183488619963, 7.868114589567758}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$28"], 
         InsetBox[
          FormBox["\"I25\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$28", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$28"], 
       TagBox[{
         TagBox[
          
          DiskBox[{4.530428108673743, 7.738466440730983}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$29"], 
         InsetBox[
          FormBox["\"I22\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$29", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$29"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.996263786206937, 7.8699251720011585`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$30"], 
         InsetBox[
          FormBox["\"I15\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$30", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$30"], 
       TagBox[{
         TagBox[
          
          DiskBox[{2.5021568056609906`, 7.942237343483865}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$31"], 
         InsetBox[
          FormBox["\"y\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$31", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$31"], 
       TagBox[{
         TagBox[
          
          DiskBox[{1.5889797456468608`, 7.965681773948111}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$32"], 
         InsetBox[
          FormBox["\"I6\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$32", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$32"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.6630745486413847, 7.600273500288654}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$33"], 
         InsetBox[
          FormBox["\"I1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$33", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$33"], 
       TagBox[{
         TagBox[
          DiskBox[{0., 7.562503673913161}, 0.08309329254766855], 
          "DynamicName", BoxID -> "VertexID$34"], 
         InsetBox[
          FormBox["\"l\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$34", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$34"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.17555874712871322`, 7.191950307790776}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$35"], 
         InsetBox[
          FormBox["\".\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$35", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$35"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.7182076767147549, 8.454108935869513}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$36"], 
         InsetBox[
          FormBox["\"I3\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$36", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$36"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.28584925645407466`, 8.921860982327491}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$37"], 
         InsetBox[
          FormBox["\"?\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$37", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$37"], 
       TagBox[{
         TagBox[
          
          DiskBox[{0.06760705094074382, 8.57479949378304}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$38"], 
         InsetBox[
          FormBox["\"v\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$38", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$38"], 
       TagBox[{
         TagBox[
          
          DiskBox[{4.0223647881441345`, 7.164826144590903}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$39"], 
         InsetBox[
          FormBox["\"I17\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$39", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$39"], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.8708681016490365`, 6.64003027573836}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$40"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$40", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$40"], 
       TagBox[{
         TagBox[
          
          DiskBox[{3.5313224649541697`, 6.9442421116639785`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$41"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$41", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$41"], 
       TagBox[{
         TagBox[
          
          DiskBox[{5.715018918063619, 8.958509792397496}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$42"], 
         InsetBox[
          FormBox["\"I20\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$42", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$42"], 
       TagBox[{
         TagBox[
          
          DiskBox[{5.2728024921352485`, 10.016523068905807`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$43"], 
         InsetBox[
          FormBox["\"I12\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$43", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$43"], 
       TagBox[{
         TagBox[
          
          DiskBox[{4.8328516266207995`, 10.906791347668994`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$44"], 
         InsetBox[
          FormBox["\"I9\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$44", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$44"], 
       TagBox[{
         TagBox[
          
          DiskBox[{4.770942427100889, 11.540434231304491`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$45"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$45", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$45"], 
       TagBox[{
         TagBox[
          
          DiskBox[{4.380002183545407, 11.356799042743802`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$46"], 
         InsetBox[
          FormBox["\",\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$46", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$46"], 
       TagBox[{
         TagBox[
          
          DiskBox[{5.255907626643264, 10.499808538918145`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$47"], 
         InsetBox[
          FormBox["\"s\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$47", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$47"], 
       TagBox[{
         TagBox[
          
          DiskBox[{5.408393250524561, 9.217108498305306}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$48"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$48", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$48"], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.806311851773953, 4.294984386406707}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$49"], 
         InsetBox[
          FormBox["\"I27\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$49", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$49"], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.714478244583489, 2.8443824680868923`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$50"], 
         InsetBox[
          FormBox["\"I24\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$50", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$50"], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.157735701601718, 2.1762761104663486`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$51"], 
         InsetBox[
          FormBox["\"I19\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$51", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$51"], 
       TagBox[{
         TagBox[
          
          DiskBox[{6.9565621422784005`, 1.6166065991601508`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$52"], 
         InsetBox[
          FormBox["\"n\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$52", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$52"], 
       TagBox[{
         TagBox[
          
          DiskBox[{6.639647625997225, 1.900775024348282}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$53"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$53", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$53"], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.979650129099947, 1.6628962880999802`}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$54"], 
         InsetBox[
          FormBox["\"I18\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$54", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$54"], 
       TagBox[{
         TagBox[
          
          DiskBox[{8.068022327524144, 0.6345104585061376}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$55"], 
         InsetBox[
          FormBox["\"I11\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$55", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$55"], 
       TagBox[{
         TagBox[
          
          DiskBox[{8.342619910109217, 0.04640316767418451}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$56"], 
         InsetBox[
          FormBox["\"t\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$56", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$56"], 
       TagBox[{
         TagBox[
          DiskBox[{7.910977771018135, 0.}, 0.08309329254766855], 
          "DynamicName", BoxID -> "VertexID$57"], 
         InsetBox[
          FormBox["\"w\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$57", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$57"], 
       TagBox[{
         TagBox[
          
          DiskBox[{8.301559366944478, 1.242147113365669}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$58"], 
         InsetBox[
          FormBox["\"r\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$58", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$58"], 
       TagBox[{
         TagBox[
          
          DiskBox[{7.762461331199756, 3.898020836002231}, 
           0.08309329254766855], "DynamicName", BoxID -> "VertexID$59"], 
         InsetBox[
          FormBox["\" \"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$59", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$59"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{11.000000000000004`, 
       358.02228357522927`}, {-142.48722235864716`, 125.88221038822587`}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->5]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drawBinaryTree", "[", "foxCode", "]"}]], "Input"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "I29", "I28", "I26", "I21", "I14", "I5", "I2", "x", "b", "I4", "m", "H",
        "I7", "\n", "\"", "I16", "u", "h", "I23", "I13", "I8", "q", "T", 
       "I10", "k", "c", "o", "I25", "I22", "I15", "y", "I6", "I1", "l", ".", 
       "I3", "?", "v", "I17", "f", "d", "I20", "I12", "I9", "a", ",", "s", 
       "e", "I27", "I24", "I19", "n", "i", "I18", "I11", "t", "w", "r", 
       " "}, {{{7, 8}, {7, 9}, {10, 11}, {10, 12}, {6, 7}, {6, 10}, {13, 
       14}, {13, 15}, {5, 6}, {5, 13}, {16, 17}, {16, 18}, {4, 5}, {4, 16}, {
       21, 22}, {21, 23}, {24, 25}, {24, 26}, {20, 21}, {20, 24}, {19, 20}, {
       19, 27}, {3, 4}, {3, 19}, {33, 34}, {33, 35}, {36, 37}, {36, 38}, {32, 
       33}, {32, 36}, {30, 31}, {30, 32}, {39, 40}, {39, 41}, {29, 30}, {29, 
       39}, {44, 45}, {44, 46}, {43, 44}, {43, 47}, {42, 43}, {42, 48}, {28, 
       29}, {28, 42}, {2, 3}, {2, 28}, {51, 52}, {51, 53}, {55, 56}, {55, 
       57}, {54, 55}, {54, 58}, {50, 51}, {50, 54}, {49, 50}, {49, 59}, {1, 
       2}, {1, 49}}, Null}, {
      EdgeLabels -> {
        DirectedEdge["I1", "l"] -> 0, DirectedEdge["I14", "I7"] -> 1, 
         DirectedEdge["I4", "H"] -> 1, DirectedEdge["I18", "r"] -> 1, 
         DirectedEdge["I12", "I9"] -> 0, DirectedEdge["I6", "I3"] -> 1, 
         DirectedEdge["I15", "I6"] -> 1, DirectedEdge["I22", "I15"] -> 0, 
         DirectedEdge["I3", "?"] -> 0, DirectedEdge["I28", "I25"] -> 1, 
         DirectedEdge["I27", " "] -> 1, DirectedEdge["I6", "I1"] -> 0, 
         DirectedEdge["I8", "T"] -> 1, DirectedEdge["I26", "I23"] -> 1, 
         DirectedEdge["I26", "I21"] -> 0, DirectedEdge["I7", "\n"] -> 0, 
         DirectedEdge["I4", "m"] -> 0, DirectedEdge["I10", "k"] -> 0, 
         DirectedEdge["I2", "x"] -> 0, DirectedEdge["I14", "I5"] -> 0, 
         DirectedEdge["I29", "I27"] -> 1, DirectedEdge["I16", "h"] -> 1, 
         DirectedEdge["I17", "d"] -> 1, DirectedEdge["I20", "e"] -> 1, 
         DirectedEdge["I28", "I26"] -> 0, DirectedEdge["I9", "a"] -> 0, 
         DirectedEdge["I17", "f"] -> 0, DirectedEdge["I15", "y"] -> 0, 
         DirectedEdge["I23", "o"] -> 1, DirectedEdge["I25", "I20"] -> 1, 
         DirectedEdge["I19", "i"] -> 1, DirectedEdge["I24", "I18"] -> 1, 
         DirectedEdge["I5", "I2"] -> 0, DirectedEdge["I21", "I16"] -> 1, 
         DirectedEdge["I8", "q"] -> 0, DirectedEdge["I3", "v"] -> 1, 
         DirectedEdge["I23", "I13"] -> 0, DirectedEdge["I27", "I24"] -> 0, 
         DirectedEdge["I11", "w"] -> 1, DirectedEdge["I5", "I4"] -> 1, 
         DirectedEdge["I10", "c"] -> 1, DirectedEdge["I9", ","] -> 1, 
         DirectedEdge["I13", "I10"] -> 1, DirectedEdge["I12", "s"] -> 1, 
         DirectedEdge["I2", "b"] -> 1, DirectedEdge["I11", "t"] -> 0, 
         DirectedEdge["I22", "I17"] -> 1, DirectedEdge["I16", "u"] -> 0, 
         DirectedEdge["I21", "I14"] -> 0, DirectedEdge["I25", "I22"] -> 0, 
         DirectedEdge["I18", "I11"] -> 0, DirectedEdge["I24", "I19"] -> 0, 
         DirectedEdge["I13", "I8"] -> 0, DirectedEdge["I29", "I28"] -> 0, 
         DirectedEdge["I1", "."] -> 1, DirectedEdge["I20", "I12"] -> 0, 
         DirectedEdge["I19", "n"] -> 0, DirectedEdge["I7", "\""] -> 1}, 
       ImagePadding -> 5, 
       Properties -> {
        "I12" -> {"order" -> 1}, "r" -> {"order" -> 2}, "w" -> {"order" -> 2},
          "i" -> {"order" -> 2}, "I14" -> {"order" -> 1}, 
         "I11" -> {"order" -> 1}, "I29" -> {"order" -> 0}, 
         "m" -> {"order" -> 1}, "I16" -> {"order" -> 2}, 
         "I13" -> {"order" -> 1}, "I18" -> {"order" -> 2}, 
         "I24" -> {"order" -> 1}, "?" -> {"order" -> 1}, 
         "I26" -> {"order" -> 1}, "I22" -> {"order" -> 1}, 
         "u" -> {"order" -> 1}, "I28" -> {"order" -> 1}, 
         "\n" -> {"order" -> 1}, "n" -> {"order" -> 1}, " " -> {"order" -> 2},
          "h" -> {"order" -> 2}, "," -> {"order" -> 2}, 
         "GraphProperties" -> {"weight" -> 1}, "d" -> {"order" -> 2}, 
         "I3" -> {"order" -> 2}, "t" -> {"order" -> 1}, "c" -> {"order" -> 2},
          "I25" -> {"order" -> 2}, "I8" -> {"order" -> 1}, 
         "T" -> {"order" -> 2}, "I1" -> {"order" -> 1}, "f" -> {"order" -> 1},
          "I27" -> {"order" -> 2}, "I10" -> {"order" -> 2}, 
         "I4" -> {"order" -> 2}, "I9" -> {"order" -> 1}, 
         "I19" -> {"order" -> 1}, "k" -> {"order" -> 1}, 
         "b" -> {"order" -> 2}, "e" -> {"order" -> 2}, 
         "I15" -> {"order" -> 1}, "." -> {"order" -> 2}, 
         "y" -> {"order" -> 1}, "I6" -> {"order" -> 2}, 
         "I2" -> {"order" -> 1}, "s" -> {"order" -> 2}, "H" -> {"order" -> 2},
          "I7" -> {"order" -> 2}, "l" -> {"order" -> 1}, 
         "I17" -> {"order" -> 2}, "q" -> {"order" -> 1}, 
         "I21" -> {"order" -> 1}, "I5" -> {"order" -> 1}, 
         "I23" -> {"order" -> 2}, "\"" -> {"order" -> 2}, 
         "a" -> {"order" -> 1}, "I20" -> {"order" -> 2}, 
         "v" -> {"order" -> 2}, "x" -> {"order" -> 1}, "o" -> {"order" -> 2}},
        VertexCoordinates -> {{
          Rational[1, 2], 1}, {
          Rational[1, 4], 
          Rational[6, 7]}, {
          Rational[1, 8], 
          Rational[5, 7]}, {
          Rational[1, 16], 
          Rational[4, 7]}, {
          Rational[1, 32], 
          Rational[3, 7]}, {
          Rational[1, 64], 
          Rational[2, 7]}, {
          Rational[1, 128], 
          Rational[1, 7]}, {
          Rational[1, 256], 0}, {
          Rational[3, 256], 0}, {
          Rational[3, 128], 
          Rational[1, 7]}, {
          Rational[5, 256], 0}, {
          Rational[7, 256], 0}, {
          Rational[3, 64], 
          Rational[2, 7]}, {
          Rational[5, 128], 
          Rational[1, 7]}, {
          Rational[7, 128], 
          Rational[1, 7]}, {
          Rational[3, 32], 
          Rational[3, 7]}, {
          Rational[5, 64], 
          Rational[2, 7]}, {
          Rational[7, 64], 
          Rational[2, 7]}, {
          Rational[3, 16], 
          Rational[4, 7]}, {
          Rational[5, 32], 
          Rational[3, 7]}, {
          Rational[9, 64], 
          Rational[2, 7]}, {
          Rational[17, 128], 
          Rational[1, 7]}, {
          Rational[19, 128], 
          Rational[1, 7]}, {
          Rational[11, 64], 
          Rational[2, 7]}, {
          Rational[21, 128], 
          Rational[1, 7]}, {
          Rational[23, 128], 
          Rational[1, 7]}, {
          Rational[7, 32], 
          Rational[3, 7]}, {
          Rational[3, 8], 
          Rational[5, 7]}, {
          Rational[5, 16], 
          Rational[4, 7]}, {
          Rational[9, 32], 
          Rational[3, 7]}, {
          Rational[17, 64], 
          Rational[2, 7]}, {
          Rational[19, 64], 
          Rational[2, 7]}, {
          Rational[37, 128], 
          Rational[1, 7]}, {
          Rational[73, 256], 0}, {
          Rational[75, 256], 0}, {
          Rational[39, 128], 
          Rational[1, 7]}, {
          Rational[77, 256], 0}, {
          Rational[79, 256], 0}, {
          Rational[11, 32], 
          Rational[3, 7]}, {
          Rational[21, 64], 
          Rational[2, 7]}, {
          Rational[23, 64], 
          Rational[2, 7]}, {
          Rational[7, 16], 
          Rational[4, 7]}, {
          Rational[13, 32], 
          Rational[3, 7]}, {
          Rational[25, 64], 
          Rational[2, 7]}, {
          Rational[49, 128], 
          Rational[1, 7]}, {
          Rational[51, 128], 
          Rational[1, 7]}, {
          Rational[27, 64], 
          Rational[2, 7]}, {
          Rational[15, 32], 
          Rational[3, 7]}, {
          Rational[3, 4], 
          Rational[6, 7]}, {
          Rational[5, 8], 
          Rational[5, 7]}, {
          Rational[9, 16], 
          Rational[4, 7]}, {
          Rational[17, 32], 
          Rational[3, 7]}, {
          Rational[19, 32], 
          Rational[3, 7]}, {
          Rational[11, 16], 
          Rational[4, 7]}, {
          Rational[21, 32], 
          Rational[3, 7]}, {
          Rational[41, 64], 
          Rational[2, 7]}, {
          Rational[43, 64], 
          Rational[2, 7]}, {
          Rational[23, 32], 
          Rational[3, 7]}, {
          Rational[7, 8], 
          Rational[5, 7]}}, VertexLabels -> {"Name"}}]], Typeset`boxes, 
    Typeset`boxes$s2d = GraphicsGroupBox[{{
       Arrowheads[0.041582132564841494`], 
       Directive[
        Opacity[0.7], 
        Hue[0.6, 0.7, 0.5]], {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$2", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$1"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$1", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$1", Automatic, Center], 
            DynamicLocation["VertexID$49", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$2"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$2", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$2", Automatic, Center], 
            DynamicLocation["VertexID$3", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$3"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$3", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$2", Automatic, Center], 
            DynamicLocation["VertexID$28", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$4"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$4", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$3", Automatic, Center], 
            DynamicLocation["VertexID$4", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$5"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$5", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$3", Automatic, Center], 
            DynamicLocation["VertexID$19", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$6"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$6", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$4", Automatic, Center], 
            DynamicLocation["VertexID$5", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$7"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$7", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$4", Automatic, Center], 
            DynamicLocation["VertexID$16", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$8"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$8", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$5", Automatic, Center], 
            DynamicLocation["VertexID$6", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$9"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$9", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$5", Automatic, Center], 
            DynamicLocation["VertexID$13", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$10"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$10", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$6", Automatic, Center], 
            DynamicLocation["VertexID$7", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$11"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$11", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$6", Automatic, Center], 
            DynamicLocation["VertexID$10", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$12"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$12", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$7", Automatic, Center], 
            DynamicLocation["VertexID$8", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$13"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$13", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$7", Automatic, Center], 
            DynamicLocation["VertexID$9", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$14"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$14", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$10", Automatic, Center], 
            DynamicLocation["VertexID$11", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$15"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$15", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$10", Automatic, Center], 
            DynamicLocation["VertexID$12", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$16"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$16", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$13", Automatic, Center], 
            DynamicLocation["VertexID$14", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$17"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$17", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$13", Automatic, Center], 
            DynamicLocation["VertexID$15", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$18"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$18", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$16", Automatic, Center], 
            DynamicLocation["VertexID$17", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$19"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$19", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$16", Automatic, Center], 
            DynamicLocation["VertexID$18", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$20"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$20", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$19", Automatic, Center], 
            DynamicLocation["VertexID$20", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$21"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$21", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$19", Automatic, Center], 
            DynamicLocation["VertexID$27", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$22"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$22", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$20", Automatic, Center], 
            DynamicLocation["VertexID$21", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$23"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$23", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$20", Automatic, Center], 
            DynamicLocation["VertexID$24", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$24"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$24", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$21", Automatic, Center], 
            DynamicLocation["VertexID$22", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$25"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$25", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$21", Automatic, Center], 
            DynamicLocation["VertexID$23", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$26"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$26", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$24", Automatic, Center], 
            DynamicLocation["VertexID$25", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$27"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$27", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$24", Automatic, Center], 
            DynamicLocation["VertexID$26", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$28"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$28", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$28", Automatic, Center], 
            DynamicLocation["VertexID$29", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$29"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$29", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$28", Automatic, Center], 
            DynamicLocation["VertexID$42", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$30"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$30", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$29", Automatic, Center], 
            DynamicLocation["VertexID$30", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$31"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$31", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$29", Automatic, Center], 
            DynamicLocation["VertexID$39", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$32"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$32", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$30", Automatic, Center], 
            DynamicLocation["VertexID$31", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$33"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$33", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$30", Automatic, Center], 
            DynamicLocation["VertexID$32", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$34"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$34", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$32", Automatic, Center], 
            DynamicLocation["VertexID$33", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$35"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$35", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$32", Automatic, Center], 
            DynamicLocation["VertexID$36", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$36"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$36", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$33", Automatic, Center], 
            DynamicLocation["VertexID$34", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$37"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$37", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$33", Automatic, Center], 
            DynamicLocation["VertexID$35", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$38"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$38", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$36", Automatic, Center], 
            DynamicLocation["VertexID$37", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$39"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$39", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$36", Automatic, Center], 
            DynamicLocation["VertexID$38", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$40"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$40", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$39", Automatic, Center], 
            DynamicLocation["VertexID$40", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$41"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$41", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$39", Automatic, Center], 
            DynamicLocation["VertexID$41", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$42"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$42", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$42", Automatic, Center], 
            DynamicLocation["VertexID$43", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$43"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$43", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$42", Automatic, Center], 
            DynamicLocation["VertexID$48", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$44"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$44", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$43", Automatic, Center], 
            DynamicLocation["VertexID$44", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$45"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$45", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$43", Automatic, Center], 
            DynamicLocation["VertexID$47", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$46"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$46", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$44", Automatic, Center], 
            DynamicLocation["VertexID$45", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$47"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$47", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$44", Automatic, Center], 
            DynamicLocation["VertexID$46", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$48"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$48", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$49", Automatic, Center], 
            DynamicLocation["VertexID$50", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$49"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$49", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$49", Automatic, Center], 
            DynamicLocation["VertexID$59", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$50"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$50", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$50", Automatic, Center], 
            DynamicLocation["VertexID$51", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$51"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$51", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$50", Automatic, Center], 
            DynamicLocation["VertexID$54", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$52"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$52", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$51", Automatic, Center], 
            DynamicLocation["VertexID$52", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$53"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$53", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$51", Automatic, Center], 
            DynamicLocation["VertexID$53", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$54"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$54", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$54", Automatic, Center], 
            DynamicLocation["VertexID$55", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$55"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$55", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$54", Automatic, Center], 
            DynamicLocation["VertexID$58", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$56"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$56", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$55", Automatic, Center], 
            DynamicLocation["VertexID$56", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$57"], 
        InsetBox[
         FormBox["0", TraditionalForm], 
         DynamicLocation["EdgeLabelID$57", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}, {
        TagBox[
         ArrowBox[
          LineBox[{
            DynamicLocation["VertexID$55", Automatic, Center], 
            DynamicLocation["VertexID$57", Automatic, Center]}]], 
         "DynamicName", BoxID -> "EdgeLabelID$58"], 
        InsetBox[
         FormBox["1", TraditionalForm], 
         DynamicLocation["EdgeLabelID$58", Automatic, 
          Scaled[0.5]], 
         ImageScaled[{0.5, 0.5}], BaseStyle -> "Graphics"]}}, {
       Directive[
        Hue[0.6, 0.2, 0.8], 
        EdgeForm[
         Directive[
          GrayLevel[0], 
          Opacity[0.7]]]], 
       TagBox[{
         TagBox[
          DiskBox[{0.5, 1.}, 0.01273], "DynamicName", BoxID -> "VertexID$1"], 
         
         InsetBox[
          FormBox["\"I29\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$1", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$1"], 
       TagBox[{
         TagBox[
          DiskBox[{0.25, 0.8571428571428571}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$2"], 
         InsetBox[
          FormBox["\"I28\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$2", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$2"], 
       TagBox[{
         TagBox[
          DiskBox[{0.125, 0.7142857142857143}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$3"], 
         InsetBox[
          FormBox["\"I26\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$3", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$3"], 
       TagBox[{
         TagBox[
          DiskBox[{0.0625, 0.5714285714285714}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$4"], 
         InsetBox[
          FormBox["\"I21\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$4", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$4"], 
       TagBox[{
         TagBox[
          DiskBox[{0.03125, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$5"], 
         InsetBox[
          FormBox["\"I14\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$5", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$5"], 
       TagBox[{
         TagBox[
          DiskBox[{0.015625, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$6"], 
         InsetBox[
          FormBox["\"I5\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$6", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$6"], 
       TagBox[{
         TagBox[
          DiskBox[{0.0078125, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$7"], 
         InsetBox[
          FormBox["\"I2\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$7", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$7"], 
       TagBox[{
         TagBox[
          DiskBox[{0.00390625, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$8"], 
         InsetBox[
          FormBox["\"x\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$8", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$8"], 
       TagBox[{
         TagBox[
          DiskBox[{0.01171875, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$9"], 
         InsetBox[
          FormBox["\"b\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$9", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$9"], 
       TagBox[{
         TagBox[
          DiskBox[{0.0234375, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$10"], 
         InsetBox[
          FormBox["\"I4\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$10", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$10"], 
       TagBox[{
         TagBox[
          DiskBox[{0.01953125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$11"], 
         InsetBox[
          FormBox["\"m\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$11", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$11"], 
       TagBox[{
         TagBox[
          DiskBox[{0.02734375, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$12"], 
         InsetBox[
          FormBox["\"H\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$12", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$12"], 
       TagBox[{
         TagBox[
          DiskBox[{0.046875, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$13"], 
         InsetBox[
          FormBox["\"I7\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$13", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$13"], 
       TagBox[{
         TagBox[
          DiskBox[{0.0390625, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$14"], 
         InsetBox[
          FormBox["\"\\n\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$14", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$14"], 
       TagBox[{
         TagBox[
          DiskBox[{0.0546875, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$15"], 
         InsetBox[
          FormBox["\"\\\"\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$15", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$15"], 
       TagBox[{
         TagBox[
          DiskBox[{0.09375, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$16"], 
         InsetBox[
          FormBox["\"I16\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$16", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$16"], 
       TagBox[{
         TagBox[
          DiskBox[{0.078125, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$17"], 
         InsetBox[
          FormBox["\"u\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$17", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$17"], 
       TagBox[{
         TagBox[
          DiskBox[{0.109375, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$18"], 
         InsetBox[
          FormBox["\"h\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$18", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$18"], 
       TagBox[{
         TagBox[
          DiskBox[{0.1875, 0.5714285714285714}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$19"], 
         InsetBox[
          FormBox["\"I23\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$19", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$19"], 
       TagBox[{
         TagBox[
          DiskBox[{0.15625, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$20"], 
         InsetBox[
          FormBox["\"I13\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$20", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$20"], 
       TagBox[{
         TagBox[
          DiskBox[{0.140625, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$21"], 
         InsetBox[
          FormBox["\"I8\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$21", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$21"], 
       TagBox[{
         TagBox[
          DiskBox[{0.1328125, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$22"], 
         InsetBox[
          FormBox["\"q\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$22", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$22"], 
       TagBox[{
         TagBox[
          DiskBox[{0.1484375, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$23"], 
         InsetBox[
          FormBox["\"T\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$23", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$23"], 
       TagBox[{
         TagBox[
          DiskBox[{0.171875, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$24"], 
         InsetBox[
          FormBox["\"I10\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$24", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$24"], 
       TagBox[{
         TagBox[
          DiskBox[{0.1640625, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$25"], 
         InsetBox[
          FormBox["\"k\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$25", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$25"], 
       TagBox[{
         TagBox[
          DiskBox[{0.1796875, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$26"], 
         InsetBox[
          FormBox["\"c\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$26", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$26"], 
       TagBox[{
         TagBox[
          DiskBox[{0.21875, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$27"], 
         InsetBox[
          FormBox["\"o\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$27", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$27"], 
       TagBox[{
         TagBox[
          DiskBox[{0.375, 0.7142857142857143}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$28"], 
         InsetBox[
          FormBox["\"I25\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$28", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$28"], 
       TagBox[{
         TagBox[
          DiskBox[{0.3125, 0.5714285714285714}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$29"], 
         InsetBox[
          FormBox["\"I22\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$29", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$29"], 
       TagBox[{
         TagBox[
          DiskBox[{0.28125, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$30"], 
         InsetBox[
          FormBox["\"I15\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$30", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$30"], 
       TagBox[{
         TagBox[
          DiskBox[{0.265625, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$31"], 
         InsetBox[
          FormBox["\"y\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$31", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$31"], 
       TagBox[{
         TagBox[
          DiskBox[{0.296875, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$32"], 
         InsetBox[
          FormBox["\"I6\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$32", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$32"], 
       TagBox[{
         TagBox[
          DiskBox[{0.2890625, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$33"], 
         InsetBox[
          FormBox["\"I1\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$33", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$33"], 
       TagBox[{
         TagBox[
          DiskBox[{0.28515625, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$34"], 
         InsetBox[
          FormBox["\"l\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$34", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$34"], 
       TagBox[{
         TagBox[
          DiskBox[{0.29296875, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$35"], 
         InsetBox[
          FormBox["\".\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$35", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$35"], 
       TagBox[{
         TagBox[
          DiskBox[{0.3046875, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$36"], 
         InsetBox[
          FormBox["\"I3\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$36", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$36"], 
       TagBox[{
         TagBox[
          DiskBox[{0.30078125, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$37"], 
         InsetBox[
          FormBox["\"?\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$37", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$37"], 
       TagBox[{
         TagBox[
          DiskBox[{0.30859375, 0.}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$38"], 
         InsetBox[
          FormBox["\"v\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$38", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$38"], 
       TagBox[{
         TagBox[
          DiskBox[{0.34375, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$39"], 
         InsetBox[
          FormBox["\"I17\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$39", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$39"], 
       TagBox[{
         TagBox[
          DiskBox[{0.328125, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$40"], 
         InsetBox[
          FormBox["\"f\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$40", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$40"], 
       TagBox[{
         TagBox[
          DiskBox[{0.359375, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$41"], 
         InsetBox[
          FormBox["\"d\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$41", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$41"], 
       TagBox[{
         TagBox[
          DiskBox[{0.4375, 0.5714285714285714}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$42"], 
         InsetBox[
          FormBox["\"I20\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$42", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$42"], 
       TagBox[{
         TagBox[
          DiskBox[{0.40625, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$43"], 
         InsetBox[
          FormBox["\"I12\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$43", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$43"], 
       TagBox[{
         TagBox[
          DiskBox[{0.390625, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$44"], 
         InsetBox[
          FormBox["\"I9\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$44", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$44"], 
       TagBox[{
         TagBox[
          DiskBox[{0.3828125, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$45"], 
         InsetBox[
          FormBox["\"a\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$45", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$45"], 
       TagBox[{
         TagBox[
          DiskBox[{0.3984375, 0.14285714285714285`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$46"], 
         InsetBox[
          FormBox["\",\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$46", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$46"], 
       TagBox[{
         TagBox[
          DiskBox[{0.421875, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$47"], 
         InsetBox[
          FormBox["\"s\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$47", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$47"], 
       TagBox[{
         TagBox[
          DiskBox[{0.46875, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$48"], 
         InsetBox[
          FormBox["\"e\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$48", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$48"], 
       TagBox[{
         TagBox[
          DiskBox[{0.75, 0.8571428571428571}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$49"], 
         InsetBox[
          FormBox["\"I27\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$49", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$49"], 
       TagBox[{
         TagBox[
          DiskBox[{0.625, 0.7142857142857143}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$50"], 
         InsetBox[
          FormBox["\"I24\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$50", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$50"], 
       TagBox[{
         TagBox[
          DiskBox[{0.5625, 0.5714285714285714}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$51"], 
         InsetBox[
          FormBox["\"I19\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$51", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$51"], 
       TagBox[{
         TagBox[
          DiskBox[{0.53125, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$52"], 
         InsetBox[
          FormBox["\"n\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$52", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$52"], 
       TagBox[{
         TagBox[
          DiskBox[{0.59375, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$53"], 
         InsetBox[
          FormBox["\"i\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$53", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$53"], 
       TagBox[{
         TagBox[
          DiskBox[{0.6875, 0.5714285714285714}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$54"], 
         InsetBox[
          FormBox["\"I18\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$54", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$54"], 
       TagBox[{
         TagBox[
          DiskBox[{0.65625, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$55"], 
         InsetBox[
          FormBox["\"I11\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$55", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$55"], 
       TagBox[{
         TagBox[
          DiskBox[{0.640625, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$56"], 
         InsetBox[
          FormBox["\"t\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$56", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$56"], 
       TagBox[{
         TagBox[
          DiskBox[{0.671875, 0.2857142857142857}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$57"], 
         InsetBox[
          FormBox["\"w\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$57", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$57"], 
       TagBox[{
         TagBox[
          DiskBox[{0.71875, 0.42857142857142855`}, 0.01273], "DynamicName", 
          BoxID -> "VertexID$58"], 
         InsetBox[
          FormBox["\"r\"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$58", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$58"], 
       TagBox[{
         TagBox[
          DiskBox[{0.875, 0.7142857142857143}, 0.01273], "DynamicName", BoxID -> 
          "VertexID$59"], 
         InsetBox[
          FormBox["\" \"", TraditionalForm], 
          Offset[{2, 2}, 
           DynamicLocation["VertexID$59", Automatic, {Right, Top}]], 
          ImageScaled[{0, 0}], BaseStyle -> "Graphics"]}, "DynamicName", 
        BoxID -> "VertexLabelID$59"]}}], $CellContext`flag}, 
    TagBox[
     DynamicBox[GraphComputation`NetworkGraphicsBox[
      3, Typeset`graph, Typeset`boxes, $CellContext`flag], {
      CachedValue :> Typeset`boxes, SingleEvaluation -> True, 
       SynchronousUpdating -> False, TrackedSymbols :> {$CellContext`flag}},
      ImageSizeCache->{{11.000000000000004`, 
       353.30977136547233`}, {-206.5897713654749, 191.0299869918562}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False,
    UnsavedVariables:>{$CellContext`flag}]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FrameTicks->None,
  ImagePadding->5]], "Output"]
}, Open  ]],

Cell["\<\
You will need to enlarge the image substantially in order to see the result \
in a readable format.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 8", "Subsection",
 CellTags->"SolutionsCE8"],

Cell[TextData[{
 "Draw the complete game tree for a game of checkers on a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "\[Times]", "4"}], TraditionalForm]]],
 " board."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We will provide a partial solution to this problem; the reader is left to \
complete the full solution. Specifically, we will create a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function called ",
 StyleBox["movePiece", "Input"],
 " that will determine all possible new checker arrangements given the \
current state of the board and the player whose turn it is. Once this \
function is created, the reader must determine how to represent these board \
positions as vertices and edges, how to determine the next level of the game \
tree, as well as the halting conditions."
}], "Text"],

Cell["\<\
Before writing this function, however, we must establish a representation of \
the board. Naturally, we will use a matrix whose size is the size of the \
board. Empty board spaces will contain 0. Board spaces in which a regular \
white or black piece is sitting will be represented by 1 or 2, respectively. \
Kings will be represented by negative values, -1 for a white king and -2 for \
a black king. The following represents an initial board before any moves have \
been made.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"checkersStart", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "2", ",", "0", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"checkersStart", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "2", "0", "2"},
     {"0", "0", "0", "0"},
     {"0", "0", "0", "0"},
     {"1", "0", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Given a matrix representing a board state and an integer representing which \
side's turn it is, the function ",
 StyleBox["movePiece", "Input"],
 " will list all of the possible results of the player's move. It operates as \
follows:"
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Initialize ",
 StyleBox["newBoards", "Input"],
 ", which will be the list of all possible boards that result from the \
current move, to the empty list."
}], "Item1Numbered"],

Cell[TextData[{
 "If ",
 StyleBox["side", "Input"],
 " is 1, then normal pieces move up the board from bottom to top and we set \
",
 StyleBox["direction", "Input"],
 " to -1, since the index of rows in a matrix decrease as we move up the \
board. If the ",
 StyleBox["side", "Input"],
 " is 2, then ",
 StyleBox["direction", "Input"],
 " is set to +1."
}], "Item1Numbered"],

Cell[TextData[{
 "Begin a pair of ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loops, with indices ",
 StyleBox["r", "Input"],
 " and ",
 StyleBox["c", "Input"],
 ". These ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loops allow us to consider each possible board location. In each position, \
we want to know if that location holds a piece belonging to the current \
player. If it is 1's turn, then that player's normal pieces are represented \
by a 1 in the position and the player's kings are represented by a -1 in the \
position. Likewise, 2's pieces are represent by 2 or by -2. Thus we can \
determine if a position holds a player's piece by comparing the absolute \
value of the matrix entry with the ",
 StyleBox["side", "Input"],
 ". If the square does not hold a piece belonging to the current player, we \
simply move on to the next location."
}], "Item1Numbered"],

Cell[TextData[{
 "Check to see if the piece is a king and set the variable ",
 StyleBox["isKing", "Input"],
 " to 1 if it is a king or 0 if not. We then begin a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop from 0 to the value of ",
 StyleBox["isKing", "Input"],
 ". If ",
 StyleBox["isKing", "Input"],
 " is 0, the loop executes only once. If ",
 StyleBox["isKing", "Input"],
 " is 1, then the loop will execute twice. The index of this loop, ",
 StyleBox["king", "Input"],
 ", is used to control ",
 StyleBox["rowDir", "Input"],
 ", the current direction being considered. ",
 StyleBox["rowDir", "Input"],
 " is either the same as ",
 StyleBox["direction", "Input"],
 " or, in the case of the second iteration for a king, the reverse \
direction."
}], "Item1Numbered"],

Cell[TextData[{
 "We now check to see if the possible moves keep the piece on the board. \
First, we make sure that ",
 StyleBox["r+rowDir", "Input"],
 ", that is, the row in which the piece would move to, is still between 1 and \
4 (the possible rows)."
}], "Item1Numbered"],

Cell[TextData[{
 "Assuming moving the piece won't take it off the top or bottom of the board, \
consider the left and right moves. We do this with a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop which sets the variable ",
 StyleBox["colDir", "Input"],
 " to -1 and then to +1. Again, we check to see that ",
 StyleBox["c+colDir", "Input"],
 ", the current column plus the proposed change to the column position, is \
still on the board."
}], "Item1Numbered"],

Cell[TextData[{
 "At this point we know that the board position ",
 StyleBox["(r+rowDir,c+colDir)", "Input"],
 " is actually a board position. There are now three possibilities: the \
position is empty, there is an enemy piece in the square, there is a friendly \
piece in the square."
}], "Item1Numbered"],

Cell[TextData[{
 "In the first case, the position is empty, we want to move the piece to that \
location. We make a copy of the ",
 StyleBox["board", "Input"],
 " matrix (so we don't modify the original board). Then we make the move and \
add the new board to the list. Note that we also check to see if the piece \
becomes a king by moving into this position."
}], "Item1Numbered"],

Cell[TextData[{
 "In the second case, there is an enemy in the square, then we test to see if \
it is possible to jump. That is, we must make sure that the landing location \
after the jump is both on the board and empty. If so, we make the jump, i.e., \
we copy the ",
 StyleBox["board", "Input"],
 " and make the necessary modifications. If not, then the move is not \
possible."
}], "Item1Numbered"],

Cell["\<\
In the third case, the move is not possible and we do nothing.\
\>", "Item1Numbered"]
}, Open  ]],

Cell["Here is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"movePiece", "[", 
   RowBox[{"board_List", ",", "side_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"newBoards", "=", 
       RowBox[{"{", "}"}]}], ",", "direction", ",", "r", ",", "c", ",", 
      "isKing", ",", "king", ",", "rowDir", ",", "colDir", ",", "newB"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"direction", "=", 
      RowBox[{"Switch", "[", 
       RowBox[{"side", ",", "1", ",", 
        RowBox[{"-", "1"}], ",", "2", ",", "1"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"r", "=", "1"}], ",", 
       RowBox[{"r", "\[LessEqual]", "4"}], ",", 
       RowBox[{"r", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"c", "=", "1"}], ",", 
         RowBox[{"c", "\[LessEqual]", "4"}], ",", 
         RowBox[{"c", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Abs", "[", 
             RowBox[{"board", "[", 
              RowBox[{"[", 
               RowBox[{"r", ",", "c"}], "]"}], "]"}], "]"}], "\[Equal]", 
            "side"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"board", "[", 
                RowBox[{"[", 
                 RowBox[{"r", ",", "c"}], "]"}], "]"}], "<", "0"}], ",", 
              RowBox[{"isKing", "=", "1"}], ",", 
              RowBox[{"isKing", "=", "0"}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"For", "[", 
             RowBox[{
              RowBox[{"king", "=", "0"}], ",", 
              RowBox[{"king", "\[LessEqual]", "isKing"}], ",", 
              RowBox[{"king", "++"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"rowDir", "=", 
                RowBox[{"Switch", "[", 
                 RowBox[{"king", ",", "0", ",", "direction", ",", "1", ",", 
                  RowBox[{"-", "direction"}]}], "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"r", "+", "rowDir"}], "\[GreaterEqual]", "1"}], "&&", 
                  RowBox[{
                   RowBox[{"r", "+", "rowDir"}], "\[LessEqual]", "4"}]}], ",",
                  "\[IndentingNewLine]", 
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"colDir", "=", 
                    RowBox[{"-", "1"}]}], ",", 
                   RowBox[{"colDir", "\[LessEqual]", "1"}], ",", 
                   RowBox[{"colDir", "=", 
                    RowBox[{"colDir", "+", "2"}]}], ",", 
                   "\[IndentingNewLine]", 
                   RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"c", "+", "colDir"}], "\[GreaterEqual]", "1"}], "&&", 
                    RowBox[{
                    RowBox[{"c", "+", "colDir"}], "\[LessEqual]", "4"}]}], 
                    ",", "\[IndentingNewLine]", 
                    RowBox[{"Which", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], ",", 
                    RowBox[{"c", "+", "colDir"}]}], "]"}], "]"}], "\[Equal]", 
                    "0"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"newB", "=", "board"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], "\[Equal]", "1"}], "&&", 
                    RowBox[{"side", "\[Equal]", "1"}]}], ")"}], "||", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], "\[Equal]", "4"}], "&&", 
                    RowBox[{"side", "\[Equal]", "2"}]}], ")"}]}], ")"}], "&&", 
                    RowBox[{
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}], ">", "0"}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], ",", 
                    RowBox[{"c", "+", "colDir"}]}], "]"}], "]"}], "=", 
                    RowBox[{
                    RowBox[{"-", "1"}], "*", 
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}]}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], ",", 
                    RowBox[{"c", "+", "colDir"}]}], "]"}], "]"}], "=", 
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}]}]}], 
                    "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
                    
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}], "=", "0"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"newBoards", ",", "newB"}], "]"}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"Abs", "[", 
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], ",", 
                    RowBox[{"c", "+", "colDir"}]}], "]"}], "]"}], "]"}], 
                    "\[NotEqual]", "side"}], ",", "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], "\[GreaterEqual]", "1"}], 
                    "&&", 
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], "\[LessEqual]", "4"}], "&&", 
                    RowBox[{
                    RowBox[{"c", "+", 
                    RowBox[{"2", "*", "colDir"}]}], "\[GreaterEqual]", "1"}], 
                    "&&", 
                    RowBox[{
                    RowBox[{"c", "+", 
                    RowBox[{"2", "*", "colDir"}]}], "\[LessEqual]", "4"}]}], 
                    ",", "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], ",", 
                    RowBox[{"c", "+", 
                    RowBox[{"2", "*", "colDir"}]}]}], "]"}], "]"}], 
                    "\[Equal]", "0"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"newB", "=", "board"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], "\[Equal]", "1"}], "&&", 
                    RowBox[{"side", "\[Equal]", "1"}]}], ")"}], "||", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], "\[Equal]", "4"}], "&&", 
                    RowBox[{"side", "\[Equal]", "2"}]}], ")"}]}], ")"}], "&&", 
                    RowBox[{
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}], ">", "0"}]}], ",", 
                    
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], ",", 
                    RowBox[{"c", "+", 
                    RowBox[{"2", "*", "colDir"}]}]}], "]"}], "]"}], "=", 
                    RowBox[{"-", 
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}]}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", 
                    RowBox[{"2", "*", "rowDir"}]}], ",", 
                    RowBox[{"c", "+", 
                    RowBox[{"2", "*", "colDir"}]}]}], "]"}], "]"}], "=", 
                    RowBox[{"board", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}]}]}], 
                    "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
                    
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{"r", ",", "c"}], "]"}], "]"}], "=", "0"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"newB", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"r", "+", "rowDir"}], ",", 
                    RowBox[{"c", "+", "colDir"}]}], "]"}], "]"}], "=", "0"}], 
                    ";", "\[IndentingNewLine]", 
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"newBoards", ",", "newB"}], "]"}]}]}], 
                    "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
                    "]"}]}], "\[IndentingNewLine]", "]"}]}], 
                    "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
                  "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
             "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
         "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "newBoards"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["We now demonstrate a few steps using the function.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"move1", "=", 
   RowBox[{"movePiece", "[", 
    RowBox[{"checkersStart", ",", "1"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "/@", "move1"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", "2"},
       {"0", "0", "0", "0"},
       {"0", "1", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", "2"},
       {"0", "0", "0", "0"},
       {"0", "1", "0", "0"},
       {"1", "0", "0", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", "2"},
       {"0", "0", "0", "0"},
       {"0", "0", "0", "1"},
       {"1", "0", "0", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"move2", "=", 
   RowBox[{"movePiece", "[", 
    RowBox[{
     RowBox[{"move1", "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "/@", "move2"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "0", "0", "2"},
       {"2", "0", "0", "0"},
       {"0", "1", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "0", "0", "2"},
       {"0", "0", "2", "0"},
       {"0", "1", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", "0"},
       {"0", "0", "2", "0"},
       {"0", "1", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"move3", "=", 
   RowBox[{"movePiece", "[", 
    RowBox[{
     RowBox[{"move2", "[", 
      RowBox[{"[", "3", "]"}], "]"}], ",", "1"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "/@", "move3"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", "0"},
       {"1", "0", "2", "0"},
       {"0", "0", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", 
        RowBox[{"-", "1"}]},
       {"0", "0", "0", "0"},
       {"0", "0", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "2", "0", "0"},
       {"0", "0", "2", "0"},
       {"0", "1", "0", "1"},
       {"0", "0", "0", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"move4", "=", 
   RowBox[{"movePiece", "[", 
    RowBox[{
     RowBox[{"move3", "[", 
      RowBox[{"[", "2", "]"}], "]"}], ",", "2"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "/@", "move4"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "0", "0", 
        RowBox[{"-", "1"}]},
       {"2", "0", "0", "0"},
       {"0", "0", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "0", "0", 
        RowBox[{"-", "1"}]},
       {"0", "0", "2", "0"},
       {"0", "0", "0", "0"},
       {"0", "0", "1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell[TextData[{
 "Write ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for finding the eccentricity of a vertex in an unrooted tree and \
for finding the center of an unrooted tree. (Eccentricity and center are \
defined in prelude to Exercise 39 of Section 11.1 of the text.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for constructing rooted Fibonacci trees. (See the prelude to \
Exercise 45 of Section 11.1 for a definition of a Fibonacci tree.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for listing the vertices of an ordered rooted tree in level order."
}], "Item1Exercise"],

Cell["\<\
Compare the performance of binary search trees to linear search as follows:\
\>", "Item1Exercise"],

Cell[CellGroupData[{

Cell[TextData[{
 "Write a function, ",
 StyleBox["linearSearch", "Input"],
 ", that takes two inputs, a list of integers and an integer to find, and \
checks each element of the list in order until the input is found, at which \
time it returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". If the desired integer is not found, it is added the end of the list."
}], "Item2Exercise"],

Cell[TextData[{
 "Use the function ",
 StyleBox["RandomSample[Range[n],n]", "Input"],
 " for a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " to create a list of the first ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " integers in random order, with an appropriately large ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Apply the ",
 StyleBox["makeBST", "Input"],
 " function to the list to create a binary search tree for the data."
}], "Item2Exercise"],

Cell[TextData[{
 "Randomly select some positive integers to search for. The ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " function could be useful here."
}], "Item2Exercise"],

Cell[TextData[{
 "Use both ",
 StyleBox["linearSearch", "Input"],
 " and ",
 StyleBox["binaryInsertion", "Input"],
 " to find the integers from part c in the list and tree, respectively. Time \
them using the typical ",
 StyleBox[ButtonBox["Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Timing"], "FunctionLink"],
 " function. Repeat this for 100 different initial lists of integers and \
compare the resulting times. Compare these data (representing average-case \
complexity) with the theoretical worst-case results of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " comparisons for ",
 StyleBox["linearSearch", "Input"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"log", "(", "n", ")"}], TraditionalForm]]],
 " for ",
 StyleBox["binaryInsertion", "Input"],
 "."
}], "Item2Exercise"]
}, Open  ]],

Cell[TextData[{
 "Construct a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for decoding a message which was encoded with a Huffman code. \
That is, given a Huffman coding tree produced by the HuffmanCode procedure \
and a message encoded by the ",
 StyleBox["encodeString", "Input"],
 " function, the algorithm should return the original string."
}], "Item1Exercise"],

Cell[TextData[{
 "Use the Shakespearean sonnets to estimate the frequency of characters used \
by Shakespeare. (See Section 7.3 of this manual to see how to read the data \
into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and make use of the functions given in the solution to Computations and \
Explorations 3 above to compute the frequencies of characters used in the \
poems.) Then create a Huffman code based on the sonnets and encode the \
ShakespeareData.txt with the Huffman code. Compare the storage space required \
by the Huffman encoded version of the file as opposed to the space that would \
be used to encode the file in ASCII format, assuming each ASCII character \
requires 7 bits."
}], "Item1Exercise"],

Cell[TextData[{
 "Construct an undirected weighted graph which has at least two different \
minimum spanning trees and for which the ",
 StyleBox["prim", "Input"],
 " and ",
 StyleBox["kruskal", "Input"],
 " algorithms will return different results."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function implementing the reverse-delete algorithm for constructing \
minimal spanning trees. (The reverse-delete algorithm is described in the \
prelude to Exercise 34 in Section 11.5.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Explore the relative complexity of ",
 StyleBox["prim", "Input"],
 ", ",
 StyleBox["kruskal", "Input"],
 ", and the reverse-delete function you created in the previous exercise. Use \
the ",
 StyleBox[ButtonBox["RandomGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomGraph"], "FunctionLink"],
 " function to experiment with their performance. The expression ",
 StyleBox["RandomGraph[{v,e}]", "Input"],
 " will produce a random weighted connected graph with ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " vertices and ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " edges. You will then need to add edge weights with a function you design \
yourself. For each algorithm, can you find properties that you can impose on \
the graphs that will ensure that the algorithm will outperform the others?"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for producing degree-constrained spanning trees, which are \
defined in the Supplementary Exercises for Chapter 11. Use this function on a \
set of randomly generated graphs to attempt to construct degree-constrained \
spanning trees in which each vertex has degree no larger than 3."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to analyze the game of Nim with different starting conditions via the \
technique of game trees. (See Example 6 in Section 11.2 for a description of \
the game of Nim.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to analyze the game of checkers on square boards of different sizes via \
the technique of game trees. (See the solution to Computations and \
Explorations 8 for the beginnings of a solution.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for finding a path through a maze using the technique of \
backtracking."
}], "Item1Exercise"],

Cell[TextData[{
 "Develop ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for solving Sudoku puzzles using the technique of backtracking."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to generate as many graceful trees as possible. (See the Supplementary \
Exercises of Chapter 11 for a definition of graceful.) Based on the examples \
you find, make conjectures about graceful trees."
}], "Item1Exercise"],

Cell[TextData[{
 "Alter the postfix expression evaluator, ",
 StyleBox["evalPostfix", "Input"],
 ", to handle prefix expressions."
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{47, Automatic}, {Automatic, 10}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section1d", "Section1e",
                "Section2a", "Section2b", "Section2c", "Section3a", 
               "Section3b", "Section3c", "Section3d", "Section4a", 
               "Section4b", "Section4c", "Section4d", "Section4e", 
               "Section5a", "Section5b", "SolutionsCE1", "SolutionsCE3", 
               "SolutionsCE8", "SolutionsCP6", "Section1", "Section2", 
               "Section3", "Section4", "Section5", "Solutions", 
               "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section1"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section2"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section3"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section4"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Section5"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Solutions"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Solutions", "Exercises"}}] NotebookLocate["Exercises"]; 
           FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section1d", "Section1e", "Section2", "Section2a", "Section2b",
                "Section2c", "Section3", "Section3a", "Section3b", 
               "Section3c", "Section3d", "Section4", "Section4a", "Section4b",
                "Section4c", "Section4d", "Section4e", "Section5", 
               "Section5a", "Section5b", "Solutions", "SolutionsCE1", 
               "SolutionsCE3", "SolutionsCE8", "SolutionsCP6"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1951, 59, 68, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[2381, 73, 60, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[33059, 829, 58, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[99805, 2752, 118, 3, 26, "Subsection",
   CellTags->"Section1c"]},
 "Section1d"->{
  Cell[106910, 3017, 66, 1, 26, "Subsection",
   CellTags->"Section1d"]},
 "Section1e"->{
  Cell[148858, 4146, 65, 1, 26, "Subsection",
   CellTags->"Section1e"]},
 "Section2"->{
  Cell[154808, 4343, 68, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[155847, 4367, 128, 5, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[198306, 5561, 62, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section2c"->{
  Cell[230211, 6441, 60, 1, 26, "Subsection",
   CellTags->"Section2c"]},
 "Section3"->{
  Cell[282478, 7842, 61, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[298030, 8228, 54, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[308840, 8524, 66, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[318608, 8863, 60, 1, 26, "Subsection",
   CellTags->"Section3c"]},
 "Section3d"->{
  Cell[352003, 9916, 73, 1, 26, "Subsection",
   CellTags->"Section3d"]},
 "Section4"->{
  Cell[358001, 10090, 61, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[410051, 11403, 64, 1, 32, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[433407, 12034, 66, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section4c"->{
  Cell[457509, 12672, 77, 1, 26, "Subsection",
   CellTags->"Section4c"]},
 "Section4d"->{
  Cell[497303, 13698, 79, 1, 26, "Subsection",
   CellTags->"Section4d"]},
 "Section4e"->{
  Cell[514545, 14182, 81, 1, 26, "Subsection",
   CellTags->"Section4e"]},
 "Section5"->{
  Cell[523604, 14480, 69, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[548328, 15133, 79, 1, 32, "Subsection",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[608862, 16570, 82, 1, 26, "Subsection",
   CellTags->"Section5b"]},
 "Solutions"->{
  Cell[632170, 17167, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCP6"->{
  Cell[632310, 17174, 68, 1, 32, "Subsection",
   CellTags->"SolutionsCP6"]},
 "SolutionsCE1"->{
  Cell[665409, 18016, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE1"]},
 "SolutionsCE3"->{
  Cell[679364, 18382, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE3"]},
 "SolutionsCE8"->{
  Cell[809291, 21600, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE8"]},
 "Exercises"->{
  Cell[837019, 22359, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 861384, 22909},
 {"Section1a", 861466, 22912},
 {"Section1b", 861552, 22915},
 {"Section1c", 861640, 22918},
 {"Section1d", 861730, 22921},
 {"Section1e", 861820, 22924},
 {"Section2", 861909, 22927},
 {"Section2a", 861995, 22930},
 {"Section2b", 862086, 22933},
 {"Section2c", 862176, 22936},
 {"Section3", 862265, 22939},
 {"Section3a", 862351, 22942},
 {"Section3b", 862441, 22945},
 {"Section3c", 862531, 22948},
 {"Section3d", 862621, 22951},
 {"Section4", 862710, 22954},
 {"Section4a", 862797, 22957},
 {"Section4b", 862888, 22960},
 {"Section4c", 862979, 22963},
 {"Section4d", 863070, 22966},
 {"Section4e", 863161, 22969},
 {"Section5", 863251, 22972},
 {"Section5a", 863338, 22975},
 {"Section5b", 863429, 22978},
 {"Solutions", 863520, 22981},
 {"SolutionsCP6", 863612, 22984},
 {"SolutionsCE1", 863709, 22987},
 {"SolutionsCE3", 863806, 22990},
 {"SolutionsCE8", 863903, 22993},
 {"Exercises", 863997, 22996}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 79, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 11}}],
Cell[CellGroupData[{
Cell[683, 27, 31, 0, 55, "Section"],
Cell[717, 29, 203, 4, 35, "Text"],
Cell[923, 35, 991, 19, 110, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1951, 59, 68, 1, 55, "Section",
 CellTags->"Section1"],
Cell[2022, 62, 334, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[2381, 73, 60, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[2444, 76, 331, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[2800, 88, 570, 13, 45, "Input",
 InitializationCell->True],
Cell[3373, 103, 3918, 92, 447, "Output"]
}, Open  ]],
Cell[7306, 198, 460, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[7791, 215, 71, 1, 29, "Input"],
Cell[7865, 218, 31, 0, 28, "Output"]
}, Open  ]],
Cell[7911, 221, 728, 16, 65, "Text"],
Cell[CellGroupData[{
Cell[8664, 241, 642, 15, 45, "Input"],
Cell[9309, 258, 3910, 92, 374, "Output"]
}, Open  ]],
Cell[13234, 353, 441, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[13700, 368, 642, 15, 45, "Input"],
Cell[14345, 385, 3920, 92, 374, "Output"]
}, Open  ]],
Cell[18280, 480, 1389, 40, 96, "Text"],
Cell[CellGroupData[{
Cell[19694, 524, 205, 4, 29, "Input"],
Cell[19902, 530, 13108, 293, 88, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[33059, 829, 58, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[33120, 832, 298, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[33443, 841, 633, 14, 45, "Input",
 InitializationCell->True],
Cell[34079, 857, 5299, 130, 268, "Output"]
}, Open  ]],
Cell[39393, 990, 447, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[39865, 1007, 73, 1, 29, "Input"],
Cell[39941, 1010, 31, 0, 28, "Output"]
}, Open  ]],
Cell[39987, 1013, 339, 10, 35, "Text"],
Cell[CellGroupData[{
Cell[40351, 1027, 538, 14, 45, "Input",
 InitializationCell->True],
Cell[40892, 1043, 5206, 128, 311, "Output"]
}, Open  ]],
Cell[46113, 1174, 244, 6, 35, "Text"],
Cell[46360, 1182, 1892, 48, 115, "Text"],
Cell[CellGroupData[{
Cell[48277, 1234, 163, 3, 29, "Input"],
Cell[48443, 1239, 2063, 48, 447, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50543, 1292, 59, 0, 18, "Subsubsection"],
Cell[50605, 1294, 488, 13, 36, "Text"],
Cell[51096, 1309, 154, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[51275, 1316, 535, 14, 45, "Input",
 InitializationCell->True],
Cell[51813, 1332, 4938, 119, 447, "Output"]
}, Open  ]],
Cell[56766, 1454, 317, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[57108, 1467, 71, 1, 29, "Input"],
Cell[57182, 1470, 32, 0, 28, "Output"]
}, Open  ]],
Cell[57229, 1473, 67, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[57321, 1477, 113, 2, 29, "Input"],
Cell[57437, 1481, 31, 0, 28, "Output"]
}, Open  ]],
Cell[57483, 1484, 477, 13, 36, "Text"],
Cell[57963, 1499, 387, 12, 45, "Input",
 InitializationCell->True],
Cell[58353, 1513, 351, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[58729, 1526, 74, 1, 28, "Input"],
Cell[58806, 1529, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58874, 1534, 116, 2, 29, "Input"],
Cell[58993, 1538, 32, 0, 28, "Output"]
}, Open  ]],
Cell[59040, 1541, 318, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[59383, 1552, 77, 1, 29, "Input"],
Cell[59463, 1555, 126, 3, 28, "Output"]
}, Open  ]],
Cell[59604, 1561, 439, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[60068, 1576, 139, 4, 29, "Input"],
Cell[60210, 1582, 73, 2, 28, "Output"]
}, Open  ]],
Cell[60298, 1587, 486, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[60809, 1604, 216, 7, 29, "Input"],
Cell[61028, 1613, 28, 0, 28, "Output"]
}, Open  ]],
Cell[61071, 1616, 683, 22, 37, "Text"],
Cell[CellGroupData[{
Cell[61779, 1642, 376, 11, 61, "Input"],
Cell[62158, 1655, 36, 0, 28, "Output"]
}, Open  ]],
Cell[62209, 1658, 59, 0, 20, "Text"],
Cell[62271, 1660, 431, 13, 29, "Input",
 InitializationCell->True],
Cell[62705, 1675, 241, 6, 21, "Text"],
Cell[62949, 1683, 301, 8, 29, "Input",
 InitializationCell->True],
Cell[63253, 1693, 102, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[63380, 1701, 143, 3, 29, "Input"],
Cell[63526, 1706, 2063, 48, 447, "Output"]
}, Open  ]],
Cell[65604, 1757, 396, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[66025, 1772, 38, 0, 28, "Input"],
Cell[66066, 1774, 5206, 128, 311, "Output"]
}, Open  ]],
Cell[71287, 1905, 433, 11, 36, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71757, 1921, 98, 3, 18, "Subsubsection"],
Cell[71858, 1926, 338, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[72221, 1938, 52, 1, 28, "Input"],
Cell[72276, 1941, 28, 0, 28, "Output"]
}, Open  ]],
Cell[72319, 1944, 131, 4, 29, "Input"],
Cell[CellGroupData[{
Cell[72475, 1952, 62, 1, 28, "Input"],
Cell[72540, 1955, 393, 9, 24, "Message"],
Cell[72936, 1966, 28, 0, 28, "Output"]
}, Open  ]],
Cell[72979, 1969, 554, 12, 66, "Text"],
Cell[73536, 1983, 537, 16, 53, "Text"],
Cell[74076, 2001, 1210, 38, 85, "Text"],
Cell[75289, 2041, 257, 7, 45, "Input"],
Cell[75549, 2050, 92, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[75666, 2056, 63, 1, 28, "Input"],
Cell[75732, 2059, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[75797, 2064, 30, 0, 28, "Input"],
Cell[75830, 2066, 28, 0, 28, "Output"]
}, Open  ]],
Cell[75873, 2069, 540, 12, 52, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76450, 2086, 91, 4, 18, "Subsubsection"],
Cell[76544, 2092, 373, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[76942, 2106, 222, 7, 29, "Input",
 InitializationCell->True],
Cell[77167, 2115, 534, 17, 195, "Output"]
}, Open  ]],
Cell[77716, 2135, 182, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[77923, 2145, 188, 6, 29, "Input"],
Cell[78114, 2153, 2147, 58, 136, "Output"]
}, Open  ]],
Cell[80276, 2214, 778, 21, 53, "Text"],
Cell[81057, 2237, 474, 14, 36, "Text"],
Cell[CellGroupData[{
Cell[81556, 2255, 225, 8, 29, "Input"],
Cell[81784, 2265, 29, 0, 28, "Output"]
}, Open  ]],
Cell[81828, 2268, 1903, 49, 101, "Text"],
Cell[CellGroupData[{
Cell[83756, 2321, 116, 2, 29, "Input"],
Cell[83875, 2325, 535, 14, 45, "Output"]
}, Open  ]],
Cell[84425, 2342, 297, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[84747, 2354, 165, 4, 29, "Input"],
Cell[84915, 2360, 179, 6, 28, "Output"]
}, Open  ]],
Cell[85109, 2369, 1111, 29, 70, "Text"],
Cell[86223, 2400, 227, 6, 21, "Text"],
Cell[86453, 2408, 1680, 45, 171, "Input",
 InitializationCell->True],
Cell[88136, 2455, 159, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[88320, 2463, 74, 1, 28, "Input"],
Cell[88397, 2466, 5607, 135, 447, "Output"]
}, Open  ]],
Cell[94019, 2604, 52, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[94096, 2608, 38, 0, 28, "Input"],
Cell[94137, 2610, 5607, 135, 447, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[99805, 2752, 118, 3, 26, "Subsection",
 CellTags->"Section1c"],
Cell[99926, 2757, 112, 3, 20, "Text"],
Cell[100041, 2762, 242, 4, 35, "Text"],
Cell[100286, 2768, 306, 9, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[100617, 2781, 119, 2, 29, "Input"],
Cell[100739, 2785, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[100807, 2790, 119, 2, 29, "Input"],
Cell[100929, 2794, 32, 0, 28, "Output"]
}, Open  ]],
Cell[100976, 2797, 1026, 27, 67, "Text"],
Cell[102005, 2826, 850, 26, 77, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[102880, 2856, 104, 2, 28, "Input"],
Cell[102987, 2860, 36, 0, 28, "Output"]
}, Open  ]],
Cell[103038, 2863, 104, 2, 28, "Input"],
Cell[103145, 2867, 171, 3, 35, "Text"],
Cell[103319, 2872, 469, 15, 45, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[103813, 2891, 106, 2, 28, "Input"],
Cell[103922, 2895, 107, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[104066, 2902, 106, 2, 28, "Input"],
Cell[104175, 2906, 45, 1, 28, "Output"]
}, Open  ]],
Cell[104235, 2910, 179, 5, 21, "Text"],
Cell[104417, 2917, 314, 9, 29, "Input",
 InitializationCell->True],
Cell[104734, 2928, 301, 9, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[105060, 2941, 109, 2, 28, "Input"],
Cell[105172, 2945, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105240, 2950, 99, 2, 28, "Input"],
Cell[105342, 2954, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105411, 2959, 99, 2, 28, "Input"],
Cell[105513, 2963, 31, 0, 28, "Output"]
}, Open  ]],
Cell[105559, 2966, 520, 17, 37, "Text"],
Cell[106082, 2985, 550, 16, 75, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[106657, 3005, 73, 1, 28, "Input"],
Cell[106733, 3008, 128, 3, 29, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[106910, 3017, 66, 1, 26, "Subsection",
 CellTags->"Section1d"],
Cell[106979, 3020, 137, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[107141, 3027, 58, 0, 18, "Subsubsection"],
Cell[107202, 3029, 408, 10, 51, "Text"],
Cell[CellGroupData[{
Cell[107635, 3043, 37, 0, 28, "Input"],
Cell[107675, 3045, 5302, 130, 268, "Output"]
}, Open  ]],
Cell[112992, 3178, 1580, 45, 81, "Text"],
Cell[114575, 3225, 338, 6, 35, "Text"],
Cell[114916, 3233, 1735, 48, 170, "Input",
 InitializationCell->True],
Cell[116654, 3283, 607, 13, 52, "Text"],
Cell[CellGroupData[{
Cell[117286, 3300, 73, 1, 28, "Input"],
Cell[117362, 3303, 183, 3, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[117582, 3311, 274, 8, 28, "Input",
 InitializationCell->True],
Cell[117859, 3321, 5514, 134, 268, "Output"]
}, Open  ]],
Cell[123388, 3458, 245, 5, 36, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[123670, 3468, 56, 0, 18, "Subsubsection"],
Cell[123729, 3470, 532, 9, 65, "Text"],
Cell[124264, 3481, 1522, 42, 154, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[125811, 3527, 81, 1, 28, "Input"],
Cell[125895, 3530, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[125964, 3535, 82, 1, 28, "Input"],
Cell[126049, 3538, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[126129, 3544, 53, 0, 18, "Subsubsection"],
Cell[126185, 3546, 568, 14, 52, "Text"],
Cell[CellGroupData[{
Cell[126778, 3564, 522, 13, 45, "Input",
 InitializationCell->True],
Cell[127303, 3579, 4707, 115, 311, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[132047, 3699, 75, 1, 29, "Input"],
Cell[132125, 3702, 146, 4, 29, "Output"]
}, Open  ]],
Cell[132286, 3709, 620, 13, 66, "Text"],
Cell[132909, 3724, 245, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[133179, 3734, 662, 17, 61, "Input",
 InitializationCell->True],
Cell[133844, 3753, 4681, 114, 374, "Output"]
}, Open  ]],
Cell[138540, 3870, 196, 4, 35, "Text"],
Cell[138739, 3876, 1137, 30, 85, "Text"],
Cell[139879, 3908, 271, 6, 36, "Text"],
Cell[140153, 3916, 2751, 74, 309, "Input",
 InitializationCell->True],
Cell[142907, 3992, 154, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[143086, 3999, 238, 5, 29, "Input"],
Cell[143327, 4006, 5470, 133, 311, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[148858, 4146, 65, 1, 26, "Subsection",
 CellTags->"Section1e"],
Cell[148926, 4149, 178, 4, 35, "Text"],
Cell[149107, 4155, 560, 9, 65, "Text"],
Cell[149670, 4166, 922, 25, 150, "Input",
 InitializationCell->True],
Cell[150595, 4193, 230, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[150850, 4207, 104, 2, 29, "Input"],
Cell[150957, 4211, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[151022, 4216, 104, 2, 28, "Input"],
Cell[151129, 4220, 28, 0, 28, "Output"]
}, Open  ]],
Cell[151172, 4223, 300, 5, 35, "Text"],
Cell[151475, 4230, 948, 27, 125, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[152448, 4261, 74, 1, 29, "Input"],
Cell[152525, 4264, 28, 0, 28, "Output"]
}, Open  ]],
Cell[152568, 4267, 765, 23, 52, "Text"],
Cell[153336, 4292, 1166, 31, 153, "Input",
 InitializationCell->True],
Cell[154505, 4325, 106, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[154636, 4333, 77, 1, 28, "Input"],
Cell[154716, 4336, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[154808, 4343, 68, 1, 35, "Section",
 CellTags->"Section2"],
Cell[154879, 4346, 518, 8, 65, "Text"],
Cell[155400, 4356, 422, 7, 50, "Text"],
Cell[CellGroupData[{
Cell[155847, 4367, 128, 5, 32, "Subsection",
 CellTags->"Section2a"],
Cell[155978, 4374, 358, 8, 50, "Text"],
Cell[CellGroupData[{
Cell[156361, 4386, 43, 0, 18, "Subsubsection"],
Cell[156407, 4388, 618, 11, 67, "Text"],
Cell[157028, 4401, 159, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[157212, 4408, 740, 17, 61, "Input",
 InitializationCell->True],
Cell[157955, 4427, 4676, 114, 374, "Output"]
}, Open  ]],
Cell[162646, 4544, 288, 8, 29, "Input",
 InitializationCell->True],
Cell[162937, 4554, 505, 9, 51, "Text"],
Cell[163445, 4565, 2113, 55, 199, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[165583, 4624, 78, 1, 29, "Input"],
Cell[165664, 4627, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[165744, 4633, 45, 0, 18, "Subsubsection"],
Cell[165792, 4635, 393, 7, 51, "Text"],
Cell[166188, 4644, 982, 31, 65, "Text"],
Cell[167173, 4677, 1848, 65, 125, "Text"],
Cell[169024, 4744, 1280, 34, 96, "Text"],
Cell[170307, 4780, 1329, 39, 106, "Text"],
Cell[171639, 4821, 3595, 94, 359, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[175259, 4919, 108, 2, 29, "Input",
 InitializationCell->True],
Cell[175370, 4923, 4910, 124, 447, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[180329, 5053, 45, 0, 18, "Subsubsection"],
Cell[180377, 5055, 234, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[180636, 5064, 108, 2, 29, "Input"],
Cell[180747, 5068, 36, 0, 28, "Output"]
}, Open  ]],
Cell[180798, 5071, 722, 14, 82, "Text"],
Cell[181523, 5087, 1358, 38, 153, "Input",
 InitializationCell->True],
Cell[182884, 5127, 1359, 38, 153, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[184268, 5169, 111, 2, 29, "Input"],
Cell[184382, 5173, 36, 0, 28, "Output"]
}, Open  ]],
Cell[184433, 5176, 112, 2, 29, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[184582, 5183, 46, 0, 18, "Subsubsection"],
Cell[184631, 5185, 413, 7, 50, "Text"],
Cell[185047, 5194, 341, 10, 37, "Text"],
Cell[185391, 5206, 743, 22, 93, "Input",
 InitializationCell->True],
Cell[186137, 5230, 868, 20, 68, "Text"],
Cell[187008, 5252, 1357, 38, 140, "Input",
 InitializationCell->True],
Cell[188368, 5292, 1359, 38, 141, "Input",
 InitializationCell->True],
Cell[189730, 5332, 440, 11, 51, "Text"],
Cell[190173, 5345, 1566, 43, 141, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[191764, 5392, 34, 0, 29, "Input"],
Cell[191801, 5394, 6444, 160, 447, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[198306, 5561, 62, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[198371, 5564, 266, 5, 35, "Text"],
Cell[198640, 5571, 255, 6, 36, "Text"],
Cell[198898, 5579, 522, 14, 37, "Text"],
Cell[199423, 5595, 1424, 41, 86, "Text"],
Cell[200850, 5638, 1249, 34, 85, "Text"],
Cell[202102, 5674, 711, 25, 37, "Text"],
Cell[202816, 5701, 61, 0, 20, "Text"],
Cell[202880, 5703, 3153, 80, 360, "Input",
 InitializationCell->True],
Cell[206036, 5785, 984, 29, 52, "Text"],
Cell[207023, 5816, 221, 7, 37, "Text"],
Cell[207247, 5825, 130, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[207402, 5833, 134, 3, 29, "Input",
 InitializationCell->True],
Cell[207539, 5838, 32, 0, 28, "Output"]
}, Open  ]],
Cell[207586, 5841, 357, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[207968, 5854, 61, 1, 29, "Input",
 InitializationCell->True],
Cell[208032, 5857, 7085, 176, 447, "Output"]
}, Open  ]],
Cell[215132, 6036, 108, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[215265, 6043, 135, 3, 29, "Input",
 InitializationCell->True],
Cell[215403, 6048, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[215471, 6053, 61, 1, 29, "Input",
 InitializationCell->True],
Cell[215535, 6056, 7085, 176, 447, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[222657, 6237, 70, 0, 18, "Subsubsection"],
Cell[222730, 6239, 268, 5, 36, "Text"],
Cell[223001, 6246, 868, 26, 92, "Input",
 InitializationCell->True],
Cell[223872, 6274, 69, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[223966, 6278, 456, 11, 61, "Input",
 InitializationCell->True],
Cell[224425, 6291, 5725, 143, 447, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[230211, 6441, 60, 1, 26, "Subsection",
 CellTags->"Section2c"],
Cell[230274, 6444, 377, 6, 50, "Text"],
Cell[CellGroupData[{
Cell[230676, 6454, 52, 0, 18, "Subsubsection"],
Cell[230731, 6456, 295, 5, 35, "Text"],
Cell[231029, 6463, 286, 6, 36, "Text"],
Cell[231318, 6471, 855, 25, 108, "Input",
 InitializationCell->True],
Cell[232176, 6498, 175, 3, 35, "Text"],
Cell[232354, 6503, 915, 27, 107, "Input",
 InitializationCell->True],
Cell[233272, 6532, 354, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[233651, 6544, 662, 18, 60, "Input",
 InitializationCell->True],
Cell[234316, 6564, 8493, 212, 67, "Output"]
}, Open  ]],
Cell[242824, 6779, 419, 7, 50, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[243280, 6791, 43, 0, 18, "Subsubsection"],
Cell[243326, 6793, 539, 13, 51, "Text"],
Cell[243868, 6808, 707, 20, 92, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[244600, 6832, 160, 4, 29, "Input",
 InitializationCell->True],
Cell[244763, 6838, 8478, 212, 67, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[253290, 7056, 44, 0, 18, "Subsubsection"],
Cell[253337, 7058, 294, 5, 35, "Text"],
Cell[253634, 7065, 736, 11, 80, "Text"],
Cell[254373, 7078, 5527, 153, 410, "Input",
 InitializationCell->True],
Cell[259903, 7233, 130, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[260058, 7241, 414, 10, 45, "Input",
 InitializationCell->True],
Cell[260475, 7253, 3650, 89, 374, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[264174, 7348, 55, 0, 18, "Subsubsection"],
Cell[264232, 7350, 927, 22, 84, "Text"],
Cell[265162, 7374, 1675, 45, 187, "Input",
 InitializationCell->True],
Cell[266840, 7421, 483, 12, 51, "Text"],
Cell[CellGroupData[{
Cell[267348, 7437, 778, 21, 76, "Input",
 InitializationCell->True],
Cell[268129, 7460, 9577, 236, 447, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[277755, 7702, 69, 0, 18, "Subsubsection"],
Cell[277827, 7704, 351, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[278203, 7715, 75, 1, 28, "Input"],
Cell[278281, 7718, 147, 3, 28, "Output"]
}, Open  ]],
Cell[278443, 7724, 760, 13, 81, "Text"],
Cell[279206, 7739, 1586, 41, 201, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[280817, 7784, 107, 2, 28, "Input"],
Cell[280927, 7788, 38, 0, 28, "Output"]
}, Open  ]],
Cell[280980, 7791, 95, 2, 20, "Text"],
Cell[281078, 7795, 1040, 26, 150, "Input",
 InitializationCell->True],
Cell[282121, 7823, 108, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[282254, 7830, 106, 2, 29, "Input"],
Cell[282363, 7834, 42, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[282478, 7842, 61, 1, 35, "Section",
 CellTags->"Section3"],
Cell[282542, 7845, 553, 10, 65, "Text"],
Cell[283098, 7857, 511, 12, 51, "Text"],
Cell[283612, 7871, 733, 18, 37, "Text"],
Cell[284348, 7891, 197, 4, 35, "Text"],
Cell[284548, 7897, 1200, 24, 61, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[285773, 7925, 70, 1, 29, "Input"],
Cell[285846, 7928, 356, 6, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[286239, 7939, 365, 9, 29, "Input",
 InitializationCell->True],
Cell[286607, 7950, 11386, 273, 339, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[298030, 8228, 54, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[298087, 8231, 299, 5, 35, "Text"],
Cell[298389, 8238, 601, 11, 66, "Text"],
Cell[298993, 8251, 564, 10, 66, "Text"],
Cell[299560, 8263, 1203, 31, 168, "Input",
 InitializationCell->True],
Cell[300766, 8296, 140, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[300931, 8305, 102, 2, 29, "Input"],
Cell[301036, 8309, 147, 3, 29, "Output"]
}, Open  ]],
Cell[301198, 8315, 534, 10, 51, "Text"],
Cell[301735, 8327, 1555, 44, 156, "Input",
 InitializationCell->True],
Cell[303293, 8373, 223, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[303541, 8384, 274, 6, 29, "Input",
 InitializationCell->True],
Cell[303818, 8392, 4822, 118, 374, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[308677, 8515, 80, 1, 29, "Input"],
Cell[308760, 8518, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[308840, 8524, 66, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[308909, 8527, 205, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[309139, 8535, 33, 0, 18, "Subsubsection"],
Cell[309175, 8537, 319, 5, 50, "Text"],
Cell[309497, 8544, 1823, 48, 200, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[311345, 8596, 68, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[311438, 8601, 35, 0, 21, "Print"],
Cell[311476, 8603, 35, 0, 21, "Print"],
Cell[311514, 8605, 35, 0, 21, "Print"],
Cell[311552, 8607, 35, 0, 22, "Print"],
Cell[311590, 8609, 35, 0, 21, "Print"],
Cell[311628, 8611, 35, 0, 21, "Print"],
Cell[311666, 8613, 35, 0, 21, "Print"],
Cell[311704, 8615, 35, 0, 22, "Print"],
Cell[311742, 8617, 35, 0, 21, "Print"],
Cell[311780, 8619, 35, 0, 21, "Print"],
Cell[311818, 8621, 35, 0, 21, "Print"],
Cell[311856, 8623, 35, 0, 22, "Print"],
Cell[311894, 8625, 35, 0, 21, "Print"],
Cell[311932, 8627, 35, 0, 21, "Print"],
Cell[311970, 8629, 35, 0, 21, "Print"],
Cell[312008, 8631, 35, 0, 21, "Print"]
}, Open  ]]
}, Open  ]],
Cell[312070, 8635, 158, 3, 20, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[312265, 8643, 34, 0, 18, "Subsubsection"],
Cell[312302, 8645, 282, 5, 35, "Text"],
Cell[312587, 8652, 1825, 48, 200, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[314437, 8704, 69, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[314531, 8709, 35, 0, 22, "Print"],
Cell[314569, 8711, 35, 0, 21, "Print"],
Cell[314607, 8713, 35, 0, 21, "Print"],
Cell[314645, 8715, 35, 0, 22, "Print"],
Cell[314683, 8717, 35, 0, 21, "Print"],
Cell[314721, 8719, 35, 0, 21, "Print"],
Cell[314759, 8721, 35, 0, 21, "Print"],
Cell[314797, 8723, 35, 0, 21, "Print"],
Cell[314835, 8725, 35, 0, 21, "Print"],
Cell[314873, 8727, 35, 0, 21, "Print"],
Cell[314911, 8729, 35, 0, 21, "Print"],
Cell[314949, 8731, 35, 0, 22, "Print"],
Cell[314987, 8733, 35, 0, 21, "Print"],
Cell[315025, 8735, 35, 0, 21, "Print"],
Cell[315063, 8737, 35, 0, 21, "Print"],
Cell[315101, 8739, 35, 0, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[315197, 8746, 32, 0, 18, "Subsubsection"],
Cell[315232, 8748, 228, 4, 35, "Text"],
Cell[315463, 8754, 2350, 62, 264, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[317838, 8820, 67, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[317930, 8825, 35, 0, 22, "Print"],
Cell[317968, 8827, 35, 0, 21, "Print"],
Cell[318006, 8829, 35, 0, 21, "Print"],
Cell[318044, 8831, 35, 0, 21, "Print"],
Cell[318082, 8833, 35, 0, 21, "Print"],
Cell[318120, 8835, 35, 0, 22, "Print"],
Cell[318158, 8837, 35, 0, 21, "Print"],
Cell[318196, 8839, 35, 0, 21, "Print"],
Cell[318234, 8841, 35, 0, 21, "Print"],
Cell[318272, 8843, 35, 0, 21, "Print"],
Cell[318310, 8845, 35, 0, 21, "Print"],
Cell[318348, 8847, 35, 0, 22, "Print"],
Cell[318386, 8849, 35, 0, 21, "Print"],
Cell[318424, 8851, 35, 0, 21, "Print"],
Cell[318462, 8853, 35, 0, 21, "Print"],
Cell[318500, 8855, 35, 0, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[318608, 8863, 60, 1, 26, "Subsection",
 CellTags->"Section3c"],
Cell[318671, 8866, 466, 9, 50, "Text"],
Cell[319140, 8877, 438, 9, 50, "Text"],
Cell[319581, 8888, 260, 4, 35, "Text"],
Cell[319844, 8894, 319, 5, 50, "Text"],
Cell[320166, 8901, 473, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[320664, 8917, 104, 3, 28, "Input"],
Cell[320771, 8922, 105, 3, 28, "Output"]
}, Open  ]],
Cell[320891, 8928, 107, 3, 20, "Text"],
Cell[321001, 8933, 771, 19, 65, "Text"],
Cell[321775, 8954, 142, 3, 20, "Text"],
Cell[321920, 8959, 157, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[322102, 8966, 69, 0, 18, "Subsubsection"],
Cell[322174, 8968, 151, 3, 20, "Text"],
Cell[322328, 8973, 96, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[322449, 8979, 57, 1, 28, "Input"],
Cell[322509, 8982, 34, 0, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322580, 8987, 57, 1, 28, "Input"],
Cell[322640, 8990, 33, 0, 29, "Output"]
}, Open  ]],
Cell[322688, 8993, 292, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[323005, 9007, 80, 2, 28, "Input"],
Cell[323088, 9011, 34, 0, 29, "Output"]
}, Open  ]],
Cell[323137, 9014, 850, 25, 53, "Text"],
Cell[CellGroupData[{
Cell[324012, 9043, 112, 3, 28, "Input"],
Cell[324127, 9048, 31, 0, 28, "Output"]
}, Open  ]],
Cell[324173, 9051, 560, 18, 37, "Text"],
Cell[CellGroupData[{
Cell[324758, 9073, 77, 2, 28, "Input"],
Cell[324838, 9077, 34, 0, 29, "Output"]
}, Open  ]],
Cell[324887, 9080, 245, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[325157, 9092, 156, 5, 28, "Input"],
Cell[325316, 9099, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[325396, 9105, 98, 2, 18, "Subsubsection"],
Cell[325497, 9109, 675, 24, 35, "Text"],
Cell[326175, 9135, 602, 16, 51, "Text"],
Cell[CellGroupData[{
Cell[326802, 9155, 146, 4, 29, "Input"],
Cell[326951, 9161, 43, 0, 28, "Output"]
}, Open  ]],
Cell[327009, 9164, 906, 18, 84, "Text"],
Cell[327918, 9184, 1079, 31, 125, "Input",
 InitializationCell->True],
Cell[329000, 9217, 139, 3, 20, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329176, 9225, 87, 1, 18, "Subsubsection",
 PageBreakWithin->False],
Cell[329266, 9228, 218, 5, 35, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[CellGroupData[{
Cell[329509, 9237, 244, 8, 29, "Input",
 InitializationCell->True],
Cell[329756, 9247, 180, 6, 28, "Output"]
}, Open  ]],
Cell[329951, 9256, 731, 27, 36, "Text"],
Cell[330685, 9285, 712, 20, 51, "Text"],
Cell[331400, 9307, 440, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[331865, 9323, 71, 1, 29, "Input"],
Cell[331939, 9326, 435, 15, 39, "Output"]
}, Open  ]],
Cell[332389, 9344, 642, 20, 37, "Text"],
Cell[CellGroupData[{
Cell[333056, 9368, 155, 4, 29, "Input"],
Cell[333214, 9374, 81, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[333332, 9381, 155, 4, 29, "Input"],
Cell[333490, 9387, 130, 4, 28, "Output"]
}, Open  ]],
Cell[333635, 9394, 373, 11, 36, "Text"],
Cell[CellGroupData[{
Cell[334033, 9409, 142, 4, 29, "Input"],
Cell[334178, 9415, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[334246, 9420, 111, 3, 29, "Input"],
Cell[334360, 9425, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[334440, 9431, 43, 0, 18, "Subsubsection"],
Cell[334486, 9433, 238, 5, 36, "Text"],
Cell[334727, 9440, 4929, 137, 379, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[339693, 9582, 42, 0, 18, "Subsubsection"],
Cell[339738, 9584, 222, 7, 36, "Text"],
Cell[339963, 9593, 802, 20, 69, "Text"],
Cell[340768, 9615, 265, 8, 36, "Text"],
Cell[341036, 9625, 394, 8, 36, "Text"],
Cell[341433, 9635, 37, 0, 20, "Text"],
Cell[341473, 9637, 2731, 74, 301, "Input",
 InitializationCell->True],
Cell[344207, 9713, 63, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[344295, 9717, 326, 10, 29, "Input",
 InitializationCell->True],
Cell[344624, 9729, 7318, 180, 447, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[352003, 9916, 73, 1, 26, "Subsection",
 CellTags->"Section3d"],
Cell[352079, 9919, 649, 11, 66, "Text"],
Cell[352731, 9932, 278, 5, 35, "Text"],
Cell[353012, 9939, 322, 5, 50, "Text"],
Cell[353337, 9946, 3602, 98, 244, "Input",
 InitializationCell->True],
Cell[356942, 10046, 200, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[357167, 10056, 280, 7, 29, "Input",
 InitializationCell->True],
Cell[357450, 10065, 213, 5, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[357700, 10075, 77, 1, 29, "Input"],
Cell[357780, 10078, 28, 0, 28, "Output"]
}, Open  ]],
Cell[357823, 10081, 129, 3, 20, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[358001, 10090, 61, 1, 35, "Section",
 CellTags->"Section4"],
Cell[358065, 10093, 1013, 24, 95, "Text"],
Cell[359081, 10119, 1890, 57, 84, "Text"],
Cell[360974, 10178, 979, 23, 66, "Text"],
Cell[CellGroupData[{
Cell[361978, 10205, 385, 10, 45, "Input"],
Cell[362366, 10217, 4406, 103, 374, "Output"]
}, Open  ]],
Cell[366787, 10323, 387, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[367199, 10337, 2047, 49, 93, "Input",
 InitializationCell->True],
Cell[369249, 10388, 7025, 158, 110, "Output"]
}, Open  ]],
Cell[376289, 10549, 374, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[376688, 10563, 177, 4, 29, "Input",
 InitializationCell->True],
Cell[376868, 10569, 241, 4, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[377146, 10578, 269, 6, 45, "Input"],
Cell[377418, 10586, 6756, 153, 447, "Output"]
}, Open  ]],
Cell[384189, 10742, 200, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[384414, 10752, 332, 7, 45, "Input"],
Cell[384749, 10761, 6768, 154, 447, "Output"]
}, Open  ]],
Cell[391532, 10918, 318, 8, 35, "Text"],
Cell[391853, 10928, 1060, 27, 68, "Text"],
Cell[CellGroupData[{
Cell[392938, 10959, 737, 20, 93, "Input",
 InitializationCell->True],
Cell[393678, 10981, 9190, 223, 110, "Output"]
}, Open  ]],
Cell[402883, 11207, 1379, 35, 83, "Text"],
Cell[404265, 11244, 664, 21, 37, "Text"],
Cell[CellGroupData[{
Cell[404954, 11269, 390, 11, 45, "Input",
 InitializationCell->True],
Cell[405347, 11282, 647, 11, 29, "Output"]
}, Open  ]],
Cell[406009, 11296, 64, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[406098, 11300, 513, 13, 61, "Input"],
Cell[406614, 11315, 1945, 40, 186, "Output"]
}, Open  ]],
Cell[408574, 11358, 120, 3, 20, "Text"],
Cell[408697, 11363, 666, 14, 51, "Text"],
Cell[409366, 11379, 410, 11, 36, "Text"],
Cell[409779, 11392, 247, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[410051, 11403, 64, 1, 32, "Subsection",
 CellTags->"Section4a"],
Cell[410118, 11406, 262, 5, 35, "Text"],
Cell[410383, 11413, 637, 13, 65, "Text"],
Cell[411023, 11428, 209, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[411257, 11437, 462, 14, 31, "Item1Numbered"],
Cell[411722, 11453, 70, 0, 16, "Item1Numbered"],
Cell[CellGroupData[{
Cell[411817, 11457, 228, 5, 29, "Item2Numbered"],
Cell[412048, 11464, 517, 11, 44, "Item2Numbered"],
Cell[412568, 11477, 378, 8, 43, "Item2Numbered"]
}, Open  ]],
Cell[412961, 11488, 613, 18, 44, "Item1Numbered"],
Cell[CellGroupData[{
Cell[413599, 11510, 674, 21, 44, "Item2Numbered"],
Cell[414276, 11533, 581, 16, 43, "Item2Numbered"]
}, Open  ]]
}, Open  ]],
Cell[414884, 11553, 43, 0, 20, "Text"],
Cell[414930, 11555, 2936, 77, 330, "Input",
 InitializationCell->True],
Cell[417869, 11634, 72, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[417966, 11638, 285, 6, 45, "Input",
 InitializationCell->True],
Cell[418254, 11646, 7120, 173, 447, "Output"]
}, Open  ]],
Cell[425389, 11822, 349, 10, 21, "Text"],
Cell[425741, 11834, 567, 18, 61, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[426333, 11856, 41, 0, 29, "Input"],
Cell[426377, 11858, 6981, 170, 118, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[433407, 12034, 66, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[433476, 12037, 325, 5, 50, "Text"],
Cell[433804, 12044, 217, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[434046, 12053, 285, 9, 17, "Item1Numbered"],
Cell[434334, 12064, 70, 0, 16, "Item1Numbered"],
Cell[CellGroupData[{
Cell[434429, 12068, 233, 5, 29, "Item2Numbered"],
Cell[434665, 12075, 333, 7, 30, "Item2Numbered"],
Cell[435001, 12084, 228, 5, 29, "Item2Numbered"]
}, Open  ]],
Cell[435244, 12092, 1044, 39, 59, "Item1Numbered"]
}, Open  ]],
Cell[436303, 12134, 290, 6, 36, "Text"],
Cell[436596, 12142, 44, 0, 20, "Text"],
Cell[436643, 12144, 2668, 71, 297, "Input",
 InitializationCell->True],
Cell[439314, 12217, 60, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[439399, 12221, 935, 26, 108, "Input",
 InitializationCell->True],
Cell[440337, 12249, 6979, 170, 110, "Output"]
}, Open  ]],
Cell[447331, 12422, 132, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[447488, 12429, 367, 10, 45, "Input"],
Cell[447858, 12441, 9266, 216, 241, "Output"]
}, Open  ]],
Cell[457139, 12660, 333, 7, 35, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[457509, 12672, 77, 1, 26, "Subsection",
 CellTags->"Section4c"],
Cell[457589, 12675, 530, 10, 65, "Text"],
Cell[458122, 12687, 453, 12, 35, "Text"],
Cell[CellGroupData[{
Cell[458600, 12703, 616, 16, 42, "Item1Numbered"],
Cell[459219, 12721, 1441, 45, 85, "Item1Numbered"],
Cell[460663, 12768, 333, 12, 17, "Item1Numbered"],
Cell[460999, 12782, 514, 14, 44, "Item1Numbered"],
Cell[461516, 12798, 753, 19, 56, "Item1Numbered"],
Cell[462272, 12819, 665, 15, 57, "Item1Numbered"]
}, Open  ]],
Cell[462952, 12837, 493, 13, 37, "Text"],
Cell[463448, 12852, 5343, 132, 535, "Input",
 InitializationCell->True],
Cell[468794, 12986, 109, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[468928, 12993, 1320, 33, 77, "Input",
 InitializationCell->True],
Cell[470251, 13028, 4233, 97, 215, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[474521, 13130, 165, 4, 29, "Input"],
Cell[474689, 13136, 4797, 111, 215, "Output"]
}, Open  ]],
Cell[479501, 13250, 111, 3, 20, "Text"],
Cell[479615, 13255, 136, 4, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[479776, 13263, 154, 4, 28, "Input"],
Cell[479933, 13269, 34, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[480004, 13274, 172, 5, 28, "Input"],
Cell[480179, 13281, 34, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[480250, 13286, 188, 5, 28, "Input"],
Cell[480441, 13293, 3916, 87, 358, "Output"]
}, Open  ]],
Cell[484372, 13383, 865, 18, 81, "Text"],
Cell[485240, 13403, 5208, 129, 505, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[490473, 13536, 167, 4, 29, "Input"],
Cell[490643, 13542, 2036, 44, 447, "Output"]
}, Open  ]],
Cell[492694, 13589, 16, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[492735, 13593, 174, 5, 28, "Input"],
Cell[492912, 13600, 4342, 92, 294, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[497303, 13698, 79, 1, 26, "Subsection",
 CellTags->"Section4d"],
Cell[497385, 13701, 787, 16, 80, "Text"],
Cell[498175, 13719, 1013, 29, 70, "Text"],
Cell[499191, 13750, 795, 14, 82, "Text"],
Cell[CellGroupData[{
Cell[500011, 13768, 174, 5, 29, "Input"],
Cell[500188, 13775, 711, 20, 104, "Output"]
}, Open  ]],
Cell[500914, 13798, 396, 8, 51, "Text"],
Cell[501313, 13808, 3409, 87, 298, "Input",
 InitializationCell->True],
Cell[504725, 13897, 254, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[505004, 13908, 176, 6, 28, "Input"],
Cell[505183, 13916, 1137, 25, 184, "Output"]
}, Open  ]],
Cell[506335, 13944, 564, 14, 68, "Text"],
Cell[506902, 13960, 1306, 35, 154, "Input",
 InitializationCell->True],
Cell[508211, 13997, 213, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[508449, 14006, 162, 5, 17, "Item1Numbered"],
Cell[508614, 14013, 645, 12, 70, "Item1Numbered"],
Cell[509262, 14027, 236, 8, 17, "Item1Numbered"],
Cell[509501, 14037, 331, 7, 31, "Item1Numbered"],
Cell[509835, 14046, 254, 6, 30, "Item1Numbered"]
}, Open  ]],
Cell[510104, 14055, 3041, 80, 296, "Input",
 InitializationCell->True],
Cell[513148, 14137, 269, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[513442, 14150, 60, 1, 28, "Input"],
Cell[513505, 14153, 991, 23, 152, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[514545, 14182, 81, 1, 26, "Subsection",
 CellTags->"Section4e"],
Cell[514629, 14185, 988, 36, 50, "Text"],
Cell[515620, 14223, 736, 21, 51, "Text"],
Cell[CellGroupData[{
Cell[516381, 14248, 186, 5, 28, "Input",
 InitializationCell->True],
Cell[516570, 14255, 124, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[516731, 14263, 64, 1, 28, "Input"],
Cell[516798, 14266, 29, 0, 28, "Output"]
}, Open  ]],
Cell[516842, 14269, 423, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[517290, 14283, 248, 6, 29, "Input",
 InitializationCell->True],
Cell[517541, 14291, 183, 3, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[517761, 14299, 155, 4, 28, "Input",
 InitializationCell->True],
Cell[517919, 14305, 93, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[518049, 14312, 99, 2, 28, "Input"],
Cell[518151, 14316, 128, 3, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[518316, 14324, 146, 4, 28, "Input"],
Cell[518465, 14330, 107, 2, 28, "Output"]
}, Open  ]],
Cell[518587, 14335, 154, 3, 20, "Text"],
Cell[518744, 14340, 4245, 108, 417, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[523014, 14452, 184, 5, 28, "Input"],
Cell[523201, 14459, 84, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[523322, 14466, 184, 5, 28, "Input"],
Cell[523509, 14473, 34, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[523604, 14480, 69, 1, 35, "Section",
 CellTags->"Section5"],
Cell[523676, 14483, 823, 21, 80, "Text"],
Cell[524502, 14506, 541, 14, 36, "Text"],
Cell[525046, 14522, 714, 14, 66, "Text"],
Cell[CellGroupData[{
Cell[525785, 14540, 3120, 77, 173, "Input",
 InitializationCell->True],
Cell[528908, 14619, 12416, 293, 374, "Output"]
}, Open  ]],
Cell[541339, 14915, 552, 16, 36, "Text"],
Cell[CellGroupData[{
Cell[541916, 14935, 123, 2, 29, "Input"],
Cell[542042, 14939, 30, 0, 28, "Output"]
}, Open  ]],
Cell[542087, 14942, 588, 13, 51, "Text"],
Cell[CellGroupData[{
Cell[542700, 14959, 105, 2, 29, "Input"],
Cell[542808, 14963, 166, 4, 28, "Output"]
}, Open  ]],
Cell[542989, 14970, 515, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[543529, 14986, 116, 3, 29, "Input"],
Cell[543648, 14991, 1193, 24, 168, "Output"]
}, Open  ]],
Cell[544856, 15018, 604, 16, 37, "Text"],
Cell[CellGroupData[{
Cell[545485, 15038, 334, 9, 45, "Input"],
Cell[545822, 15049, 2469, 79, 184, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[548328, 15133, 79, 1, 32, "Subsection",
 CellTags->"Section5a"],
Cell[548410, 15136, 204, 4, 35, "Text"],
Cell[548617, 15142, 1037, 25, 68, "Text"],
Cell[549657, 15169, 577, 17, 61, "Input",
 InitializationCell->True],
Cell[550237, 15188, 484, 16, 37, "Text"],
Cell[550724, 15206, 557, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[551306, 15225, 129, 3, 29, "Input",
 InitializationCell->True],
Cell[551438, 15230, 29513, 616, 212, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[580988, 15851, 196, 4, 29, "Input"],
Cell[581187, 15857, 31, 0, 28, "Output"]
}, Open  ]],
Cell[581233, 15860, 262, 5, 35, "Text"],
Cell[581498, 15867, 333, 8, 37, "Text"],
Cell[581834, 15877, 958, 17, 98, "Text"],
Cell[582795, 15896, 2723, 68, 283, "Input",
 InitializationCell->True],
Cell[585521, 15966, 111, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[585657, 15973, 206, 5, 30, "Item1Numbered"],
Cell[585866, 15980, 414, 9, 31, "Item1Numbered"],
Cell[586283, 15991, 281, 9, 16, "Item1Numbered"],
Cell[586567, 16002, 176, 3, 29, "Item1Numbered"]
}, Open  ]],
Cell[586758, 16008, 4644, 130, 394, "Input",
 InitializationCell->True],
Cell[591405, 16140, 105, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[591535, 16147, 194, 5, 29, "Input",
 InitializationCell->True],
Cell[591732, 16154, 8856, 210, 374, "Output"]
}, Open  ]],
Cell[600603, 16367, 403, 8, 36, "Text"],
Cell[601009, 16377, 2321, 60, 268, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[603355, 16441, 70, 1, 29, "Input"],
Cell[603428, 16444, 581, 10, 29, "Output"]
}, Open  ]],
Cell[604024, 16457, 277, 6, 36, "Text"],
Cell[604304, 16465, 995, 27, 93, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[605324, 16496, 140, 3, 29, "Input"],
Cell[605467, 16501, 3346, 63, 450, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[608862, 16570, 82, 1, 26, "Subsection",
 CellTags->"Section5b"],
Cell[608947, 16573, 376, 6, 50, "Text"],
Cell[609326, 16581, 902, 16, 81, "Text"],
Cell[610231, 16599, 1261, 34, 139, "Input",
 InitializationCell->True],
Cell[611495, 16635, 89, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[611609, 16641, 219, 6, 17, "Item1Numbered"],
Cell[611831, 16649, 165, 6, 17, "Item1Numbered"],
Cell[611999, 16657, 340, 9, 31, "Item1Numbered"],
Cell[612342, 16668, 406, 10, 44, "Item1Numbered"],
Cell[612751, 16680, 258, 9, 16, "Item1Numbered"]
}, Open  ]],
Cell[613024, 16692, 3743, 100, 376, "Input",
 InitializationCell->True],
Cell[616770, 16794, 216, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[617011, 16802, 200, 5, 29, "Input",
 InitializationCell->True],
Cell[617214, 16809, 8856, 210, 374, "Output"]
}, Open  ]],
Cell[626085, 17022, 261, 6, 36, "Text"],
Cell[626349, 17030, 2098, 53, 282, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[628472, 17087, 143, 3, 29, "Input"],
Cell[628618, 17092, 3346, 63, 450, "Output"]
}, Open  ]],
Cell[631979, 17158, 142, 3, 20, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[632170, 17167, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[632310, 17174, 68, 1, 32, "Subsection",
 CellTags->"SolutionsCP6"],
Cell[632381, 17177, 133, 3, 42, "Problem"],
Cell[632517, 17182, 648, 18, 50, "Text"],
Cell[633168, 17202, 372, 6, 50, "Text"],
Cell[633543, 17210, 367, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[633935, 17221, 739, 15, 45, "Input",
 InitializationCell->True],
Cell[634677, 17238, 669, 13, 28, "Output"]
}, Open  ]],
Cell[635361, 17254, 234, 4, 35, "Text"],
Cell[635598, 17260, 226, 7, 21, "Text"],
Cell[635827, 17269, 143, 4, 29, "Input",
 InitializationCell->True],
Cell[635973, 17275, 148, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[636146, 17282, 71, 1, 29, "Input"],
Cell[636220, 17285, 31, 0, 28, "Output"]
}, Open  ]],
Cell[636266, 17288, 266, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[636557, 17297, 261, 8, 29, "Input",
 InitializationCell->True],
Cell[636821, 17307, 28, 0, 28, "Output"]
}, Open  ]],
Cell[636864, 17310, 1361, 30, 131, "Text"],
Cell[638228, 17342, 1161, 32, 141, "Input",
 InitializationCell->True],
Cell[639392, 17376, 100, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[639517, 17382, 78, 1, 29, "Input"],
Cell[639598, 17385, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[639666, 17390, 197, 4, 29, "Input"],
Cell[639866, 17396, 8789, 211, 294, "Output"]
}, Open  ]],
Cell[648670, 17610, 63, 0, 20, "Text"],
Cell[648736, 17612, 2001, 52, 234, "Input",
 InitializationCell->True],
Cell[650740, 17666, 1495, 28, 128, "Text"],
Cell[652238, 17696, 3575, 83, 283, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[655838, 17783, 143, 3, 29, "Input"],
Cell[655984, 17788, 9376, 222, 294, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[665409, 18016, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE1"],
Cell[665492, 18019, 55, 0, 42, "Problem"],
Cell[665550, 18021, 531, 9, 65, "Text"],
Cell[666084, 18032, 462, 12, 36, "Text"],
Cell[666549, 18046, 1539, 41, 198, "Input",
 InitializationCell->True],
Cell[668091, 18089, 158, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[668274, 18096, 335, 11, 29, "Input",
 InitializationCell->True],
Cell[668612, 18109, 1553, 38, 30, "Output"]
}, Open  ]],
Cell[670180, 18150, 309, 10, 28, "Input",
 InitializationCell->True],
Cell[670492, 18162, 274, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[670791, 18174, 402, 12, 45, "Input"],
Cell[671196, 18188, 8119, 188, 463, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[679364, 18382, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE3"],
Cell[679447, 18385, 152, 3, 42, "Problem"],
Cell[679602, 18390, 445, 8, 50, "Text"],
Cell[680050, 18400, 372, 7, 51, "Text"],
Cell[CellGroupData[{
Cell[680447, 18411, 203, 4, 61, "Input",
 InitializationCell->True],
Cell[680653, 18417, 146, 1, 61, "Output"]
}, Open  ]],
Cell[680814, 18421, 210, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[681049, 18431, 128, 3, 29, "Input",
 InitializationCell->True],
Cell[681180, 18436, 1945, 27, 77, "Output"]
}, Open  ]],
Cell[683140, 18466, 395, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[683560, 18482, 117, 3, 28, "Input"],
Cell[683680, 18487, 159, 6, 39, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[683876, 18498, 116, 3, 28, "Input"],
Cell[683995, 18503, 157, 6, 39, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[684189, 18514, 175, 5, 28, "Input"],
Cell[684367, 18521, 32, 0, 28, "Output"]
}, Open  ]],
Cell[684414, 18524, 481, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[684920, 18540, 103, 2, 29, "Input"],
Cell[685026, 18544, 2069, 62, 61, "Output"]
}, Open  ]],
Cell[687110, 18609, 315, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[687450, 18620, 104, 2, 29, "Input"],
Cell[687557, 18624, 29, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[687623, 18629, 348, 11, 29, "Input",
 InitializationCell->True],
Cell[687974, 18642, 2819, 92, 195, "Output"]
}, Open  ]],
Cell[690808, 18737, 114, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[690947, 18745, 255, 6, 29, "Input",
 InitializationCell->True],
Cell[691205, 18753, 58324, 1388, 293, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[749566, 20146, 73, 1, 29, "Input"],
Cell[749642, 20149, 59474, 1440, 423, "Output"]
}, Open  ]],
Cell[809131, 21592, 123, 3, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[809291, 21600, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE8"],
Cell[809374, 21603, 183, 6, 42, "Problem"],
Cell[809560, 21611, 663, 14, 66, "Text"],
Cell[810226, 21627, 502, 8, 65, "Text"],
Cell[CellGroupData[{
Cell[810753, 21639, 544, 15, 44, "Input",
 InitializationCell->True],
Cell[811300, 21656, 659, 19, 88, "Output"]
}, Open  ]],
Cell[811974, 21678, 265, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[812264, 21688, 192, 5, 17, "Item1Numbered"],
Cell[812459, 21695, 374, 12, 31, "Item1Numbered"],
Cell[812836, 21709, 994, 23, 83, "Item1Numbered"],
Cell[813833, 21734, 833, 23, 59, "Item1Numbered"],
Cell[814669, 21759, 275, 6, 30, "Item1Numbered"],
Cell[814947, 21767, 521, 12, 43, "Item1Numbered"],
Cell[815471, 21781, 306, 6, 30, "Item1Numbered"],
Cell[815780, 21789, 382, 7, 43, "Item1Numbered"],
Cell[816165, 21798, 402, 8, 43, "Item1Numbered"],
Cell[816570, 21808, 95, 2, 16, "Item1Numbered"]
}, Open  ]],
Cell[816680, 21813, 37, 0, 20, "Text"],
Cell[816720, 21815, 11154, 254, 688, "Input",
 InitializationCell->True],
Cell[827877, 22071, 66, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[827968, 22075, 241, 7, 44, "Input",
 InitializationCell->True],
Cell[828212, 22084, 2090, 59, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[830339, 22148, 291, 9, 44, "Input",
 InitializationCell->True],
Cell[830633, 22159, 2090, 59, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[832760, 22223, 291, 9, 44, "Input",
 InitializationCell->True],
Cell[833054, 22234, 2114, 60, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[835205, 22299, 291, 9, 44, "Input",
 InitializationCell->True],
Cell[835499, 22310, 1459, 42, 77, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[837019, 22359, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[837096, 22364, 316, 7, 33, "Item1Exercise"],
Cell[837415, 22373, 243, 6, 33, "Item1Exercise"],
Cell[837661, 22381, 180, 5, 20, "Item1Exercise"],
Cell[837844, 22388, 108, 2, 20, "Item1Exercise"],
Cell[CellGroupData[{
Cell[837977, 22394, 443, 10, 43, "Item2Exercise"],
Cell[838423, 22406, 499, 15, 43, "Item2Exercise"],
Cell[838925, 22423, 252, 6, 16, "Item2Exercise"],
Cell[839180, 22431, 807, 24, 57, "Item2Exercise"]
}, Open  ]],
Cell[840002, 22458, 386, 9, 47, "Item1Exercise"],
Cell[840391, 22469, 730, 13, 85, "Item1Exercise"],
Cell[841124, 22484, 271, 7, 35, "Item1Exercise"],
Cell[841398, 22493, 289, 7, 33, "Item1Exercise"],
Cell[841690, 22502, 863, 21, 74, "Item1Exercise"],
Cell[842556, 22525, 397, 8, 46, "Item1Exercise"],
Cell[842956, 22535, 267, 7, 33, "Item1Exercise"],
Cell[843226, 22544, 291, 7, 33, "Item1Exercise"],
Cell[843520, 22553, 185, 6, 20, "Item1Exercise"],
Cell[843708, 22561, 176, 5, 20, "Item1Exercise"],
Cell[843887, 22568, 299, 7, 33, "Item1Exercise"],
Cell[844189, 22577, 151, 4, 21, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
